#!/bin/sh
# the next line restarts using tclsh \
    exec tclsh "$0" "$@"

set scriptdir [file dirname [info script]]
set libDirectory [file join $scriptdir lib]
set auto_path [linsert $auto_path 0 $libDirectory]
set servercfg server.cfg

package require buildsystem
namespace import buildsystem::*

#parse commandline options
buildsystem::parseConfigSourceArgs $::argv
array set globaloptions [buildsystem::getGlobalOptions]
if { $globaloptions("createPackage") == 1 } {
    set createPackage 1
} else {
    set createPackage 0
}
if { $globaloptions("buildPackage") == 1 } {
    set buildPackage 1
} else {
    set buildPackage 0
}
if { $globaloptions("nosync") == 1 } {
    puts "### --- Configsource called with nosync set to 1"
}

buildsystem::parse_defconfig $globaloptions("defconfig")
# read opensource include list
# this is for forcefully including none opensource components in a opensource build

buildsystem::parse_include_list $globaloptions("opensource_include")

# read the component exclude list
buildsystem::parse_component_exclude_list $globaloptions("component_exclude") 

#read the build.cfg
if { $buildPackage == 0 } {
    array set globalbuildcfg [buildsystem::read_servers $globaloptions("buildconfigfile")]
} else {
    array set globalbuildcfg [buildsystem::read_servers $servercfg]
}

#read componentlist
array set globalcomponentlst [buildsystem::read_components "$scriptdir/component.lst"]

#read rootpath.cfg
buildsystem::read_rootpathcfg "rootpath.cfg"

# we directly read out a dependency file which acts more as a meta package
set cnames [buildsystem::read_componentdeps $globaloptions("projectdependenciesfile")]
buildsystem::read_cs_deps $cnames
buildsystem::check_deps
########################################
# generate buildconfig.default and Component.in
buildsystem::ConvertBuildconfigInfoToConfigIn
buildsystem::buildConfigAndComponentAll $scriptdir

########################################

########################################
# create staging and release dir
if { $buildPackage == 0 } {
    buildsystem::createStagingAndReleaseDir 0
} else {
    buildsystem::createStagingAndReleaseDir 2 
}

########################################

########################################
# generate component Makefile
if { $buildPackage == 0 } {
    buildsystem::generateCompMakeFile 0
} else {
    buildsystem::generateCompMakeFile 2
}
########################################
