#!/bin/bash


usage() {
  echo "Usage:" >&2
  echo "  $0 <config directory> <output path>" >&2
  exit 1
}

if [ -z "$1" ] || [ -z "$2" ] || [ ! -d "$1" ]; then
  usage
fi

if [ -f server.cfg ]
then
	servercfgfile="server.cfg"
else
	echo "** ERROR: server.cfg not found."
	exit 1
fi

echo "** Server configuration file: $servercfgfile"



config_path=$1
tar_path=$2
config=`find $config_path -name project.list | sed 's/\/project\.list//g'`

echo "** Config: $config"

del_list=""

wsbasedir=`pwd | sed 's,build.*,,g' | sed 's,/*[^/]\+/*$,,'`

n1_ws=`cat $servercfgfile | grep n1 | sed 's/^.*1999\///'`
w1_ws=`cat $servercfgfile | grep w1 | sed 's/^.*1666\///'`

n1_ws_dir="$wsbasedir/$n1_ws"
w1_ws_dir="$wsbasedir/$w1_ws"

echo "** n1_ws_dir: $n1_ws_dir"
echo "** w1_ws_dir: $w1_ws_dir"

if [ "$n1_ws_dir" == "$w1_ws_dir" ]
then
	workspace_path="$n1_ws_dir"
else	
	workspace_path="$w1_ws_dir $n1_ws_dir"
fi

echo "workspace path: $workspace_path"

mkdir -p $tar_path

config_name=`echo $config | sed 's/^.*config\///' | sed 's/\//\_/g'`
echo "Config Name $config_name"
echo "Fetching $config"

for ws in $workspace_path
do
	rm -rf $ws/*
done

./build.sh $config nobuild

echo "Appending Component.lic to the list"
lic_list=""
for ws in $workspace_path
do
	echo "Workspace $ws"
	tmp_list=`find $ws -name Component.lic`
	lic_list="$lic_list $tmp_list"
done

echo "lic_list = $lic_list"


echo "Looking for :opensource:"
for cp in $lic_list
do
	grep ":opensource:" $cp > /dev/null
	if [ $? -eq 1 ]
	then
		cp_dir=`dirname $cp`
		echo "Removing $cp_dir"
		rm -rf $cp_dir
	else	
		grep "sah:proprietary" $cp > /dev/null
		if [ $? -eq 0 ]
		then
			cp_dir=`dirname $cp`
			echo "!!! WARNING : both proprietary and opensource code in component $cp_dir"
			echo "Removing $cp_dir"
			rm -rf $cp_dir
		else
			echo "Keeping $cp"
		fi
	fi
done

echo "Removing special components"
for i in $del_list
do
	echo "Removing $i"
	rm -rf $i
done

echo "Removing empty directories"
for ws in $workspace_path
do
	find $ws/ -type d -empty -delete
done

echo "Removing symlink to output directory"
rm -rf $workspace_path/output

to=${tar_path}/$config_name.tar.bz

cd $workspace_path
echo "Creating list of components with their opensource licence"
rgrep :opensource: . > licences.txt
echo "Backing up $workspace_path to $to"
tar -cjf $to *
