package provide buildsystem 1.0

namespace eval buildsystem {
    variable globaloptions
        variable globalbuildcfg
        variable globalcomponentlst
        variable globalrootpathlst

        set packagetargetdir "./output/package"
        set package_buildconfig_default "buildconfig.default.m4"

        set globaloptions("buildconfigfile") "$::env(HOME)/.server.cfg"

        namespace export parse_cs_commandline read_cs_deps generatePackage
}

# parse configsource commandline options
# options are returned in commandlineoptions as an associative list
proc buildsystem::parseConfigPackagesArgs {argv} {
    variable globaloptions
    set options {
        {defconfig.arg  "set to defconfig"}
        {p.arg          "set to projectdependenciesfile"}
        {h.arg          "set to hardcofile"}
        {s.arg          "set to configurationfile"}
        {o.arg          "set to outputdirectory"}
        {b.arg          "set to buildconfig.default file"}
        {v.arg          "set to version_info file"}
        {lic.arg        "set to licence_info file"}
        {cl.arg         "set to changelist_info file"}
        {l.arg          "use specific release directory "}
        {g.arg          "use specific staging directory "}
        {f              "enable parallel building"}
        {r              "get the rootpath from the version control system"}
        {t              "enable tracing"}
        {ci             "continuous integration"}
        {c              "use ccache"}
        {x              "differentComponentReleaseDir"}
        {y              "use legacy releasedir "}
        {z              "create packaige"}
        {n              "do not track in workspace"}
        {nf             "do not force sync operations"}
        {opensource_only     "Build only the opensource packages"}
    }

    set globaloptions("gendefaultbuildconfig") 1
    set globaloptions("ccache") 0
    set globaloptions("differentComponentReleaseDir") 0
    set globaloptions("createPackage") 0
    set globaloptions("buildPackage") 0
    set globaloptions("version_info") "versioninfo.txt"
    set globaloptions("licence_info") "licenceinfo.txt"
    set globaloptions("changelist_info") "changelistinfo.txt"
    set globaloptions("parallel_make") 0
    set globaloptions("continuous_integration") 0
    set globaloptions("notrackws") 0
    set globaloptions("modular_packages") 0
    set globaloptions("noforcesync") 0
    set globaloptions("regendefconfig") 0
    set globaloptions("nosync") 0
    set globaloptions("os_as_source") 0


    if {[llength $argv]} {
        while {[llength $argv]} {

            set err [::cmdline::getopt argv {defconfig.arg s.arg p.arg h.arg o.arg b.arg v.arg lic.arg l.arg cl.arg g.arg n nf f r t ci c d x y z opensource_only} optVar optValue]

                if {$err == 1} {
                    switch $optVar {
                        "defconfig" {
                            set globaloptions("defconfig") [file join $optValue]
                        }
                        "p" {
                            set globaloptions("projectdependenciesfile") [file join $optValue]
                        }
                        "h" {
                            set globaloptions("hardcofile") $optValue
                        }
                        "s" {
                            set globaloptions("buildconfigfile") $optValue
                        }
                        "o" {
                            set globaloptions("outputdirectory") $optValue
                        }
                        "b" {
                            set globaloptions("buildconfigdefault") $optValue
                        }
                        "v" {
                            set globaloptions("version_info") $optValue
                        }
                        "lic" {
                            set globaloptions("licence_info") $optValue
                        }
                        "cl" {
                            set globaloptions("changelist_info") $optValue
                        }
                        "r" {
                            set globaloptions("rootpathfromvcs") "1"
                        }
                        "t" {
                            log::lvSuppress debug 0
                        }
                        "ci" {
                            set globaloptions("continuous_integration") 1
                        }
                        "f" {
                            set globaloptions("parallel_make") 1
                        }
                        "l" {
                            set globaloptions("releasedir") $optValue
                        }
                        "g" {
                            set globaloptions("stagingdir") $optValue
                        }
                        "c" {
                            set globaloptions("ccache") 1
                        }
                        "x" {
                            set globaloptions("differentComponentReleaseDir") 1
                        }
                        "y" {
                            set globaloptions("legacyreleasedir") 1
                        }
                        "z" {
                            set globaloptions("createPackage") 1
                        }
                        "opensource_only" {
                            setOpenSourceBuild 1
                        }
                        "n" {
                            set globaloptions("notrackws") 1
                        }
                        "nf" {
                            set globaloptions("noforcesync") 1
                        }
                        "modular_packages" {
                            set globaloptions("modular_packages") 1
                        }
                    }
                } elseif {$err < 0 } {
                    set usage [::cmdline::usage $options]
                        puts $usage
                        exit 1
                } else {
                    exit 1
                }
        }
    } else {
        set usage [::cmdline::usage $options]
            puts $usage
            exit 1
    }
}

