package provide buildsystem 1.0

namespace eval buildsystem {
    variable globaloptions
    variable globalbuildcfg
    variable globalcomponentlst
    variable checkpath
    variable globalrootpathlst
    variable versionpath

    set globaloptions("buildconfigfile") "$::env(HOME)/.server.cfg"

  namespace export parse_ct_commandline
}

# parse configsource commandline options
# options are returned in commandlineoptions as an associative list
proc buildsystem::parseConfigTestArgs {argv} {
    variable globaloptions
    set options {
        {p.arg     "set to projectdependenciesfile"}
        {h.arg     "set to hardcofile"}
	{s.arg     "set to configurationfile"}
	{o.arg     "set to outputdirectory"}
	{b.arg     "set to buildconfig.default file"}
	{l.arg     "use specific release directory "}
	{g.arg     "use specific staging directory "}
	{r         "get the rootpath from the version control system"}
    {nosync              "nosync"}
	{t         "enable tracing"}
	{c	   "use ccache"}
	{y     "use legacy releasedir "}
	{x     "install all components in their own private release dir"}
    }

    set globaloptions("gendefaultbuildconfig") 1
    set globaloptions("ccache") 0
    set globaloptions("differentComponentReleaseDir") 0
    set globaloptions("continuous_integration") 0
    set globaloptions("buildPackage") 0
    set globaloptions("nosync") 0

    if {[llength $argv]} {
	while {[llength $argv]} {

	    set err [::cmdline::getopt argv {s.arg p.arg h.arg o.arg b.arg l.arg g.arg r nosync t c d y x} optVar optValue]

	    if {$err == 1} {
		switch $optVar {
		    "p" {
			set globaloptions("projectdependenciesfile") $optValue
		    }
		    "h" {
			set globaloptions("hardcofile") $optValue
		    }
		    "s" {
			set globaloptions("buildconfigfile") $optValue
		    }
		    "o" {
			set globaloptions("outputdirectory") $optValue
		    }
		    "b" {
		        set globaloptions("buildconfigdefault") $optValue
		    }
		    "r" {
			set globaloptions("rootpathfromvcs") "1"
		    }
            "nosync" {
            set globaloptions("nosync") 1
            }
		    "t" {
			log::lvSuppress debug 0
		    }
		    "l" {
            		set globaloptions("releasedir") $optValue
            	    }
            	    "g" {
            		set globaloptions("stagingdir") $optValue
                    }
		    "c" {
			set globaloptions("ccache") 1
	            }
		    "x" {
    			set globaloptions("differentComponentReleaseDir") 1
		    }
		}
	    } elseif {$err < 0 } {
		set usage [::cmdline::usage $options]
		puts $usage
		exit 1
	    } else {
		exit 1
	    }
	}
    } else {
	set usage [::cmdline::usage $options]
	puts $usage
	exit 1
    }
}

proc buildsystem::getGlobalOptions {} {
        variable globaloptions
        return [array get globaloptions]
}


# Create runtest.sh
proc buildsystem::createRunTestScript {} {
    file mkdir "output/test"

    set runtest_script [open "output/test/runtest.sh" "w"]

    puts $runtest_script "#!/bin/bash"
    puts $runtest_script "# runtest.sh"

    # switch to the test directory
    puts $runtest_script "DIRECTORY=\$(cd `dirname \$0` && pwd)"
    puts $runtest_script "echo \"Switching to \$DIRECTORY\""
    puts $runtest_script "cd \$DIRECTORY"
    puts $runtest_script ""
    puts $runtest_script "source env.sh"
    puts $runtest_script ""


    # print header    
    puts $runtest_script "echo \"**********************************************************\""
    puts $runtest_script "echo \"Running tests in `pwd`\""
    puts $runtest_script "echo \"Current date is `date`\""
    puts $runtest_script "echo \"**********************************************************\""
    puts $runtest_script ""


    # initialize test script
    puts $runtest_script "./initializetest.sh"
    puts $runtest_script ""

    # loop over all tests
    puts $runtest_script "echo \"**** Launching unit tests\""
    puts $runtest_script "CONFIGLOCATION=`cat test.configlocation`"
    puts $runtest_script "CONFIGVERSION=`cat test.configversion`"
    puts $runtest_script "SPIN=`cat test.spin`"
    puts $runtest_script "echo \"<?xml version=\\\"1.0\\\"?>\" > \$TESTRESULTSDIR/result.xml"
    puts $runtest_script "echo \"<testrun config=\\\"\$CONFIGLOCATION\\\" configversion=\\\"\$CONFIGVERSION\\\" spinnumber=\\\"\$SPIN\\\">\" >> \$TESTRESULTSDIR/result.xml"
    puts $runtest_script "for dir in *"
    puts $runtest_script "do"
    puts $runtest_script "  if \[ -d \"\$dir\" \]; then"
    puts $runtest_script "    NEWCOMPONENT=1"
    puts $runtest_script "    echo \"**** Launching unit test in \$dir\""
    puts $runtest_script "    for file in \"\$dir\"/*_testplan.sh"
    puts $runtest_script "    do"
    puts $runtest_script "      TESTPLANTYPE=`\"\$file\" --type;`"
    puts $runtest_script "      if \[ -f \"\$file\" \] && \[ \$TESTPLANTYPE = \$TESTTYPE \]; then"
    puts $runtest_script "        if \[ \$NEWCOMPONENT = 1 \]; then"
    puts $runtest_script "          NEWCOMPONENT=0"
    puts $runtest_script "          CVERSION=`cat \$dir/component.version | sed s/export\\ COMPONENT_VERSION=//`"
    puts $runtest_script "          CID=`cat \$dir/component.name | sed s/export\\ COMPONENT_NAME=//`"
    puts $runtest_script "          echo \"<component id=\\\"\$CID\\\" version=\\\"\$CVERSION\\\">\" >> \$TESTRESULTSDIR/result.xml"
    puts $runtest_script "        fi"
    puts $runtest_script "        echo \"**** Launching testplan \$file\""
    puts $runtest_script "        \"\$file\";"
    puts $runtest_script "        TESTPLAN_NAME=`basename \$file .sh`;"
    puts $runtest_script "        cat \"\$TESTRESULTSDIR/\$dir/\${TESTPLAN_NAME}_result.xml\" | grep -v \"<?xml version=\" | grep -v \"testrun>\" | grep -v \"<testrun config\" | grep -v \"<component id\" | grep -v \"</component\" >> \$TESTRESULTSDIR/result.xml;"
    puts $runtest_script "      fi"
    puts $runtest_script "    done"
    puts $runtest_script "    if \[ \$NEWCOMPONENT = 0 \]; then"
    puts $runtest_script "      NEWCOMPONENT=0"
    puts $runtest_script "      echo \"</component>\" >> \$TESTRESULTSDIR/result.xml"
    puts $runtest_script "    fi"
    puts $runtest_script "  fi"
    puts $runtest_script "done"
    puts $runtest_script "echo \"</testrun>\" >> \$TESTRESULTSDIR/result.xml"
    puts $runtest_script ""
       
    # run cleanup test script
    puts $runtest_script "./cleanuptest.sh"
    puts $runtest_script ""
    puts $runtest_script ""
    puts $runtest_script ""
    close $runtest_script
  
    # make the script executable
    file attributes "output/test/runtest.sh" -permissions "rwxrwxrwx"

}

proc checkconfig {arg1 arg2} {
	if {[string match "$arg1=*" $arg2] == 1} {
		return 1
	}
return 0
}
proc setconfig {arg1} {
	set y [split $arg1 "="]
	return [lindex $y 1]	
}

# Create initializetest.sh
proc buildsystem::createInitializeTestScript {} {
    file mkdir "output/test"
    set inittest_script [open "output/test/initializetest.sh" "w"]

    puts $inittest_script "#!/bin/bash"
    puts $inittest_script "# initializetest.sh"
    puts $inittest_script ""
    puts $inittest_script "source env.sh"
    puts $inittest_script ""


    # initialize
    puts $inittest_script "rm \"\$TESTRESULTSDIR/result.xml\" 2>/dev/null"
    puts $inittest_script "rm \"\$TESTRESULTSDIR/source.dirs\" 2>/dev/null"
   

    set testtype "remote"
    if {[file exists "output/test/test.config"] == 1} {
      #  Slurp up the data file
      set fp [open "output/test/test.config" r]
      set file_data [read $fp]
      close $fp
      #  Process data file
      set data [split $file_data "\n"]
      foreach line $data {
        if {[string length $line] > 1 } {
          set x [checkconfig "TESTTYPE" $line]
          if {$x == 1} { set testtype [setconfig $line] }
        }
      }
    }

    # remote initialisation
    if {[string match *remote* $testtype] == 1} {
      puts $inittest_script "mycmd=\"\\\"\\"
      puts $inittest_script "export HOSTIP=\$HOSTIP;\\"
      puts $inittest_script "export NFSMOUNT=\$NFSMOUNT;\\"
      puts $inittest_script "mount -t nfs -o async,noatime,exec,rw,nfsvers=2,nolock \$HOSTIP:'\$NFSMOUNT' /mnt/nfs;\\"
      puts $inittest_script "\\\"\""
      puts $inittest_script "eval \"ssh -i \$TESTDIR/id_rsa -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no root@\$TARGETIP \$mycmd\""
    }
    close $inittest_script
  
    # make the script executable
    file attributes "output/test/initializetest.sh" -permissions "rwxrwxrwx"

}

# Create env.sh
proc buildsystem::createInitializeEnvironmentTestScript {} {
    variable globaloptions
    file mkdir "output/test"

    set envtest_script [open "output/test/env.sh" "w"]

    puts $envtest_script "#!/bin/bash"
    puts $envtest_script "# env.sh"
    puts $envtest_script ""

    # initialize
    set outputdir $globaloptions("outputdirectory")
    puts $envtest_script "export STAGINGDIR=$outputdir/staging"
    puts $envtest_script "export TESTDIR=$outputdir/test"
    puts $envtest_script "export TESTRESULTSDIR=$outputdir/testresults"
    puts $envtest_script "export REMOTETESTDIR=\"\$(echo \"$outputdir/test\" | sed 's/\\/home\\/\[a-zA-Z0-9\]*\\//\\/mnt\\/nfs\\//')\""
    puts $envtest_script "export REMOTETESTRESULTSDIR=\"\$(echo \"$outputdir/testresults\" | sed 's/\\/home\\/\[a-zA-Z0-9\]*\\//\\/mnt\\/nfs\\//')\""
    puts $envtest_script "export NFSMOUNT=\"$::env(HOME)\""

    set rc [catch {exec ifconfig | grep {192\.168\.1\.}}]
    if {$rc == 0} {
      set hostip [exec ifconfig | grep {192\.168\.1\.} | grep -Eo {([0-9]{1,3}\.){3}[0-9]{1,3}} | head -1]
    } else {   
      set hostip "100.0.0.1"
    }

    if {[file exists "output/test/test.config"] == 1} {
      #  Slurp up the data file    
      set fp [open "output/test/test.config" r]
      set file_data [read $fp]
      close $fp
      #  Process data file
      set data [split $file_data "\n"]
      foreach line $data {
         if {[string length $line] > 1 } {
           puts $envtest_script "export $line"
         }
      }
      puts $envtest_script "export HOSTIP=\"$hostip\""
    } else {
      # generate default env settings
      puts $envtest_script "export HOSTIP=\"$hostip\""
      puts $envtest_script "export TESTTYPE=\"remote\""
      puts $envtest_script "export VALGRIND="
      puts $envtest_script "export TARGETIP=\"192.168.1.1\""
    }
    puts $envtest_script ""
    close $envtest_script
  
    # make the script executable
    file attributes "output/test/env.sh" -permissions "rwxrwxrwx"

}

# Create cleanuptest.sh
proc buildsystem::createCleanupTestScript {} {
    file mkdir "output/test"

    set cleantest_script [open "output/test/cleanuptest.sh" "w"]

    puts $cleantest_script "#!/bin/bash"
    puts $cleantest_script "# cleanuptest.sh"
    puts $cleantest_script ""
    puts $cleantest_script "source env.sh"
    puts $cleantest_script ""

    # cleanup
    puts $cleantest_script "echo \"**** Cleaning up test environment\""
    puts $cleantest_script ""

    set testtype "remote"
    if {[file exists "output/test/test.config"] == 1} {
      #  Slurp up the data file
      set fp [open "output/test/test.config" r]
      set file_data [read $fp]
      close $fp
      #  Process data file
      set data [split $file_data "\n"]
      foreach line $data {
        if {[string length $line] > 1 } {
          set x [checkconfig "TESTTYPE" $line]
          if {$x == 1} { set testtype [setconfig $line] }
        }
      }
    }

    # remote cleanup
    if {[string match *remote* $testtype] == 1} {
      puts $cleantest_script "ssh -i \$TESTDIR/id_rsa -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no root@\$TARGETIP 'umount /mnt/nfs'"
    }

    
    close $cleantest_script
  
    # make the script executable
    file attributes "output/test/cleanuptest.sh" -permissions "rwxrwxrwx"

}

# Create testsources.sh
proc buildsystem::createTestSourcesScript {} {
    file mkdir "output/test"

    set testsource_script [open "output/test/testsources.sh" "w"]

    puts $testsource_script "#!/bin/bash"
    puts $testsource_script "# testsources.sh"
    puts $testsource_script ""
    puts $testsource_script "# load the environment variables"
    puts $testsource_script "source ../env.sh"
    puts $testsource_script "source ./component.location"
    puts $testsource_script "source ./component.name"
    puts $testsource_script "source ./component.version"
    puts $testsource_script ""
    puts $testsource_script "case \$TESTTYPE in"
    puts $testsource_script "  remote)"
    puts $testsource_script "    # setup pcb environment"
    puts $testsource_script "    export PCB_SYS_BUS=\"pcb://ipc:{/var/run/pcb_sys}\""
    puts $testsource_script "    # variable used in the .so unit tests"
    puts $testsource_script "    export TESTRESULTFILE=\$REMOTETESTRESULTSDIR/\$COMPONENT_NAME/result.xml"
    puts $testsource_script "    ;;"
    puts $testsource_script "  remote_standalone)"
    puts $testsource_script "    # setup pcb environment"
    puts $testsource_script "    export PCB_SYS_BUS=\"pcb://ipc:{/var/run/pcb_sys}\""
    puts $testsource_script "    # variable used in the .so unit tests"
    puts $testsource_script "    export TESTRESULTFILE=\$REMOTETESTRESULTSDIR/\$COMPONENT_NAME/result.xml"
    puts $testsource_script "    ;;"
    puts $testsource_script "  local)"
    puts $testsource_script "    ;;"
    puts $testsource_script "  local_standalone)"
    puts $testsource_script "    # setup pcb environment"
    puts $testsource_script "    export PCB_SYS_BUS=\"pcb://ipc:{/tmp/sysbus}\""
    puts $testsource_script "    export SYS_BUS=\"/tmp/sysbus\""
    puts $testsource_script "    export PCB_PLUGIN_PATH=\$STAGINGDIR/usr/lib/pcb"
    puts $testsource_script "    export PATH=\$PATH:\$STAGINGDIR/bin"
    puts $testsource_script "    export LD_LIBRARY_PATH=\$STAGINGDIR/lib"
    puts $testsource_script "    # variable used in the .so unit tests"
    puts $testsource_script "    export TESTRESULTFILE=\$TESTRESULTSDIR/\$COMPONENT_NAME/result.xml"
    puts $testsource_script "    ;;"
    puts $testsource_script "  *)"
    puts $testsource_script "    echo \"*** Unknown test type (\$TESTTYPE)\""
    puts $testsource_script "    ;;"
    puts $testsource_script "esac"


echo "*** Scheduler unit test done!"

    close $testsource_script
  
    # make the script executable
    file attributes "output/test/testsources.sh" -permissions "rwxrwxrwx"
}










