namespace eval buildsystem {
    variable bs_components
    variable bs_projlist
    variable bs_is_opensource
    set bs_projlist ""
}

proc buildsystem::getPackageBuildDir {} {
    variable packagetargetdir

    set relative_build_location   [buildsystem::getComponent_relpath "build"]
    set package_build_dir "$packagetargetdir$relative_build_location"
    return $package_build_dir
}

proc buildsystem::cpMakePath {name version} {
    set rootpath    [buildsystem::get_root_path $name]
    set serverpath  [buildsystem::get_server_path $name]
    set vcs [buildsystem::getComponentVcs $name]
    if { "$vcs" == "p4" } {
        set path [file join $rootpath/$serverpath/$version]
    } else {
        set path [file join $rootpath/$serverpath/]
    }
    return $path
}

proc buildsystem::getComponent_relpath {name} {
    variable bs_components

    array set componentstruct   [lindex $bs_components($name) 0]
    set srvpath [buildsystem::get_server_path $name]
    set vcs [buildsystem::getComponentVcs $name]
    if { "$vcs" == "p4" } {
        return [file join "$srvpath/$componentstruct(version)"]
    } else {
        return [file join "$srvpath/"]
    }
}

proc buildsystem::setComponent_license {name license} {
    variable bs_components

    array set componentstruct       [lindex $bs_components($name) 0]

    set componentstruct(license)    $license
    set bs_components($name)        [list [array get componentstruct]]
}

proc buildsystem::setComponent_changelist {name changelist} {
    variable bs_components

    array set componentstruct       [lindex $bs_components($name) 0]

    set componentstruct(changelist)    $changelist
    set bs_components($name)        [list [array get componentstruct]]
}

proc buildsystem::getComponent_changelist {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(changelist)
}

proc buildsystem::setComponent_rel_as_src {name} {
    variable bs_components

    array set componentstruct       [lindex $bs_components($name) 0]

    set componentstruct(release_as_binary)    0
    set bs_components($name)        [list [array get componentstruct]]
}

proc buildsystem::getComponent_raw_dependencies {name} {
    variable bs_components

    array set componentstruct   [lindex $bs_components($name) 0]
    return $componentstruct(dependencies)
}

proc buildsystem::setComponent_dependencies {name dependencies} {
    variable bs_components

    array set componentstruct   [lindex $bs_components($name) 0]

    set componentstruct(dependencies)   $dependencies

    set bs_components($name)    [list [array get componentstruct]]
}

proc buildsystem::getComponent_enable_debug {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(enable_debug)
}

proc buildsystem::getComponent_debug_options {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    set debugoptions ""
    foreach ldebug $componentstruct(debug_options) {
        #lappend will generate a list and this will result in an empty list {} in the Makefile.components
        set debugoptions "$debugoptions $ldebug"
    }
    return $debugoptions
}

proc buildsystem::getComponent_rel_as_bin {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(release_as_binary)
}

proc buildsystem::getComponent_deppath {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(deppath)
}

proc buildsystem::getComponent_dependencies_with_suffix {name suffix} {
    set new_deps ""
    set deps [buildsystem::getComponent_dependencies $name]

    foreach dep $deps {
        lappend new_deps ${dep}${suffix}
    }
    return $new_deps
}

proc buildsystem::isComponentDepOptional {name dep_name} {
    variable    bs_components
    variable    bs_is_opensource
    set         dep_list        ""
    array set componentstruct   [lindex $bs_components($name) 0]

    foreach {deps assgm version} $componentstruct(dependencies) {
        if {![defconfig_name_is_set $deps]} {
            continue
        }

        if {![string compare $deps $dep_name]} {
            if {![string compare $version "?"]} {
                return 1
            }
        }
    }
    return 0
}

proc buildsystem::getComponent_dependencies {name} {
    variable    bs_components
    variable    bs_is_opensource
    set         dep_list        ""
    array set componentstruct   [lindex $bs_components($name) 0]

    foreach {deps assgm version} $componentstruct(dependencies) {
        if {$bs_is_opensource} {
            if {![defconfig_name_is_set $deps]} {
                continue
            }
            #Always add the components when the version is not set to ?
            if {[string compare $version "?"]} {
                lappend dep_list $deps
            #If the version is set to ? and the component doesn't exist.
            #Then don't add it to the list
            } elseif {[componentExists $deps]} {
                lappend dep_list $deps
            }
        } else {
            if {[buildsystem::componentExists $deps]} {
                lappend dep_list $deps
            }
        }
    }
    return $dep_list
}

proc buildsystem::getComponent_licfile {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(licfile)
}

proc buildsystem::getComponent_mkfile {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(mkfile)
}

proc buildsystem::getComponent_infile {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(infile)
}

proc buildsystem::getComponent_depfile {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(depfile)
}

proc buildsystem::getComponent_assignment {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(assignment)
}

proc buildsystem::getComponent_version {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(version)
}

proc buildsystem::getComponent_license {name} {
    variable bs_components
    array set componentstruct   [lindex $bs_components($name) 0]

    return $componentstruct(license)
}

proc buildsystem::isComponent_opensource {name} {
    
    if {[opensrc_is_excluded $name]} {
        return 0
    }

    if {[opensrc_is_included $name]} {
        return 1
    }

    #if {[isComponent_opensource_bsd $name]} {
    #puts "LicenceType = BSD -> Forcing component $name to be not opensource!!!"
    #return 0
    #}

    if {[isComponent_opensource_gpl $name]} {
        return 1
    }

    foreach {license licenseType} [getComponent_license $name] {
        if {![string compare $license "opensource"]} {
            return 1
        }
    }
    return 0
}

proc buildsystem::isComponent_completely_opensource {name} {
    
    set opensource false

    foreach {license licenseType} [getComponent_license $name] {
        if {[licenseIsGPLV2Style $license $licenseType]} {
            return 1
        }
    }

    foreach {license licenseType} [getComponent_license $name] {
        if {![string compare $license "opensource"]} {
            set opensource true
        } elseif {![string compare $licenseType "proprietary"]} {
            return 0
        }
    }
    return $opensource
}

proc buildsystem::isComponent_opensource_bsd {name} {
    set cp_is_forced [opensrc_is_included $name]
    if {$cp_is_forced} {
        return 1
    }

    foreach {license licenseType} [getComponent_license $name] {
        if {[licenseIsBSDStyle $license $licenseType]} {
            return 1
        }
    }
    return 0
}

proc buildsystem::isComponent_opensource_gpl {name} {
    set cp_is_forced [opensrc_is_included $name]
    if {$cp_is_forced} {
        return 1
    }

    foreach {license licenseType} [getComponent_license $name] {
        if {[licenseIsGPLV2Style $license $licenseType]} {
            return 1
        }
    }
    return 0
}

proc buildsystem::makefile_getComponents {suffix} {
    set line ""
    foreach name [getComponents] {
        lappend line ${name}${suffix}
    }
    return $line
}

proc buildsystem::getComponents {} {
    variable bs_components

    set line ""
    foreach name [array names bs_components] {
        lappend line ${name}
    }
    return $line
}

proc buildsystem::makefile_getProjComponents {suffix} {
    variable bs_components
    variable bs_projlist
    variable bs_is_opensource

    if {$bs_is_opensource} {
        return [makefile_getComponents $suffix]
    } else {
        set line ""

#foreach name [array names bs_components]
        foreach name $bs_projlist {
            array set componentstruct   [lindex $bs_components($name) 0]
            if {[info exists componentstruct(proj_component)]} {
                lappend line ${name}${suffix}
            }
        }
        return $line
    }
}

proc buildsystem::delComponents {name} {
    variable bs_components
    array unset bs_components $name
}

proc buildsystem::is_undefined_version {version} {
    return [regexp {^\*|\?$} $version]
}

proc buildsystem::addComponent {name filename assignment version rel_as_bin debug_opt} {
    variable bs_components
    variable globaloptions
    variable bs_projlist

    #
    # When BUILD_MODE_RELEASE is set, versions which are not releases 
    # are not allowed in component dependencies.
    # In project.lst all types of versions are allowed.
    #
    
    if {[string compare $filename $globaloptions("projectdependenciesfile")]} {
        if {[defconfig_name_is_set "BUILD_MODE_RELEASE"]} {
            set vcs [buildsystem::getComponentVcs $name]
            if {$vcs == "p4"} {
                if {![regexp {^.*REL\/\d+-\d+-\d+_V.*$} $version]} {
                    if {![is_undefined_version $version]} {
                       puts "\033\[31mError: Incorrect P4 component version detected! ($name=$version)\033\[0m"
#                       exit 1
                    } 
                }
            } elseif {$vcs == "git"} {
                if {![regexp {^refs\/tags\/.*_v.*$} $version]} {
                    if {![is_undefined_version $version]} {
                       puts "\033\[33mWarning: Incorrect GIT component version detected! ($name=$version)\033\[0m"
                       #exit 1
                    } 
                }
            } else {
                error "\033\[31mFATAL ERROR : unknown server protocol $proto\033\[0m"
            }
        }
    }

    set componentstruct(version)            $version
    set componentstruct(dependencies)       ""
    set componentstruct(assignment)         $assignment
    set componentstruct(depfile)            $filename
    set componentstruct(deppath)            [cpMakePath $name $version]
    set componentstruct(licfile)            [file join $componentstruct(deppath) "Component.lic"]
    set componentstruct(infile)             [file join $componentstruct(deppath) "Component.in"]
    set componentstruct(mkfile)             [file join $componentstruct(deppath) "Component.mk"]
    set componentstruct(license)            0
    set componentstruct(release_as_binary)  $rel_as_bin
    set componentstruct(changelist)         ""

    if {![string compare $filename $globaloptions("projectdependenciesfile")]} {
        set componentstruct(proj_component) 1
        lappend bs_projlist $name
    }

    if {[llength debug_opt] > 0} {
        set componentstruct(enable_debug)       1
        set componentstruct(debug_options)      $debug_opt
    } else {
        set componentstruct(enable_debug)       0
        set componentstruct(debug_options)      ""
    }
    set bs_components($name)                [list [array get componentstruct]]
    return 1
}

proc buildsystem::componentExists {name} {
    variable bs_components
    return [info exists bs_components($name)]
}

proc buildsystem::setComponent {name filename assignment version rel_as_bin debug_opt} {
    variable bs_components

    if {![info exists bs_components($name)]} {
        # new component => add it
        return [addComponent $name $filename $assignment $version $rel_as_bin $debug_opt]
    }

    array set componentstruct   [lindex $bs_components($name) 0]
    set bs_cp_version           $componentstruct(version)
    set bs_cp_dependencies      $componentstruct(dependencies)
    set bs_cp_assignment        $componentstruct(assignment)
    set bs_cp_depfile           $componentstruct(depfile)
    set bs_cp_deppath           $componentstruct(deppath)
    set bs_cp_is_forced         [expr [string compare "EQUALS" $bs_cp_assignment] == 0 ]

    set result [string compare $version $bs_cp_version]

    switch $assignment {
        "EQUALS" {
            if {$result != 0} {
                if {$bs_cp_is_forced} {
                    puts "\033\[31mError parsing $filename - component $name: Can't force the same component twice to a different version\033\[0m"
                    exit 1
                } else {
                    puts "\033\[33mWarning: override $name version $bs_cp_version with $version (forced by $filename)\033\[0m"
                    return [addComponent $name $filename $assignment $version $rel_as_bin $debug_opt]
                }
            }
        }
        "LESSER" {
            if {$result < 0} {
                if {$bs_cp_is_forced} {
                    puts "\033\[33mWarning: ignoring $name <= $version because it is forced to $bs_cp_version (forced by $filename)\033\[0m"
                } else {
                    return [addComponent $name $filename $assignment $version $rel_as_bin $debug_opt]
                }
            }
        }
        "GREATER" {
            if {$result > 0} {
                if {$bs_cp_is_forced} {
                    puts "\033\[33mWarning: ignoring $name >= $version because it is forced to $bs_cp_version (forced by $filename)\033\[0m"
                } else {
                    return [addComponent $name $filename $assignment $version $rel_as_bin $debug_opt]
                }
            }
        }
    }
    return 0
}

proc buildsystem::getAllComponentsInfo {} {
    variable bs_components
    return [array get bs_components]
}

proc buildsystem::clearAllComponents {} {
    variable bs_components
    array unset bs_components
    return 0
}

proc buildsystem::getModularComponents {} {
    set rv ""
    foreach comp [getComponents] {
        if { [string first "sah_packages_" $comp] != -1 } {
            puts "found $comp"
            lappend rv $comp
        }
    }
    return $rv
}
