package provide buildsystem 1.0

namespace eval buildsystem {
    variable globaloptions
        variable globalbuildcfg
        variable globalcomponentlst
        variable globalrootpathlst

        set packagetargetdir "./output/package"
        set package_buildconfig_default "buildconfig.default.m4"

        set globaloptions("buildconfigfile") "$::env(HOME)/.server.cfg"

        namespace export parse_cs_commandline read_cs_deps generatePackage
}


# parse genPkg commandline options
# options are returned in commandlineoptions as an associative list
proc buildsystem::parseGenPkgArgs {argv} {
    variable globaloptions
    set options {
        {defconfig.arg  "set to defconfig"}
        {osrc.arg           "set to opensource configuration file"}
        {compexcl.arg   "set the component exclude file"}
        {p.arg          "set to projectdependenciesfile"}
        {h.arg          "set to hardcofile"}
        {s.arg          "set to configurationfile"}
        {o.arg          "set to outputdirectory"}
        {b.arg          "set to buildconfig.default file"}
        {v.arg          "set to version_info file"}
        {lic.arg        "set to licence_info file"}
        {cl.arg         "set to changelist_info file"}
        {l.arg          "use specific release directory "}
        {g.arg          "use specific staging directory "}
        {f              "enable parallel building"}
        {r              "get the rootpath from the version control system"}
        {t              "enable tracing"}
        {ci             "continuous integration"}
        {c              "use ccache"}
        {nosync              "nosync"}
        {w		        "build package"}
        {x              "differentComponentReleaseDir"}
        {y              "use legacy releasedir "}
        {z              "create package"}
        {opensource_only     "Build only the opensource packages"}
    }

    set globaloptions("gendefaultbuildconfig") 1
    set globaloptions("ccache") 0
    set globaloptions("differentComponentReleaseDir") 0
    set globaloptions("createPackage") 0
    set globaloptions("buildPackage") 0
    set globaloptions("nosync") 0
    set globaloptions("version_info") "versioninfo.txt"
    set globaloptions("licence_info") "licenceinfo.txt"
    set globaloptions("changelist_info") "changelistinfo.txt"
    set globaloptions("parallel_make") 0
    set globaloptions("continuous_integration") 0
    set globaloptions("opensource_include") 0
    set globaloptions("os_as_source") 0
    set globaloptions("component_exclude") 0
    set globaloptions("opensource_only") 0

    if {[llength $argv]} {
        while {[llength $argv]} {

            set err [::cmdline::getopt argv {defconfig.arg osrc.arg compexcl.arg s.arg p.arg h.arg o.arg b.arg v.arg lic.arg cl.arg l.arg g.arg f r t ci c d nosync w x y z opensource_only} optVar optValue]
                if {$err == 1} {
                    switch $optVar {
                        "compexcl" {
                            set globaloptions("component_exclude") [file join $optValue]
                        }
                        "defconfig" {
                            set globaloptions("defconfig") [file join $optValue]
                        }
                        "osrc" {
                            set globaloptions("opensource_include") [file join $optValue]
                        }

                        "p" {
                            set globaloptions("projectdependenciesfile") [file join $optValue]
                        }
                        "h" {
                            set globaloptions("hardcofile") $optValue
                        }
                        "s" {
                            set globaloptions("buildconfigfile") $optValue
                        }
                        "o" {
                            set globaloptions("outputdirectory") $optValue
                        }
                        "b" {
                            set globaloptions("buildconfigdefault") $optValue
                        }
                        "v" {
                            set globaloptions("version_info") $optValue
                        }
                        "lic" {
                            set globaloptions("licence_info") $optValue
                        }
                        "cl" {
                            set globaloptions("changelist_info") $optValue
                        }
                        "r" {
                            set globaloptions("rootpathfromvcs") "1"
                        }
                        "t" {
                            log::lvSuppress debug 0
                        }
                        "ci" {
                            set globaloptions("continuous_integration") 1
                        }
                        "f" {
                            set globaloptions("parallel_make") 1
                        }
                        "l" {
                            set globaloptions("releasedir") $optValue
                        }
                        "g" {
                            set globaloptions("stagingdir") $optValue
                        }
                        "c" {
                            set globaloptions("ccache") 1
                        }
                        "nosync" {
                            set globaloptions("nosync") 1
                        }
                        "w" {
                            set globaloptions("buildPackage") 1
                        }
                        "x" {
                            set globaloptions("differentComponentReleaseDir") 1
                        }
                        "y" {
                            set globaloptions("legacyreleasedir") 1
                        }
                        "z" {
                            set globaloptions("createPackage") 1
                        }
                        "opensource_only" {
                            set globaloptions("opensource_only") 1
                            setOpenSourceBuild 1
                        }
                    }
                } elseif {$err < 0 } {
                    set usage [::cmdline::usage $options]
                        puts $usage
                        exit 1
                } else {
                    exit 1
                }

        }
    } else {
        set usage [::cmdline::usage $options]
            puts $usage
            exit 1
    }
}


proc buildsystem::prepare_ossKernel {} {
    puts "\n\n---- Building Kernel with prepare_oss target ----\n"
    foreach name [buildsystem::getComponents] {
        if { [string match *kernel_broadcom* $name] } {
            puts "Building  kernel here ..."
            exec make -f Makefile.components sah_kernel_broadcom_prepare_oss >&@stdout
        } 
    }
}

# Create staging dir, release dir and Makefile.global and sahcommon.mk
proc buildsystem::createStagingAndReleaseDir {generateForPackageCreation} {
    variable globaloptions
    variable packagetargetdir

    if { [info exists globaloptions("projectdependenciesfile")] } {
        if { $generateForPackageCreation == 1 } {
                set package_build_dir   [buildsystem::getPackageBuildDir]
                set stagingdir "__CWD__/output/staging"
                set releasedir "__CWD__/output/release"
        } else {
            if {[info exists globaloptions("stagingdir")]} {
                set stagingdir [string trim "$globaloptions("stagingdir")"]
            } else {
                set stagingdir [string trim "$globaloptions("outputdirectory")/staging"]
            }
            if {[info exists globaloptions("releasedir")]} {
                set releasedir [string trim "$globaloptions("releasedir")"]
            } else {
                set releasedir [string trim "$globaloptions("outputdirectory")/release"]
            }
        }

        if { $globaloptions("ccache") } {
            set ccache "ccache ";
        } else {
            set ccache "";
        }

        if {$generateForPackageCreation == 1} {
            #file mkdir "$package_build_dir/output/release"
            set global_makefile [open "$package_build_dir/Makefile.global.m4" "w"]
        } else {
            file mkdir $stagingdir
            file mkdir $releasedir

            puts "using releasedir $releasedir"
            set global_makefile [open "Makefile.global" "w"]
        }

        puts $global_makefile "# Makefile.global"
        puts $global_makefile ""

        if { $generateForPackageCreation == 1 } {
            puts $global_makefile "export GLOBALMAKEFILE=__CWD__/Makefile.global"
        } else {
            puts $global_makefile "export GLOBALMAKEFILE=$::env(PWD)/Makefile.global"
        }
        puts $global_makefile "export STAGINGDIR=$stagingdir"
        puts $global_makefile "export RELEASEDIR=$releasedir"
        puts $global_makefile "export ROOT_FS_DIR=$releasedir/root_fs"
        puts $global_makefile "export PKG_CONFIG_LIBDIR=$stagingdir/build/pkg-config"
        puts $global_makefile "export PKG_CONFIG_TOP_BUILD_DIR=$stagingdir"
        puts $global_makefile "export PKG_CONFIG_SYSROOT_DIR=$stagingdir"
        puts $global_makefile "export PKG_CONFIG_PATH="
        if {$generateForPackageCreation == 1} {
            puts $global_makefile "export OUTPUTDIR=__CWD__/output"
        } else {
            puts $global_makefile "export OUTPUTDIR=$globaloptions("outputdirectory")"
        }
        if { $globaloptions("ccache") } {
# we can only set one BASEDIR, so set it to the first
# rootpath and cross our fingers
            variable globalrootpathlst
            set myrootpath [lindex [array get globalrootpathlst] 1]
            puts $global_makefile "export CCACHE_BASEDIR=$myrootpath"
        }
        puts $global_makefile ""
        puts $global_makefile "quote := \""
        puts $global_makefile "empty :="
        puts $global_makefile ""

        puts $global_makefile "export CROSS_COMPILE=\$(subst \$(quote),\$(empty),\$(CONFIG_CROSS_COMPILE))"
        puts $global_makefile "SYSROOT = \$(addprefix \$(empty) ,\$(if \$(subst \$(quote),\$(empty),\$(CONFIG_BUILD_CROSS_COMPILE_SYSROOT)), --sysroot \$(CONFIG_BUILD_CROSS_COMPILE_SYSROOT)))"
        puts $global_makefile "CC_OPTIONS  = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_BUILD_CC_OPTIONS)))"
        puts $global_makefile "CXX_OPTIONS = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_BUILD_CXX_OPTIONS)))"
        puts $global_makefile "USER_CROSS_COMPILE_CPPFLAGS  = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_USER_CROSS_COMPILE_CPPFLAGS)))"
        puts $global_makefile "USER_CROSS_COMPILE_CFLAGS    = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_USER_CROSS_COMPILE_CFLAGS)))"
        puts $global_makefile "USER_CROSS_COMPILE_CXXFLAGS  = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_USER_CROSS_COMPILE_CXXFLAGS)))"
        puts $global_makefile "USER_CROSS_COMPILE_LDFLAGS   = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_USER_CROSS_COMPILE_LDFLAGS)))"
        puts $global_makefile "CROSS_COMPILE_CPPFLAGS       = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_BUILD_CROSS_COMPILE_CPPFLAGS)))"
        puts $global_makefile "CROSS_COMPILE_CFLAGS         = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_BUILD_CROSS_COMPILE_CFLAGS)))"
        puts $global_makefile "CROSS_COMPILE_CXXFLAGS       = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_BUILD_CROSS_COMPILE_CXXFLAGS)))"
        puts $global_makefile "CROSS_COMPILE_LDFLAGS        = \$(addprefix \$(empty) ,\$(subst \$(quote),\$(empty),\$(CONFIG_BUILD_CROSS_COMPILE_LDFLAGS)))"

        puts $global_makefile "CROSS_COMPILE_OPTIMIZE_FLAGS = \$(subst \$(quote),\$(empty),\$(if \$(filter y,\$(CONFIG_BUILD_MODE_RELWITHDBGINFO)),\$(CONFIG_BUILD_CROSS_COMPILE_OPTIMIZE_FLAGS_RELWITHDBGINFO))\$(if \$(filter y,\$(CONFIG_BUILD_MODE_RELEASE)),\$(CONFIG_BUILD_CROSS_COMPILE_OPTIMIZE_FLAGS_RELEASE))\$(if \$(filter y,\$(CONFIG_BUILD_MODE_DEBUG)),\$(CONFIG_BUILD_CROSS_COMPILE_OPTIMIZE_FLAGS_DEBUG)))"
        puts $global_makefile "AUTOCONF_HOST  = \$(shell \$(CROSS_COMPILE)gcc -dumpmachine)"
        puts $global_makefile "AUTOCONF_BUILD = \$(shell gcc -dumpmachine)"

        puts $global_makefile "# legacy DEV/STB build compat"
        puts $global_makefile "CONFIG_DEBUG = \$(CONFIG_BUILD_MODE_DEBUG)"
        puts $global_makefile ""

        puts $global_makefile "export AS            = \$(CROSS_COMPILE)as"
        puts $global_makefile "export LD            = \$(CROSS_COMPILE)ld\$(SYSROOT)"
        puts $global_makefile "export CC            = $ccache\$(CROSS_COMPILE)gcc\$(CC_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export CXX           = $ccache\$(CROSS_COMPILE)g++\$(CXX_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export C++           = $ccache\$(CROSS_COMPILE)c++\$(CXX_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export G++           = $ccache\$(CROSS_COMPILE)g++\$(CXX_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export REAL_CC       = \$(CROSS_COMPILE)gcc\$(CC_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export REAL_CXX      = \$(CROSS_COMPILE)g++\$(CXX_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export REAL_C++      = \$(CROSS_COMPILE)c++\$(CXX_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export REAL_G++      = \$(CROSS_COMPILE)g++\$(CXX_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export NASM          = \$(CROSS_COMPILE)nasm"
        puts $global_makefile "export AR            = \$(CROSS_COMPILE)ar"
        puts $global_makefile "export NM            = \$(CROSS_COMPILE)nm"
        puts $global_makefile "export STRIP         = \$(CROSS_COMPILE)strip"
        puts $global_makefile "export OBJCOPY       = \$(CROSS_COMPILE)objcopy"
        puts $global_makefile "export OBJDUMP       = \$(CROSS_COMPILE)objdump"
        puts $global_makefile "export RANLIB        = \$(CROSS_COMPILE)ranlib"
        puts $global_makefile "export READELF       = \$(CROSS_COMPILE)readelf"
        puts $global_makefile "export CC_NOCCACHE   = \$(CROSS_COMPILE)gcc\$(CC_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export CXX_NOCCACHE  = \$(CROSS_COMPILE)g++\$(CXX_OPTIONS)\$(SYSROOT)"
        puts $global_makefile "export INSTALL       = /usr/bin/install"
        puts $global_makefile "export VFSL_INSTALL  = \$(OUTPUTDIR)/../scripts/install/vfsl_install.sh"

        puts $global_makefile "export CPPFLAGS      = \$(CROSS_COMPILE_CPPFLAGS) \$(USER_CROSS_COMPILE_CPPFLAGS)"
        puts $global_makefile "export CFLAGS        = \$(CROSS_COMPILE_CFLAGS) \$(CROSS_COMPILE_OPTIMIZE_FLAGS) \$(USER_CROSS_COMPILE_CFLAGS)"
        puts $global_makefile "export CXXFLAGS      = \$(CROSS_COMPILE_CXXFLAGS) \$(CROSS_COMPILE_OPTIMIZE_FLAGS) \$(USER_CROSS_COMPILE_CXXFLAGS)"
        puts $global_makefile "export LDFLAGS       = \$(CROSS_COMPILE_LDFLAGS) \$(USER_CROSS_COMPILE_LDFLAGS)"
        puts $global_makefile "export AUTOCONF_HOST"
        puts $global_makefile "export AUTOCONF_BUILD"

        if { ($generateForPackageCreation == 1) || ($generateForPackageCreation == 2) } {
            puts $global_makefile "export BINARY_PACKAGE_CREATION = y"
        }
        puts $global_makefile ""

        set relativeBuildLocation [buildsystem::getRelativeBuildLocation]

        if { [info exists globaloptions("hardcofile")] } {
            if { [file exists $globaloptions("hardcofile")] } {
                read_hardcocfg $globaloptions("hardcofile") hardcoconfig
                if {[info exists hardcoconfig(hardco_hal)]} {
                    set hardco_real_cp $hardcoconfig(hardco_hal)
                } else {
                    set hardco_real_cp "not set"
                }

                if { [componentExists $hardco_real_cp] } {
                  if { $generateForPackageCreation == 1 } {
                    set component_path [buildsystem::getComponent_relpath $hardcoconfig(hardco_hal)]
                    set hardco_hal_dir "__CWD__/$relativeBuildLocation$component_path"
                  } else {
                    set hardco_hal_dir [buildsystem::getComponent_deppath $hardcoconfig(hardco_hal)]
                  }
                } else {
                    set hardco_hal_dir ""
                }

                if { [info exists hardcoconfig(hardco_kernel)] && [string length $hardcoconfig(hardco_kernel)] } {
                  if { $generateForPackageCreation == 1 } {
                    set component_path [buildsystem::getComponent_relpath $hardcoconfig(hardco_kernel)]
                    set hardco_kernel_dir "__CWD__/$relativeBuildLocation$component_path"
                  } else {
                    set hardco_kernel_dir [buildsystem::getComponent_deppath $hardcoconfig(hardco_kernel)]
                  }
                } else {
                    set hardco_kernel_dir ""
                }

                if { [info exists hardcoconfig(hardco_toolchain)] } {
                  if { $generateForPackageCreation == 1 } {
                    set component_path [buildsystem::getComponent_relpath $hardcoconfig(hardco_toolchain)]
                    set hardco_toolchain_dir "__CWD__/$relativeBuildLocation$component_path"
                  } else {
                    set hardco_toolchain_dir [buildsystem::getComponent_deppath $hardcoconfig(hardco_toolchain)]
                  }
                } else {
                    set hardco_toolchain_dir ""
                }

                puts $global_makefile "export HARDCO_HAL_DIR=$hardco_hal_dir"
                puts $global_makefile "export HARDCO_KERNEL_DIR=$hardco_kernel_dir"
                puts $global_makefile "export HARDCO_TOOLCHAIN_DIR=$hardco_toolchain_dir"
                if { [componentExists $hardco_real_cp] } {
                    puts $global_makefile "include \$(HARDCO_HAL_DIR)/Makefile.hal"
                }
            }
        }

        puts $global_makefile ""
        puts $global_makefile "include \$(STAGINGDIR)/components.config"
        puts $global_makefile ""
        puts $global_makefile "ifndef MAKEFILE_GLOBAL_EVALUATED"
        puts $global_makefile "export MAKEFILE_GLOBAL_EVALUATED = 1"
        puts $global_makefile "# The ifndef avoids evaluating these lines in recursive makefiles"
        puts $global_makefile "# We must still use immediate evaluation for shell commands, else they would get evaluated on each recipe execution in the top level makefile"
        puts $global_makefile ""
        puts $global_makefile "export PATH               := \$(STAGINGDIR)/utils:\$(PATH)"
        puts $global_makefile ""
        puts $global_makefile "export COMPRESSOR         := \$(shell which yui-compressor)"
        puts $global_makefile ""
        puts $global_makefile "export DUMP_CMD           := \$(shell \$(CC) -dumpmachine | awk '{split(\$\$0,array,\"-\")} END{print array\[1\]}' | tr \[:upper:\] \[:lower:\] )"
        puts $global_makefile "export TARGET_MACHINE_CMD := case \"\$(DUMP_CMD)\" in \"arm\") echo \"ARM\" ;; \"mips\") echo \"MIPS\" ;; \"i486\") echo \"INTEL\" ;; \"i686\") echo \"INTEL\" ;; *) echo \"UNKNOWN\" ;; esac"
        puts $global_makefile "export TARGET_MACHINE     := \$(shell \$(TARGET_MACHINE_CMD))"
        puts $global_makefile ""
        puts $global_makefile "endif"

        close $global_makefile

    # create sahcommon.mk file
    if {$generateForPackageCreation == 1} {
        set global_sahcommon [open "$package_build_dir/sahcommon.mk" "w"]
    } else {
        set global_sahcommon [open "$stagingdir/sahcommon.mk" "w"]
    }
    puts $global_sahcommon "include \$(STAGINGDIR)/components.config"
    puts $global_sahcommon "include \$(STAGINGDIR)/install_scripts.mk"
    close $global_sahcommon
    }

}


proc buildsystem::generatePackage {scriptdir} {
    variable globaloptions
    variable packagetargetdir
    variable package_buildconfig_default

    puts "\n\n---- GeneratePackage ----\n"

    set fp [open "createPackageCopyStagingDir.sh" "w"]
    puts $fp "#!/bin/bash\n"

    # clear the package directory
    exec rm -rf "$packagetargetdir"

# loop through all the components
    foreach name [buildsystem::getComponents] {
        set relative_package_location   [buildsystem::getComponent_relpath $name]
        set comp_location               [buildsystem::getComponent_deppath $name]
        set release_as_binary           [buildsystem::getComponent_rel_as_bin $name]
        set line_sah_debug              [buildsystem::getComponent_debug_options $name]
        set enable_debug                [buildsystem::getComponent_enable_debug $name]
        set debug_options               [buildsystem::getComponent_debug_options $name]

        if { ! [file isdirectory $comp_location] } {
            continue
        }

        file mkdir $packagetargetdir$relative_package_location

        puts "$name : release_as_binary($release_as_binary) - enable_debug($enable_debug) - debug_options($debug_options)"

        if {$release_as_binary > 0} {
            set absolutepackagetargetdir [exec readlink -f $packagetargetdir]
            #system "cd $comp_location && find . -name \"*.mk\" -o -name mk.inc -o -name mk.spec |  xargs -I '{}' cp -pfa --parents '{}' $absolutepackagetargetdir$relative_package_location"
            system "cd $comp_location && find . -path ./output -prune -type f -o -name \"*.conf\" -o -name \"*.sh\" -o -name \"*.mk\" -o -name \"*.m4\" -o -name \"*.inc\" -o -name \"*.include\" -o -name mk.spec -o -iname \"makefile*\" -o -iname \"Makedefs*\" |  xargs -I '{}' cp -pfa --parents '{}' $absolutepackagetargetdir$relative_package_location"
            if [file exists $comp_location/Makefile] {
                exec cp -pf $comp_location/Makefile $packagetargetdir$relative_package_location
            }
            if [file exists $comp_location/Makefile.hal] {
                exec cp -pf $comp_location/Makefile.hal $packagetargetdir$relative_package_location
            }
            if [file exists $comp_location/Component.lic] {
                exec cp -pf $comp_location/Component.lic $packagetargetdir$relative_package_location
            }
            if [file exists $comp_location/Component.dep] {
                exec cp -pf $comp_location/Component.dep $packagetargetdir$relative_package_location
            }
            if [file exists $comp_location/Component.in] {
                exec cp -pf $comp_location/Component.in $packagetargetdir$relative_package_location
            }
            if [file exists $comp_location/staging.tar] {
                exec cp -pf $comp_location/staging.tar $packagetargetdir$relative_package_location
            }
        } else {
            if [file exists $comp_location/.git] {
                # TODO add option to create source packages with git clone.
                if { 1 } {
                    exec cp -prf {*}[glob $comp_location/*] $packagetargetdir$relative_package_location/
                } else {
                    exec rm -rf $packagetargetdir$relative_package_location/
                    # WARNING: this may contain unwanted branches !
                    exec git clone $comp_location $packagetargetdir$relative_package_location/
                }
            } else {
                exec cp -prf $comp_location $packagetargetdir$relative_package_location/..
            }
        }
    }

    puts "\n"
    buildsystem::createStagingAndReleaseDir 1
    #buildsystem::generateCompMakeFile 1

    # -> at this point we have all the necessary information, thus create a file to be
    # executed when the staging_dir is ready to be copied to the package directory
    if { [ info exists globaloptions("stagingdir") ] } {
        set stagingdir [ string trim "$globaloptions("stagingdir")" ]
    } else {
        set stagingdir [string trim "$globaloptions("outputdirectory")/staging"]
    }

    set relative_build_location   [buildsystem::getComponent_relpath "build"]
    set package_build_dir "$packagetargetdir$relative_build_location"
    puts $fp "cp -prf ./config $package_build_dir/"
    puts $fp "cp -prf ./lib $package_build_dir/"
    puts $fp "cp -prf ./scripts $package_build_dir/"
    puts $fp "cp -prf ./tools $package_build_dir/"
    puts $fp "cp -prf ./utils $package_build_dir/"
    puts $fp "cp -pf autoconf.config.in $package_build_dir/autoconf.config.in"
    puts $fp "cp -pf autoconf.h.in $package_build_dir/autoconf.h.in"
    puts $fp "cp -pf autogen.sh $package_build_dir/autogen.sh"
    puts $fp "cp -pf build_package.sh $package_build_dir/build_package.sh"
    puts $fp "cp -pf build_rootpathcfg $package_build_dir/build_rootpathcfg"
    puts $fp "cp -pf build_rootpathcfg_package $package_build_dir/build_rootpathcfg_package"
    puts $fp "cp -pf build.sh $package_build_dir/build.sh"
    puts $fp "cp -pf Component.dep $package_build_dir/Component.dep"
    puts $fp "cp -pf Component.in $package_build_dir/Component.in"
    puts $fp "cp -pf Component.lic $package_build_dir/Component.lic"
    puts $fp "cp -pf component.lst $package_build_dir/component.lst"
    puts $fp "cp -pf Component.mk $package_build_dir/Component.mk"
    puts $fp "cp -pf configsource.tcl $package_build_dir/configsource.tcl"
    puts $fp "cp -pf configure $package_build_dir/configure"
    puts $fp "cp -pf configure.ac $package_build_dir/configure.ac"
    puts $fp "cp -pf create_package.sh $package_build_dir/create_package.sh"
    puts $fp "cp -pf generate_opensource_tree.sh $package_build_dir/generate_opensource_tree.sh"
    puts $fp "cp -pf gentestconfig.tcl $package_build_dir/gentestconfig.tcl"
    puts $fp "cp -pf getsource.tcl $package_build_dir/getsource.tcl"
    puts $fp "cp -pf Global.in $package_build_dir/Global.in"
    puts $fp "cp -pf Makefile $package_build_dir/Makefile"
    puts $fp "cp -pf Makefile.maps $package_build_dir/Makefile.maps"
    puts $fp "cp -pf Makefile.menu $package_build_dir/Makefile.menu"
    puts $fp "cp -pf Makefile.recipes $package_build_dir/Makefile.recipes"
    puts $fp "cp -pf modular_packages.tcl $package_build_dir/modular_packages.tcl"
    puts $fp "cp -pf README $package_build_dir/README"
    puts $fp "cp -pf server.cfg.sample $package_build_dir/server.cfg"
    puts $fp "cp -pf sonar_bash.sh $package_build_dir/sonar_bash.sh"
    puts $fp "cp -pf sonar_project.sh $package_build_dir/sonar_project.sh"
    puts $fp "cp -pf ./scripts/install_scripts/install_to_staging.sh $package_build_dir/scripts/install_scripts/install_to_staging.sh"
    puts $fp "cp -pf $package_buildconfig_default $package_build_dir/$package_buildconfig_default"
    puts $fp "if \[ -e output/release/delivery/version.txt \]"
    puts $fp "then"
    puts $fp "cp -pf output/release/delivery/version.txt $packagetargetdir/version.txt"
    puts $fp "fi"
    close $fp
    exec chmod +x createPackageCopyStagingDir.sh
}
