package provide buildsystem 1.0

namespace eval buildsystem {
        variable globalcomponentlst
        array set p4actions {getDep getP4Dep getLic getP4Lic}
        array set gitactions {getDep getGitDep getLic getGitLic}
        set vcsactions(p4,getDep) "getP4Dep"
        set vcsactions(p4,getLic) "getP4Lic"
        set vcsactions(p4,getSource) "getP4ComponentSource"
        set vcsactions(p4,cleanFully) "cleanP4ComponentWorkspaceFully"
        set vcsactions(git,getDep) "getGitDep"
        set vcsactions(git,getLic) "getGitLic"
        set vcsactions(git,getSource) "getGitComponentSource"
        set vcsactions(git,cleanFully) "cleanGitWorkingTreeFully"

        namespace export parse_gs_commandline read_gs_deps getComponentVcs writeCurProjectList
}

# parse getsource commandline options
# options are returned in commandlineoptions as an associative list
proc buildsystem::parseGetSourceArgs {argv} {
    variable globaloptions

        set options {
            {defconfig.arg      "set to defconfig"}
            {p.arg              "set to project dependencies file"}
            {s.arg              "set to configuration file"}
            {osrc.arg           "set to opensource configuration file"}
            {compexcl.arg       "set the component exclude file"}
            {cl.arg             "set to changelist_info file"}
            {t                  "enable tracing"}
            {ci                 "continuous integration"}
            {n                  "do not track in workspace"}
            {nf                 "do not force sync operations"}
            {w                  "build package"}
            {z                  "create package"}
            {opensource_only    "Build only the opensource packages"}
            {fullclean          "remove all non tracked files"}
        }

        set globaloptions("regendefconfig") 0
        set globaloptions("notrackws") 0
        set globaloptions("noforcesync") 0
        set globaloptions("gendefaultbuildconfig") 1
        set globaloptions("createPackage") 0
	    set globaloptions("buildPackage") 0
        set globaloptions("nosync") 0
        set globaloptions("continuous_integration") 0
        set globaloptions("fullclean") 0
        set globaloptions("opensource_include") 0
        set globaloptions("os_as_source") 0
        set globaloptions("component_exclude") 0
        set globaloptions("changelist_info") "changelistinfo.txt"


        if {[llength $argv]} {
            while {[llength $argv]} {

                set err [::cmdline::getopt argv {defconfig.arg s.arg p.arg osrc.arg compexcl.arg cl.arg t ci n nf r w z fullclean opensource_only} optVar optValue]

                    if {$err == 1} {
                        switch $optVar {
                            "compexcl" {
                                set globaloptions("component_exclude") [file join $optValue]
                            }
                            "osrc" {
                                set globaloptions("opensource_include") [file join $optValue]
                            }
                            "defconfig" {
                                set globaloptions("defconfig") [file join $optValue]
                            }
                            "r" {
                                set globaloptions("regendefconfig") [file join $optValue]
                            }
                            "p" {
                                set globaloptions("projectdependenciesfile") [file join $optValue]
                            }
                            "s" {
                                set globaloptions("buildconfigfile") [file join $optValue]
                            }
                            "n" {
                                set globaloptions("notrackws") 1
                            }
                            "nf" {
                                set globaloptions("noforcesync") 1
                            }
                            "t" {
                                log::lvSuppress debug 0
                            }
                            "ci" {
                                set globaloptions("continuous_integration") 1
                            }
                            "w" {
                            set globaloptions("buildPackage") 1
                            }     
                            "z" {
                                set globaloptions("createPackage") 1
                            }
                            "opensource_only" {
                            setOpenSourceBuild 1
                            }
                            "fullclean" {
                                set globaloptions("fullclean") 1
                            }
                        }
                    } elseif {$err < 0 } {
                        set usage [::cmdline::usage $options]
                            puts $usage
                            exit 1
                    } else {
                        exit 1
                    }
            } 
        } else {
            set usage [::cmdline::usage $options]
                puts $usage
                exit 1
        }

}

proc buildsystem::getP4File {name version filename} {
    set servername  [buildsystem::get_server_name $name]
    set serverpath  [buildsystem::get_server_path $name]
    set proto       [buildsystem::get_vcs_srv_address $servername]
    set workspace   [buildsystem::get_vcs_wks_name $servername]

### we get our rootpath ###
    set rootpath [buildsystem::get_root_path $name]

    set path $serverpath/$version
    set componentpath //$workspace/[string range $path 1 [string length $path]]

    set updated 0

    log::log debug "Executing perforce command: p4 -p $proto -c $workspace print $componentpath/$filename"
    set tempfile [fileutil::tempfile]
    if { [catch { exec p4 -p $proto -c $workspace print -q $componentpath/$filename > $tempfile} msg] } {
        puts "\033\[31mError getting P4 file : $msg\033\[0m"
    }
    return $tempfile
}

proc buildsystem::getP4Lic {name version} {
    return [ getP4File $name $version Component.lic ]
}

proc buildsystem::getP4Dep {name version} {
    return [ getP4File $name $version Component.dep ]
}

#read all dependencies recursively
proc buildsystem::read_gs_deps {componentnames} {
    return [buildsystem::read_deps $componentnames 1]
}

proc buildsystem::getGitFile {name version filepath} {
    variable globalbuildcfg
    set servername [buildsystem::get_server_name $name]
    array set server $globalbuildcfg($servername)
    set componentsubpath [buildsystem::get_server_path $name]
    set repopath $server(serveraddress)$componentsubpath
    set tempfile [fileutil::tempfile]
    
    set componentpath [buildsystem::get_vcs_rootpath $servername]$componentsubpath
    buildsystem::fetchComponentGitHistory $name $componentpath $repopath "--all"

    if {! [runCommandInRepo $componentpath git show $version:$filepath > $tempfile]} {
        log::log error "\033\[31mError: Could not get file from git\033\[0m"
    }
    return $tempfile
}

proc buildsystem::getGitDep {name version} {
    return [getGitFile $name $version Component.dep]
}

proc buildsystem::getGitLic {name version} {
    return [getGitFile $name $version Component.lic]
}

proc buildsystem::getComponentVcs {name} {
    variable globalbuildcfg
    array set generalcomponent $buildsystem::globalcomponentlst($name)
    set servername $generalcomponent(servername)
    array set server $globalbuildcfg($servername)
    set vcs $server(proto)
    return $vcs
}

proc buildsystem::runVcsAction {operation name version} {
    variable vcsactions
    set vcs [getComponentVcs $name]
    return [$vcsactions($vcs,$operation) $name $version]
}


proc buildsystem::getP4ComponentSource {name dummy} {
    variable globalcomponentlst
    variable globaloptions

    set deppath     [buildsystem::getComponent_deppath $name]
    set servername  [buildsystem::get_server_name $name]
    set proto       [buildsystem::get_vcs_srv_address $servername]
    set workspace   [buildsystem::get_vcs_wks_name $servername]
    set changelist_info [open $globaloptions("changelist_info") "a+"]

    if { $globaloptions("notrackws") == "1" } {
        set syncopt "-p"
    } elseif { $globaloptions("noforcesync") == "1" } {
# -q is needed so that p4 sync does not show "file(s) up-to-date" on stderr, but it might hide some real failures
        set syncopt "-q"
    } else {
        set syncopt "-f"
    }

    log::log debug "Executing perforce command: p4 -p $proto -c $workspace sync $syncopt $deppath..."
    puts  "Syncing $deppath"
    if { $globaloptions("regendefconfig") == "1" } {
        set errFileNotInDepot [catch {exec p4 fstat [file join $deppath Makefile.hal] } ]
        if { $name eq "sah_hal" } {
            if { [catch {exec p4 -p $proto -c $workspace sync $syncopt [file join $deppath Makefile.hal] } msg] } {
                puts "\033\[31mError syncing P4 file : $msg\033\[0m"
            }
        }
        if { [catch {set result [exec p4 -p $proto -c $workspace sync $syncopt [file join $deppath Component.*]] } msg] } {
            puts "\033\[31mError syncing P4 file : $msg\033\[0m"
        }
    } else {
        if { [catch {set result [exec p4 -p $proto -c $workspace sync $syncopt [file join $deppath ...]] } msg] } {
            puts "\033\[31mError syncing P4 file : $msg\033\[0m"
        }
    }
    if { [catch { set changes [exec p4 -p $proto -c $workspace changes -m1 [file join $deppath ...]] } msg] }  {
        puts "\033\[31mError : $msg\033\[0m"
        exit 1
    }
    #set changes [exec p4 -p $proto -c $workspace changes -m1 [file join $deppath ...]]
    set list [split $changes]
    set cl [lindex $list 1]
    if { [string equal $cl ""] } {
        set cl "Unknown"
    }
    buildsystem::setComponent_changelist $name $cl
    puts $changelist_info "$name=$cl"
    close $changelist_info
}

proc buildsystem::cleanP4ComponentWorkspaceFully {name dummy} {
    variable globalcomponentlst
    variable globaloptions

    set deppath     [buildsystem::getComponent_deppath $name]
    set servername [buildsystem::get_server_name $name]
    set proto       [buildsystem::get_vcs_srv_address $servername]
    set workspace   [buildsystem::get_vcs_wks_name $servername]

    puts "Cleaning $deppath"
    set tempfile [fileutil::tempfile]
    log::log debug "Executing: find $deppath -type f | p4 -p $proto -c $workspace -x - fstat 2> $tempfile > /dev/null"
    catch {exec find $deppath -type f | p4 -p $proto -c $workspace -x - fstat 2> /$tempfile > /dev/null }
    log::log debug "Executing: sed \"s/ -.*$//\" $tempfile | xargs rm -f"
    catch {exec sed "s/ -.*$//" $tempfile | xargs rm -f }
    catch {exec rm $tempfile }
    # Note this will not delete empty directories
}

proc buildsystem::runCommandInRepo {workingdir args} {
    set result 1
    if { ! [file exists "$workingdir"]} {
        log::log error "\033\[31mFATAL ERROR: Could not go into $workingdir\033\[0m"
        exit 1
    }
    set previouspath [pwd]
    cd $workingdir
    set command "$args"
    log::log debug "Executing command in $workingdir: $command"
    if { [catch {exec -ignorestderr {*}$args} msg] } {
        log::log error "\033\[31mERROR executing '$command': $msg\033\[0m"
        set result 0
    }
    cd $previouspath
    return $result
}

proc buildsystem::createRemotes {name componentpath repopath} {
    variable globalbuildcfg
    set servername [buildsystem::get_server_name $name]
    foreach extension {upstream review cache} {
        set extensionalias $servername-$extension
        if { [info exists globalbuildcfg($extensionalias)]} {
            array set server $globalbuildcfg($extensionalias)
            runCommandInRepo $componentpath git remote add $extension $server(serveraddress)[buildsystem::get_server_path $name]
        } elseif { ! [string equal "cache" $extension] } {
            # create default remote for extensions except cache
            runCommandInRepo $componentpath git remote add $extension $repopath
        }
    }
}

# Clone or update a git repository. Return 1 if the repository was created.
proc buildsystem::fetchGitHistory {name componentpath repopath version options} {
    set created 0
    log::log debug "Fetching $version in $componentpath"
    if { ! [file exists $componentpath/.git] && !  [file exists $componentpath/refs/heads] } {
        # no git repository or bare repository yet in that directory
        if { [string equal $version "--all"] } {
            log::log debug "Executing command: git clone $options -n $repopath $componentpath"
            if { [catch {exec -ignorestderr git clone {*}$options -n $repopath $componentpath} msg] } { 
                log::log error "\033\[31mERROR executing command: $msg\033\[0m"
                exit 1
            }
            return 1
        } else {
            file mkdir $componentpath
            if {! [runCommandInRepo $componentpath git init {*}$options]} {exit 1}
        }
        set created 1
    }
    if [string equal $version "--all"] {
        set localrefs "refs/remotes/origin/*"
        if [string equal [lindex $options 0] "--bare"] {
            set localrefs "refs/heads/*"
        }
        if {! [runCommandInRepo $componentpath git fetch $repopath +refs/heads/*:$localrefs]} { exit 1 }
    } else {
        set remoteversion [lindex [split $version /] end]
        if {! [runCommandInRepo $componentpath git fetch $repopath $remoteversion:$version]} { exit 1 }
    }
    return $created
}

proc buildsystem::fetchComponentGitHistory {name componentpath repopath version} {
    variable globalbuildcfg
    variable fetched_components
    if {[info exists fetched_components($name)] && ([string equal $fetched_components($name) "--all"] || [string equal $fetched_components($name) $version])} {
        # already fetched. Skip.
        return
    }
    set servername [buildsystem::get_server_name $name]
    set originrepo $repopath
    if { [info exists globalbuildcfg($servername-cache)]} {

            array set server $globalbuildcfg($servername-cache)
            set originrepo $server(serveraddress)[buildsystem::get_server_path $name]
            puts "Checking for file lock in $originrepo ..."
            set done false
            while {!$done} {
                if { [catch {open $originrepo/.busy {WRONLY EXCL CREAT}} f] } {
                    if { [file isdirectory $originrepo] } {
                        # entry in gitcache exists, so file is locked -> wait for the lockfile to be released
                        # check if file is older than 5 minutes -> if so delete, since other process has most likely crashed
                        if { [ exec find $originrepo/.busy -mmin +5 ] == "$originrepo/.busy" } {
                            file delete $originrepo/.busy
                        }
                        puts "Waiting ... Other process is busy in $originrepo"
                        sleep 1
                    } else {
                        # entry in gitcache doesn't exist yet, so go ahead wit the fetch
                        buildsystem::fetchGitHistory $name $originrepo $repopath $version --bare
                        set done true
                    }
                } else {
                    buildsystem::fetchGitHistory $name $originrepo $repopath $version --bare
                    close $f
                    if { [file exists $originrepo/.busy] } {
                        file delete $originrepo/.busy
                    }
                    set done true
                }
            }
    }
    set fetched_components($name) $version
    if [buildsystem::fetchGitHistory $name $componentpath $originrepo $version {}] {
        buildsystem::createRemotes $name $componentpath $repopath
    }
}

proc buildsystem::getGitComponentSource {name dummy} {
    variable globalcomponentlst
    variable globalbuildcfg
    variable globaloptions

    set servername [buildsystem::get_server_name $name]
    array set server $globalbuildcfg($servername)
    set version [buildsystem::getComponent_version $name]
    set componentsubpath [buildsystem::get_server_path $name]
    set repopath $server(serveraddress)$componentsubpath
    set deppath [buildsystem::getComponent_deppath $name]
    
    set localpath [buildsystem::get_vcs_rootpath $servername]
    set componentpath $localpath$componentsubpath
    set changelist_info [open $globaloptions("changelist_info") "a+"]
    puts  "Syncing $deppath"

    buildsystem::fetchComponentGitHistory $name $componentpath $repopath "--all"
    puts "checking out $componentpath from GIT"
    if { $globaloptions("noforcesync") == "0" } {
        if {! [runCommandInRepo $componentpath git checkout -f $version] } { exit 1 }
		if { [catch { system "cd $componentpath && find . -type f -name \"*.in\" -o -name \"*.ac\" -o -name \"*.am\" | xargs -I '{}' touch -r configure '{}' 2>/dev/null"} msg] } {
			puts "\033\[33mWarning : $msg\033\[0m"
	    	}
        return
    }
    if {! [runCommandInRepo $componentpath git checkout $version] } { exit 1 }
    	if { [catch { system "cd $componentpath && find . -type f -name \"*.in\" -o -name \"*.ac\" -o -name \"*.am\" | xargs -I '{}' touch -r configure '{}' 2>/dev/null"} msg] } {
        	puts "\033\[33mWarning : $msg\033\[0m"
    	}
    set cl "Unknown Git"
    buildsystem::setComponent_changelist $name $cl
    puts $changelist_info "$name=$cl"
    close $changelist_info
    
}

proc buildsystem::cleanGitWorkingTreeFully {name dummy} {
    set componentpath [buildsystem::getComponent_deppath $name]

    puts "Cleaning $componentpath"
    if [file exists $componentpath/.git] {
        runCommandInRepo $componentpath git clean "-fdx" 
    } else {
        log::log debug "Executing command: rm -rf $componentpath"
        catch {exec rm -rf $componentpath}
    }
}

proc buildsystem::getSource {} {
    variable globalcomponentlst
    variable globalbuildcfg
    variable globaloptions

    foreach name [buildsystem::getComponents] {
        runVcsAction getSource $name "todo"
    }
}

proc buildsystem::cleanFully {} {
    variable globalcomponentlst
    variable globalbuildcfg
    variable globaloptions

    foreach name [buildsystem::getComponents] {
        if { "$name" ne "build" } {
            runVcsAction cleanFully $name "todo"
        }
    }
}

proc buildsystem::writeCurProjectList {scriptdir} {
        variable bs_is_opensource
        set cur_prj [open "$scriptdir/cur_prj.list" "w"] 

        if {$bs_is_opensource} {
            puts $cur_prj "#! package_default=source\n"
        }    

        foreach name [lsort [buildsystem::getComponents]] {
            if {$bs_is_opensource} {
                if { $name eq "build" } {
                    puts $cur_prj      "b:$name=[buildsystem::getComponent_version $name]"
                } else {
                    puts $cur_prj      "s:$name=[buildsystem::getComponent_version $name]"
                }
                continue
            }

            set mode [buildsystem::getComponent_rel_as_bin $name]
            switch $mode {
                0 { 
                    puts $cur_prj      "s:$name=[buildsystem::getComponent_version $name]"
                } 
                1 {
                    puts $cur_prj      "b:$name=[buildsystem::getComponent_version $name]"
                }
                2 {
                    puts $cur_prj      "p:$name=[buildsystem::getComponent_version $name]"
                }
            }
        }    
        close $cur_prj
}

