package provide buildsystem 1.0

namespace eval buildsystem {
      variable globalbuildcfg
    
  namespace export read_servers

}

# fills in globalbuildcfg array

proc buildsystem::read_servers {filename} {
    variable globalbuildcfg

    log::log debug "Reading build config" 
    log::log debug "Opening $filename"
    set infile [open $filename r]
    array set globaloptions [buildsystem::getGlobalOptions]

    while { [gets $infile line] >= 0} {
        set wks_path        "not set"
        set wks_name        "not set"
        set proto           "not set"
        set serveraddress   "not set"

        # is line not empty
        if [string length $line] {

            # line starting with # are treated as comment
            if [regexp {^#} $line] {
                continue
            }

            #split name from the server *uri*
            if {![regexp {^(.*)=(\w+):(.*)$} $line match name proto serveraddress]} {
                puts "\033\[31mError parsing $line in $filename\033\[0m"
                exit 1
            }

            #now check if the server part has a workspace included, otherwise add the content of the P4CLIENT environment variable
            if [string match "p4" $proto] {
                if {[regexp ".+/.+" $serveraddress] != 1} {
                    continue
                }
                lassign [split $serveraddress "/"] serveraddress wks_name
                
                if { $globaloptions("buildPackage") == 1 || $globaloptions("nosync") == 1} {
                    set wks_path    "not set"
                } else {  
                    if { [catch { set wks_path [string trim [exec "p4" "-p" "$serveraddress" "-c" "$wks_name" "client" "-o" "|" "sed" "-n" {s/^Root:[ \/t]*//p}]] } msg] }  {
                        puts "\033\[31mPerforce Error : $msg\033\[0m"
                        exit 1
                    }
                }
            } elseif [string match "git" $proto] {
                if {![regexp {(.*),(.*)} $serveraddress match serveraddress wks_path]} {
                    puts "\033\[31mError parsing workspace path from  $line\033\[0m"
                    puts "Expected Syntax:"
                    puts "gerrit=git:ssh://gerrit.be.softathome.com:29418,/home/sahXXXX/GIT_WORKSPACE"
                    puts "gerrit-cache=git:/home/sahXXXX/gitcache,/home/sahXXXX/GIT_WORKSPACE"
                    exit 1
                }
            }

           log::log debug "Servername: $name Serverprotocol: $proto Path: $serveraddress"

           set server(proto)            $proto
           set server(serveraddress)    $serveraddress
           set server(wks_path)         $wks_path
           set server(wks_name)         $wks_name
           set globalbuildcfg($name)    [array get server]
           
           unset line
        }
    }

    close $infile

    log::log debug "Reading build config done" 
    return [array get globalbuildcfg]
}

# returns arraylist of globalbuildcfg

proc buildsystem::getGlobalbuildcfg {} {
    variable globalbuildcfg
    return [array get globalbuildcfg]

}
