package provide buildsystem 1.0

namespace eval buildsystem {
    variable package_default 1;       # default is release as binary
    namespace export read_componentdeps get_componentdeps_string get_componentdeps_string_release
}

#expected options_string: d(.*)
proc buildsystem::parse_debug_options {options_string} {
    set result_string ""

# we are only interested in the part between the brackets
        if [regexp -all {\((.*)\)} $options_string match debuginfo] {
            set result_string "SAH_DEBUG=1"
                foreach command [split $debuginfo ","] {
                    set command [string trim $command]
                        if { [split $command "=" ] == "$command" } {
                            set command "$command=1"
                        }
                    set result_string "$result_string SAH_DEBUG_$command"
                }
        }
    return $result_string
}

# parse component.dep files
proc buildsystem::read_componentdeps {filename} {
    variable package_default
    variable globaloptions

    set cnames " "

    log::log debug "Reading component dependencies"
    if [file exists $filename] {
        log::log debug "Opening $filename"
        set infile [open $filename r]
        set names "test"
        set lineNumber 0
        while { [gets $infile line] >= 0} {
# is line not empty
            if [string length $line] {
# line starting with #! is treated as additional information
                if [regexp {^#! package_default=source} $line] {
                    log::log debug "Default package behavior: release as source"
                        set package_default 0
                        continue
                } elseif [regexp {^#! package_default=binary} $line] {
                    log::log debug "Default package behavior: release as binary"
                        set package_default 1
                        continue
                } elseif [regexp {^#! package_default=binary_no_headers} $line] {
                    log::log debug "Default package behavior: release as binary without the header files"
                        set package_default 2
                        continue
                } elseif [regexp {^#! opensource_as_source} $line] {
                    puts "####### -> release all opensource components as source"
                    log::log debug "release all opensource components as source"
                        set globaloptions("os_as_source") 1
                        continue
                } elseif [regexp {^#} $line] {
# line starting with # are treated as comment
                    continue
                }

                set debug_options "";                          # store all debug options here
                set release_as_binary $package_default;        # do we need to package the component as binary or as source
                set extra_info ""

                if { [string first "#" $line] > 0 } {
                    set parts [split $line "#"]
                    lassign $parts line comment
                }
# lets see whether there is a : character
                if { [string first ":" $line] != -1 } {
                    set parts [split $line ":"]
                    lassign $parts extra_info line
# let's see whether we can find a debug section-> d(.....)
                    if [regexp -all {(d\(.*\))} $extra_info match debuginfo] {
# found the section we were looking for, get the options
                        set debug_options [parse_debug_options $debuginfo]
# remove the section from the option_string
                        regsub {d\(.*\)} $extra_info "" extra_info
                    }

# let's investigate the rest of the option_string
                    for { set i 0 } { $i < [string length $extra_info] } { incr i } {
                        set char [string index $extra_info $i ]
                        if { $char == "d"} {
                            set debug_options "$debug_options SAH_DEBUG=1"
                        } elseif { $char == "p"} {
                            set release_as_binary 2; # binary package without header files
                        } elseif { $char == "b"} {
                            set release_as_binary 1; # binary package with header files
                        } elseif { $char == "s"} {
                            set release_as_binary 0; # source package
                        } else {
                            puts "\n\033\[33mWarning ($filename): skipping invalid option \"$char\" in line \"$line\"\033\[0m\n"
#break
                        }
                    }
                }

                if { [string first "=" $line] > 0 } {
#split name from the version
                    set parts [split $line "="]
                    set assgm "EQUALS"
                } elseif { [string first "<" $line] > 0 } {
#split name from the version
                    set parts [split $line "<"]
                    set assgm "LESSER"
                } elseif { [string first ">" $line] > 0} {
#split name from the version
                    set parts [split $line ">"]
                    set assgm "GREATER"
                } else {
                    continue
                }
                lassign $parts componentname componentversion
                set componentversion [string trim $componentversion]
                 
                if { [string match *Component.dep* $filename] } {
                    if { (![string match *REL* $componentversion]) && (![string match refs/tags/* $componentversion]) && ([string length $componentversion] > 1) } {
                        puts "\n\033\[31mERROR in $filename: invalid componentversion $componentversion\033\[0m\n"
                        #exit 1
                    }
                }
                log::log debug "Componentname: $componentname Componentversion: $componentversion"
                if { $globaloptions("continuous_integration") == "1" } {
                    #-## TODO: add support for git
                    set depversion $componentversion
                    set vcs [buildsystem::getComponentVcs $componentname]
                    if { "$vcs" == "p4" } {
                        if { [regsub {REL/[^/]*} $componentversion "MAIN" componentversion] > 0 } {
                            log::log debug "P4 -> CI: using $componentversion instead of $depversion"
                        }
                    } elseif { "$vcs" == "git" } {
                        set sub1 [regsub {refs/tags/} $componentversion "remotes/origin/" componentversion]
                        set sub2 [regsub {_v\d{1,3}\.\d{1,3}\.\d{1,3}} $componentversion "" componentversion]
                        if { $sub1 > 0 || $sub2 > 0 } {
                            log::log debug "GIT -> CI: using $componentversion instead of $depversion"
                        }
                    } else {
                        puts "\033\[31mERROR - CI - Version control system not defined for component $componentname\033\[0m"
                        exit 1
                    }   
                }
                lappend cnames $componentname $filename $assgm $componentversion $release_as_binary [list $debug_options]
            }
        }
        close $infile
    }
    log::log debug "Reading component dependencies done"
    return $cnames
}

proc buildsystem::getComponentNamesFromList {cnames} {
    set dep_list ""
    #create dependency list
    foreach {name filename assgm version release_as_binary debug_options} $cnames {
        lappend dep_list $name $assgm $version
    }
    return $dep_list
}

# look for the component with name 'cname' in the list 'cnames'
proc buildsystem::getComponentFromList {cnames cname} {
    set component ""
    foreach {name filename assgm version release_as_binary debug_options} $cnames {
        if {![string compare $cname $name]} {
            lappend component $name $assgm $version $release_as_binary $debug_options
            return $component
        }
    }
    return -1
}

proc buildsystem::addComponentToList {cnames component} {
    set new_cnames ""
    lassign $component new_name new_filename new_assgm new_version new_release_as_binary new_debug_options
    foreach {name filename assgm version release_as_binary debug_options} $cnames {
        # if we find a match, skip it. We'll add it at the end
        if {![string compare $new_name $name]} {
            puts "Overwriting component '$new_name'"
            puts "$version --> $new_version"
        } else {
        # all other data just add as is
            lappend new_cnames $name $filename $assgm $version $release_as_binary $debug_options
        }
    }

    # add the new data at the end of the list
    lappend new_cnames $new_name $new_filename $new_assgm $new_version $new_release_as_binary $new_debug_options
    
    # return the new array
    return $new_cnames
}

#read all dependencies recursively
proc buildsystem::read_deps {componentnames sync {indent ""}} {
   array set printmap {"EQUALS" "=" "LESSER" "<=" "GREATER" ">="}
   array set iprint {0 "|- " 1 "`- "}
   array set isub   {0 "|  " 1 "   "}
   # list of new or update components whose dependencies need to be checked
   set newdeps ""

    foreach {name filename assgm version release_as_binary debug_options} $componentnames {
        #print dependency map
        set dep_list [getComponentNamesFromList $componentnames]
        set ilast [expr {"$name" == "[lindex $dep_list end]"}]

        if {![buildsystem::setComponent $name $filename $assgm $version $release_as_binary $debug_options]} {
            # this component version is older => skip
            puts "${indent}$iprint($ilast)$name $printmap($assgm) $version"
            continue
        }
        # compare if version is set to '*' or '?
        if [is_undefined_version $version] {
            puts "${indent}$iprint($ilast)$name $printmap($assgm) $version"
            continue 
        }
        
        # newer version => need to check dependencies
        lappend newdeps $name $version
    }
    
    foreach {name version} $newdeps {
        puts "${indent}$iprint($ilast)$name $printmap($assgm) $version"
        if { ! [string equal [getComponent_version $name] $version]} {
            # the version has changed, no need to analyze deps anymore
            continue
        }
        if {$sync} {
            set depfile [runVcsAction getDep $name $version]
            set licfile [runVcsAction getLic $name $version]
        } else {
            set deppath     [buildsystem::getComponent_deppath $name]
            set depfile     [file join $deppath Component.dep]
            set licfile     [getComponent_licfile $name]
        }

        set cnames      [read_componentdeps $depfile]
        set dep_list    [getComponentNamesFromList $cnames]
        setComponent_dependencies $name $dep_list

        set license [parseComponentLic $licfile]
        setComponent_license $name $license

        if {$sync} {
            exec rm -f $depfile
            exec rm -f $licfile
        }
        read_deps $cnames $sync "$indent$isub($ilast)"
    }
}


