package provide buildsystem 1.0 

namespace eval buildsystem {
    variable defconfig_param
    namespace export parse_defconfig defconfig_name_is_set write_defconfig
    #Pre set buildsystem 
    set defconfig_param(CONFIG_BUILD) "y"
}

proc buildsystem::parse_defconfig {fileName} {
    variable defconfig_param

    if {![file exists $fileName]} {
        puts "\033\[31mError opening $fileName\033\[0m"
        return 0
    }

    set fd [open $fileName "r"]
    while {[gets $fd line] >= 0} {
        if { [regexp {(.*)=(.*)$} $line match key value]} {
            set defconfig_param($key) [string trim $value]
        } elseif {[regexp {# (.*) (is not set)$} $line match key value]} {
            set defconfig_param($key) [string trim $value]
        }
    }
    close $fd
}

proc buildsystem::write_defconfig {filename} {
    variable defconfig_param

    set fd [open $filename w]
    foreach key [lsort [array names defconfig_param]] {
        if {![string compare $defconfig_param($key) "is not set"]} {
            puts $fd "# $key $defconfig_param($key)"
        } else {
            puts $fd "$key=$defconfig_param($key)"
        }
    }   
    close $fd
}

proc buildsystem::defconfig_name_is_set {name} {
    variable defconfig_param
    variable globaloptions

    set key "CONFIG_[string toupper $name]"
    if {[info exists defconfig_param($key)]} {
        if {![string compare $defconfig_param($key) "y"]} {
            return 1
        }
    }
    return 0
}

proc buildsystem::defconfig_enable_component {name} {
    variable defconfig_param
    variable globaloptions

    set key "CONFIG_[string toupper $name]"
    defconfig_key_set $key y
    return 0
}

proc buildsystem::defconfig_disable_component {name} {
    variable defconfig_param
    variable globaloptions

    set key "CONFIG_[string toupper $name]"
    set defconfig_param($key) "is not set"
    return 0
}

proc buildsystem::defconfig_key_set {key value} {
    variable defconfig_param
    variable globaloptions
    
    set defconfig_param($key) [string trim $value]
    return 0
}

proc buildsystem::defconfig_get_all_values {} {
    variable defconfig_param
    return [array get defconfig_param]
}

proc buildsystem::defconfig_clear_all_values {} {
    variable defconfig_param
    array unset defconfig_param
    return 0
}
