package provide buildsystem 1.0

namespace eval buildsystem {
    variable deliver_sources_for
    variable deliver_sources_for_opt
    variable bs_is_opensource

    set bs_is_opensource            0
    set deliver_sources_for        "gpl lgpl gplv3 gplv2* lgplv2*" 

    set deliver_sources_for_opt    "bsd openssl mit mplv*"
}

proc buildsystem::setOpenSourceBuild {arg} {
    variable bs_is_opensource
    set bs_is_opensource $arg
    puts "Opensource build $arg"
}

proc buildsystem::delNoneOpensourceComponents {} { 
    set fail_build 0
    set del_cp_list ""
    set keep_cp_list ""
    foreach name [buildsystem::getComponents] {
        if {![isComponent_opensource $name]} {
            lappend del_cp_list $name
            continue
            if {[isDepNotOpenSource_gplv2 $name]} {
                lappend del_cp_list $name
            }
            continue
        }

        if {[isComponent_opensource_gpl $name]} {
            if {[isDepNotOpenSource $name]} {
                set fail_build 1
            } else {
                foreach keep_name [getComponent_dependencies $name] {
                    set optional [buildsystem::isComponentDepOptional $name $keep_name]
                    if {!$optional} {
                        lappend keep_cp_list $keep_name
                    }
                }
            }
        } else {
            lappend del_cp_list $name
        }
    }
    if {$fail_build} {
        exit 1
    }

    foreach name [lsort -unique $keep_cp_list] {
        puts "Keeping $name"
        set del_cp_list [lsearch -all -inline -not -exact $del_cp_list $name]
    }

    foreach name [lsort -unique $del_cp_list] {
        set lic     [getComponent_license $name]
        puts "Removing component $name License: $lic"
        defconfig_disable_component $name
        delComponents $name
    }
}

proc buildsystem::isDepNotOpenSource_gplv2 { name } { 
    set fail_build 0

    foreach dep_name [getComponent_dependencies $name] {
        if {![defconfig_name_is_set $dep_name]} {
            puts "Component $dep_name is disabled - skipping license check!"
            continue
        }

        if {![componentExists $dep_name]} {
            puts "\033\[31mError Component $name ($version) depends on a not existing component $dep_name \033\[0m"
            set fail_build 1
            continue
        }

        if {[opensrc_is_included $dep_name]} {
            continue
        }
        
        set version [buildsystem::getComponent_version $name]
        set dep_version [buildsystem::getComponent_version $dep_name]
        set optional [buildsystem::isComponentDepOptional $name $dep_name]

        if {$optional} {
            set opt_str "yes"
        } else {
            set opt_str "no"
        }

        if {[isComponent_opensource_gpl $dep_name]} {
            set lic     [getComponent_license $name]
            set dep_lic [getComponent_license $dep_name]
            if {$optional} {
                puts "\033\[33mWarning: component $name ($version) ($lic) is depending on $dep_name ($dep_version) which is $dep_lic licensed!\033\[0m" 
                puts "\033\[33mIs dependency $dep_name optional $opt_str\033\[0m"
            } else {
                puts "\033\[31mError: component $name ($version) ($lic) is depending on $dep_name ($dep_version) which is $dep_lic licensed!\033\[0m" 
                puts "\033\[31mIs dependency $dep_name optional $opt_str\033\[0m"
                set fail_build 1
            }
        } else {
            set res [isDepNotOpenSource_gplv2 $dep_name]
            set fail_build [expr $fail_build || $res]
        }   
    }
    return $fail_build
    }

proc buildsystem::isDepNotOpenSource { name } { 
    set fail_build 0

    foreach dep_name [getComponent_dependencies $name] {
        if {![defconfig_name_is_set $dep_name]} {
            puts "Component $dep_name is disabled - skipping license check!"
            continue
        }

        
        set version [buildsystem::getComponent_version $name]
        if {![componentExists $dep_name]} {
            puts "\033\[31mError Component $name ($version) depends on a not existing component $dep_name \033\[0m"
            set fail_build 1
            continue
        }

        set dep_version [buildsystem::getComponent_version $dep_name]
        set optional [buildsystem::isComponentDepOptional $name $dep_name]

        if {$optional} {
            set opt_str "yes"
        } else {
            set opt_str "no"
        }

        if {![isComponent_opensource_gpl $dep_name] && ![isComponent_opensource $dep_name]} {
            set lic     [getComponent_license $name]
            set dep_lic [getComponent_license $dep_name]
            if {$optional} {
                puts "\033\[33mWarning: component $name ($version) ($lic) is depending on $dep_name ($dep_version) which is $dep_lic licensed!\033\[0m" 
                puts "\033\[33mIs dependency $dep_name optional $opt_str\033\[0m"
            } else {
                puts "\033\[31mError: component $name ($version) ($lic) is depending on $dep_name ($dep_version) which is $dep_lic licensed!\033\[0m" 
                puts "\033\[31mIs dependency $dep_name optional $opt_str\033\[0m"
                set fail_build 1
            }
        } else {
            set res [isDepNotOpenSource $dep_name]
            set fail_build [expr $fail_build || $res]
        }   
    }
    return $fail_build
}

proc buildsystem::licenseIsGPLV2Style {license licenseType} {
    foreach lType [getDeliveryLicenseType] {
        if {[string match $lType $licenseType]} {
            return 1
        }
    }
    return 0
}

proc buildsystem::licenseIsBSDStyle {license licenseType} {
    foreach lType [getOptDeliveryLicenseTypes] {
        if {[string match $lType $licenseType]} {
            return 1
        }
    }
    return 0
}

proc buildsystem::getOptDeliveryLicenseTypes {} {
    variable deliver_sources_for_opt

    return $deliver_sources_for_opt
}

proc buildsystem::getDeliveryLicenseType {} {
    variable deliver_sources_for

    return $deliver_sources_for 
}

proc buildsystem::parseComponentLic { fileName } { 
    set license ""

    if {![file exists $fileName]} {
        puts "\033\[33mWarning license file doesn't exist: $fileName\033\[0m"
        return 0 
    }
    if {[catch {set fd [ open $fileName "r" ]} errMsg]} {
        puts "\033\[31mError opening $fileName: $errMsg\033\[0m"
        return 0 
    }
    if { 0 >= $fd } {
        puts "\033\[31mError: Illegal filedescriptor $fileName\033\[0m"
        return 0 
    }

    while {[gets $fd line] >= 0} {
        set line    [string tolower $line]
        set l       [split $line ":"]
        set lic     [lindex $l 1]
        set licType [lindex $l 2]
        if {[string length $lic] && [string length $licType]} {
            foreach lType [split $licType ","] {
                lappend license $lic $lType
            }
        }
    }   
    close $fd 
    return $license
}

