package provide buildsystem 1.0

namespace eval buildsystem {
    variable globalbuildcfg
    variable globalrootpathlst
    variable globaloptions
    variable globalcomponentlst
    namespace export get_perforce_rootpath read_rootpathcfg
}

proc buildsystem::get_server_path {name} {
    variable globalcomponentlst
    array set servercomponent $globalcomponentlst($name)
    return $servercomponent(serverpath)
}

proc buildsystem::get_server_name {name} {
    variable globalcomponentlst
    array set servercomponent $globalcomponentlst($name)
    return $servercomponent(servername)
}

proc buildsystem::get_root_path {name} {
    variable globaloptions
    variable globalcomponentlst
    variable globalrootpathlst

    if { [catch {array set servercomponent $globalcomponentlst($name)}] } {
    puts stderr "\033\[31mERROR -- could not find $name in component.lst\033\[0m"
    exit 1
}

    if {[info exists globaloptions("rootpathfromvcs")] == 1} {
        set rootpath [get_vcs_rootpath $servercomponent(servername)]
    } else {
        set rootpath $globalrootpathlst($servercomponent(servername))
    }
    return $rootpath
}

# get perforce rootpath
proc buildsystem::get_git_rootpath {servername} {
    variable globalbuildcfg

    array set server $globalbuildcfg($servername)
    return $server(wks_path)
}

# get perforce rootpath
proc buildsystem::get_perforce_rootpath {servername} {
    variable globalbuildcfg

    array set server $globalbuildcfg($servername)
    return $server(wks_path)
}

# get version control server address
proc buildsystem::get_vcs_srv_address {servername} {
    variable globalbuildcfg

    array set server $globalbuildcfg($servername)
    return $server(serveraddress)
}

# get version control workspace name
proc buildsystem::get_vcs_wks_name {servername} {
    variable globalbuildcfg

    array set server $globalbuildcfg($servername)
    return $server(wks_name)
}

# get version control root path
proc buildsystem::get_vcs_rootpath {servername} {
    variable globalbuildcfg
    variable globaloptions
    array set server $globalbuildcfg($servername)
    if {$server(proto) == "p4"} {
        return [buildsystem::get_perforce_rootpath $servername]
    } elseif {$server(proto) == "git"} {
        return [buildsystem::get_git_rootpath $servername]
    } else {
        error "\033\[31mFATAL ERROR : unknown server protocol $proto\033\[0m"
    }
}

proc buildsystem::getRootPaths {} {
    variable globalrootpathlst
    set pathrootlst ""
    foreach path [array names globalrootpathlst] {
        lappend pathrootlst $globalrootpathlst($path)
    }
    return $pathrootlst
}

proc buildsystem::getRootPaths_cvs {} {
    return [join [buildsystem::getRootPaths] ","]
}

proc buildsystem::compute_rootpathlst {} {
    variable globalbuildcfg
    variable globalrootpathlst
    log::log debug "Computing rootpath list"
    foreach name [array names globalbuildcfg] {
        set rootpath [get_vcs_rootpath $name]
        set globalrootpathlst($name) "$rootpath"
        log::log debug "rootpath $name=$rootpath"
    }
}

proc buildsystem::writeRootPathFile {filename} {
    variable globalrootpathlst
    parray globalrootpathlst
    set rootpathfile [open $filename w]
    foreach name [array names globalrootpathlst] {
        if { $globalrootpathlst($name) != "not set"} {
            puts $rootpathfile "$name=$globalrootpathlst($name)"
        }
    }
    close $rootpathfile
}

proc buildsystem::read_rootpathcfg {filename} {
    variable globalrootpathlst

    log::log debug "Reading rootpath configuration"
    if [file exists $filename] { 
	log::log debug "Opening $filename"
	set infile [open $filename r]

	while { [gets $infile line] >= 0} {
	    # is line not empty
	    if [string length $line] {
		# line starting with # are treated as comment
		if [regexp {^#} $line] {
		    continue
		}
	    }

	    set parts [split $line "="]
	    lassign $parts server rootpath
	    set globalrootpathlst($server) $rootpath
	}
     close $infile
     } else {
	puts "\033\[31mError : no $filename found\033\[0m"
	exit 1
     }
     return [array get globalrootpathlst]
}
