#!/bin/sh
# the next line restarts using tclsh \
    exec tclsh "$0" "$@"

set scriptdir [file dirname [info script]]
set libDirectory [file join $scriptdir lib]
set auto_path [linsert $auto_path 0 $libDirectory]

package require buildsystem
namespace import buildsystem::*

#parse commandline options
buildsystem::parseConfigPackagesArgs $::argv
array set globaloptions [buildsystem::getGlobalOptions]

# create original config and package global parameters
set original_configdir "[file dirname $globaloptions("projectdependenciesfile")]/../../.."
set package_configdir "[file dirname $globaloptions("projectdependenciesfile")]"
set package_projectlist [buildsystem::read_componentdeps "$package_configdir/project.list"]

# parse defconfig
buildsystem::parse_defconfig "$package_configdir/defconfig"
set package_defconf [buildsystem::defconfig_get_all_values]
buildsystem::parse_defconfig "$original_configdir/defconfig"
array set original_defconf [buildsystem::defconfig_get_all_values]

#read the server.cfg
array set globalbuildcfg [buildsystem::read_servers $globaloptions("buildconfigfile")]

# read opensource include list
# this is for forcefully including none opensource components in a opensource build
buildsystem::parse_include_list "$original_configdir/opensource_include.lst"

#read componentlist
array set globalcomponentlst [buildsystem::read_components "$scriptdir/component.lst"]

#compute root pathlist
buildsystem::compute_rootpathlst

# we directly read out a dependency file which acts more as a meta package
set original_projectlist [buildsystem::read_componentdeps "$original_configdir/project.list"]
buildsystem::read_gs_deps $original_projectlist
array set original_components [buildsystem::getAllComponentsInfo]
buildsystem::check_deps

# start handling package info
buildsystem::clearAllComponents

# merge original project.list with the package project.list
set merged_projectlist $original_projectlist
foreach {new_name new_filename new_assgm new_version new_release_as_binary new_debug_options} $package_projectlist {
    set new_component ""
    lappend new_component $new_name $new_filename $new_assgm $new_version $new_release_as_binary $new_debug_options
    set merged_projectlist [buildsystem::addComponentToList $merged_projectlist $new_component]
}

# read out all components from the merged list
buildsystem::read_gs_deps $merged_projectlist
set merged_components [buildsystem::getAllComponentsInfo]

# Check all components from the merged list; against the one from the original config.
# If a changed/added component is found, check if it's present in the package project.list
# If not, we forgot to add it!
puts "*** Crosschecking project.lists ***"
puts ""
foreach {m_cname m_cinfo} $merged_components {
    array set m_componentstruct [lindex $m_cinfo 0]
    if { [info exists original_components($m_cname)] } {
        array set o_componentstruct [lindex $original_components($m_cname) 0]
        if { [string compare $o_componentstruct(version) $m_componentstruct(version)] } {
            puts "Component '$m_cname' has changed: $o_componentstruct(version) --> $m_componentstruct(version)"
            if {[buildsystem::getComponentFromList $package_projectlist $m_cname] == -1} {
                puts "Error: changed component not in package project.list!"
                exit 1
            }
        }
    } else {
        puts "Component '$m_cname' has been added."
        if { [string compare $m_componentstruct(version) "?"] } {
            if {[buildsystem::getComponentFromList $package_projectlist $m_cname] == -1} {
                puts "Error: added component not in package project.list!"
                puts "Error: version needed $m_componentstruct(version)"
                exit 1
            }
        }
    }
}

# create all necessary config/make files
buildsystem::check_deps
buildsystem::writeRootPathFile "rootpath.cfg"
buildsystem::getSource

buildsystem::ConvertBuildconfigInfoToConfigIn
buildsystem::buildConfigAndComponentAll $scriptdir

buildsystem::createStagingAndReleaseDir 0
buildsystem::generateCompMakeFile 0
