#/bin/bash
#arguments:
# $1 = project
# $2 = version
# $3 = include dirs
# $4 = artifact

function sonar_generate_make_env() {
    local MF="./Makefile.global" TOOLCHAIN CROSS_COMPILE
    # we need the toolchain to create the cross compile var
    TOOLCHAIN=$(grep HARDCO_TOOLCHAIN_DIR "${MF}" | cut -d= -f2)
    CROSS_COMPILE=$(grep CONFIG_CROSS_COMPILE= "./output/staging/components.config" | cut -d= -f2)
    CROSS_COMPILE=${CROSS_COMPILE/\$(HARDCO_TOOLCHAIN_DIR)/${TOOLCHAIN}}
    echo "export CROSS_COMPILE=${CROSS_COMPILE}" > "./sonar.env"
    cat "${MF}" |\
        sed -n -e '/export CROSS_COMPILE/ d' \
               -e '/export PATH/ d' \
               -e '/export TARGET_MACHINE/ d' \
               -e '/export DUMP_CMD/ d' \
               -e '/\+\+/ d' \
               -e '/^export / s/ *= *\(.*\)/="\1"/p' \
               -e "/:=/ s/ *:= *\(.*\)/='\1'/p" |\
        sed -e 's/\$(/${/g' -e 's/)/}/g' >> "./sonar.env"
        
    build_system=$(cat buildconfig.default | grep ^BUILD=)
    build_component=$(echo $build_system | awk -F'=' '{print $1}')
    build_path=$(echo $build_system | awk -F'=' '{print $2}')

    echo "export BUILDDIR=$build_path" >> ./sonar.env
}

function sonar_properties_create() {
  mkdir -p output/staging/sonar
  
  project=$1
  version=$2
  artifact=$3
  
  echo "# requiered metadata" > ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo "sonar.projectKey=com.softathome:$project" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo "sonar.projectName=$project" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo "sonar.projectVersion=$version" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties

  if [ "$artifact" != "" ]; then
    echo "# to monitor size of artifact" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
    echo "sonar.artifact.path=output/release/delivery/$artifact" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
    echo >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  fi

  echo "# path to source directories (required)"  >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo "sonar.sources=." >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  
  echo "# The value of the property must be the key of the language." >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo "sonar.language=c++" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties

  echo "sonar.c.cppcheck.path=$(which cppcheck)" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
}

function sonar_properties_add_include_dirs() {
  component=$1
  path=$2
  toolchain_include_dirs=$3
  kernel_include_dirs=$4
  extra_include_dirs=$5

  includes="sonar.cxx.include_directories=/usr/lib/gcc/i686-linux-gnu/4.7/include,"
  separator=""
  
  if [ "$(echo $extra_include_dirs | grep "%KERNEL%")" != "" ]; then
      kernel_root=$(cat ${BUILDDIR}/buildconfig.default | grep "SAH_KERNEL=" | awk -F'=' '{print $2}')
      dirs=$(echo $kernel_include_dirs | tr "," " ")
      temp=""
      for i in $dirs; do
        temp=$temp$separator$kernel_root$i
        separator=","
      done
      separator=""
      extra_include_dirs=$(echo $extra_include_dirs | sed "s#%KERNEL%#$temp#g")
  fi
  
  if [ "$toolchain_include_dirs" != "" ]; then
    toolchain_dir=$(dirname $CC)/../..
      dirs=$(echo $toolchain_include_dirs | tr "," " ")
    for i in $dirs; do
        includes=$includes$separator$toolchain_dir$i
        separator=","
    done
  fi
  includes=$includes$separator${BUILDDIR}/output/staging/include
  if [ "$extra_include_dirs" != "" ]; then
    includes=$includes","$extra_include_dirs
  fi

  includes=$includes","$path
  includes=$includes","$path"/include,."
  
  # on x86 make sure that /usr/include,/usr/include/i386-linux-gnu are added as extra include dirs
  echo "# Optional comma-separated list of additional libraries folders (such as /usr/include)" >>  ${BUILDDIR}/output/staging/sonar/$component/sonar-project.properties
  echo "$includes"  >> ${BUILDDIR}/output/staging/sonar/$component/sonar-project.properties
  echo >> ${BUILDDIR}/output/staging/sonar/$component/sonar-project.properties
}

function sonar_properties_add_modules() {
  echo "# List of the module identifiers" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties

  modules="sonar.modules="
  separator=""

  for i in $(cat buildconfig.default); do
    component=$(echo $i | awk -F'=' '{print $1}')
    path=$(echo $i | awk -F'=' '{print $2}')
    echo "Component = $component, Path = $path"
    
    COMPONONENT_IS_SET=$(grep CONFIG_$component=y "./output/staging/components.config" | cut -d= -f2)
    if [ "$COMPONONENT_IS_SET" == "y" ] && [ "$component" != "BUILD" ]; then
      if [ -e $path/sonar-project.properties ]; then
        echo "Component $component has sonar properties"
        modules=$modules$separator$component
        separator=","
        
        mkdir -p output/staging/sonar/$component
        component_include_dirs=$(cat $path/sonar-project.properties | grep "sonar\.c\.library\.directories=")
        toolchain_dir=$(dirname $CC)/../..
        staging_dir=${BUILDDIR}/output/staging/include
        
        chmod 644 $path/sonar-project.properties
        cat $path/sonar-project.properties | sed  -e '/sonar\.c\.library\.directories=/ d' | \
                                             sed "s#%COMPONENT_NAME%#$component#g" | \
                                             sed "s#%COMPONENT_PATH%#$path#g" | \
                                             sed "s#%TOOLCHAIN%%#$toolchain_dir#g" | \
                                             sed "s#%STAGING%#$staging_dir#g" >  ${BUILDDIR}/output/staging/sonar/$component/sonar-project.properties
        echo >> ${BUILDDIR}/output/staging/sonar/$component/sonar-project.properties

        echo "sonar.language=c++" >> $path/sonar-project.properties

        echo "" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
        echo "$component.sonar.projectBaseDir=$path" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
        echo "" >>  ${BUILDDIR}/output/staging/sonar/sonar-project.properties

        if [ "$component_include_dirs" != "" ]; then
          sonar_properties_add_include_dirs $component $path $TOOLCHAIN_INCLUDE_DIRS $KERNEL_INCLUDE_DIRS $(echo $component_include_dirs | awk -F'=' '{print $2}' | \
                                                                                                                                     sed "s###g" | \
                                                                                                                                     sed "s#%COMPONENT_PATH%#$path#g" | \
                                                                                                                                     sed "s#%TOOLCHAIN%#$toolchain_dir#g" | \
                                                                                                                                     sed "s#%STAGING%#$staging_dir#g")
        else
          sonar_properties_add_include_dirs $component $path $TOOLCHAIN_INCLUDE_DIRS
        fi
        
        mv ${BUILDDIR}/output/staging/sonar/$component/sonar-project.properties $path/sonar-project.properties
      fi
    fi
  done

  echo ""  >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo "project.home=${BUILDDIR}/output/staging/sonar/" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
  echo ""  >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties

  echo "$modules" >> ${BUILDDIR}/output/staging/sonar/sonar-project.properties
}

sonar_generate_make_env
. "./sonar.env"
. "${BUILDDIR}/output/staging/buildconfig.info"
if [ -e "${BUILD_DEF}/sonar.config" ]; then
  . "${BUILD_DEF}/sonar.config"
elif [ -e "${BUILDDIR}/${BUILD_DEF}/sonar.config" ]; then
  . "${BUILDDIR}/${BUILD_DEF}/sonar.config"
fi

if [ "$1" != "" ]; then
  PROJECT=$1
fi

if [ "$3" != "" ]; then
  TOOLCHAIN_INCLUDE_DIRS=$3
fi

sonar_properties_create $PROJECT $2 $4

sonar_properties_add_modules
