#!/usr/bin/perl

use strict;
use WWW::Bugzilla::Search;

require 'lib/core.pl';
require 'lib/file.pl';
require 'lib/string.pl';
require 'lib/compare.pl';
require 'lib/vcs.pl';

my $url     = 'bugzilla.softathome.com/bugzilla';
my $user    = 'david.cluytens_ext@softathome.com';
my $password= $ENV{P_SCRIPT_PASSWD};

my $sop_target  = "7.0";
my $sop_ver	= "10.11.0";
my $sop_rel	= "2015-03-25_V10.11.0";
my $sop_path	= "/config";

my $bz = WWW::Bugzilla::Search->new(
        protocol => 'https',
        server => $url, 
        email => $user,
        password => $password);

$bz->classification('Home Gateway');
$bz->product('MAINLINE_GW');
$bz->fixedversion($sop_ver);
$bz->resolution('CONFIG_UPDATED');
my @bugs = $bz->summary_search();

printHeader($sop_target, $sop_ver, $sop_rel);

print "\nThe following bugs were reported on MAINLINE_GW\n";
my %bz_bugs = ();
foreach my $bug (@bugs) {
	my $bug_id 	= $bug->{'bug_number'};
	my $bug_desc	= $bug->{'summary'};
	$bz_bugs{$bug_id} = trim($bug_desc);
}

my $i = 1;
foreach my $bug_id (sort(keys(%bz_bugs))) {
	print "$i Bugno: " . $bug_id . " - " . $bz_bugs{$bug_id} . "\n";
	$i++;
}

my $i = 1;
my %p4_bug_list = ();
getP4Bugs(\%p4_bug_list, $sop_path, $sop_rel);
print "\nThe following bugs were also integrated.\n";
foreach my $bug_id (sort(keys(%p4_bug_list))) {
	if (!exists $bz_bugs{$bug_id}) {
		print "$i: Bug " . $bug_id . " - " . $p4_bug_list{$bug_id} . "\n";
		$i++;
	}
}

sub getP4Bugs {
	my $bug_list	= shift;
	my $sop_path 	= shift;
	my $sop_rel	= shift;
	my %bug_list 	= ();
	my $ch_info 	= p4GetChangeList("w1", $sop_path, $sop_rel);

	parseChangeLists($bug_list, $ch_info);
}

sub parseChangeLists {
	my $bug_list 	= shift;
	my $ch_info 	= shift;
	while ($ch_info =~ /Bug\s+(\d+)\s+-?(.*)/g) {
		my $bug_id = $1;
		my $bug_desc = $2;
		$bug_list->{$bug_id} = trim($bug_desc);
		parseChangeLists($bug_list, $bug_desc);
	}
}

sub printHeader {
	my $sop_target	= shift;
	my $sop_ver	= shift;
	my $sop_rel	= shift;
    my $url     = "";
    my $first   = 1;

    (my $year, my $month, my $date) = $sop_rel =~ /(\d+)-(\d+)-(\d+)_V.*/;
    $url = "$year/$month/$sop_rel";
	print "
HGW Release $sop_ver is available, intended for the V$sop_target target.

Highlights

Images can be found here:
http://builds.be.softathome.com/REL/$url/

Test results:
https://builds.be.softathome.com:8080/view/TESTS/view/RELEASES/


";
}

exit 0;
