#!/usr/bin/perl

use strict;
use Getopt::Long;

require 'tools/lib/core.pl';
require 'tools/lib/file.pl';
require 'tools/lib/string.pl';
require 'tools/lib/compare.pl';


our @files      = ();

my $config_name;
my $help		= 0;
my $proj_file;
my $cp_file     = "component.lst";

GetOptions('help'           => sub { $help = 1 },
		'file=s'         => \$proj_file,
		) or printSyntax();


if ($help) {
	printSyntax();
} elsif (defined($proj_file)) {
	if ($proj_file =~ /^.*(\/config\/MAIN\/.*)\/project\.list$/) {
		$config_name = $1;
		print "//depot${config_name}... //jenkins${config_name}...\n";

		parseComponentLst($cp_file);
		parseConfig($proj_file);
		exit 0
	}
}
printSyntax();
exit 1;

sub printSyntax {
    print "cJenkinsView.pl   --file  <project.list>\
Example:\
		./tools/cJenkinsView.pl --file ~/ws_w1/config/CI/sah/generic/sagem_lbv3/sah_multi_sip_normal/project.list\n\n";
}

sub parseConfig {
	my $fh;
	my $file = shift;
	my @lines = readFile($file);
	foreach my $line (@lines) {
		parseLine($line);
	}
}

sub parseLine {
	our $configs;
	my $line = shift;
	my $cp;
	my $path;
	my $version;
	my $location;
	my $server;
	my $uri;
	my $cur;
	my $latest;
	my $b_type;
	my $p   = 1;
	my $ch_info = "";
	my $operator = "";
#   printf("DEBUG LINE $line");
	if ($line =~ /^:?(.*)(=|>)(.*)\/(REL|DEV)\/(.*)$/) {
		$cp         = $1;
		$operator	= $2;
		$path       = $3;
		$b_type     = $4;
		$version    = $5;
		if ($cp =~ /:(.*)/) {
			$cp = $1;
		}
		$server     = getComponentServer($cp);
		$location   = getComponentLocation($cp) . "/" . $path;
#       printf("DEBUG $cp $version\n");
		if ($server ne "" && $location ne "" ) {
			p4PrintURI($server, $location);
		}
	} elsif ($line =~ /^:?(.*)(=|>)(.*)/) {
		$cp         = $1;
		$operator	= $2;
		$version    = $3;
		if ($cp =~ /:(.*)/) {
			$cp = $1;
		}
		$server     = getComponentServer($cp);
		$location   = getComponentLocation($cp);
#        printf("DEBUG $cp $version\n");
		if ($server ne "" && $location ne "" ) {
			p4PrintURI($server, $location);
		}
	}
	if ($p) {
#print $line
	}
}

sub p4PrintURI {
	my $server      = shift;
	my $location    = shift;
	my $ver			= "MAIN";
	my $uri			= "";
	my $cp_file		= "";
	my @lines		= ();

	if (cp_cache_exist($server, $location)) {
		return;
	} 

	cp_cache_add($server, $location, $ver);
	if ($server eq "w1") {
		$uri 		= "//depot${location}/MAIN/... //jenkins${location}/MAIN/...";
		$cp_file 	= "//depot${location}/MAIN/Component.dep";
		@lines = `p4 print $cp_file 2>/dev/null`;
		foreach my $line (@lines) {
			parseLine($line);
		}
		print "$uri\n";
	} 
	return;
}

