#!/usr/bin/perl
###############################################################################
#
# Tool for comparing the versions of components in the project list
# with the latest available version in perforce.
# Email <david.cluytens_ext@softathome.com>
#
###############################################################################
use strict;
use Getopt::Long;

require 'tools/lib/core.pl';
require 'tools/lib/string.pl';
require 'tools/lib/file.pl';
require 'tools/lib/compare.pl';

my $cp_file     = "component.lst";
my $version_txt = 'version.txt';
my $cp_txt      = 'cp.txt';

my $help        = 0;
my $proj_file;
our @files      = ();
our $upgrade    = 0;

GetOptions('help'           => sub { $help = 1 },
           'file=s'         => \$proj_file,
            ) or printSyntax();

parseComponentLst($cp_file);
my @version_arr = parseVersionTxt($version_txt);
parseCpTxt($cp_txt, \@version_arr);
exit 0;

sub parseCpTxt {
    my $fh;
    my $file        = shift;
    my @version_arr = @{$_[0]};
    my @lines       = readFile($file);

    foreach my $line (@lines) {
        $line = trim($line);
        $line = removeTrailSlash($line);
        if (!parseCpLine($line, \@version_arr)) {
            print "$line cp not found\n";
        }
    }
}

sub parseVersionTxt {
    my $fh;
    my $file = shift;
    my @version_arr = ();
    my @lines = readFile($file);
    foreach my $line (@lines) {
        my $ver = parseVersionLine($line);
        if ($ver ne "") {
            push(@version_arr, $ver);
        }
    }
    return @version_arr;
}

sub parseCpLine {
    our $configs;
    my $line        = shift;
    my @version_arr = @{$_[0]};

    (my $line_path, my $line_rel) = getReleaseStringFromP4Uri($line);
    (my $ldate, my $lmajor, my $lminor, my $lmicro, my $lpico) = parseVersionString($line_rel);
    foreach my $version (@version_arr) {
        (my $ver_path, my $ver_rel) = getReleaseStringFromP4Uri($version);
        if ($ver_path eq $line_path) {
            (my $vdate, my $vmajor, my $vminor, my $vmicro, my $vpico) = parseVersionString($ver_rel);
            my $cmp_result = compareVersionString($ldate, $vdate, $lmajor, $vmajor, $lminor, $vminor, $lpico, $vpico);
            if ($cmp_result > 0) {
                return 1;
            } elsif ($cmp_result == 0) {
                return 1;
            } else {
                print "Less $version $line\n";
                return 1;
            }
        }
    }
    return 0;
}

sub parseVersionLine {
    our $configs;
    my $line    = shift;
    my $ver     = "";

    if ($line =~ /^:?(.*)=(.*)$/) {
        my $cp         = $1;
        my $version    = $2; 

        my $server     = getComponentServer($cp);
        my $location   = getComponentLocation($cp);
        if ($server ne "" && $location ne "" ) {
            $ver = "//depot${location}/${version}";
            $ver = removeTrailSlash($ver);
        }
    }
    return $ver
}
