#!/usr/bin/perl
use strict;
use Getopt::Long;
my $help = 0;
my $path;
my $ver;
my $spin    = "00";
my $p4_args;
my $p4_workspace;

GetOptions('help'           => sub { $help = 1 },
        'path=s'            => \$path,
        'version=s'         => \$ver,
        'spin=s'            => \$spin,
        'p4_args=s'         => \$p4_args,
        'p4_workspace'      => \$p4_workspace,
        ) or printSyntax();

if ($path =~ /^(.*)\/\.\.\.$/) {
    $path =$1;
}

if ($p4_args =~ /^(.*),(.*)$/) {
    $ver    = $1;
    $spin   = $2;
} else {
    print "Error invalid p4_args: $p4_args\n";
}

if (-d $path && $ver) {
    updateVersionString($path, $ver, $spin);
} else {
    print "Error invalid or missing arguments: path=$path, version=$ver, spin=$spin\n";
    printSyntax();
}

sub printSyntax {
	print "Recursively change the release number and reset the spin number in the defconfig of the projects.\n";
	print "Usage: ./chver.pl --path <config path> --version <version_string> [--spin <spin_number>]\n";
	print "Example: ./chver.pl --path /home/sah5056/ws_w1/config/MAIN --version 3.50.1\n";
	print "Example: ./chver.pl --path /home/sah5056/ws_w1/config/MAIN --version 3.50.1 --spin 03\n";
	exit 1;
}

sub updateVersionString {
    my $path = shift;
    my $ver  = shift;
    my $spin = shift;
    my $what = "defconfig";

    my $spin_length = length($spin);
    if (!$spin_length) {
        $spin = "00";
    } elsif (1 == $spin_length) {
        $spin = "0".$spin;
    }

    my $chlist = p4CreateChangeList("\tSetting version to $ver spin $spin");
    print "Recursively setting version to '$ver', spin='$spin'\n";

    my @files = `find $path -name $what`;

    foreach my $name (@files) {
        chomp $name;
        print "\nScanning $name\n";
        my $touched = 0;

#read in file
        open FILE, $name or die $!;
        my @lines = <FILE>;
        close FILE;

# check if we (need to) replace something
        my @newfile = ();
        foreach my $line (@lines) {
            if ($line =~ m/CONFIG_VERSION_NAME_MAJOR=/) {
                my $newline = "CONFIG_VERSION_NAME_MAJOR=\"$ver\"\n";
                print "$line -> $newline";
                $line = $newline;
                $touched = 1;
            }
            elsif ($line =~ m/CONFIG_BUILD_SPIN_NUMBER=/) {
                my $newline = "CONFIG_BUILD_SPIN_NUMBER=\"$spin\"\n";
                print "$line -> $newline";
                $line = $newline;
                $touched = 1;
            }
            push(@newfile, $line);
        }

# if we edit, p4 edit and rewrite
        if ($touched) {
            print ">>>>Editing $name\n";
            if ($chlist) {
                system "p4 edit -c $chlist $name";
            } else {
                system "p4 edit $name";
            }
            open FILE, "+>", $name or die $!;
            print FILE @newfile;
            close FILE;
        }
    }
}

sub p4CreateChangeList {
    my $description = shift;
    my @spec        = (); 
    my $ch_spec     = ""; 

    $description = escapeString($description);
    @spec = `p4 change -o | sed '/^Files:/q'`;
    foreach my $line (@spec) {
        $line =~ s#^\s+<enter description here>$#$description#g;
        $ch_spec = $ch_spec . $line;
    }   
    my  $chlist = `echo "$ch_spec" | p4 change -i | cut -d ' ' -f 2`;
    chomp $chlist;
    return $chlist; 
}

sub escapeString {
    my $string = shift;
    $string  =~ s/\(/\\(/g;
                $string  =~ s/\)/\\)/g;
    $string  =~ s/\'/\\'/g;
    $string  =~ s/\`/\\`/g;
    $string  =~ s/\"/\\"/g;
    $string  =~ s/\./\\./g;
    return $string;
}

