#!/usr/bin/perl

use strict;
use Getopt::Long;

require 'tools/lib/core.pl';
require 'tools/lib/file.pl';
require 'tools/lib/string.pl';
require 'tools/lib/compare.pl';


our @files      = ();
our %CPLIST     = {};

my $config_name;
my $help		= 0;
my $proj_path;
my $cp_file     = "component.lst";

GetOptions('help'           => sub { $help = 1 },
		'path=s'         => \$proj_path,
		) or printSyntax();


if ($help) {
	printSyntax();
} elsif (defined($proj_path)) {
    my @projs=`find $proj_path -name project.list`;
    foreach my $proj_file (@projs) {
		parseComponentLst($cp_file);
		parseConfig($proj_file);
	}
    printCP();
    exit 0;
} 
printSyntax();
exit 1;

sub printSyntax {
    print $ARGV[0] . "   --path  <config_path>\n";
    exit 1;
}

sub parseConfig {
	my $fh;
	my $file = shift;
	my @lines = readFile($file);
	foreach my $line (@lines) {
		parseLine($line);
	}
}

sub parseLine {
	our $configs;
	my $line = shift;
	my $cp;
	my $path;
	my $version;
	my $location;
	my $server;
	my $cur;
	my $latest;
	my $b_type;
	my $p   = 1;
	my $ch_info = "";
	my $operator = "";

    if ($line =~ /^(.*)?#.*$/) {
        $line = $1;
    }

	if ($line =~ /^:?(.*)(=|>)(.*)\/(REL|DEV)\/(.*)$/) {
		$cp         = $1;
		$operator	= $2;
		$path       = $3;
		$b_type     = $4;
		$version    = $5;
		if ($cp =~ /:(.*)/) {
			$cp = $1;
		}
        addCP($cp);
		$server     = getComponentServer($cp);
		$location   = getComponentLocation($cp) . "/" . $path;
		if ($server ne "" && $location ne "" ) {
			p4PrintURI($server, $location);
		}
	} elsif ($line =~ /^:?(.*)(=|>)(.*)/) {
		$cp         = $1;
		$operator	= $2;
		$version    = $3;
		if ($cp =~ /:(.*)/) {
			$cp = $1;
		}
		$server     = getComponentServer($cp);
		$location   = getComponentLocation($cp);
        addCP($cp);
		if ($server ne "" && $location ne "" ) {
			p4PrintURI($server, $location);
		}
	}
}

sub addCP {
    my $cp = shift;
    our $CPLIST;

    if (exists($CPLIST->{$cp})){
        return;
    }
    $CPLIST->{$cp} = 1;
}

sub printCP {
    our $CPLIST;

    foreach my $cp (sort keys($CPLIST)) {
        print $cp . "\n";
    }
}

sub p4PrintURI {
	my $server      = shift;
	my $location    = shift;
	my $ver			= "MAIN";
	my $cp_file		= "";
	my @lines		= ();

	if (cp_cache_exist($server, $location)) {
		return;
	} 

	cp_cache_add($server, $location, $ver);
	if ($server eq "w1") {
		$cp_file 	= "//depot${location}/MAIN/Component.dep";
		@lines = `p4 -p sahwp4s01.be.softathome.com:1666 print $cp_file 2>/dev/null`;
		foreach my $line (@lines) {
			parseLine($line);
		}
    } elsif ($server eq "n1") {
		$cp_file 	= "/${location}/MAIN/Component.dep";
		@lines = `p4 -p sahwp4s01.be.softathome.com:1999 print $cp_file 2>/dev/null`;
		foreach my $line (@lines) {
			parseLine($line);
		}
    } 
    return;
}

