#!/usr/bin/perl
##
#
#   david.cluytens@softathome.com
#
##
use strict;
use Time::Piece;
use Getopt::Long;

my $cDate   = Time::Piece->new->strftime('%Y-%m-%d_%H:%M:S');
my $help    = 0;

GetOptions('help'           => sub { $help = 1 },
        'uri=s'             => \my $uri,
        ) or printSyntax();


if(!defined($uri)) {
    print("Error invalid arguments\n");
    exit 1;
}

my @p4rels = p4GetRels($uri);
print("# Do not alter!!!\n");
print("# This config file is automatic generated\n");
print("# Generated @" . $cDate . "\n");
print("\n");

print("[\@repo]\n");
print("description = A Git Fusion repo config which maps the Talkhouse project Perforce branches to Git branches.\n");
print("charset = utf8\n");
print("enable-git-branch-creation = yes\n");
print("enable-git-merge-commits = yes\n");
print("ignore-author-permissions = no\n");
print("change-owner = author\n");
print("\n");

foreach my $item (@p4rels) {
    print("[myrepo-$item]\n");
    print("git-branch-name = REL_" . $item . "\n");
    print("view = " . $uri . "/" . $item . "/... ...\n");
    print("\n");
}

print ("[myrepo-master]\n");
print ("git-branch-name = master\n");
print ("view = " . $uri . "/MAIN/... ...\n");

exit 0;

sub trim {
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

sub p4GetRels {
    my $location    = shift;
    my $fh;
    my $p4location;
    my @p4rels      = ();

    $p4location = "${location}/REL/";
    open($fh, "p4 -p sahwp4s01.be.softathome.com:1666 dirs ${p4location}* |");
    while (!eof($fh)) {
        my $line = <$fh>;
        my $len  = length $p4location;
        my $cver = substr $line, $len;
        my $update_ver = 0;
        $cver = trim($cver);
        if ($cver =~ /^([\d]+-[\d]+-[\d]+)_V([\d]+)\.([\d]+)\.([\d]+)$/) {
            push(@p4rels, $cver);
        } 
    }   
    close $fh;
    return(@p4rels);
}

