sub parseVersionString {
    my $cver        = shift;
    my $cur_date    = 0;
    my $cur_major   = 0;
    my $cur_minor   = 0;
    my $cur_micro   = 0;
    my $cur_pico    = 0;

    $cver = trim($cver);
    if ($cver =~ /^(.*)_[v|V](\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
        $cur_date   = $1;
        $cur_major  = $2;
        $cur_minor  = $3;
        $cur_micro  = $4;
        $cur_pico   = $5;
    } elsif ($cver =~ /^(.*)_[v|V](\d+)\.(\d+)\.(\d+)$/) {
        $cur_date   = $1;
        $cur_major  = $2;
        $cur_minor  = $3;
        $cur_micro  = $4;
        $cur_pico   = 0;
    } elsif ($cver =~ /^(.*)_[v|V](\d+)\.(\d+)$/) {
        $cur_date   = $1;
        $cur_major  = $2;
        $cur_minor  = $3;
        $cur_micro  = 0;
        $cur_pico   = 0;
    } else {
        print "Warning: Parsing invalid current version ${cver} skipping\n";
    }
    return ($cur_date, $cur_major, $cur_minor, $cur_micro, $cur_pico);
}

# compares cur against l
# returns 1 if greater
# returns 0 if not greater
sub compareVersionString {
    my $ldate           = shift;
    my $cur_date        = shift;
    my $lmajor          = shift;
    my $cur_major       = shift;
    my $lminor          = shift;
    my $cur_minor       = shift;
    my $lmicro          = shift;
    my $cur_micro       = shift;
    my $lpico           = shift;
    my $cur_pico        = shift;

    if ($cur_major > $lmajor) {
        return 5;
    } elsif ($cur_major == $lmajor) {
        if ($cur_minor > $lminor) {
            return 4;
        } elsif ($cur_minor == $lminor) {
            if ($cur_micro > $lmicro) {
                return 3;
            } elsif ($cur_micro == $lmicro) {
                if ($cur_pico > $lpico) {
                    return 2;
                } elsif ($cur_pico == $lpico) {
                    if ($cur_date gt $ldate) {
                        return 1;
                    } elsif ($cur_date eq $ldate) {
                        return 0;
                    } else {
                        return -1;
                    }
                } else {
                    return -2;
                }
            } else {
                return -3;
            }
        } else {
            return -4;
        }
    } else {
        return -5;
    }
    return 0;
}

sub compareVersion {
    my $cver    = shift;
    my $lver    = shift;

    (my $cur_date, my $cur_major, my $cur_minor, my $cur_micro, my $cur_pico)   = parseVersionString($cver);
    (my $ldate, my $lmajor, my $lminor, my $lmicro, my $lpico)                  = parseVersionString($lver);

    my $cmp_result = compareVersionString($ldate, $cur_date, $lmajor, $cur_major, $lminor, $cur_minor, $lmicro, $cur_micro, $lpico, $cur_pico);
    return $cmp_result;
}


1;
