sub fixRepoName {
    my $cp  = shift;
    $cp         =~ s/^sah_services_mtd_utils$/sah_services_mtd-utils/;
    $cp         =~ s/^sah_services_bridge_utils$/sah_services_bridge-utils/;
    return $cp;
}

sub transformTagToGit {
    my $tag     = shift;
    my $new_tag = $tag;

    if ($tag =~ /^PROJ\/(.*)$/) {
        $new_tag    = "proj_" . $1;
    } elsif ($tag =~ /^GEN\/(.*)$/) {
        $new_tag    = "gen_" . $1;
    } elsif ($tag =~ /^REL\/(.*)_[v|V](.*)$/) {
        $new_tag    = "master_v" . $2;
    }

    if ($new_tag =~ /^(.*)\/REL\/.*_[v|V](.*)$/) {
        $new_tag = $1 . "_v" . $2;
    }

    return "refs/tags/" . $new_tag;
}

sub makeRepoName {
    my $cp_name     = shift;
    my $version     = shift;
    my $new_tag     = "unknown";
    my $repo_path   = "unknown";

    if ($version =~ /^(.*)\/REL\/(.*)_[v|V](.*)$/) {
        $version = $1 . "_v" . $3;
    }

    if ($version =~ /^REL\/.*_[v|V](.*)$/) {
        $version = "master_v" . $1;
    }

    $version    = lc $version;
    $version    =~ s/\//_/g;
    return($cp_name, $version);
}

sub changeComponentDep {
    our $components;
    our $new_cplst;
    my $file_name   = shift;
    my @file        = readFile($file_name);

    my @newfile = ();
    foreach my $line (@file) {
        if ($line =~ /^(.*)\#.*$/) {
            $line = $1;
            $line = trim($line) . "\n";
        }
        if ($line =~ /^(.*)(<|>|=)(.*)$/) {
            my $cp          = $1;
            my $operator    = $2;
            my $version     = $3;
            if (($version eq "?") || ($version eq "*")) {
                push(@newfile, $line);
                next;
            }
# for component version use as default the version set in version.txt
# if needed this will be over written later by gitListTagsRepo
            $version     = transformTagToGit($components->{$cp}->{'tag'});
            my $repo_loc    = $components->{$cp}->{'repo_path'};
            if ($repo_loc ne "") {
                my @tags = gitListTagsRepo($repo_loc);
                foreach my $tag (@tags) {
                    $tag = "refs/tags/" . $tag;
                    if ($tag ne $version) {
                        $version = $tag;
                    }
                }
            }

            if (!(exists($components->{$cp}->{'tag'}))) {
                $new_cplst->{$cp} = "unknown";
            }

            $line           = $cp . $operator . $version . "\n";
        }
        push(@newfile, $line);
    }

    `chmod 644 $file_name`;
    open FILE, "+>$file_name" or die "writing to $file_name. Error: " . $!;
    print FILE @newfile;
    close FILE;
}

1;
