#!/usr/bin/perl
sub getLatestComponent {
    my $server      = shift;
    my $location    = shift;
    my $cur_ver     = shift;

    if ($server eq "w1") {
        return p4GetLatestComponent($server, $location);
    } elsif ($server eq "n1") {
        return p4GetLatestComponent($server, $location);
    } elsif ($server eq "wg1") {
        return gitGetLatestComponent($server, $location, $cur_ver);
    } elsif ($server eq "gerrit") {
        return gitGetLatestComponent($server, $location, $cur_ver);
    }
}

sub gitGetLatestComponent {
    our $GIT_CACHE_PATH;

    my $server      = shift;
    my $location    = shift;
    my $cur_version = shift;

    my $fh;
    my $gitlocation;
    my $cur_prefix  = 0;
    my $cur_major   = 0;
    my $cur_minor   = 0;
    my $cur_micro   = 0;
    my $cur_pico    = 0;
    my $lmajor      = 0;
    my $lminor      = 0;
    my $lmicro      = 0;
    my $lpico       = 0;
    my $lprefix     = 0;
    my $lver        = "";
    my $prefix      = "";

    if ($cur_version =~ /^(.*)_[a|A|v|V].*$/) {
        $prefix = $1;
    } else {
        print "Error parsing $cur_version\n";
        return;
    }
   
    my $git_location = $location . "/" . $prefix;
    if (cp_cache_exist($server, $git_location)) {
        my $d = cp_cache_get($server, $git_location);
        return cp_cache_get($server, $git_location);
    }

    if ($server eq "wg1") {
        my $git_component_dir = $GIT_CACHE_PATH . "$location";
#        print "DEBUG Testing $git_component_dir\n";
        if (-e $git_component_dir and -d $git_component_dir) {
#            print "DEBUG chdir to $git_component_dir\n";
            chdir $git_component_dir;
            my $cmd = "git fetch origin +refs/heads/*:refs/heads/*";
            system($cmd);
        } else {
            $git_component_dir = dirname($git_component_dir);
#            print "DEBUG chdir to $git_component_dir\n";
            make_path  $git_component_dir;
            chdir "$git_component_dir";

            my $uri = "ssh://git.be.softathome.com/srv/git".$location;
            my $cmd = "git clone --bare $uri";
            system($cmd);
        }
        open($fh, "git tag -l |");
    } elsif ($server eq "gerrit") {
        my $git_component_dir = $GIT_CACHE_PATH . "$location";
#print "DEBUG Testing $git_component_dir\n";
        if (-e $git_component_dir and -d $git_component_dir) {
#print "DEBUG chdir to $git_component_dir\n";
            chdir $git_component_dir;
            my $cmd = "git fetch origin +refs/heads/*:refs/heads/*";
            system($cmd);
#$git_component_dir=dirname(dirname($git_component_dir));
        } else {
#print ("DEBUG reseting git_component_dir to dirname($git_component_dir)");
            $git_component_dir = dirname($git_component_dir);
#print "DEBUG chdir to $git_component_dir\n";
            make_path  $git_component_dir;
            chdir "$git_component_dir";
            my $uri = "ssh://gerrit.be.softathome.com:29418".$location;
            my $cmd = "git clone --bare " . $uri . " " . $git_component_dir . $location;
# print "DEBUG running cmd " .$cmd ."\n";
            system($cmd);
# reset git_component_dir back to git cache path, this because we're in GIT_CACHE_PATH/core or something else for gerrit!
#$git_component_dir=dirname($git_component_dir);
        }
        $git_component_dir=$GIT_CACHE_PATH;
#        print ("DEBUG git --git-dir=$git_component_dir" .$location ." tag -l\n");
        open($fh, "git --git-dir=$git_component_dir" .$location . " tag -l|");
    } else {
        print "Error handling request gitGetLatestComponent($server, $location)";
        exit 1;
    }

    while (!eof($fh)) {
        my $line = <$fh>;
        my $len  = length $gitlocation;
        my $cver = substr $line, $len;
        $cver = trim($cver);

        ($cur_prefix, $cur_major, $cur_minor, $cur_micro, $cur_pico) = parseVersionString($cver);
        if ($cur_prefix ne $prefix) {
            next;
        }

        my $cmp_result = compareVersionString($lprefix, $cur_prefix, $lmajor, $cur_major, $lminor, $cur_minor, $lmicro, $cur_micro, $lpico, $cur_pico);
        if ($cmp_result > 0) {
            $lmajor = $cur_major;
            $lminor = $cur_minor;
            $lmicro = $cur_micro;
            $lpico  = $cur_pico;
            $lprefix  = $cur_prefix;
            $lver = $cver;
        }
    }
    close $fh;
# print "DEBUG LVER $lver\n";
    $git_location = $location . "/". $lprefix;
    cp_cache_add($server, $git_location, $lver);
    return $lver;
}

sub p4GetLatestComponent {
    my $server      = shift;
    my $location    = shift;
    my $fh;
    my $p4location;
    my $cur_date    = 0;
    my $cur_major   = 0;
    my $cur_minor   = 0;
    my $cur_micro   = 0;
    my $cur_pico    = 0;
    my $lmajor      = 0;
    my $lminor      = 0;
    my $lmicro      = 0;
    my $lpico       = 0;
    my $ldate       = 0;
    my $lver        = "";

    if (cp_cache_exist($server, $location)) {
        return cp_cache_get($server, $location);
    }

    if ($server eq "w1") {
        $p4location = "//depot${location}/REL/";
        open($fh, "p4 -p sahwp4s01.be.softathome.com:1666 dirs ${p4location}* |");
    } elsif ($server eq "n1") {
        $p4location = "/${location}/REL/";
        open($fh, "p4 -p sahwp4s01.be.softathome.com:1999 dirs ${p4location}* |");
    } else {
        print "Error handling request p4GetLatestComponent($server, $location)";
        exit 1;
    }

    while (!eof($fh)) {
        my $line = <$fh>;
        my $len  = length $p4location;
        my $cver = substr $line, $len;

        $cver = trim($cver);
        ($cur_date, $cur_major, $cur_minor, $cur_micro, $cur_pico) = parseVersionString($cver);
        my $cmp_result = compareVersionString($ldate, $cur_date, $lmajor, $cur_major, $lminor, $cur_minor, $lmicro, $cur_micro, $lpico, $cur_pico);
        if ($cmp_result > 0) {
            $lmajor = $cur_major;
            $lminor = $cur_minor;
            $lmicro = $cur_micro;
            $lpico  = $cur_pico;
            $ldate  = $cur_date;
            $lver = $cver;
        }
    }
    close $fh;
# print "DEBUG LVER $lver\n";
    cp_cache_add($server, $location, $lver);
    return $lver;
}

sub getChangeLists {
    my $server      = shift;
    my $location    = shift;
    my $cver        = shift;
    my $lver        = shift;


    if ($server eq "w1") {
        return p4GetChangeLists($server, $location, $cver, $lver);
    }  elsif ($server eq "n1") {
        return p4GetChangeLists($server, $location, $cver, $lver);
    } elsif ($server eq "wg1") {
        return gitGetChangeLists($server, $location, $cver, $lver);
    } elsif ($server eq "gerrit") {
        return gitGetChangeLists($server, $location, $cver, $lver);
    } else {
        return "Error unknown server protocol specified $server";
    }   
}

sub p4GetChangeLists {
    my $server      = shift;
    my $location    = shift;
    my $cver        = shift;
    my $lver        = shift;
    my @ver_list    = p4GetVersionsComponent($server, $location, $cver, $lver);
    my $ch_info     = "";
   
    foreach my $version (sort(@ver_list)) {
        $ch_info = $ch_info . p4GetChangeList($server, $location, $version);
    }
    return $ch_info;
}

sub p4GetChangeList {
    our $GIT_CACHE_PATH;
    my $server      = shift;
    my $location    = shift;
    my $lver        = shift;
    my $fh;
    my $p4location;
    my $ch_info;


    if ($server eq "w1") {
        $p4location = "//depot${location}/REL/$lver/...";
        open($fh, "p4 -p sahwp4s01.be.softathome.com:1666 changes -l ${p4location} |");
    }  elsif ($server eq "n1") {
        $p4location = "/${location}/REL/$lver/...";
        open($fh, "p4 -p sahwp4s01.be.softathome.com:1999 changes -l ${p4location} |");
    }

    while (!eof($fh)) {
        my $line = <$fh>;
        $line = trim($line);
        if ($line =~ /^Change \d+ on \d+\/\d+\/\d+ by \w+@\w+$/) {
            next;
        } elsif ($line =~ /\w+.*/) {
            $ch_info = $ch_info . "\t" . $line . "\n";
        }
    }
    close $fh;
    return $ch_info;
}

sub gitGetChangeLists {
    our $GIT_CACHE_PATH;
    my $server      = shift;
    my $location    = shift;
    my $cver        = shift;
    my $lver        = shift;
    my $fh;
    my $p4location;
    my $ch_info;

    if ($server eq "wg1") {
        my $git_component_dir = $GIT_CACHE_PATH . $location;
#        print "DEBUG Changelist Testing $git_component_dir\n";
        if (-e $git_component_dir and -d $git_component_dir) {
#            print "DEBUG Changelist chdir to $git_component_dir\n";
            chdir $git_component_dir;
            open($fh, "git log  --pretty=medium $cver..$lver |");
        } else {
            return "Error retrieving git changelist for tag $cver..$lver";
        }
    } elsif ($server eq "gerrit") {
        my $git_component_dir = $GIT_CACHE_PATH . $location;
#        print "DEBUG Changelist Testing $git_component_dir\n";
        if (-e $git_component_dir and -d $git_component_dir) {
#            print "DEBUG Changelist chdir to $git_component_dir\n";
            chdir $git_component_dir;
            open($fh, "git log  --pretty=medium $cver..$lver |");
        } else {
            return "Error retrieving git changelist for tag $cver..$lver";
        }
    } else {
        return "Error unknown server protocol specified $server";
    }   

    while (!eof($fh)) {
        my $line = <$fh>;
        $ch_info = $ch_info . "\t" . $line;
    }
    close $fh;
    return $ch_info;
}

sub p4GetVersionsComponent {
    my $server      = shift;
    my $location    = shift;
    my $start_ver   = shift;
    my $stop_ver    = shift;

    my $fh;
    my $p4location;
    my @ver_list    = ();

    if ($server eq "w1") {
        $p4location = "//depot${location}/REL/";
        open($fh, "p4 -p sahwp4s01.be.softathome.com:1666 dirs ${p4location}* |");
    } elsif ($server eq "n1") {
        $p4location = "/${location}/REL/";
        open($fh, "p4 -p sahwp4s01.be.softathome.com:1999 dirs ${p4location}* |");
    } else {
        print "Error handling request p4GetLatestComponent($server, $location)";
        exit 1;
    }

    (my $start_date, my $start_major, my $start_minor, my $start_micro, my $start_pico) = parseVersionString($start_ver);
    (my $stop_date, my $stop_major, my $stop_minor, my $stop_micro, my $stop_pico) = parseVersionString($stop_cver);

    while (!eof($fh)) {
        my $line = <$fh>;
        my $len  = length $p4location;
        my $cver = substr $line, $len;

        $cver = trim($cver);
        (my $cur_date, my $cur_major, my $cur_minor, my $cur_micro, my $cur_pico) = parseVersionString($cver);
        my $start_cmp_result    = compareVersionString($start_date, $cur_date, $start_major, $cur_major, $start_minor, $cur_minor, $start_micro, $cur_micro, $start_pico, $cur_pico);
        my $stop_cmp_result     = compareVersionString($stop_date, $cur_date, $stop_major, $cur_major, $stop_minor, $cur_minor, $stop_micro, $cur_micro, $stop_pico, $cur_pico);
#        print "Comparing $start_ver > $cver && $cver >= $stop_ver\n";
#        print "Comparing ($start_cmp_result > 0) && ($stop_cmp_result >= 0)\n";

#Cur version must be greater than start version and smaller or equal than stop version
        if (($start_cmp_result > 0) && ($stop_cmp_result >= 0)) {
            push(@ver_list, $cver);
        }
    }
    close $fh;
    return @ver_list;
}

sub p4Submit {
    my $ch_list = shift;
    print "Submiting: $ch_list\n";
    my $output = `p4 submit -c $ch_list`;
}
1;
