#!/usr/bin/perl

use strict;
use Git::Repository;
use Git::Repository::Command;
use List::Util qw( min max );

our $fd_cp_lst;
our $git_repo_path  = "/home/sah5056/gits";
our $index_path     = "/home/sah5056/git_repos";
our $proj_file      = "config/MAIN/sah/cssp/siligence_pkg/project.list";
our @proj_lines     = readFile($proj_file);
our $cplst          = {};

# Create repositories 
parseComponentLst("build/REL/2014-09-02_V8.4.7/component.lst");
open_cp_lst();
scanDir($index_path);
close_cp_lst();
write_proj();

# Check the versions of the Component.dep(s) and compare them with the tags
# found in the git repositories. If the version doesn't exist change it to an existing tag
parseComponentLst("$git_repo_path/component.lst");
check_cp_deps($git_repo_path);

exit 0;

sub stripName {
    my $repo_name = shift;
    $repo_name =~ s/^(.*)\/[^\/]*$/\1/;
    return $repo_name;
}

sub tstName {
    my $repo_name = shift;
    if($repo_name =~ /\//) {
        return 1;
    }
    return 0;
}

sub getComponentName {
    our $index_path;

    my $path        = shift;
    my $len         = (length $index_path) + 1;
    my $repo_name   = substr $path, $len;
    my $tst         = 1;

    for ($repo_name = stripName($repo_name); $tst; $repo_name = stripName($repo_name)) {
        my $cp_name  = $repo_name;

        $cp_name =~ s/\//_/g;
        $cp_name =~ s/^sah_tests_applications_hal(.*)$/sah_tests_tools\1/;
        if(existsComponent($cp_name)) {
            return $cp_name;
        }
        $tst = tstName($repo_name);
    }
    return "unknown";
}

sub write_proj {
    our @proj_lines;

    open FILE, "+>/home/sah5056/gits/project.list" or die $!; 
    print FILE @proj_lines;
    close FILE;
}

sub update_proj {
    our $proj_file;
    our @proj_lines;

    my $repo_name    = shift;
    my $repo_tag     = shift;


    my @newfile = (); 
    foreach my $line (@proj_lines) {
        $line  =~ s/sah_services_mtd_utils/sah_services_mtd-utils/g;
        $line  =~ s/sah_services_bridge_utils/sah_services_bridge-utils/g;
        if ($line =~ /^(.*:)?$repo_name.*$/) {
            $line = $1 . $repo_name. "=refs/tags/" . $repo_tag . "\n";
        } else {
            my $old_repo_name = $repo_name;
#            $repo_name  =~ s/-/_/g;
            if ($line =~ /^(.*:)?$repo_name.*$/) {
                $line = $1 . $old_repo_name. "=refs/tags/" . $repo_tag . "\n";
            }
        }
        push(@newfile, $line);
    } 
    @proj_lines = @newfile;
}

sub open_cp_lst {
    open FILE_LST, ">$git_repo_path/component.lst" || die print "Unable to create file";
}

sub write_cp_lst {
    my $buf = shift;
    print FILE_LST "$buf\n";
}

sub close_cp_lst {
    close FILE_LST;
}

sub create_repo {
    our $git_repo_path;
    my $source_path = shift;
    my $repo_name   = shift;
    my $repo_tag    = shift;
    my $r;

    my $new_path = $git_repo_path . "/" . $repo_name;
    `cp -rp $source_path $new_path`;

    changeComponentDep("$new_path/Component.dep");

    Git::Repository->run(init => $new_path);
    $r = Git::Repository->new(work_tree => $new_path );
    $r->run( add => '.' );
    $r->run( commit => '-m', 'Creating repo for component' . $repo_name . ' release ' . $repo_tag);
    $r->run( tag => '-m', 'Creating repo for component'. $repo_name . ' release ' . $repo_tag, $repo_tag );
}

sub update_repo {
    our $git_repo_path;
    my $source_path = shift;
    my $repo_name   = shift;
    my $repo_tag    = shift;
    my $r;

    my $new_path = $git_repo_path . "/" . $repo_name;
    `cp -rp $source_path $new_path`;

    changeComponentDep("$new_path/Component.dep");

    $r = Git::Repository->new(work_tree => $new_path );
    $r->run( add => '.' );
    $r->run( commit => '-m', 'Updating repo for component' . $repo_name . ' release ' . $repo_tag);
    $r->run( tag => '-m', 'Updating repo for component'. $repo_name . ' release ' . $repo_tag, $repo_tag );
}

sub reset_git_tag_by_location {
    my $new_path    = shift;
    my $repo_tag    = shift;
    my $r;

    if (-d $new_path) {
        $r = Git::Repository->new(work_tree => $new_path );
        $r->run( tag => '-d', $repo_tag );
        $r->run( add => '.' );
        $r->run( commit => '-m', 'Updating repo for component' . $new_path . ' release ' . $repo_tag);
        $r->run( tag => '-m', 'Updating repo for component'. $new_path . ' release ' . $repo_tag, $repo_tag );
    } else {
        print "Error reset_git_tag_by_location invalid path $new_path\n";
        exit 1;
    }
}

sub repo_tags_by_location {
    my $repo_loc   = shift;
    my $r;

    $r = Git::Repository->new(work_tree => $repo_loc);
    my $cmd = $r->run( tag => '-l');

    my @tags = split '\n', $cmd;
    return @tags;
}

sub repo_tag_exists {
    our $git_repo_path;
    my $repo_name   = shift;
    my $repo_tag    = shift;
    my $r;

    my $new_path = $git_repo_path . "/" . $repo_name;
    $r = Git::Repository->new(work_tree => $new_path );
    my $cmd = $r->run( tag => '-l');

    my @tags = split '\n', $cmd;
    foreach my $tag (@tags) {
        if ($tag eq $repo_tag) {
            return 1;
        }
    }
    return 0;
}

sub repo_exists {
    our $git_repo_path;
    my $repo_name   = shift;

    my $path = $git_repo_path . "/" . $repo_name;
    if (-d $path) {
        return 1;
    } else {
        return 0;
    }
}

sub transformTagToGit {
    my $tag     = shift;
    my $new_tag = $tag;

    if ($tag =~ /^PROJ\/(.*)$/) {
        $new_tag    = "proj_" . $1;
    } elsif ($tag =~ /^GEN\/(.*)$/) {
        $new_tag    = "gen_" . $1;
    } elsif ($tag =~ /^REL\/(.*)_[v|V](.*)$/) {
        $new_tag    = "master_v" . $2;
    }

    if ($new_tag =~ /^(.*)\/REL\/.*_[v|V](.*)$/) {
        $new_tag = $1 . "_v" . $2;
    }

    if ($new_tag ne $tag) {
        return "refs/tags/" . $new_tag;
    }
    return $new_tag;
}

sub changeComponentDep {
    my $file_name   = shift;
    my @file        = readFile($file_name); 

    my @newfile = (); 
    foreach my $line (@file) {
        if ($line =~ /^(.*)(<|>|=)(.*)$/) {
            my $cp = $1;
            my $operator = $2;
            my $tag = transformTagToGit($3);
            $cp  =~ s/sah_services_mtd_utils/sah_services_mtd-utils/g;
            $cp  =~ s/sah_services_bridge_utils/sah_services_bridge-utils/g;
            $line = $cp . $operator . $tag . "\n";
        } 
        push(@newfile, $line);
    } 

    `chmod 644 $file_name`;
    open FILE, "+>$file_name" or die "writing to $file_name. Error: " . $!; 
    print FILE @newfile;
    close FILE;
}

sub getRepoName {
    our $index_path;
    our $git_repo_path;
    my $path        = shift;
    my $new_tag     = "unknown";
    my $repo_path   = "unknown";

    my $len         = (length $index_path) + 1;
    my $repo_name   = substr $path, $len;
    my $cp_name     = getComponentName($path);

    my $cp_path     = getComponentLocation($cp_name);
    $len            = (length $cp_path);
    $repo_name      = substr $repo_name, $len;

    print "Handling TAG $repo_name\n";
    if ($repo_name =~ /^(.*)\/REL\/(.*)_[v|V](.*)$/) {
        $new_tag    = $1 . "_v" . $3;
    }

    if ($repo_name =~ /^REL\/.*_[v|V](.*)$/) {
        $new_tag = "master_v" . $1;
    }

    $new_tag    = lc $new_tag;
    $repo_name  = lc $repo_name;
    $repo_name  =~ s/\//_/g;
    $new_tag    =~ s/\//_/g;

    $repo_path  = $git_repo_path . "/" . $cp_name;
    return($cp_name, $repo_path, $new_tag);
}

sub scanDir {
    my $scanPath = shift;
    my $fileName;
    my $newPath;

    opendir(my $dirh, $scanPath) || die "Can't opendir $scanPath";
    while(readdir $dirh) {
        $fileName = $_; 
        if ($fileName eq "." || $fileName eq "..") {
            next;
        }
        $newPath = $scanPath . "/" . $fileName;

#Ignoring special files and symlinks
        if (-l $newPath || -p $newPath || -S $newPath || -b $newPath || -c $newPath) {
            next;
        }

        if (-d $newPath) {

            if ($newPath =~ /^.*\/REL\/.*$/) {
                (my $repo_name, my $repo_path, my $new_tag) = getRepoName($newPath); 
                print "Path $newPath Repo name: $repo_name tag $new_tag\n";
                if ($repo_name eq "unknown") {
                    print "Error Unknown repo found\n";
                    exit 1;
                }
                update_proj($repo_name, $new_tag);
                write_cp_lst("$repo_name=g1:$repo_path");

                my $re = repo_exists($repo_name);
                print "Repo $repo_name exists $re\n";
                if (!$re) {
                    create_repo($newPath, $repo_name, $new_tag);
                } else {
                    my $te = repo_tag_exists($repo_name, $new_tag);
                    print "Repo $repo_name tag $new_tag exists $te\n";
                    if (!$te) {
                        update_repo($newPath, $repo_name, $new_tag);
                    }
                }
            } else {
                scanDir($newPath);
            }
        } 
    }   
}

sub readFile {
    my $name = shift;
    open FILE, $name or print "Warning opening $name: $!";
    my @lines = <FILE>;
    close FILE;
    return @lines;
}

sub parseComponentLst {
    my  $file   = shift;
    our $cplst  = ();
    my  $cp;
    my  $location;
    my  $server;

    my @lines = readFile($file);
    foreach my $line (@lines) {
        if ($line =~ /^(.*)=(.*):(.*)$/) {
            $cp         = $1;
            $server     = $2;
            $location   = $3;
            $cp         =~ s/^sah_services_mtd_utils$/sah_services_mtd-utils/;
            $cp         =~ s/^sah_services_bridge_utils$/sah_services_bridge-utils/;
            $cplst->{$cp}->{"server"}   = $server;
            $cplst->{$cp}->{"location"} = $location;
        }
    }
}

sub getComponentServer {
    our $cplst;
    my $cp = shift;
    return $cplst->{$cp}->{"server"};
}

sub getComponentLocation {
    our $cplst;
    my $cp = shift;
    return $cplst->{$cp}->{"location"};
}

sub existsComponent {
    our $cplst;
    my $cp = shift;
    my $tst = exists($cplst->{$cp}->{"location"}) ? 1 : 0;
    return $tst;
}

sub check_cp_deps {
    my $start_path  = shift;
    my @dep_list    = `find $start_path -name Component.dep`;
    
    foreach my $cp (@dep_list) {
        changeComponentDepVersion($cp);    
    }
}

sub changeComponentDepVersion {
    my $file_name   = shift;
    my @file        = readFile($file_name); 
    my $update_repo = 0;

    my @newfile = (); 
    foreach my $line (@file) {
        if ($line =~ /^(.*)(<|>|=)(.*)$/) {
            my $cp          = $1;
            my $operator    = $2;
            my $version     = $3;
            my $repo_loc    = getComponentLocation($cp);

            if (($repo_loc ne "") && ($version ne "?") && ($version ne "*")) {
                my @tags = repo_tags_by_location($repo_loc);
                foreach my $tag (@tags) {
                    $tag = "refs/tags/" . $tag;
                    if ($tag ne $version) {
                        print "File $file_name CP $cp VERSION $version TAG $tag\n";
                        $version = $tag;
                        $update_repo = 1;
                    }
                }
            }
            $line = $cp . $operator . $version . "\n";
        }
        push(@newfile, $line);
    } 

    if ($update_repo) {
        `chmod 644 $file_name`;
        open F, "+>$file_name" or die "writing to $file_name. Error: " . $!; 
        print F @newfile;
        close F;

        my $repo_loc = stripName($file_name);
        print "DEBUG REPO LOC $repo_loc\n";
        my @tags = repo_tags_by_location($repo_loc);
        my $tag  = max(@tags);
        reset_git_tag_by_location($repo_loc, $tag);
    }
}
