#!/usr/bin/perl
##
# p4 to Rel
# Quick integration script for integrating a REL to V-branch
#
# Install
# In p4v select Tools -> Manage Custom Tools
# Select New -> Tool
#   Under 
#       Name:                               p4ToVbranc
#       Add to applicable context menus:    YES
#       Application:                        /home/sah5056/ws_w1/build/MAIN/tools/p4ToVbranc.pl
#       Arguments:                          --base %D 
#       Start In:                           /home/sah5056/ws_w1/build/MAIN
#       Run tool in terminal:               this is up to you!
#       Refresh P4V upon completion:        YES
#       Press OK 
#       And Press OK again!
#
#       Right click on a BRANCH p4ToVbranch should now be visable!
#
#       Email: <david.cluytens_ext@softathome.com>
##
use strict;
use v5.10.1;
use Time::Piece;
use Getopt::Long;

my $p4From;
my $p4Dir;
my $p4To;
my $rDate;
my $rMajor;
my $rMinor;
my $rMicro;
my $chlist          = 0;
my $help            = 0;
my $update_mode     = 0;
my $cDate           = Time::Piece->new->strftime('%Y-%m-%d');

GetOptions('help'           => sub { $help = 1 },
        'base=s'            => \$p4From,
        'update-latest'     => sub { $update_mode = 1 },
        'update-nano'       => sub { $update_mode = 2 },
        ) or printSyntax();

if ($p4From =~ /^(.*)\/REL\/(.*)_V([\d]+)\.([\d]+)\.([\d]+)\/\.\.\.$/) {
    $p4Dir  = $1;
    $rDate  = $2;
    $rMajor = $3;
    $rMinor = $4;
    $rMicro = $5;
} else {
    printSyntax();
}

$chlist     = p4CreateChangeList("\tBranched from $p4From");
$p4To = $p4Dir . "/V" . $rMajor . "." . $rMinor . "." . $rMicro . "/MAIN/...";

if ($chlist) {
    print "p4 integrate -c $chlist -d -o $p4From $p4To\n";
    system("p4 integrate -c $chlist -d -o $p4From $p4To");
} else {
    print "p4 integrate -d -o $p4From $p4To\n";
    system("p4 integrate -d -o $p4From $p4To");
}
exit $?;

sub printSyntax {
    print "p4ToVbranch --base <//depot/URL/...> \n\n";
    exit 1;
}

sub p4CreateChangeList {
    my $description = shift;
    my @spec        = (); 
    my $ch_spec     = ""; 

    $description = escapeString($description);
    @spec = `p4 change -o | sed '/^Files:/q'`;
    foreach my $line (@spec) {
        $line =~ s#^\s+<enter description here>$#$description#g;
        $ch_spec = $ch_spec . $line;
    }   
    my  $chlist = `echo "$ch_spec" | p4 change -i | cut -d ' ' -f 2`;
    chomp $chlist;
    return $chlist; 
}

sub escapeString {
    my $string = shift;
    $string  =~ s/\(/\\(/g;
    $string  =~ s/\)/\\)/g;
    $string  =~ s/\'/\\'/g;
    $string  =~ s/\`/\\`/g;
    $string  =~ s/\"/\\"/g;
    $string  =~ s/\./\\./g;
    return $string;
}

