#!/usr/bin/perl
use strict;
use Getopt::Long;

our @config      = ();
my $proj_file;
my $section;     
my $add_cp;    
my $remove_cp;
my $print;
my $list;

GetOptions( 'section=s'  => \$section,
            'add=s'      => \$add_cp,
            'remove=s'   => \$remove_cp,
            'file=s'     => \$proj_file,
            'print'      => sub { $print = 1 },
            'list'       => sub { $list = 1 } 
) or printSyntax();

if (!defined($proj_file)) {
    printSyntax();
}

parseConfig($proj_file);
if (!defined($print) && !defined($list)) {
    if (defined($add_cp) && ($add_cp ne "") && !defined($remove_cp)) {
        addComponent($section, $add_cp);
    } elsif (defined($remove_cp) && ($remove_cp ne "") && !defined($add_cp)) {
        removeComponent($remove_cp);
    } else {
        printSyntax();
    }
}

if ($list) {
    printList();
} else {
    printConfig();
}
exit 0;

sub printSyntax {
print "pconfig.pl --file <project.list>\
\t--print     print project.list (sorted)\
\t--list      list available sections\
\t--section   section name\
\t--add       add component to project.list\
\t--remove    remove component from project.list\
\t--file      project.list\n\
Example \
\tAdd component to project.list\
\tpconfig --section \"optional services\" --add sah_services_dropbear=REL/2013-04-08_V1.2.0 --file project.list\n\
\tRemove compontent from project.list\
\tpconfig --section \"optional services\" --remove sah_services_dropbear --file project.list\n\
";
exit 1;
}

sub readFile {
    my $name = shift;
    open FILE, $name or die $!;
    my @lines = <FILE>;
    close FILE;
    return @lines;
}

sub parseConfig {
    our $config;
    my $fh;
    my $file = shift;
    my $cp;
    my $version;
    my $section = "unknown";
    my $order   = -1;
    my @lines   = readFile($file);
    
    foreach my $line (@lines) {
        if ($line =~ /^#(.*)$/) {
            $section = $1;
            $order++; 
            $config[$order]->{'section'} = $section;
            $config[$order]->{'cp'}      = {};
        } elsif ($line =~ /^(.*)=(.*)$/) {
            $cp         = $1;
            $version    = $2;
            if ($order < 0) {
                $order = 0;
                $config[$order]->{'section'} = " $cp";
            }
            $config[$order]->{'cp'}->{$cp} = $version; 
        } 
    }
}

sub printList {
    our $config;
    foreach my $item (@config) {
        print $item->{'section'} . "\n";
    }
}

sub printConfig {
    our $config;
    my $order = 0;
    foreach my $item (@config) {
        if (exists($item->{'cp'})) {
            #re-enable if you don't want to see empty sections!
            #if (keys($item->{'cp'})) { 
                my $section = $item->{'section'};
                if($section =~ /^[#]+(.*)$/) {
                    $section = $1;
                } elsif ($section =~ /^\s+(.*)$/) {
                    if ($order) {
                        print "\n";
                    }
                }
                print "#" . $section . "\n";
                $order++;
            #}
            foreach my $cp (sort keys $item->{'cp'}) {
                print $cp . "=" . $item->{'cp'}->{$cp} . "\n";
            }
        }
    }
}

sub addComponent {
    our $config;
    my  $section = shift;
    my  $toadd   = shift;
    my  $cp;
    my  $rel;
    my  $found   = 0;

    if ($toadd =~ /^(.*)=(.*)$/) {
        $cp     = $1;
        $rel    = $2;
    } else {
        return;
    }

    foreach my $item (@config) {
        if ($item->{'section'} eq $section) {
            $found = 1;    
            $item->{'cp'}->{$cp}=$rel;
        }
    }
    if (!$found) {
        my $new             = {};
        $new->{'section'}   = $section;
        $new->{'cp'}->{$cp} = $rel; 
        push(@config, $new);
    }
}

sub removeComponent {
    our $config;
    my  $rcp     = shift;
    foreach my $item (@config) {
        foreach my $cp (sort keys $item->{'cp'}) {
            if ($cp eq $rcp) {
                delete($item->{'cp'}->{$cp});
            }
        }
    }
}

