#!/usr/bin/perl

use strict;
use Getopt::Long;

require 'tools/lib/core.pl';
require 'tools/lib/file.pl';
require 'tools/lib/string.pl';
require 'tools/lib/compare.pl';
require 'tools/lib/repo.pl';
require 'tools/lib/git.pl';
require 'tools/lib/helpers.pl';


our @files          = ();
our $components     = {};
our $pkg_dir;
our $git_repo_path;

my $config_name;
my $help		    = 0;
my $proj_file;
my $version_file;
my $cp_file;

GetOptions('help'        => sub { $help = 1 },
		'version=s'     => \$version_file,
		'proj=s'        => \$proj_file,
        'lst=s'         => \$cp_file,
        'pkg_dir=s'     => \$pkg_dir,
        'git_repo_path' => \$git_repo_path,
		) or printSyntax();


if ($help) {
	printSyntax();
} elsif (defined($proj_file) && defined($version_file) && defined($cp_file) && defined($pkg_dir) && defined($git_repo_path)) {
    `find $pkg_dir -name .gitignore -exec rm -f {} \\;`;
    open_proj($proj_file);
    parseComponentLst($cp_file);
    parseConfig($version_file);
    write_proj($git_repo_path . "/project.list");

    updateGitRepos();
    tagGitRepos();
    write_cp_lst($git_repo_path . "/component.lst");
    exit 0;
} 
printSyntax();
exit 1;

sub printSyntax {
    print "pkgToGit.pl   --version  <version.txt> --proj <project.list> --lst <component.lst> --pkg_dir <location of the package dir> --git_repo_path <location of the git repositories>\n\n";
}

sub parseConfig {
    my $fh;
    my $file = shift;
    my @lines = readFile($file);
    foreach my $line (@lines) {
        parseLine($line);
    }
}

sub parseLine {
    our $new_cplst;
    our $git_repo_path;
    our $components;
    my $line = shift;

    if ($line =~ /^(.*)=(.*)$/) {
        my $cp              = $1;
        my $version         = $2;
        my $server          = getComponentServer($cp);
        my $location        = getComponentLocation($cp);
        my $src_repo_path   = $pkg_dir . $location;
        print "$version\n";
        if ($version =~ /.*refs\/tags.*/) {
            $location = $location;
        } else {
            $location = $location . "/" . $version;
        }

        if (existsComponent($cp)) {
            (my $repo_name, my $repo_tag) = makeRepoName($cp, $version);
            if ($repo_name eq "unknown") {
                print "Error Unknown repo found\n";
                exit 1;
            }
            my $src_path    = $pkg_dir . $location;
            my $repo_path   = $git_repo_path . "/" . fixRepoName($repo_name);
            print "$cp = $src_path -> $repo_name=$repo_tag -> $repo_path\n";

            update_proj($repo_name, $repo_tag);

            $new_cplst->{$repo_name}                    = "GIT_REPO_PATH" . "/" . fixRepoName($repo_name);
            $components->{$repo_name}->{'tag'}          = $repo_tag;
            $components->{$repo_name}->{'repo_path'}    = $repo_path;
            $components->{$repo_name}->{'src_path'}     = $src_path;

        } else {
            print "Warning $cp doesn't exist.\n";
        }
    } 
}

sub updateGitRepos {
    our $components;

    foreach my $repo_name (sort keys $components) {
        my $repo_tag    = $components->{$repo_name}->{'tag'};
        my $repo_path   = $components->{$repo_name}->{'repo_path'};
        my $src_path    = $components->{$repo_name}->{'src_path'};
        my $cp_file     = $repo_path . "/Component.dep";
        my $re          = gitExistsRepo($repo_path);

        if (!$re) {
            print "Creating repo $repo_name at $repo_path\n";
            copyDir($src_path, $repo_path);
            gitCreateRepo($repo_path, $repo_name, $repo_tag);
        } else {
            my $te = gitTagExistsRepo($repo_path, $repo_tag);
            if (!$te) {
                print "Updating repo $repo_name at $repo_path\n";
                copyDir($src_path, $repo_path);
                gitUpdateRepo($repo_path, $repo_name, $repo_tag);
            }
        }
    }
}

sub tagGitRepos {
    our $components;

    foreach my $repo_name (sort keys $components) {
        my $repo_tag    = $components->{$repo_name}->{'tag'};
        my $repo_path   = $components->{$repo_name}->{'repo_path'};
        my $src_path    = $components->{$repo_name}->{'src_path'};
        my $cp_file     = $repo_path . "/Component.dep";

        my $te = gitTagExistsRepo($repo_path, $repo_tag);
        if (!$te) {
            if (-f $cp_file) {
                changeComponentDep($cp_file);
                gitUpdateRepo($repo_path, $repo_name, $repo_tag);
            }
            print "Tagging repo $repo_path name $repo_name tag $repo_tag\n";
            gitTagRepo($repo_path, $repo_name, $repo_tag);
        }
    }
}
