#!/usr/bin/perl
###############################################################################
#
# Tool for verifying the project.list on syntax errors
# Email <david.cluytens_ext@softathome.com>
#
###############################################################################
use strict;
use Getopt::Long;

my $help;
my $error = 0;
my $changelist = 0;

GetOptions('help'           => sub { $help = 1 },
           'changelist=s'   => \$changelist,
           ) or printSyntax();

if ($help) {
    printSyntax();
} elsif ($changelist > 0) {
    my @files = parseChangeList($changelist);
    foreach my $file (@files) {
        $error += verifyProj($file);    
    }
} else {
    printSyntax();
}
exit $error;

sub printSyntax {
    print "cat project.list | verify_proj.pl\n";
    exit 1;
}

sub parseChangeList {
    my $changelist = shift;
    my @lines = `p4 describe $changelist`;
    my $parse = 0;
    my @files = ();
    foreach my $line (@lines) {
        if ($line =~ /^Affected files \.\.\.$/) {
            $parse = 1;
        }
        if ($line =~ /^Differences \.\.\.$/) {
            $parse = 0;
        }
        if ($parse) {
            if ($line =~ /^\.\.\. (.*project\.list#\d+)\s+\w+$/) {
                my $file = $1;
                print "$file\n";
                push(@files, $file);
            }
        }
    }
    return @files;
}

sub verifyProj {
    my $fh;
    my $file  = shift;
    my @lines = `p4 print $file`;
    my $error = 0;
    foreach my $line (@lines) {
        if ($line =~ /^:?.*=(.*\/)?DEV\/.*$/) {
            print "Error $line not accepted!\n";
            $error = 1;
        } elsif ($line =~ /^:?.*=(.*\/)?MAIN$/) {
            print "Error $line not accepted!\n";
            $error = 1;
        } 
    }
    return $error;
}
