include $(STAGINGDIR)/components.config

LIBRARIES=date_time,filesystem,serialization,system,thread

LIBBOOSTVER=1.55.0

PKCONFIG_FILES=

PKCONFIG_FILES += boost_date_time.pc
PKCONFIG_FILES += boost_filesystem.pc
PKCONFIG_FILES += boost_serialization.pc
PKCONFIG_FILES += boost_system.pc
PKCONFIG_FILES += boost_thread.pc

all: b2
	./b2 install --prefix=$(STAGINGDIR) || echo "WARNING: ./b2 returned non-zero value"

b2: tools/build/v2/user-config.jam
	./bootstrap.sh \
	  --prefix=$(STAGINGDIR) \
	  --with-toolset=gcc \
	  --with-libraries=$(LIBRARIES)

tools/build/v2/user-config.jam: tools/build/v2/user-config.jam.m4
	( [ $(DUMP_CMD) == arm ] || [ $(DUMP_CMD) == mips ] ) || ( echo "Unknown target (DUMP_CMD=$(DUMP_CMD))" && [ true == false ] )
	m4 -DDUMP_CMD=$(DUMP_CMD) -DCPP=$(CONFIG_CROSS_COMPILE)g++ tools/build/v2/user-config.jam.m4 > $@

# FIXME: Should parse "bin.v2/project-cache.jam" to retreive the value of "lockfree boost::atomic_flag".

boost_date_time.pc : boost_date_time.pc.m4
	m4 -I$(STAGINGDIR) -DDUMP_CMD=$(DUMP_CMD) $< > $@

boost_filesystem.pc : boost_filesystem.pc.m4
	m4 -I$(STAGINGDIR) -DDUMP_CMD=$(DUMP_CMD) $< > $@

boost_serialization.pc : boost_serialization.pc.m4
	m4 -I$(STAGINGDIR) -DDUMP_CMD=$(DUMP_CMD) $< > $@

boost_system.pc : boost_system.pc.m4
	m4 -I$(STAGINGDIR) -DDUMP_CMD=$(DUMP_CMD) $< > $@

boost_thread.pc : boost_thread.pc.m4
	m4 -I$(STAGINGDIR) -DDUMP_CMD=$(DUMP_CMD) $< > $@

clean:
	rm -f tools/build/v2/user-config.jam
	rm -Rf bin.v2 stage tools/build/v2/engine/bin.linuxx86 tools/build/v2/engine/bootstrap b2 bjam bootstrap.log cscope.out project-config.jam*
	rm -f $(PKCONFIG_FILES)

install: $(PKCONFIG_FILES)
	$(INSTALL) -d $(PKG_CONFIG_LIBDIR)
	$(INSTALL) -D -m 0644 $(PKCONFIG_FILES) $(PKG_CONFIG_LIBDIR)/

release:
	$(INSTALL) -d $(ROOT_FS_DIR)/lib/
	if [ -f $(STAGINGDIR)/lib/libboost_atomic.so.$(LIBBOOSTVER) ]; \
	then \
	  $(INSTALL) -m 644 $(STAGINGDIR)/lib/libboost_atomic.so.$(LIBBOOSTVER) $(ROOT_FS_DIR)/lib/ ; \
	fi
	$(INSTALL) -m 644 $(STAGINGDIR)/lib/libboost_date_time.so.$(LIBBOOSTVER) $(ROOT_FS_DIR)/lib/
	$(INSTALL) -m 644 $(STAGINGDIR)/lib/libboost_filesystem.so.$(LIBBOOSTVER) $(ROOT_FS_DIR)/lib/
	$(INSTALL) -m 644 $(STAGINGDIR)/lib/libboost_serialization.so.$(LIBBOOSTVER) $(ROOT_FS_DIR)/lib/
	$(INSTALL) -m 644 $(STAGINGDIR)/lib/libboost_system.so.$(LIBBOOSTVER) $(ROOT_FS_DIR)/lib/
	$(INSTALL) -m 644 $(STAGINGDIR)/lib/libboost_thread.so.$(LIBBOOSTVER) $(ROOT_FS_DIR)/lib/
	$(INSTALL) -m 644 $(STAGINGDIR)/lib/libboost_wserialization.so.$(LIBBOOSTVER) $(ROOT_FS_DIR)/lib/
	if [ -f $(ROOT_FS_DIR)/lib/libboost_atomic.so.$(LIBBOOSTVER) ]; \
	then \
	  $(STRIP) $(ROOT_FS_DIR)/lib/libboost_atomic.so.$(LIBBOOSTVER) ; \
	fi
	$(STRIP) $(ROOT_FS_DIR)/lib/libboost_date_time.so.$(LIBBOOSTVER)
	$(STRIP) $(ROOT_FS_DIR)/lib/libboost_filesystem.so.$(LIBBOOSTVER)
	$(STRIP) $(ROOT_FS_DIR)/lib/libboost_serialization.so.$(LIBBOOSTVER)
	$(STRIP) $(ROOT_FS_DIR)/lib/libboost_system.so.$(LIBBOOSTVER)
	$(STRIP) $(ROOT_FS_DIR)/lib/libboost_thread.so.$(LIBBOOSTVER)
	$(STRIP) $(ROOT_FS_DIR)/lib/libboost_wserialization.so.$(LIBBOOSTVER)

.PHONY: clean install release

