# include components.config
ifdef STAGINGDIR
include $(STAGINGDIR)/sahcommon.mk
endif

INSTALL ?= install
TARGET = libfw_client.a
OBJECTS = fw_client.o

CFLAGS += -fPIC -g -Os -I. -Wall -Wextra -I$(STAGINGDIR)/include $(shell pkg-config --cflags sahtrace pcb)

ifeq ($(CONFIG_SAH_SERVICES_FW),y)
CFLAGS += -I$(STAGINGDIR)/include/fw
endif

all: $(TARGET)
	@echo -en "\033[1m  ## Make firewall client ##\n\033[0m"

$(TARGET): $(OBJECTS)
	@echo -en "\033[1m  ## Link $(TARGET) ##\n\033[0m"
	$(AR) rcs $(@) $(OBJECTS)

-include $(OBJECTS:.o=.d)

%.o: %.c
	@echo -en "\033[1m  ## Compile $@ ##\n\033[0m"
	$(CC) $(CFLAGS) -c -o $@ $<
	@$(CC) $(CFLAGS) -MM -MP -MT '$(@) $(@:.o=.d)' -MF $(@:.o=.d) $(<)

clean:
	@echo -en "\033[1m  ## Clean firewall files ##\n\033[0m"
	rm -f $(TARGET) $(OBJECTS)
	rm -f *.d

install:
	@echo -en "\033[1m  ## Install firewall files ##\n\033[0m"
	$(INSTALL) -d $(STAGINGDIR)/lib
	$(INSTALL) -d $(STAGINGDIR)/include/fw
	$(INSTALL) -d $(PKG_CONFIG_LIBDIR)
	$(INSTALL) -m 0644 libfw_client.a $(STAGINGDIR)/lib
	$(INSTALL) -p -m 0644 fw_client.h $(STAGINGDIR)/include/fw
	$(INSTALL) -m 0644 fw_client.pc $(PKG_CONFIG_LIBDIR)

release:
	@echo -en "\033[1m  ## Release firewall files ##\n\033[0m"

.PHONY: all clean install release 
