#ifndef __fw_client_h__
#define __fw_client_h__

#ifdef __cplusplus
    extern "C"
{
#endif

#include <stdbool.h>
#include <stdint.h>
#include "components.h"

#ifdef CONFIG_SAH_SERVICES_FW
#include "fw_common.h"
#else
typedef struct {
	bool enableIPv4;
	bool enableIPv6;
} service_enable_t;

typedef struct {
	char *name;
	char *port;
} service_result_t;
#endif

variant_t *service_enable_t2variant(const service_enable_t *data);

bool save(peer_info_t *dest, const char *objectPath);
bool reset(peer_info_t *dest, const char *objectPath);
bool commit(peer_info_t *dest, const char *objectPath);
service_result_t* setService(peer_info_t *dest, const char *objectPath, const char *id, const char *sourceInterface, const char *destinationPort, const char *protocol, int32_t ipversion, bool enable);
bool deleteService(peer_info_t *dest, const char *objectPath, const char *id);
variant_list_t *getService(peer_info_t *dest, const char *objectPath, const char *id);
bool setRespondToPing(peer_info_t *dest, const char *objectPath, const char *sourceInterface, const service_enable_t *service_enable);
service_enable_t * getRespondToPing(peer_info_t *dest, const char *objectPath, const char *sourceInterface);
char *setPortForwarding(peer_info_t *dest, const char *objectPath, const char *id, const char *origin, const char *sourceInterface, const char *externalPort, const char *internalPort, const char *destinationIPAddress, const char *sourcePrefix, const char *protocol, bool enable, bool persistent, const char *description);
bool deletePortForwarding(peer_info_t *dest, const char *objectPath, const char *id, const char *origin, const char* destinationIPAddress);
variant_list_t * getPortForwarding(peer_info_t *dest, const char *objectPath, const char *id, const char *origin);
char *setPinhole(peer_info_t *dest, const char *objectPath, const char *id, const char *origin, const char *sourceInterface, const char *sourcePort, const char *destinationPort, const char *destinationIPAddress, const char *sourcePrefix, const char *protocol, int32_t ipversion, bool enable, bool persistent, const char *description);
bool deletePinhole(peer_info_t *dest, const char *objectPath, const char *id, const char *origin);
variant_list_t * getPinhole(peer_info_t *dest, const char *objectPath, const char *id, const char *origin);
char *setDMZ(peer_info_t *dest, const char *objectPath, const char *id, const char *sourceInterface, const char *destinationIPAddress, const char *sourcePrefix, bool enable);
bool deleteDMZ(peer_info_t *dest, const char *objectPath, const char *id);
variant_list_t * getDMZ(peer_info_t *dest, const char *objectPath, const char *id);
char *setCustomRule(peer_info_t *dest, const char *objectPath, const char *id, const char *chain, const char *action, const char *destinationPort, const char *sourcePort, const char *destinationPrefix, const char *sourcePrefix, const char *protocol, int32_t ipversion, bool enable, const char* description);
bool deleteCustomRule(peer_info_t *dest, const char *objectPath, const char *id, const char *chain);
variant_list_t * getCustomRule(peer_info_t *dest, const char *objectPath, const char *id, const char *chain);
bool setFirewallLevel(peer_info_t *dest, const char *objectPath, const char *level);

bool setPolicyLevel(peer_info_t *dest, const char *objectPath, const char *id, const char *sourceInterface, const char *destinationInterface, const char *level, int32_t ipversion, bool enable);

bool deletePolicy(peer_info_t *dest, const char *objectPath, const char *id);

char *getPolicyLevel(peer_info_t *dest, const char *objectPath, const char *id);
#ifdef __cplusplus
}
#endif

#endif
