/*
 * Copyright (c) 2012 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <debug/sahtrace.h>

#include <pcb/utils.h>
#include <pcb/core.h>

#include "nemo/client.h"
#include "client/util.h"

variant_t *nemo_getRAOptions(const char *intf, uint8_t tag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "getRAOptions", request_common_path_key_notation | request_function_args_by_name);
	request_addUInt16Parameter(req, "tag", tag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_t *retval = calloc(1, sizeof(variant_t));
	variant_initialize(retval, variant_type_unknown);
	variant_copy(retval, nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}


variant_t *nemo_getRARouters(const char *intf, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "getRARouters", request_common_path_key_notation | request_function_args_by_name);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_t *retval = calloc(1, sizeof(variant_t));
	variant_initialize(retval, variant_type_unknown);
	variant_copy(retval, nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

nemo_query_t *nemo_openQuery_getRAOptions(const char *intf, const char *subscriber, uint8_t tag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	nemo_argument_valueAddUInt16(&args, "tag", tag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "getRAOptions", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}


nemo_query_t *nemo_openQuery_getRARouters(const char *intf, const char *subscriber, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "getRARouters", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}

