include $(STAGINGDIR)/components.config

CFLAGS += -Wall -fPIC -Os -g -Iinclude -I$(STAGINGDIR)/include
ifeq ($(notdir $(CC)),gcc)
CFLAGS += -I/usr/src/linux-headers-$(shell uname -r)/include -D_GNU_SOURCE
else
CFLAGS += -I$(HARDCO_KERNEL_DIR)/include
endif


LDFLAGS += -L$(STAGINGDIR)/lib -lm -L.

OBJECTS  = lib/utils.o lib/data.o lib/cache.o lib/msg.o lib/genl/mngt.o lib/genl/family.o lib/genl/genl.o
OBJECTS += lib/genl/ctrl.o lib/addr.o lib/cache_mngt.o lib/cache_mngr.o lib/socket.o lib/attr.o
OBJECTS += lib/object.o lib/error.o lib/handlers.o lib/doc.o lib/nl.o
OBJECTS += lib/fib_lookup/request.o lib/fib_lookup/lookup.o

OBJECTS += lib/route/cls_api.o lib/route/sch/htb.o lib/route/sch/sfq.o lib/route/sch/prio.o
OBJECTS += lib/route/sch/netem.o lib/route/sch/dsmark.o lib/route/sch/blackhole.o lib/route/sch/tbf.o
OBJECTS += lib/route/sch/fifo.o lib/route/sch/cbq.o lib/route/sch/red.o
OBJECTS += lib/route/route.o lib/route/route_utils.o
OBJECTS += lib/route/tc.o lib/route/cls.o lib/route/class_api.o lib/route/rule.o lib/route/addr.o
OBJECTS += lib/route/rtnl.o
OBJECTS += lib/route/class.o lib/route/route_obj.o lib/route/link.o lib/route/cls_obj.o lib/route/neightbl.o
OBJECTS += lib/route/qdisc.o lib/route/nexthop.o lib/route/qdisc_api.o lib/route/qdisc_obj.o
OBJECTS += lib/route/link/api.o
OBJECTS += lib/route/link/vlan.o lib/route/class_obj.o lib/route/neigh.o

OBJECTS += lib/netfilter/queue_msg.o lib/netfilter/log_msg.o lib/netfilter/log_obj.o lib/netfilter/ct_obj.o
OBJECTS += lib/netfilter/ct.o lib/netfilter/queue_obj.o lib/netfilter/log.o lib/netfilter/log_msg_obj.o
OBJECTS += lib/netfilter/netfilter.o lib/netfilter/nfnl.o lib/netfilter/queue_msg_obj.o lib/netfilter/queue.o

OBJECTS += lib/route/cls/ematch.o lib/route/cls/u32.o
OBJECTS += lib/route/cls/police.o lib/route/cls/fw.o lib/route/cls/cgroup.o lib/route/cls/basic.o
OBJECTS += lib/route/cls/ematch/container.o lib/route/cls/ematch/cmp.o

LIBS-all = libnl.so libnl.a

ifeq ($(CONFIG_SAH_LIB_NETLINK_CMDTOOLS), y)
CLI_OBJECTS  = src/lib/addr.o src/lib/ct.o src/lib/link.o src/lib/neigh.o src/lib/qdisc.o
CLI_OBJECTS += src/lib/route.o src/lib/rule.o src/lib/utils.o

OBJS-nl-list-sockets = src/nl-list-sockets.o
OBJS-nf-monitor = src/nf-monitor.o
OBJS-nf-queue = src/nf-queue.o
OBJS-nf-ct-list = src/nf-ct-list.o
OBJS-all = $(OBJS-nl-list-sockets) $(OBJS-nf-monitor) $(OBJS-nf-queue) $(OBJS-nf-ct-list)
BINS-all = nf-queue nf-monitor nl-list-sockets nf-ct-list
LIBS-all += libnlcli.so libnlcli.a
else
BINS-all =
OBJS-all =
endif

libnl.so: $(OBJECTS)
	$(CC) -Wl,-soname,libnl.so -shared -o $(@) $(OBJECTS) $(LDFLAGS)

libnl.a: $(OBJECTS)
	$(AR) rcs $(@) $(OBJECTS)

ifeq ($(CONFIG_SAH_LIB_NETLINK_CMDTOOLS), y)
libnlcli.so: $(CLI_OBJECTS)
	$(CC) -Wl,-soname,libnlcli.so -shared -o $(@) $(CLI_OBJECTS) $(LDFLAGS)

libnlcli.a: $(CLI_OBJECTS)
	$(AR) rcs $(@) $(CLI_OBJECTS)

$(OBJS-all): %.o: %.c Component.mk $(STAGINGDIR)/components.config
	$(CC) $(CFLAGS) -c -o $(@) $(<)

$(BINS-all): $(OBJS-all)
	$(CC) $(CFLAGS) -o $@ $(OBJS-$@) $(LDFLAGS) -lnl -lnlcli
endif

%.o: %.c
	$(CC) $(CFLAGS) -c -o $(@) $(<)

all: $(LIBS-all) $(BINS-all)

clean:
	rm -f $(LIBS-all) $(BINS-all)
	rm -f $(OBJECTS) $(OBJS-all)

install: all
	@echo "Installing to: " $(STAGINGDIR)
	$(INSTALL) -d $(STAGINGDIR)/include
	$(INSTALL) -d $(STAGINGDIR)/include/netlink
	cp -a -f include/* $(STAGINGDIR)/include/netlink
	$(INSTALL) -d $(STAGINGDIR)/lib
	$(INSTALL) -m 0644 libnl.so $(STAGINGDIR)/lib/libnl.so
	$(INSTALL) -m 0644 libnl.a $(STAGINGDIR)/lib/
	#ln -sf libnl.a $(STAGINGDIR)/lib/libnl_pic.a # branched for bug 15138
ifeq ($(CONFIG_SAH_LIB_NETLINK_CMDTOOLS), y)
	$(INSTALL) -m 0644 libnlcli.so $(STAGINGDIR)/lib/libnlcli.so
	$(INSTALL) -d $(STAGINGDIR)/usr/bin
	$(INSTALL) -m 0755 nl-list-sockets $(STAGINGDIR)/usr/bin
	$(INSTALL) -m 0755 nf-monitor $(STAGINGDIR)/usr/bin
	$(INSTALL) -m 0755 nf-queue $(STAGINGDIR)/usr/bin
	$(INSTALL) -m 0755 nf-ct-list $(STAGINGDIR)/usr/bin
endif

release:
	@echo "Releasing to: " $(ROOT_FS_DIR)
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	$(INSTALL) -m 0644 $(STAGINGDIR)/lib/libnl.so $(ROOT_FS_DIR)/lib
ifeq ($(CONFIG_SAH_LIB_NETLINK_CMDTOOLS), y)
	$(INSTALL) -d $(ROOT_FS_DIR)/usr/bin
	$(INSTALL) -m 0755 $(STAGINGDIR)/usr/bin/nl-list-sockets $(ROOT_FS_DIR)/usr/bin
	$(INSTALL) -m 0755 $(STAGINGDIR)/usr/bin/nf-monitor $(ROOT_FS_DIR)/usr/bin
	$(INSTALL) -m 0755 $(STAGINGDIR)/usr/bin/nf-queue $(ROOT_FS_DIR)/usr/bin
	$(INSTALL) -m 0755 $(STAGINGDIR)/usr/bin/nf-ct-list $(ROOT_FS_DIR)/usr/bin
	$(INSTALL) -m 0644 $(STAGINGDIR)/lib/libnlcli.so $(ROOT_FS_DIR)/lib
endif

.PHONY: all clean install release

