################################################################
# project.lst build flags
#
# SAH_DEBUG..........: No optimalization, trace level set to 500
# SAH_DEBUG_NO_STRIP.: No stripping during release step
#
################################################################

ifndef STAGINGDIR
  $(error STAGINGDIR not defined, stopping build)
endif

include $(STAGINGDIR)/components.config

INSTALL ?= install
STRIP ?= strip
PKG_CONFIG_LIBDIR ?= $(STAGINGDIR)/build/pkg-config

# define the name of your component here
export COMPONENT = pcb-libs

all:
	$(MAKE) -C src pcb_utils
	$(MAKE) -C src pcb_sl
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(MAKE) -C src pcb_ssl
endif
ifneq ($(CONFIG_PCB_UTILS_ONLY),y)
	$(MAKE) -C src pcb_dm
endif

clean:
	-$(MAKE) -C src clean
	-$(MAKE) -C test/pcb_utils clean
	-$(MAKE) -C test/pcb_dm clean
	rm -rf html
	rm -f Makefile
	rm -f pcb.pro.user
	rm -f pkg-config-utils.pc
	rm -f pkg-config-sockets.pc
	rm -f pkg-config-secure-sockets.pc
	rm -f pkg-config.pc
	rm -f pkg-config-ssl.pc

pkg-config-utils.pc: pkg-config-utils.pc.m4 $(STAGINGDIR)/components.config
	cat $(STAGINGDIR)/components.config.m4 $< | m4 > $@

pkg-config-sockets.pc: pkg-config-sockets.pc.m4 $(STAGINGDIR)/components.config
	cat $(STAGINGDIR)/components.config.m4 $< | m4 > $@

pkg-config-secure-sockets.pc: pkg-config-secure-sockets.pc.m4 $(STAGINGDIR)/components.config
	cat $(STAGINGDIR)/components.config.m4 $< | m4 > $@

pkg-config.pc: pkg-config.pc.m4 $(STAGINGDIR)/components.config
	cat $(STAGINGDIR)/components.config.m4 $< | m4 > $@

pkg-config-ssl.pc: pkg-config-ssl.pc.m4 $(STAGINGDIR)/components.config
	cat $(STAGINGDIR)/components.config.m4 $< | m4 > $@

install: pkg-config-utils.pc pkg-config-sockets.pc pkg-config-secure-sockets.pc pkg-config.pc pkg-config-ssl.pc
	$(INSTALL) -d $(STAGINGDIR)/include
ifneq ($(CONFIG_PCB_UTILS_ONLY),y)
	for FILE in $$(find include/pcb/core -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	$(INSTALL) -p -m 0644 -D include/pcb/core.h $(STAGINGDIR)/include/pcb/core.h
	$(INSTALL) -p -m 0644 -D include/pcb/pcb_client.h $(STAGINGDIR)/include/pcb/pcb_client.h
endif
	for FILE in $$(find include/pcb/common/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	for FILE in $$(find include/pcb/utils/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	for FILE in $$(find include/pcb/plugin/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	for FILE in $$(find include/pcb/client/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	$(INSTALL) -p -m 0644 -D include/pcb/common.h $(STAGINGDIR)/include/pcb/common.h
	$(INSTALL) -p -m 0644 -D include/pcb/utils.h $(STAGINGDIR)/include/pcb/utils.h
	$(INSTALL) -d $(STAGINGDIR)/lib
	$(INSTALL) -d $(STAGINGDIR)/slib
	$(INSTALL) -m 0755 src/libpcb_utils.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_sl.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0644 src/libpcb_utils.a $(STAGINGDIR)/slib
	$(INSTALL) -m 0644 src/libpcb_sl.a $(STAGINGDIR)/slib
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(INSTALL) -m 0755 src/libpcb_ssl.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0644 src/libpcb_ssl.a $(STAGINGDIR)/slib
	ln -sf libpcb_ssl.a $(STAGINGDIR)/lib/libpcb_ssl_pic.a
endif
	$(INSTALL) -D -m 0644 pkg-config-utils.pc $(PKG_CONFIG_LIBDIR)/pcb_utils.pc
	$(INSTALL) -D -m 0644 pkg-config-sockets.pc $(PKG_CONFIG_LIBDIR)/pcb_sockets.pc
	$(INSTALL) -D -m 0644 pkg-config-secure-sockets.pc $(PKG_CONFIG_LIBDIR)/pcb_secure_sockets.pc
ifneq ($(CONFIG_PCB_UTILS_ONLY),y)
	$(INSTALL) -m 0755 src/libpcb_dm.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0644 src/libpcb_dm.a $(STAGINGDIR)/slib
	$(INSTALL) -D -m 0644 pkg-config.pc $(PKG_CONFIG_LIBDIR)/pcb.pc
	$(INSTALL) -D -m 0644 pkg-config-ssl.pc $(PKG_CONFIG_LIBDIR)/pcbssl.pc
endif

release:
ifndef ROOT_FS_DIR
	$(error ROOT_FS_DIR not defined, stopping release)
endif
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_utils.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_utils.so
endif
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_sl.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_sl.so
endif
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_ssl.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_ssl.so
endif
endif
ifneq ($(CONFIG_PCB_UTILS_ONLY),y)
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_dm.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_dm.so
endif
endif

doc:
	$(INSTALL) -d $(STAGINGDIR)/doc/api/pcblibs
	doxygen documentation/pcb.doxy

test:
	make -C test/pcb_utils
	make -C test/pcb_dm

runtest:
	make -C test/pcb_utils test
	make -C test/pcb_dm test

.PHONY: all clean install release doc test runtest
