/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <stdlib.h>
#include <pcb/core/acl.h>

#include "core_priv.h"

void acl_list_destroy(llist_t *list) {
    llist_iterator_t *it = llist_takeFirst(list);
    ACL_t *acl = NULL;
    while(it) {
        acl = llist_item_data(it,ACL_t,it);
        free(acl);
        it = llist_takeFirst(list);
    }
}

ACL_t *acl_first(const llist_t *list) {
    if (!list) {
        return NULL;
    }

    llist_iterator_t *it = llist_first(list);
    ACL_t *rv = NULL;
    if (it) {
        rv = llist_item_data(it,ACL_t,it);
    }

    return rv;
}

ACL_t *acl_next(ACL_t *acl) {
    if (!acl) {
        return NULL;
    }

    llist_iterator_t *it = llist_iterator_next(&acl->it);
    ACL_t *rv = NULL;
    if (it) {
        rv = llist_item_data(it,ACL_t,it);
    }

    return rv;
}

uint32_t acl_id(ACL_t *acl) {
    if (!acl) {
        return UINT32_MAX;
    }

    return acl->id;
}

uint16_t acl_flags(ACL_t *acl) {
    if (!acl) {
        return 0;
    }

    return acl->acl_flags;
}

bool acl_canRead(ACL_t *acl) {
    if (!acl) {
        return false;
    }

    return (acl->acl_flags & acl_read);
}

bool acl_canWrite(ACL_t *acl) {
    if (!acl) {
        return false;
    }

    return (acl->acl_flags & acl_write);
}

bool acl_canExecute(ACL_t *acl) {
    if (!acl) {
        return false;
    }

    return (acl->acl_flags & acl_execute);
}

bool acl_isGroup(ACL_t *acl) {
    if (!acl) {
        return false;
    }

    return (acl->acl_flags & acl_group_id);
}
