/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_CORE_PRIV_H)
#define PCB_CORE_PRIV_H

#ifdef __cplusplus
extern "C"
{
#endif

/**
@file
@brief
Header file with private common definitions and private common functions of the core module
*/

#include <pcb/utils/string.h>
#include <pcb/utils/linked_list.h>
#include <pcb/core/acl.h>

/**
  Access control information structure
*/
struct _ACL_info {
    uint16_t acl_flags;           /** read, write, execute, user/group */
    uint32_t id;                  /** the id (user or group) */
    llist_iterator_t it;          /** iterator used to store in the ACL list */
};

#if !defined(NO_DOXYGEN)
bool __attribute__ ((visibility("hidden"))) isValidName(string_t *name);
bool __attribute__ ((visibility("hidden"))) isValidNameChar(const char *name);
ACL_t __attribute__ ((visibility("hidden"))) *aclFind(llist_t *ACL, uint32_t id, uint16_t flags);
bool __attribute__ ((visibility("hidden"))) aclSet(llist_t *ACL, uint32_t id, uint16_t flags);
bool __attribute__ ((visibility("hidden"))) aclCheck(llist_t *ACL, uint32_t uid, uint16_t flags);
void __attribute__ ((visibility("hidden"))) aclDestroy(llist_t *ACL);
#else
bool isValidName(string_t *name);
bool isValidNameChar(string_t *name);
ACL_t *aclFind(llist_t *ACL, uint32_t id, uint16_t flags);
bool aclSet(llist_t *ACL, uint32_t id, uint16_t flags);
bool aclCheck(llist_t *ACL, uint32_t uid, uint32_t gid, uint16_t flags);
void aclDestroy(llist_t *ACL);
#endif

#ifdef __cplusplus
}
#endif

#endif
