/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_DATAMODEL_PRIV_H)
#define PCB_DATAMODEL_PRIV_H

#ifdef __cplusplus
extern "C"
{
#endif

/**
@file
@brief
Header file with private data model definitions and private data model functions
*/

#include "object_priv.h"
#include <pcb/core/datamodel.h>

/**
@brief
Data model attributes. These are for internal use only, and can not be modified using any of the public functions
*/
typedef enum _datamodel_attributes {
    datamodel_attr_default    = 0x00000000,      /**< Default data model (connection less, no caching, no dispatching) */
    datamodel_attr_connected  = 0x00000001,      /**< This datamodel has a connection */
    datamodel_attr_cache      = 0x00000002,      /**< This datamodel is used as cache */
    datamodel_attr_dispatch   = 0x00000004,      /**< This datamodel can only dispatch requests, does not contain any real objects */
    datamodel_attr_no_resolve = 0x00000008,      /**< Do not do automatic function resolving */
} datamodel_attribute_t;

typedef struct _notify_handlers {
    llist_iterator_t it;
    datamodel_notify_handler_t datamodel_changed;
} datamodel_notify_handlers_t;

/**
@brief
The data model structure definition
*/
struct _datamodel {
    object_t                    root;       /**< The root object of the datamodel*/
    datamodel_attribute_t       attributes; /**< The datamodel attributes, these are not accessible, for internal use only */
    llist_t                     notify;     /**< List of callback functions, they are called when something has changed to the datamodel */
    void *                      userData;   /**< User data */
    char *                      mibdir;     /**< Directory used to search mibs in */
    object_t                    mibRoot;    /**< Definition of mibs are added under this object */
};


#if !defined(NO_DOXYGEN)
bool __attribute__ ((visibility("hidden"))) datamodel_initialize(datamodel_t *dm);
void __attribute__ ((visibility("hidden"))) datamodel_cleanup(datamodel_t *dm);
#else
bool datamodel_initialize(datamodel_t *dm);
void datamodel_cleanup(datamodel_t *dm);
#endif

#ifdef __cplusplus
}
#endif

#endif

