/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_FUNCTION_PRIV_H)
#define PCB_FUNCTION_PRIV_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <pcb/core/function.h>
#include <pcb/utils/linked_list.h>
#include <pcb/utils/string.h>
#include <pcb/utils/variant.h>

struct _function_argument {
    llist_iterator_t it;            /**< The place in the function argumtents list */
    string_t name;                  /**< Name of the argument */
    string_t typeName;              /**< Name of the type, used for custom types */
    argument_type_t type;           /**< Type of the argument */
    uint32_t attributes;            /**< Argument attributes */
};

typedef struct _function_def {
    uint32_t attributes;               /**< Function attributes */
    function_type_t returnType;        /**< Function return type */
    llist_t arguments;                 /**< List of arguments */
#ifdef PCB_HELP_SUPPORT
    char *description;                 /**< Function help text */
#endif

    uint32_t refcount;                 /**< Number of times the definition is used */
} function_def_t;

struct _function {
    llist_iterator_t it;               /**< The place in the owners function list */
    function_def_t *definition;        /**< Pointer to the function definition */
    llist_t ACL;                       /**< Access Control List */
    object_destination_t *destination; /**< Function mapping information */
    char *plugin_name;                 /**< Plug-in function name */
    function_handler_t handler;        /**< The real function pointer */
    function_destroy_handler_t destroy;/**< The destroy handler */
    void *userdata;                    /**< Pointer to some user data */
};

struct _function_call {
    request_t *req;
    peer_info_t *source;
    function_t *function;
    object_t *object;
    function_cancel_handler_t cancelExecution;
    function_exec_state_t state;
    void *userdata;
};

function_t *function_allocate(const char *name, const char *typeName, function_type_t returnType, uint32_t attributes);

#ifdef __cplusplus
}
#endif

#endif
