/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include "datamodel_priv.h"

int dm_initialize() {
    datamodel_t dm;

    assert_bool(datamodel_initialize(&dm) == true);
    assert_bool(datamodel_root(&dm) == &dm.root);
    assert_bool(dm.root.attributes == (object_attr_default | object_attr_persistent));
    assert_bool(dm.attributes == datamodel_attr_default);
    assert_bool(llist_isEmpty(&dm.notify));
    assert_bool(dm.root.state == object_state_ready);

    datamodel_cleanup(&dm);
    sahtest_success();
}

int dm_initialize_null() {
    assert_bool(datamodel_initialize(NULL) == false);
    assert_bool(datamodel_root(NULL) == NULL);

    sahtest_success();
}

int dm_create() {
    datamodel_t *dm = datamodel_create();
    assert_not_null(dm);
    assert_bool(dm->root.attributes == (object_attr_default | object_attr_persistent));
    assert_bool(dm->attributes == datamodel_attr_default);
    assert_bool(llist_isEmpty(&dm->notify));
    assert_bool(dm->root.state == object_state_ready);

    datamodel_destroy(dm);
    sahtest_success();
}

int dm_cleanup_null() {
    datamodel_cleanup(NULL);
    datamodel_clear(NULL);

    sahtest_success();
}

int dm_cleanup() {
    datamodel_t dm;

    assert_bool(datamodel_initialize(&dm) == true);
    assert_bool(datamodel_root(&dm) == &dm.root);

    object_t *sub = object_create(&dm.root,"test",0);
    assert_not_null(sub);
    sub = object_create(sub,"test",0);
    assert_not_null(sub);

    datamodel_cleanup(&dm);

    sahtest_success();
}

int dm_destroy_null() {
    datamodel_destroy(NULL);

    sahtest_success();
}
