/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#ifndef DM_LOAD_SAVE_H
#define DM_LOAD_SAVE_H

void dm_load_save_setup();
void dm_load_save_teardown();

int dm_load_null();
int dm_load_unknown_format();
int dm_load_no_handler();
int dm_load();
int dm_load_prefix_path();
int dm_load_auto_resolve_yes();
int dm_load_auto_resolve_no();
int dm_load_auto_resolve_invalid_value();
int dm_load_no_auto_resolve_set_resolve_to_false();
int dm_load_open_failing();
int dm_load_failing();
int dm_load_definition_null();
int dm_load_definition();

int dm_verify_null();
int dm_verify_unknown_format();
int dm_verify_no_handler();
int dm_verify();
int dm_verify_prefix_path();
int dm_verify_open_failing();
int dm_verify_failing();

int dm_save_null();
int dm_save_unknown_format();
int dm_save_no_handler();
int dm_save();
int dm_save_open_failing();
int dm_save_failing();

sahtest_group_begin(dm_loading,dm_load_save_setup,dm_load_save_teardown)
    sahtest_add_test(dm_load_null,"Load datamodel using null pointer datamodel")
    sahtest_add_test(dm_load_unknown_format,"Load datamodel using an unknown format")
    sahtest_add_test(dm_load_no_handler,"Load datamodel using a format without load handler")
    sahtest_add_test(dm_load,"Load datamodel")
    sahtest_add_test(dm_load_prefix_path,"Load datamodel using a prefix path")
    sahtest_add_test(dm_load_auto_resolve_yes,"Load datamodel setting auto resolve to yes")
    sahtest_add_test(dm_load_auto_resolve_no,"Load datamodel setting auto resolve to no")
    sahtest_add_test(dm_load_auto_resolve_invalid_value,"Load datamodel setting auto resolve to invalid value")
    sahtest_add_test(dm_load_no_auto_resolve_set_resolve_to_false,"Load datamodel after setting auto resolve to false and auto resolve env not defined")
    sahtest_add_test(dm_load_open_failing,"Load datamodel failing to open the file")
    sahtest_add_test(dm_load_failing,"Load datamodel failing to load")
    sahtest_add_test(dm_load_definition_null,"Load datamodel definition using null pointer datamodel")
    sahtest_add_test(dm_load_definition,"Load datamodel definition")
sahtest_group_end()

sahtest_group_begin(dm_verifying,dm_load_save_setup,dm_load_save_teardown)
    sahtest_add_test(dm_verify_null,"Verify datamodel using null pointer datamodel")
    sahtest_add_test(dm_verify_unknown_format,"Verify datamodel using an unknown format")
    sahtest_add_test(dm_verify_no_handler,"Verify datamodel using a format without verify handler")
    sahtest_add_test(dm_verify,"Verify datamodel")
    sahtest_add_test(dm_verify_prefix_path,"Verify datamodel using a prefix path")
    sahtest_add_test(dm_verify_open_failing,"Verify datamodel failing to open the file")
    sahtest_add_test(dm_verify_failing,"Verify datamodel verifying failed")
sahtest_group_end()

sahtest_group_begin(dm_saving,dm_load_save_setup,dm_load_save_teardown)
    sahtest_add_test(dm_save_null,"Save datamodel using null pointer object")
    sahtest_add_test(dm_save_unknown_format,"Save datamodel using an unknown format")
    sahtest_add_test(dm_save_no_handler,"Save datamodel using a format without save handler")
    sahtest_add_test(dm_save,"Save datamodel")
    sahtest_add_test(dm_save_open_failing,"Save datamodel failing to open the file")
    sahtest_add_test(dm_save_failing,"Save datamodel saving failed")
sahtest_group_end()

sahtest_group_begin(dm_load_save,NULL,NULL)
    sahtest_add_group(dm_verifying,"Test verifying data model")
    sahtest_add_group(dm_loading,"Test loading data model")
    sahtest_add_group(dm_saving,"Test saving data model")
sahtest_group_end()

#endif // DM_LOAD_SAVE_H
