/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include "datamodel_priv.h"

static datamodel_t dm;

bool test_datamodel_notify_handler1(datamodel_t *dm, object_t *object, parameter_t *parameter, notification_t *notify) {
    (void)dm;
    (void)object;
    (void)parameter;
    (void)notify;

    return true;
}

bool test_datamodel_notify_handler2(datamodel_t *dm, object_t *object, parameter_t *parameter, notification_t *notify) {
    (void)dm;
    (void)object;
    (void)parameter;
    (void)notify;

    return true;
}

void dm_notify_handlers_add_setup() {
    datamodel_initialize(&dm);
}

void dm_notify_handlers_add_teardown() {
    datamodel_cleanup(&dm);
}

void dm_notify_handlers_remove_setup() {
    datamodel_initialize(&dm);
    datamodel_addNotifyHandler(&dm,test_datamodel_notify_handler1);
    datamodel_addNotifyHandler(&dm,test_datamodel_notify_handler2);
}

void dm_notify_handlers_remove_teardown() {
    datamodel_cleanup(&dm);
}

int dm_notify_handlers_add_null() {
    assert_bool(datamodel_addNotifyHandler(NULL,test_datamodel_notify_handler1) == false);

    sahtest_success();
}

int dm_notify_handlers_add_null_fn() {
    assert_bool(datamodel_addNotifyHandler(&dm,NULL) == false);
    assert_bool(llist_isEmpty(&dm.notify));

    sahtest_success();
}

int dm_notify_handlers_add() {
    assert_bool(datamodel_addNotifyHandler(&dm,test_datamodel_notify_handler1) == true);
    assert_bool(llist_size(&dm.notify) == 1);

    llist_iterator_t *it = llist_first(&dm.notify);
    datamodel_notify_handlers_t *handler = llist_item_data(it,datamodel_notify_handlers_t,it);
    assert_bool(handler->datamodel_changed == test_datamodel_notify_handler1);

    sahtest_success();
}

int dm_notify_handlers_add_same_handler_twice() {
    assert_bool(datamodel_addNotifyHandler(&dm,test_datamodel_notify_handler1) == true);
    assert_bool(datamodel_addNotifyHandler(&dm,test_datamodel_notify_handler1) == true);

    assert_bool(llist_size(&dm.notify) == 1);

    llist_iterator_t *it = llist_first(&dm.notify);
    datamodel_notify_handlers_t *handler = llist_item_data(it,datamodel_notify_handlers_t,it);
    assert_bool(handler->datamodel_changed == test_datamodel_notify_handler1);

    sahtest_success();
}

int dm_notify_handlers_add_two_handlers() {
    assert_bool(datamodel_addNotifyHandler(&dm,test_datamodel_notify_handler1) == true);
    assert_bool(datamodel_addNotifyHandler(&dm,test_datamodel_notify_handler2) == true);

    assert_bool(llist_size(&dm.notify) == 2);

    llist_iterator_t *it = llist_first(&dm.notify);
    datamodel_notify_handlers_t *handler = llist_item_data(it,datamodel_notify_handlers_t,it);
    assert_bool(handler->datamodel_changed == test_datamodel_notify_handler1);

    it = llist_iterator_next(it);
    handler = llist_item_data(it,datamodel_notify_handlers_t,it);
    assert_bool(handler->datamodel_changed == test_datamodel_notify_handler2);

    sahtest_success();
}

int dm_notify_handlers_remove_null() {
    assert_bool(datamodel_removeNotifyHandler(NULL,test_datamodel_notify_handler1) == false);

    sahtest_success();
}

int dm_notify_handlers_remove_null_fn() {
    assert_bool(datamodel_removeNotifyHandler(&dm,NULL) == false);
    assert_bool(llist_size(&dm.notify) == 2);

    sahtest_success();
}

int dm_notify_handlers_remove() {
    assert_bool(datamodel_removeNotifyHandler(&dm,test_datamodel_notify_handler1) == true);
    assert_bool(llist_size(&dm.notify) == 1);

    llist_iterator_t *it = llist_first(&dm.notify);
    datamodel_notify_handlers_t *handler = llist_item_data(it,datamodel_notify_handlers_t,it);
    assert_bool(handler->datamodel_changed == test_datamodel_notify_handler2);

    sahtest_success();
}

int dm_notify_handlers_remove_same_handler_twice() {
    assert_bool(datamodel_removeNotifyHandler(&dm,test_datamodel_notify_handler1) == true);
    assert_bool(datamodel_removeNotifyHandler(&dm,test_datamodel_notify_handler1) == true);

    assert_bool(llist_size(&dm.notify) == 1);

    llist_iterator_t *it = llist_first(&dm.notify);
    datamodel_notify_handlers_t *handler = llist_item_data(it,datamodel_notify_handlers_t,it);
    assert_bool(handler->datamodel_changed == test_datamodel_notify_handler2);

    sahtest_success();
}

int dm_notify_handlers_remove_two_handlers() {
    assert_bool(datamodel_removeNotifyHandler(&dm,test_datamodel_notify_handler1) == true);
    assert_bool(datamodel_removeNotifyHandler(&dm,test_datamodel_notify_handler2) == true);

    assert_bool(llist_size(&dm.notify) == 0);

    sahtest_success();
}
