/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdarg.h>

#include "mock.h"

mock_struct_declare(getenv,50);
mock_struct_declare(open,50);
mock_struct_declare(rename,50);
mock_struct_declare(dirname,50);
mock_struct_declare(fsync,50);
mock_struct_declare(close,50);

typedef char *(*getenv_func_t)(const char *name);
typedef int (*open_func_t)(const char *pathname, int flags, ...);
typedef int (*rename_func_t)(const char *oldpath, const char *newpath);
typedef char * (*dirname_func_t)(char *path);
typedef int (*fsync_func_t)(int fd);
typedef int (*close_func_t)(int fd);

char *getenv(const char *name) {
    if (sahtest_isRunning()) {
        if (strcmp(mock_get_expectation(getenv)->varname,name) != 0) {
            return NULL;
        }
        const char *value = mock_get_expectation(getenv)->value;
        return value?value:NULL;
    }

    getenv_func_t real_getenv = mock_real_function("libc.so.6","getenv");
    if (!real_getenv) {
        return NULL;
    }

    return real_getenv(name);
}

int open(const char *pathname, int flags, ...) {
    if (sahtest_isRunning()) {
        T_CMP(mock_get_expectation(open)->pathname,pathname);
        T_VERIFY(mock_get_expectation(open)->flags == flags);
        return mock_get_expectation(open)->fd;
    }

    open_func_t real_open = mock_real_function("libc.so.6","open");
    if (!real_open) {
        return -1;
    }

    va_list arguments;
    va_start ( arguments, flags );
    int fd = real_open(pathname, flags, va_arg ( arguments, mode_t ));
    va_end ( arguments );

    return fd;
}

int rename(const char *oldpath, const char *newpath) {
    if (sahtest_isRunning()) {
        assert_string_equals(mock_get_expectation(rename)->oldpath,oldpath);
        assert_string_equals(mock_get_expectation(rename)->newpath,newpath);
        return mock_get_expectation(rename)->retval;
    }

    rename_func_t real_rename = mock_real_function("libc.so.6","rename");
    if (!real_rename) {
        return -1;
    }

    return real_rename(oldpath,newpath);
}

char *dirname(char *path) {
    if (sahtest_isRunning()) {
        if (strcmp(mock_get_expectation(dirname)->path,path) != 0) {
            sahtest_report(__LINE__,__FILE__,__FUNCTION__, "test failed");
            return ".";
        }
        return mock_get_expectation(dirname)->dirname;
    }

    dirname_func_t real_dirname = mock_real_function("libc.so.6","dirname");
    if (!real_dirname) {
        return ".";
    }

    return real_dirname(path);
}

int fsync(int fd) {
    if (sahtest_isRunning()) {
        assert_bool(mock_get_expectation(fsync)->fd = fd);
        return mock_get_expectation(fsync)->retval;
    }

    fsync_func_t real_fsync = mock_real_function("libc.so.6","fsync");
    if (!real_fsync) {
        return -1;
    }

    return real_fsync(fd);
}

int close(int fd) {
    if (sahtest_isRunning()) {
        assert_bool(mock_get_expectation(close)->fd = fd);
        return mock_get_expectation(close)->retval;
    }

    close_func_t real_close = mock_real_function("libc.so.6","close");
    if (!real_close) {
        return -1;
    }

    return real_close(fd);
}
