/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include <pcb/core/object.h>
#include <pcb/utils.h>
#include "datamodel_priv.h"
#include "object_priv.h"

static datamodel_t dm;
static object_t *root = NULL;
static object_t *testroot[5];

void obj_navigation_setup() {
    datamodel_initialize(&dm);
    root = datamodel_root(&dm);
    testroot[0] = object_create(root,"TestRoot1",object_attr_template);
    object_create(testroot[0],"Child",0);
    object_createInstance(testroot[0],0,NULL);

    testroot[1] = object_create(root,"TestRoot2",0);
    object_create(testroot[1],"Child",0);

    testroot[2] = object_create(root,"TestRoot3",0);

    testroot[3] = object_create(root,"TestRoot4",object_attr_template);
    object_createInstance(testroot[3],0,NULL);
    object_createInstance(testroot[3],0,NULL);
    object_createInstance(testroot[3],0,NULL);

    testroot[4] = object_create(root,"TestRoot5",object_attr_template);

    object_commit(root);
}

void obj_navigation_teardown() {
    datamodel_cleanup(&dm);
}

int obj_root() {
    object_t *obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_root(obj) == root);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_root(obj) == root);

    assert_bool(object_root(testroot[0]) == root);
    assert_bool(object_root(testroot[1]) == root);
    assert_bool(object_root(testroot[2]) == root);

    sahtest_success();
}

int obj_root_null() {
    assert_bool(object_root(NULL) == NULL);

    sahtest_success();
}

int obj_datamodel() {
    object_t *obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_datamodel(obj) == &dm);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_datamodel(obj) == &dm);

    assert_bool(object_datamodel(testroot[0]) == &dm);
    assert_bool(object_datamodel(testroot[1]) == &dm);
    assert_bool(object_datamodel(testroot[2]) == &dm);

    sahtest_success();
}

int obj_datamodel_null() {
    assert_bool(object_datamodel(NULL) == NULL);

    sahtest_success();
}

int obj_parent() {
    object_t *obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    object_t *parent = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(parent);
    assert_bool(object_parent(obj) == parent);
    assert_bool(object_parent(parent) == testroot[0]);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_parent(obj) == testroot[1]);

    assert_bool(object_parent(testroot[0]) == root);
    assert_bool(object_parent(testroot[1]) == root);
    assert_bool(object_parent(testroot[2]) == root);

    sahtest_success();
}

int obj_parent_null() {
    assert_bool(object_parent(NULL) == NULL);

    sahtest_success();
}

int obj_first_child() {
    object_t *obj = NULL;
    object_t *inst = NULL;
    assert_bool(object_firstChild(root) == testroot[0]);

    obj = object_getObject(root,"TestRoot1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_firstChild(testroot[0]) == obj);
    inst = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(inst);
    obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_firstChild(inst) == obj);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_firstChild(testroot[1]) == obj);

    assert_bool(object_firstChild(testroot[2]) == NULL);

    assert_bool(object_firstChild(testroot[3]) == NULL);

    sahtest_success();
}

int obj_first_child_null() {
    assert_bool(object_firstChild(NULL) == NULL);

    sahtest_success();
}

int obj_last_child() {
    object_t *obj = NULL;
    object_t *inst = NULL;

    assert_bool(object_lastChild(root) == testroot[4]);

    obj = object_getObject(root,"TestRoot1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_lastChild(testroot[0]) == obj);
    inst = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(inst);
    obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_lastChild(inst) == obj);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_lastChild(testroot[1]) == obj);

    assert_bool(object_lastChild(testroot[2]) == NULL);

    assert_bool(object_lastChild(testroot[3]) == NULL);

    sahtest_success();
}

int obj_last_child_null() {
    assert_bool(object_lastChild(NULL) == NULL);

    sahtest_success();
}

int obj_next_sibling() {
    object_t *obj = NULL;

    assert_bool(object_nextSibling(root) == NULL);
    assert_bool(object_nextSibling(testroot[0]) == testroot[1]);
    assert_bool(object_nextSibling(testroot[1]) == testroot[2]);
    assert_bool(object_nextSibling(testroot[2]) == testroot[3]);
    assert_bool(object_nextSibling(testroot[3]) == testroot[4]);
    assert_bool(object_nextSibling(testroot[4]) == NULL);

    obj = object_getObject(root,"TestRoot1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_nextSibling(obj) == NULL);
    obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_nextSibling(obj) == NULL);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_nextSibling(obj) == NULL);

    obj = object_getObject(root,"TestRoot4.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_nextSibling(obj) == NULL);

    sahtest_success();
}

int obj_next_sibling_null() {
    assert_bool(object_nextSibling(NULL) == NULL);

    sahtest_success();
}

int obj_prev_sibling() {
    object_t *obj = NULL;

    assert_bool(object_prevSibling(root) == NULL);
    assert_bool(object_prevSibling(testroot[0]) == NULL);
    assert_bool(object_prevSibling(testroot[1]) == testroot[0]);
    assert_bool(object_prevSibling(testroot[2]) == testroot[1]);
    assert_bool(object_prevSibling(testroot[3]) == testroot[2]);
    assert_bool(object_prevSibling(testroot[4]) == testroot[3]);

    obj = object_getObject(root,"TestRoot1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_prevSibling(obj) == NULL);
    obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_prevSibling(obj) == NULL);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_prevSibling(obj) == NULL);

    obj = object_getObject(root,"TestRoot4.3",0,NULL);
    assert_not_null(obj);
    assert_bool(object_prevSibling(obj) == NULL);

    sahtest_success();
}

int obj_prev_sibling_null() {
    assert_bool(object_prevSibling(NULL) == NULL);

    sahtest_success();
}

int obj_has_children() {
    object_t *obj = NULL;

    assert_bool(object_hasChildren(root) == true);
    assert_bool(object_hasChildren(testroot[0]) == true);
    assert_bool(object_hasChildren(testroot[1]) == true);
    assert_bool(object_hasChildren(testroot[2]) == false);
    assert_bool(object_hasChildren(testroot[3]) == false);

    obj = object_getObject(root,"TestRoot1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_hasChildren(obj) == false);
    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_hasChildren(obj) == true);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_hasChildren(obj) == false);

    obj = object_getObject(root,"TestRoot4.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_hasChildren(obj) == false);

    sahtest_success();
}

int obj_has_children_null() {
    assert_bool(object_hasChildren(NULL) == false);

    sahtest_success();
}

int obj_child_count() {
    object_t *obj = NULL;

    assert_bool(object_childCount(root) == 5);
    assert_bool(object_childCount(testroot[0]) == 1);
    assert_bool(object_childCount(testroot[1]) == 1);
    assert_bool(object_childCount(testroot[2]) == 0);
    assert_bool(object_childCount(testroot[3]) == 0);

    obj = object_getObject(root,"TestRoot1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_childCount(obj) == 0);
    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_childCount(obj) == 1);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_childCount(obj) == 0);

    obj = object_getObject(root,"TestRoot4.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_childCount(obj) == 0);

    sahtest_success();
}

int obj_child_count_null() {
    assert_bool(object_childCount(NULL) == 0);

    sahtest_success();
}

int obj_first_instance() {
    object_t *obj = NULL;
    assert_bool(object_firstInstance(root) == NULL);

    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_firstInstance(testroot[0]) == obj);
    assert_bool(object_firstInstance(obj) == NULL);
    obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_firstInstance(obj) == NULL);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_firstInstance(testroot[1]) == NULL);
    assert_bool(object_firstInstance(obj) == NULL);

    assert_bool(object_firstInstance(testroot[2]) == NULL);

    obj = object_getObject(root,"TestRoot4.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_firstInstance(testroot[3]) == obj);

    assert_bool(object_firstInstance(testroot[4]) == NULL);

    sahtest_success();
}

int obj_first_instance_null() {
    assert_bool(object_firstInstance(NULL) == NULL);

    sahtest_success();
}

int obj_last_instance() {
    object_t *obj = NULL;
    assert_bool(object_lastInstance(root) == NULL);

    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_lastInstance(testroot[0]) == obj);
    assert_bool(object_lastInstance(obj) == NULL);
    obj = object_getObject(root,"TestRoot1.1.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_lastInstance(obj) == NULL);

    obj = object_getObject(root,"TestRoot2.Child",0,NULL);
    assert_not_null(obj);
    assert_bool(object_lastInstance(testroot[1]) == NULL);
    assert_bool(object_lastInstance(obj) == NULL);

    assert_bool(object_lastInstance(testroot[2]) == NULL);

    obj = object_getObject(root,"TestRoot4.3",0,NULL);
    assert_not_null(obj);
    assert_bool(object_lastInstance(testroot[3]) == obj);

    assert_bool(object_lastInstance(testroot[4]) == NULL);

    sahtest_success();
}

int obj_last_instance_null() {
    assert_bool(object_lastInstance(NULL) == NULL);

    sahtest_success();
}

int obj_next_instance() {
    object_t *obj = NULL;
    object_t *inst[3];
    assert_bool(object_nextInstance(root) == NULL);

    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_nextInstance(obj) == NULL);
    assert_bool(object_nextInstance(testroot[0]) == NULL);

    inst[0] = object_getObject(root,"TestRoot4.1",0,NULL);
    assert_not_null(inst[0]);
    inst[1] = object_getObject(root,"TestRoot4.2",0,NULL);
    assert_not_null(inst[1]);
    inst[2] = object_getObject(root,"TestRoot4.3",0,NULL);
    assert_not_null(inst[2]);

    assert_bool(object_nextInstance(inst[0]) == inst[1]);
    assert_bool(object_nextInstance(inst[1]) == inst[2]);
    assert_bool(object_nextInstance(inst[2]) == NULL);

    sahtest_success();
}

int obj_next_instance_null() {
    assert_bool(object_nextInstance(NULL) == NULL);

    sahtest_success();
}

int obj_prev_instance() {
    object_t *obj = NULL;
    object_t *inst[3];
    assert_bool(object_prevInstance(root) == NULL);

    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_prevInstance(obj) == NULL);
    assert_bool(object_prevInstance(testroot[0]) == NULL);

    inst[0] = object_getObject(root,"TestRoot4.1",0,NULL);
    assert_not_null(inst[0]);
    inst[1] = object_getObject(root,"TestRoot4.2",0,NULL);
    assert_not_null(inst[1]);
    inst[2] = object_getObject(root,"TestRoot4.3",0,NULL);
    assert_not_null(inst[2]);

    assert_bool(object_prevInstance(inst[0]) == NULL);
    assert_bool(object_prevInstance(inst[1]) == inst[0]);
    assert_bool(object_prevInstance(inst[2]) == inst[1]);

    sahtest_success();
}

int obj_prev_instance_null() {
    assert_bool(object_prevInstance(NULL) == NULL);

    sahtest_success();
}

int obj_has_instances() {
    object_t *obj = NULL;
    assert_bool(object_hasInstances(root) == false);
    assert_bool(object_hasInstances(testroot[0]) == true);
    assert_bool(object_hasInstances(testroot[1]) == false);
    assert_bool(object_hasInstances(testroot[2]) == false);
    assert_bool(object_hasInstances(testroot[3]) == true);
    assert_bool(object_hasInstances(testroot[4]) == false);

    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_hasInstances(obj) == false);

    sahtest_success();
}

int obj_has_instances_null() {
    assert_bool(object_hasInstances(NULL) == false);

    sahtest_success();
}

int obj_instance_count() {
    object_t *obj = NULL;
    assert_bool(object_instanceCount(root) == 0);
    assert_bool(object_instanceCount(testroot[0]) == 1);
    assert_bool(object_instanceCount(testroot[1]) == 0);
    assert_bool(object_instanceCount(testroot[2]) == 0);
    assert_bool(object_instanceCount(testroot[3]) == 3);
    assert_bool(object_instanceCount(testroot[4]) == 0);

    obj = object_getObject(root,"TestRoot1.1",0,NULL);
    assert_not_null(obj);
    assert_bool(object_instanceCount(obj) == 0);

    sahtest_success();
}

int obj_instance_count_null() {
    assert_bool(object_instanceCount(NULL) == 0);

    sahtest_success();
}
