/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include <pcb/core/object.h>
#include <pcb/utils.h>
#include "datamodel_priv.h"
#include "object_priv.h"

static datamodel_t dm;
static object_t *root = NULL;
static object_t *instance;
static object_t *instance_child;

void obj_path_name_setup() {
    datamodel_initialize(&dm);
    datamodel_setNamespace(&dm,"Bar");
    root = datamodel_root(&dm);
    object_t *obj = object_create(root,"TestRoot1",object_attr_template);
    object_create(obj,"Child",0);
    instance = object_createInstance(obj,5,"Foo");
    instance_child = object_firstChild(instance);

    object_commit(root);
}

void obj_path_name_teardown() {
    datamodel_cleanup(&dm);
}

int obj_path_index() {
    string_t path;
    string_initialize(&path,0);

    object_path(instance_child,&path,path_attr_default);
    assert_string_equals(string_buffer(&path),"TestRoot1.5.Child");

    char *p = object_pathChar(instance_child,path_attr_default);
    assert_string_equals(p,"TestRoot1.5.Child");

    free(p);
    string_cleanup(&path);

    sahtest_success();
}

int obj_path_key() {
    string_t path;
    string_initialize(&path,0);

    object_path(instance_child,&path,path_attr_key_notation);
    assert_string_equals(string_buffer(&path),"TestRoot1.Foo.Child");

    char *p = object_pathChar(instance_child,path_attr_key_notation);
    assert_string_equals(p,"TestRoot1.Foo.Child");

    free(p);
    string_cleanup(&path);

    sahtest_success();
}

int obj_path_index_slash() {
    string_t path;
    string_initialize(&path,0);

    object_path(instance_child,&path,path_attr_slash_seperator);
    assert_string_equals(string_buffer(&path),"TestRoot1/5/Child");

    char *p = object_pathChar(instance_child,path_attr_slash_seperator);
    assert_string_equals(p,"TestRoot1/5/Child");

    free(p);
    string_cleanup(&path);

    sahtest_success();
}

int obj_path_key_slash() {
    string_t path;
    string_initialize(&path,0);

    object_path(instance_child,&path,path_attr_key_notation | path_attr_slash_seperator);
    assert_string_equals(string_buffer(&path),"TestRoot1/Foo/Child");

    char *p = object_pathChar(instance_child,path_attr_key_notation | path_attr_slash_seperator);
    assert_string_equals(p,"TestRoot1/Foo/Child");

    free(p);
    string_cleanup(&path);

    sahtest_success();
}

int obj_path_include_namespace() {
    string_t path;
    string_initialize(&path,0);

    object_path(instance_child,&path,path_attr_include_namespace);
    assert_string_equals(string_buffer(&path),"Bar.TestRoot1.5.Child");

    char *p = object_pathChar(instance_child,path_attr_include_namespace);
    assert_string_equals(p,"Bar.TestRoot1.5.Child");

    free(p);
    string_cleanup(&path);

    sahtest_success();
}

int obj_path_parent() {
    string_t path;
    string_initialize(&path,0);

    object_path(instance_child,&path,path_attr_parent);
    assert_string_equals(string_buffer(&path),"TestRoot1.5");

    char *p = object_pathChar(instance_child,path_attr_parent);
    assert_string_equals(p,"TestRoot1.5");

    free(p);
    string_cleanup(&path);

    sahtest_success();
}

int obj_path_null() {
    string_t path;
    string_initialize(&path,0);

    object_path(NULL,&path,path_attr_default);
    assert_bool(string_length(&path) == 0);
    object_path(instance_child,NULL,path_attr_default);

    char *p = object_pathChar(NULL,path_attr_parent);
    assert_null(p);

    string_cleanup(&path);
    sahtest_success();
}

int obj_name_index() {
    assert_string_equals(object_name(instance_child,path_attr_default),"Child");
    assert_string_equals(object_name(instance,path_attr_default),"5");
    assert_null(object_name(root,path_attr_default));

    sahtest_success();
}

int obj_name_key() {
    assert_string_equals(object_name(instance_child,path_attr_key_notation),"Child");
    assert_string_equals(object_name(instance,path_attr_key_notation),"Foo");
    assert_null(object_name(root,path_attr_key_notation));

    sahtest_success();
}

int obj_name_null() {
    assert_null(object_name(NULL,path_attr_key_notation));

    sahtest_success();
}
