/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include <pcb/core/object.h>
#include <pcb/utils.h>
#include "datamodel_priv.h"
#include "object_priv.h"

static datamodel_t dm;
static object_t *root = NULL;
static object_t *testroot[5];
static object_t *instances_child[6];

void obj_get_find_setup() {
    datamodel_initialize(&dm);
    root = datamodel_root(&dm);
    testroot[0] = object_create(root,"TestRoot1",object_attr_template);
    object_create(testroot[0],"Child",0);
    object_t *obj = object_createInstance(testroot[0],0,"Key1");
    instances_child[0] = object_firstChild(obj);
    obj = object_createInstance(testroot[0],0,"Key2");
    instances_child[1] = object_firstChild(obj);
    obj = object_createInstance(testroot[0],0,"Key3");
    instances_child[2] = object_firstChild(obj);

    testroot[1] = object_create(root,"TestRoot2",0);
    object_create(testroot[1],"Child",0);

    testroot[2] = object_create(root,"TestRoot3",0);

    testroot[3] = object_create(root,"TestRoot4",object_attr_template);
    obj = object_createInstance(testroot[3],0,"Key1");
    instances_child[3] = obj;
    obj = object_createInstance(testroot[3],0,"Key2");
    instances_child[4] = obj;
    obj = object_createInstance(testroot[3],0,"Key3");
    instances_child[5] = obj;

    testroot[4] = object_create(root,"TestRoot5",object_attr_template);

    object_commit(root);
}

void obj_get_find_teardown() {
    datamodel_cleanup(&dm);
}

int obj_get_index_path() {
    object_t *obj = object_getObject(root,"TestRoot1.1.Child",path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[0]);
    obj = object_getObject(testroot[0],"3.Child",path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[2]);

    sahtest_success();
}

int obj_get_key_path() {
    object_t *obj = object_getObject(root,"TestRoot1.Key1.Child",path_attr_key_notation,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[0]);
    obj = object_getObject(testroot[0],"Key3.Child",path_attr_key_notation,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[2]);

    sahtest_success();
}

int obj_get_index_slash_path() {
    object_t *obj = object_getObject(root,"TestRoot1/1/Child",path_attr_slash_seperator,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[0]);
    obj = object_getObject(testroot[0],"3/Child",path_attr_slash_seperator,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[2]);

    sahtest_success();
}

int obj_get_key_slash_path() {
    object_t *obj = object_getObject(root,"TestRoot1/Key1/Child",path_attr_key_notation | path_attr_slash_seperator,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[0]);
    obj = object_getObject(testroot[0],"Key3/Child",path_attr_key_notation | path_attr_slash_seperator,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[2]);

    sahtest_success();
}

int obj_get_partial_match() {
    bool exactMatch = false;
    object_t *obj = object_getObject(root,"TestRoot1.1.Child.SubChild.Extra",path_attr_partial_match,&exactMatch);
    assert_not_null(obj);
    assert_bool(exactMatch == false);
    assert_bool(obj == instances_child[0]);
    obj = object_getObject(testroot[0],"3.Child",path_attr_partial_match,&exactMatch);
    assert_not_null(obj);
    assert_bool(exactMatch == true);
    assert_bool(obj == instances_child[2]);
    obj = object_getObject(testroot[0],"2",path_attr_partial_match,&exactMatch);
    assert_not_null(obj);
    assert_bool(exactMatch == true);
    assert_bool(obj == object_parent(instances_child[1]));
    obj = object_getObject(testroot[0],"2.Test",path_attr_partial_match,&exactMatch);
    assert_not_null(obj);
    assert_bool(exactMatch == true);
    assert_bool(obj == object_parent(instances_child[1]));

    sahtest_success();
}

int obj_get_empty_path() {
    bool exactMatch = false;
    object_t *obj = object_getObject(root,"",path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == root);
    obj = object_getObject(testroot[0],"",path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == testroot[0]);
    obj = object_getObject(testroot[1],"",path_attr_partial_match,&exactMatch);
    assert_not_null(obj);
    assert_bool(obj == testroot[1]);
    assert_bool(exactMatch == true);

    sahtest_success();
}

int obj_get_null_path() {
    object_t *obj = object_getObject(root,NULL,path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == root);
    obj = object_getObject(testroot[0],NULL,path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == testroot[0]);

    sahtest_success();
}

int obj_get_null_object() {
    object_t *obj = object_getObject(NULL,"TestRoot1.1.Child",path_attr_default,NULL);
    assert_null(obj);

    sahtest_success();
}

int obj_get_invalid_path() {
    object_t *obj = object_getObject(root,"TestRoot1.1.Child.SubChild.Extra",path_attr_default,NULL);
    assert_null(obj);

    sahtest_success();
}

int obj_get_use_path_type_modifiers() {
    object_t *obj = object_getObject(root,"TestRoot1.$Key1.Child",path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[0]);
    obj = object_getObject(testroot[0],"$Key3.Child",path_attr_default,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[2]);

    obj = object_getObject(root,"TestRoot1.%1.Child",path_attr_key_notation,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[0]);
    obj = object_getObject(testroot[0],"%3.Child",path_attr_key_notation,NULL);
    assert_not_null(obj);
    assert_bool(obj == instances_child[2]);

    sahtest_success();
}

int obj_get_by_key() {
    object_t *obj = object_getObjectByKey(root,"TestRoot1.%s.Child","Key1");
    assert_not_null(obj);
    assert_bool(obj == instances_child[0]);

    obj = object_getObjectByKey(root,"TestRoot1.%%%d.%s",2,"Child");
    assert_not_null(obj);
    assert_bool(obj == instances_child[1]);

    sahtest_success();
}

int obj_find_index_path() {
    object_list_t *objs = object_findObjects(root,"TestRoot?.1.*",-1,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 1);
    llist_iterator_t *it = llist_first(objs);
    object_t *obj = object_item(it);
    assert_bool(obj == instances_child[0]);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?.[13]",-1,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 4);
    it = llist_first(objs);
    obj = object_item(it);
    assert_bool(obj == object_parent(instances_child[0]));
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == object_parent(instances_child[2]));
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == instances_child[3]);
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == instances_child[5]);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(testroot[0],"TestRoot?.[13]",-1,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 2);
    llist_cleanup(objs);
    free(objs);

    sahtest_success();
}

int obj_find_key_path() {
    object_list_t *objs = object_findObjects(root,"TestRoot?.Key1.*",-1,search_attr_key_notation);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 1);
    llist_iterator_t *it = llist_first(objs);
    object_t *obj = object_item(it);
    assert_bool(obj == instances_child[0]);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?.Key[13]",-1,search_attr_key_notation);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 4);
    it = llist_first(objs);
    obj = object_item(it);
    assert_bool(obj == object_parent(instances_child[0]));
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == object_parent(instances_child[2]));
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == instances_child[3]);
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == instances_child[5]);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(testroot[0],"TestRoot?.Key[13]",-1,search_attr_key_notation);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 2);
    llist_cleanup(objs);
    free(objs);

    sahtest_success();
}

int obj_find_slash_path() {
    object_list_t *objs = object_findObjects(root,"TestRoot?/1/*",-1,search_attr_slash_seperator);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 1);
    llist_iterator_t *it = llist_first(objs);
    object_t *obj = object_item(it);
    assert_bool(obj == instances_child[0]);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?/[13]",-1,search_attr_slash_seperator);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 4);
    it = llist_first(objs);
    obj = object_item(it);
    assert_bool(obj == object_parent(instances_child[0]));
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == object_parent(instances_child[2]));
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == instances_child[3]);
    it = llist_iterator_next(it);
    obj = object_item(it);
    assert_bool(obj == instances_child[5]);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(testroot[0],"TestRoot?/[13]",-1,search_attr_slash_seperator);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 2);
    llist_cleanup(objs);
    free(objs);

    sahtest_success();
}

int obj_find_null_object() {
    object_list_t *objs = object_findObjects(NULL,"TestRoot?.1.*",-1,search_attr_default);
    assert_null(objs);

    object_t *obj = object_item(NULL);
    assert_null(obj);

    sahtest_success();
}

int obj_find_different_depths() {
    object_list_t *objs = object_findObjects(root,"TestRoot?.1.*",0,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 0);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?.1.*",1,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 0);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?.1.*",3,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 1);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?*",0,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 0);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?",1,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 5);
    llist_cleanup(objs);
    free(objs);

    objs = object_findObjects(root,"TestRoot?",2,search_attr_default);
    assert_not_null(objs);
    assert_bool(llist_size(objs) == 5);
    llist_cleanup(objs);
    free(objs);

    sahtest_success();
}
