/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include <pcb/core/object.h>
#include <pcb/core/parameter.h>
#include <pcb/utils.h>
#include "datamodel_priv.h"
#include "object_priv.h"
#include "parameter_priv.h"

static datamodel_t dm;
static object_t *root = NULL;
static object_t *object = NULL;
static parameter_t *params[5];

void param_iterating_setup() {
    datamodel_initialize(&dm);
    root = datamodel_root(&dm);
    object = object_create(root,"TestRoot1",object_attr_default);

    params[0] = parameter_create(root,"Foo1",parameter_type_string,parameter_attr_default);
    params[1] = parameter_create(root,"Foo2",parameter_type_string,parameter_attr_default);
    params[2] = parameter_create(root,"Foo3",parameter_type_string,parameter_attr_default);
    params[3] = parameter_create(root,"Foo4",parameter_type_string,parameter_attr_default);
    params[4] = parameter_create(root,"Foo5",parameter_type_string,parameter_attr_default);

}

void param_iterating_teardown() {
    datamodel_cleanup(&dm);
}

int obj_first_parameter() {
    assert_bool(object_firstParameter(root) == params[0]);
    assert_bool(object_firstParameter(object) == NULL);
    assert_bool(object_firstParameter(NULL) == NULL);

    sahtest_success();
}

int obj_last_parameter() {
    assert_bool(object_lastParameter(root) == params[4]);
    assert_bool(object_lastParameter(object) == NULL);
    assert_bool(object_lastParameter(NULL) == NULL);

    sahtest_success();
}

int obj_next_parameter() {
    assert_bool(object_nextParameter(params[0]) == params[1]);
    assert_bool(object_nextParameter(params[1]) == params[2]);
    assert_bool(object_nextParameter(params[2]) == params[3]);
    assert_bool(object_nextParameter(params[3]) == params[4]);
    assert_bool(object_nextParameter(params[4]) == NULL);
    assert_bool(object_nextParameter(NULL) == NULL);

    sahtest_success();
}

int obj_prev_parameter() {
    assert_bool(object_prevParameter(params[0]) == NULL);
    assert_bool(object_prevParameter(params[1]) == params[0]);
    assert_bool(object_prevParameter(params[2]) == params[1]);
    assert_bool(object_prevParameter(params[3]) == params[2]);
    assert_bool(object_prevParameter(params[4]) == params[3]);
    assert_bool(object_prevParameter(NULL) == NULL);

    sahtest_success();
}

int obj_has_parameters() {
    assert_bool(object_hasParameters(root) == true);
    assert_bool(object_hasParameters(NULL) == false);

    sahtest_success();
}

int obj_parameter_count() {
    assert_bool(object_parameterCount(root) == 5);
    assert_bool(object_parameterCount(NULL) == 0);

    sahtest_success();
}

int obj_get_parameter() {
    assert_bool(object_getParameter(root,"Foo1") == params[0]);
    assert_bool(object_getParameter(root,"Foo2") == params[1]);
    assert_bool(object_getParameter(root,"Foo3") == params[2]);
    assert_bool(object_getParameter(root,"Foo4") == params[3]);
    assert_bool(object_getParameter(root,"Foo5") == params[4]);
    assert_bool(object_getParameter(root,"Foo6") == NULL);
    assert_bool(object_getParameter(NULL,"Foo1") == NULL);
    assert_bool(object_getParameter(root,NULL) == NULL);

    sahtest_success();
}
