/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include <pcb/core/object.h>
#include <pcb/core/parameter.h>
#include <pcb/utils.h>
#include "datamodel_priv.h"
#include "object_priv.h"
#include "parameter_priv.h"

static datamodel_t dm;
static object_t *root = NULL;
static parameter_t *param;

void param_rename_cast_setup() {
    datamodel_initialize(&dm);
    root = datamodel_root(&dm);

    param = parameter_create(root,"Foo",parameter_type_string,parameter_attr_default);
}

void param_rename_cast_teardown() {
    datamodel_cleanup(&dm);
}

int param_rename() {
    assert_bool(parameter_rename(param,"Bar") == true);
    assert_string_equals(parameter_name(param),"Bar");

    sahtest_success();
}

int param_rename_null() {
    assert_bool(parameter_rename(NULL,"Bar") == false);
    assert_bool(parameter_rename(param,NULL) == false);
    assert_bool(parameter_rename(param,"") == false);

    sahtest_success();
}

int param_rename_invalid_name() {
    assert_bool(parameter_rename(param,"Foo.Bar") == false);

    sahtest_success();
}

int param_rename_same_name() {
    assert_bool(parameter_rename(param,"Foo") == true);

    sahtest_success();
}

int param_rename_longer_name() {
    assert_bool(parameter_rename(param,"FooBar") == true);
    assert_string_equals(parameter_name(param),"FooBar");
    assert_bool(parameter_rename(param,"FooBarExtended") == true);
    assert_string_equals(parameter_name(param),"FooBarExtended");

    sahtest_success();
}

int param_cast() {
    assert_bool(parameter_cast(param,parameter_type_uint32) == true);
    assert_bool(parameter_type(param) == parameter_type_uint32);

    sahtest_success();
}

int param_cast_null() {
    assert_bool(parameter_cast(NULL,parameter_type_uint32) == false);

    sahtest_success();
}
