/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/core/datamodel.h>
#include <pcb/core/object.h>
#include <pcb/core/parameter.h>
#include <pcb/utils.h>
#include "datamodel_priv.h"
#include "object_priv.h"
#include "parameter_priv.h"

static datamodel_t dm;
static object_t *root = NULL;
static parameter_t *params[11];

void param_values_setup() {
    datamodel_initialize(&dm);
    root = datamodel_root(&dm);

    params[0] = parameter_create(root,"Foo1",parameter_type_string,parameter_attr_default);
    params[1] = parameter_create(root,"Foo2",parameter_type_int8,parameter_attr_default);
    params[2] = parameter_create(root,"Foo3",parameter_type_int16,parameter_attr_default);
    params[3] = parameter_create(root,"Foo4",parameter_type_int32,parameter_attr_default);
    params[4] = parameter_create(root,"Foo5",parameter_type_int64,parameter_attr_default);
    params[5] = parameter_create(root,"Foo6",parameter_type_uint8,parameter_attr_default);
    params[6] = parameter_create(root,"Foo7",parameter_type_uint16,parameter_attr_default);
    params[7] = parameter_create(root,"Foo8",parameter_type_uint32,parameter_attr_default);
    params[8] = parameter_create(root,"Foo9",parameter_type_uint64,parameter_attr_default);
    params[9] = parameter_create(root,"Foo10",parameter_type_bool,parameter_attr_default);
    params[10] = parameter_create(root,"Foo11",parameter_type_date_time,parameter_attr_default);
}

void param_values_teardown() {
    datamodel_cleanup(&dm);
}

int obj_param_value_string_type() {
    const variant_t *value = parameter_getValue(params[0]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_string);

    value = object_parameterValue(root,"Foo1");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_string);

    sahtest_success();
}

int obj_param_value_int8_type() {
    const variant_t *value = parameter_getValue(params[1]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int8);

    value = object_parameterValue(root,"Foo2");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int8);

    sahtest_success();
}

int obj_param_value_int16_type() {
    const variant_t *value = parameter_getValue(params[2]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int16);

    value = object_parameterValue(root,"Foo3");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int16);

    sahtest_success();
}

int obj_param_value_int32_type() {
    const variant_t *value = parameter_getValue(params[3]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int32);

    value = object_parameterValue(root,"Foo4");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int32);

    sahtest_success();
}

int obj_param_value_int64_type() {
    const variant_t *value = parameter_getValue(params[4]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int64);

    value = object_parameterValue(root,"Foo5");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_int64);

    sahtest_success();
}

int obj_param_value_uint8_type() {
    const variant_t *value = parameter_getValue(params[5]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint8);

    value = object_parameterValue(root,"Foo6");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint8);

    sahtest_success();
}

int obj_param_value_uint16_type() {
    const variant_t *value = parameter_getValue(params[6]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint16);

    value = object_parameterValue(root,"Foo7");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint16);

    sahtest_success();
}

int obj_param_value_uint32_type() {
    const variant_t *value = parameter_getValue(params[7]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint32);

    value = object_parameterValue(root,"Foo8");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint32);

    sahtest_success();
}

int obj_param_value_uint64_type() {
    const variant_t *value = parameter_getValue(params[8]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint64);

    value = object_parameterValue(root,"Foo9");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_uint64);

    sahtest_success();
}

int obj_param_value_bool_type() {
    const variant_t *value = parameter_getValue(params[9]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_bool);

    value = object_parameterValue(root,"Foo10");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_bool);

    sahtest_success();
}

int obj_param_value_date_time_type() {
    const variant_t *value = parameter_getValue(params[10]);
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_date_time);

    value = object_parameterValue(root,"Foo11");
    assert_not_null(value);
    assert_bool(variant_type(value) == variant_type_date_time);

    sahtest_success();
}

int obj_param_value_object_null() {
    const variant_t *value = object_parameterValue(NULL,"Foo1");
    assert_null(value);

    sahtest_success();
}

int obj_param_value_parameter_null() {
    const variant_t *value = parameter_getValue(NULL);
    assert_null(value);
    value = object_parameterValue(root,NULL);
    assert_null(value);
    value = object_parameterValue(root,"");
    assert_null(value);

    sahtest_success();
}

