/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#ifndef LLIST_INSERT_H
#define LLIST_INSERT_H

void llist_inserting_setup();
void llist_inserting_teardown();

void llist_insert_build_list_setup();
void llist_insert_build_list_teardown();

int llist_insert_append_null_list();
int llist_insert_append_null_it();
int llist_insert_append_orphan();
int llist_insert_append_iterator_twice();
int llist_insert_append_multiple();

int llist_insert_prepend_null_list();
int llist_insert_prepend_null_it();
int llist_insert_prepend_orphan();
int llist_insert_prepend_iterator_twice();
int llist_insert_prepend_multiple();

int llist_insert_before_null_list();
int llist_insert_before_null_reference();
int llist_insert_before_null_it();
int llist_insert_before_first();
int llist_insert_before_last();
int llist_insert_before_second();
int llist_insert_before_reference_is_insert_it();
int llist_insert_before_in_empty_list();
int llist_insert_before_using_orphan_reference();
int llist_insert_before_move_iterator_in_list();

int llist_insert_after_null_list();
int llist_insert_after_null_reference();
int llist_insert_after_null_it();
int llist_insert_after_first();
int llist_insert_after_last();
int llist_insert_after_second();
int llist_insert_after_reference_is_insert_it();
int llist_insert_after_in_empty_list();
int llist_insert_after_using_orphan_reference();
int llist_insert_after_move_iterator_in_list();

int llist_insert_indexed_null_list();
int llist_insert_indexed_empty_list_index_0();
int llist_insert_indexed_empty_list_index_2();
int llist_insert_indexed_list_index_out_of_boundry();
int llist_insert_indexed_list_index_0();
int llist_insert_indexed_list_index_2();
int llist_insert_indexed_list_index_last();

sahtest_group_begin(ll_append,llist_inserting_setup,llist_inserting_teardown)
    sahtest_add_test(llist_insert_append_null_list,"Test appending iterator to null list")
    sahtest_add_test(llist_insert_append_null_it,"Test appending null iterator to list")
    sahtest_add_test(llist_insert_append_orphan,"Test appending a valid iterator")
    sahtest_add_test(llist_insert_append_iterator_twice,"Test appending the same iterator twice")
    sahtest_add_test(llist_insert_append_multiple,"Test appending multiple iterators after each other")
sahtest_group_end()

sahtest_group_begin(ll_prepend,llist_inserting_setup,llist_inserting_teardown)
    sahtest_add_test(llist_insert_prepend_null_list,"Test prepending iterator to null list")
    sahtest_add_test(llist_insert_prepend_null_it,"Test prepending null iterator to list")
    sahtest_add_test(llist_insert_prepend_orphan,"Test prepending a valid iterator")
    sahtest_add_test(llist_insert_prepend_iterator_twice,"Test prepending the same iterator twice")
    sahtest_add_test(llist_insert_prepend_multiple,"Test prepending multiple iterators before each other")
sahtest_group_end()

sahtest_group_begin(insert_before,llist_insert_build_list_setup,llist_insert_build_list_teardown)
    sahtest_add_test(llist_insert_before_null_list,"Test inserting in a null list")
    sahtest_add_test(llist_insert_before_null_reference,"Test inserting before null reference")
    sahtest_add_test(llist_insert_before_null_it,"Test inserting null it")
    sahtest_add_test(llist_insert_before_first,"Test inserting before first item")
    sahtest_add_test(llist_insert_before_last,"Test inserting before last item")
    sahtest_add_test(llist_insert_before_second,"Test inserting before second item")
    sahtest_add_test(llist_insert_before_reference_is_insert_it,"Test inserting item which is used as reference")
    sahtest_add_test(llist_insert_before_in_empty_list,"Test inserting item in an empty list")
    sahtest_add_test(llist_insert_before_using_orphan_reference,"Test inserting using a reference not in the list")
    sahtest_add_test(llist_insert_before_move_iterator_in_list,"Test inserting item which is in the list")
sahtest_group_end()

sahtest_group_begin(insert_after,llist_insert_build_list_setup,llist_insert_build_list_teardown)
    sahtest_add_test(llist_insert_after_null_list,"Test inserting in a null list")
    sahtest_add_test(llist_insert_after_null_reference,"Test inserting after null reference")
    sahtest_add_test(llist_insert_after_null_it,"Test inserting null it")
    sahtest_add_test(llist_insert_after_first,"Test inserting after first item")
    sahtest_add_test(llist_insert_after_last,"Test inserting after last item")
    sahtest_add_test(llist_insert_after_second,"Test inserting after second item")
    sahtest_add_test(llist_insert_after_reference_is_insert_it,"Test inserting item which is used as reference")
    sahtest_add_test(llist_insert_after_in_empty_list,"Test inserting item in an empty list")
    sahtest_add_test(llist_insert_after_using_orphan_reference,"Test inserting using a reference not in the list")
    sahtest_add_test(llist_insert_after_move_iterator_in_list,"Test inserting item which is in the list")
sahtest_group_end()

sahtest_group_begin(insert_indexed,llist_insert_build_list_setup,llist_insert_build_list_teardown)
    sahtest_add_test(llist_insert_indexed_null_list,"Test inserting in a null list using index")
    sahtest_add_test(llist_insert_indexed_empty_list_index_0,"Test inserting in a empty list with index 0")
    sahtest_add_test(llist_insert_indexed_empty_list_index_2,"Test inserting in a empty list with index 2")
    sahtest_add_test(llist_insert_indexed_list_index_out_of_boundry,"Test inserting in a list with index higher then size of list")
    sahtest_add_test(llist_insert_indexed_list_index_0,"Test inserting in a list with index 0")
    sahtest_add_test(llist_insert_indexed_list_index_2,"Test inserting with index set to 2")
    sahtest_add_test(llist_insert_indexed_list_index_last,"Test inserting with index set to last item")
sahtest_group_end()

sahtest_group_begin(inserting,NULL,NULL)
    sahtest_add_group(ll_append,"Test appending items")
    sahtest_add_group(ll_prepend,"Test prepending items")
    sahtest_add_group(insert_before,"Test inserting items before a reference item")
    sahtest_add_group(insert_after,"Test inserting items after a reference item")
    sahtest_add_group(insert_indexed,"Test inserting items using index")
sahtest_group_end()

#endif // LLIST_INSERT_H
