/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/linked_list.h>

static llist_t list;
static llist_iterator_t it[5];
static llist_iterator_t orphan;

void llist_iterating_setup() {
    int i;
    llist_initialize(&list);
    for(i = 0; i <5; i++) {
        llist_iterator_initialize(&it[i]);
        llist_append(&list,&it[i]);
    }
    llist_iterator_initialize(&orphan);
}

void llist_iterating_teardown() {
    llist_cleanup(&list);
}

int llist_iterating_first_null() {
    T_VERIFY(llist_first(NULL) == NULL);

    return 0;
}

int llist_iterating_last_null() {
    T_VERIFY(llist_last(NULL) == NULL);

    return 0;
}

int llist_iterating_next_null() {
    T_VERIFY(llist_iterator_next(NULL) == NULL);

    return 0;
}

int llist_iterating_prev_null() {
    T_VERIFY(llist_iterator_prev(NULL) == NULL);

    return 0;
}

int llist_iterating_first() {
    T_VERIFY(llist_first(&list) == &it[0]);

    return 0;
}

int llist_iterating_last() {
    T_VERIFY(llist_last(&list) == &it[4]);

    return 0;
}

int llist_iterating_first_prev() {
    llist_iterator_t *iter = llist_first(&list);
    T_VERIFY(llist_iterator_prev(iter) == NULL);

    return 0;
}

int llist_iterating_last_next() {
    llist_iterator_t *iter = llist_last(&list);
    T_VERIFY(llist_iterator_next(iter) == NULL);

    return 0;
}

int llist_iterating_first_next() {
    llist_iterator_t *iter = llist_first(&list);
    T_VERIFY(llist_iterator_next(iter) == &it[1]);

    return 0;
}

int llist_iterating_last_prev() {
    llist_iterator_t *iter = llist_last(&list);
    T_VERIFY(llist_iterator_prev(iter) == &it[3]);

    return 0;
}

int llist_iterating_orphan_next() {
    T_VERIFY(llist_iterator_next(&orphan) == NULL);

    return 0;
}

int llist_iterating_orphan_prev() {
    T_VERIFY(llist_iterator_prev(&orphan) == NULL);

    return 0;
}

int llist_iterating_loop_forward() {
    llist_iterator_t *iter = NULL;
    int i = 0;
    llist_for_each(iter, &list) {
        T_VERIFY(i < 5);
        T_VERIFY(iter == &it[i]);
        i++;
    }

    return 0;
}

int llist_iterating_loop_reverse() {
    llist_iterator_t *iter = NULL;
    int i = 4;
    llist_for_each_reverse(iter, &list) {
        T_VERIFY(i >= 0);
        T_VERIFY(iter == &it[i]);
        i--;
    }

    return 0;
}
