/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/linked_list.h>

static llist_t list;
static llist_iterator_t it[5];
static llist_iterator_t orphan;

void llist_remove_setup() {
    int i;
    llist_initialize(&list);
    for(i = 0; i <5; i++) {
        llist_iterator_initialize(&it[i]);
        llist_append(&list,&it[i]);
    }
    llist_iterator_initialize(&orphan);
}

void llist_remove_teardown() {
    llist_cleanup(&list);
}

int llist_remove_take_first_null() {
    T_VERIFY(llist_takeFirst(NULL) == NULL);
    return 0;
}

int llist_remove_take_last_null() {
    T_VERIFY(llist_takeLast(NULL) == NULL);
    return 0;
}

int llist_remove_take_first_empty_list() {
    llist_cleanup(&list);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_takeFirst(&list) == NULL);

    return 0;
}

int llist_remove_take_last_empty_list() {
    llist_cleanup(&list);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_takeLast(&list) == NULL);
    return 0;
}

int llist_remove_take_first() {
    llist_iterator_t *taken = llist_takeFirst(&list);
    T_VERIFY(taken == &it[0]);
    T_VERIFY(llist_iterator_next(taken) == NULL);
    T_VERIFY(llist_iterator_prev(taken) == NULL);
    T_VERIFY(llist_first(&list) == &it[1]);

    return 0;
}

int llist_remove_take_last() {
    llist_iterator_t *taken = llist_takeLast(&list);
    T_VERIFY(taken == &it[4]);
    T_VERIFY(llist_iterator_next(taken) == NULL);
    T_VERIFY(llist_iterator_prev(taken) == NULL);
    T_VERIFY(llist_last(&list) == &it[3]);

    return 0;
}
