/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdarg.h>

#include "mock.h"

mock_struct_declare(getenv,50);
mock_struct_declare(fcntl,50);

typedef char *(*getenv_func_t)(const char *name);
typedef int (*fcntl_func_t)(int fd, int cmd, ... /* arg */ );

char *getenv(const char *name) {
    if (sahtest_isRunning()) {
        if (strcmp(mock_get_expectation(getenv)->varname,name) != 0) {
            return NULL;
        }
        const char *value = mock_get_expectation(getenv)->value;
        //return value?strdup(value):NULL;
        return value;
    }

    getenv_func_t real_getenv = mock_real_function("libc.so.6","getenv");
    if (!real_getenv) {
        return NULL;
    }

    return real_getenv(name);
}

int fcntl(int fd, int cmd, ... /* arg */ ) {
    if (sahtest_isRunning()) {
        assert_bool(mock_get_expectation(fcntl)->fd == fd);
        assert_bool(mock_get_expectation(fcntl)->cmd == cmd);
        int retval = mock_get_expectation(fcntl)->retval;
        if (retval == -1) {
            errno = mock_get_expectation(fcntl)->error;
        }
        return retval;
    }

    fcntl_func_t real_fcntl = mock_real_function("libc.so.6","fcntl");
    if (!real_fcntl) {
        return -1;
    }


    va_list arguments;
    va_start ( arguments, cmd );
    int retval = real_fcntl(fd,cmd,va_arg ( arguments, int ));
    va_end ( arguments );

    return retval;
}
