/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#define _XOPEN_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <pcb/utils/string.h>
#include <sahtest/sahtest.h>

static string_t string1;
static string_t string2;

void test_string_append_setup() {
    string_initialize(&string1,6);
    string_initialize(&string2,6);
}

void test_string_append_teardown() {
    string_cleanup(&string1);
    string_cleanup(&string2);
}

int test_append_to_null_string() {
    string_t temp;
    string_initialize(&temp,0);
    T_VERIFY(string_appendChar(&string1,"Hallo") == 5);
    T_VERIFY(string_appendChar(NULL,"Hallo") == 0);
    string_cleanup(&temp);
    return 0;
}

int test_append_null_to_string() {
    string_fromChar(&string1,"Hallo");
    T_VERIFY(string_appendChar(&string1,NULL) == 5);
    T_VERIFY (strcmp(string_buffer(&string1),"Hallo") == 0);

    return 0;
}

int test_append_empty_const_char_to_string() {
    string_fromChar(&string1,"Hallo");
    T_VERIFY(string_appendChar(&string1,"") == 5);
    T_VERIFY (strcmp(string_buffer(&string1),"Hallo") == 0);

    return 0;
}

int test_empty_string_append_char() {
    T_VERIFY(string_appendChar(&string1,"Hallo") == 5);
    T_VERIFY (strcmp(string_buffer(&string1),"Hallo") == 0);

    return 0;
}

int test_string_append_char() {
    string_fromChar(&string1,"Hallo");
    T_VERIFY(string_appendChar(&string1," world") == 5 + 6);
    T_VERIFY (strcmp(string_buffer(&string1),"Hallo world") == 0);

    return 0;
}

int test_string_append_string() {
    string_fromChar(&string1,"Hallo");
    string_fromChar(&string2," world");

    T_VERIFY(string_append(&string1,&string2) == 5 + 6);
    T_VERIFY (strcmp(string_buffer(&string1),"Hallo world") == 0);

    return 0;
}

int test_string_append_string_null() {
    string_fromChar(&string1,"Hallo");

    T_VERIFY(string_prepend(&string1,NULL) == 5);
    T_VERIFY (strcmp(string_buffer(&string1),"Hallo") == 0);

    return 0;
}

int test_string_append_format() {
    string_fromChar(&string1,"Hallo");
    T_VERIFY(string_appendFormat(&string1," %s", "World") == 11);
    T_VERIFY(strcmp(string_buffer(&string1),"Hallo World") == 0);

    return 0;
}

int test_string_append_format_using_null_format() {
    string_fromChar(&string1,"Hallo");
    T_VERIFY(string_appendFormat(&string1, NULL, " World") == 5);
    T_VERIFY(strcmp(string_buffer(&string1),"Hallo") == 0);

    return 0;
}

int test_string_append_format_using_empty_dest() {
    string_t empty;
    string_initialize(&empty,0);
    T_VERIFY(string_appendFormat(&empty, "%s", "Hallo") == 5);
    T_VERIFY(strcmp(string_buffer(&empty),"Hallo") == 0);
    string_cleanup(&empty);

    return 0;
}

int test_string_append_format_using_null_dest() {
    T_VERIFY(string_appendFormat(NULL, "%s", "Hallo") == 0);

    return 0;
}

int test_string_append_format_using_empty_dest_initialized_buffer() {
    T_VERIFY(string_appendFormat(&string1, "%s", "Hallo") == 5);
    printf("%s\n",string_buffer(&string1));
    T_VERIFY(strcmp(string_buffer(&string1),"Hallo") == 0);

    return 0;
}
