/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#define _XOPEN_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <pcb/utils/string.h>
#include <sahtest/sahtest.h>

static string_t haystack;
static string_t needle;

void test_string_find_setup() {
    //                         1234567890123456789012345678901234567890123456
    string_fromChar(&haystack,"HaystackHaystackHaystackNeedleHaystackHaystack");
    string_fromChar(&needle,"Needle");
}

void test_string_find_teardown() {
    string_cleanup(&haystack);
    string_cleanup(&needle);
}

int test_find_casesensitive() {
    T_VERIFY(string_find(&haystack,0,&needle,string_case_sensitive) == 24);
    T_VERIFY(string_find(&haystack,24,&needle,string_case_sensitive) == 24);
    T_VERIFY(string_find(&haystack,26,&needle,string_case_sensitive) == UINT32_MAX);

    return 0;
}

int test_find_caseinsensitive() {
    string_fromChar(&needle,"NEEDLE");

    T_VERIFY(string_find(&haystack,0,&needle,string_case_insensitive) == 24);
    T_VERIFY(string_find(&haystack,24,&needle,string_case_insensitive) == 24);
    T_VERIFY(string_find(&haystack,26,&needle,string_case_insensitive) == UINT32_MAX);

    return 0;
}

int test_find_start_out_of_boundery() {
    T_VERIFY(string_find(&haystack,100,&needle,string_case_sensitive) == UINT32_MAX);

    return 0;
}

int test_find_not_available() {
    string_fromChar(&needle,"Gold");
    T_VERIFY(string_find(&haystack,0,&needle,string_case_sensitive) == UINT32_MAX);

    return 0;
}

int test_find_needle_null() {
    T_VERIFY(string_find(&haystack,0,NULL,string_case_sensitive) == 0);

    return 0;
}

int test_find_haystack_null_string() {
    T_VERIFY(string_find(NULL,0,&needle,string_case_sensitive) == UINT32_MAX);

    return 0;
}

int test_find_haystack_empty_string_initialized_size() {
    string_t empty;
    string_initialize(&empty,10);
    T_VERIFY(string_find(&empty,0,&needle,string_case_sensitive) == UINT32_MAX);
    string_cleanup(&empty);

    return 0;
}

int test_find_haystack_empty_string_initialized_0() {
    string_t empty;
    string_initialize(&empty,0);
    T_VERIFY(string_find(&empty,0,&needle,string_case_sensitive) == UINT32_MAX);
    string_cleanup(&empty);

    return 0;
}

int test_find_needle_empty_string_initialized_size() {
    string_t empty;
    string_initialize(&empty,10);
    T_VERIFY(string_find(&haystack,0,&empty,string_case_sensitive) == 0);
    string_cleanup(&empty);

    return 0;
}

int test_find_needle_empty_string_initialized_0() {
    string_t empty;
    string_initialize(&empty,0);
    T_VERIFY(string_find(&haystack,0,&empty,string_case_sensitive) == 0);
    string_cleanup(&empty);

    return 0;
}

int test_find_char_needle_null() {
    T_VERIFY(string_findChar(&haystack,0,NULL,string_case_sensitive) == 0);

    return 0;
}
