/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#define _XOPEN_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <pcb/utils/string.h>
#include <sahtest/sahtest.h>

static string_t string1;
static string_t string2;

void test_string_from_type_setup() {
    string_initialize(&string1,0);
    string_initialize(&string2,0);
}

void test_string_from_type_teardown() {
    string_cleanup(&string1);
    string_cleanup(&string2);
}

int test_string_from_bool() {
    T_VERIFY(string_fromBool(&string1,true) == 1);
    T_VERIFY(strcmp(string1.buffer,"1") == 0);

    T_VERIFY(string_fromBool(&string1,false) == 1);
    T_VERIFY(strcmp(string1.buffer,"0") == 0);

    return 0;
}

int test_NULL_string_from_bool() {
    T_VERIFY(string_fromBool(NULL,true) == 0);

    return 0;
}

int test_string_from_char() {
    T_VERIFY(string_fromChar(&string1,"Hallo") == 5);
    T_VERIFY(strcmp(string1.buffer,"Hallo") == 0);
    T_VERIFY(string1.bufferSize == 6);

    T_VERIFY(string_fromChar(&string1,"Hallo world") == 11);
    T_VERIFY(strcmp(string1.buffer,"Hallo world") == 0);
    T_VERIFY(string1.bufferSize == 12);

    return 0;
}

int test_NULL_string_from_char() {
    T_VERIFY(string_fromChar(NULL,"Hallo") == 0);

    return 0;
}

int test_string_from_NULL_char() {
    T_VERIFY(string_fromChar(&string1,NULL) == 0);
    T_VERIFY(string1.buffer == NULL);

    T_VERIFY(string_fromChar(&string1,"Hallo") == 5);
    T_VERIFY(string_fromChar(&string1,NULL) == 0);
    T_VERIFY(strcmp(string1.buffer,"") == 0);

    return 0;
}

int test_string_from_int8() {
    T_VERIFY(string_fromInt8(&string1,127) == 3);
    T_VERIFY(strcmp(string1.buffer,"127") == 0);

    T_VERIFY(string_fromInt8(&string1,-1) == 2);
    T_VERIFY(strcmp(string1.buffer,"-1") == 0);

    T_VERIFY(string_fromInt8(&string1,-127) == 4);
    T_VERIFY(strcmp(string1.buffer,"-127") == 0);

    return 0;
}

int test_NULL_string_from_int8() {
    T_VERIFY(string_fromInt8(NULL,100) == 0);

    return 0;
}

int test_string_from_int16() {
    T_VERIFY(string_fromInt16(&string1,32767) == 5);
    T_VERIFY(strcmp(string1.buffer,"32767") == 0);

    T_VERIFY(string_fromInt16(&string1,-1) == 2);
    T_VERIFY(strcmp(string1.buffer,"-1") == 0);

    T_VERIFY(string_fromInt16(&string1,-32767) == 6);
    T_VERIFY(strcmp(string1.buffer,"-32767") == 0);

    return 0;
}

int test_NULL_string_from_int16() {
    T_VERIFY(string_fromInt16(NULL,100) == 0);

    return 0;
}

int test_string_from_int32() {
    T_VERIFY(string_fromInt32(&string1,2147483647) == 10);
    T_VERIFY(strcmp(string1.buffer,"2147483647") == 0);

    T_VERIFY(string_fromInt32(&string1,-1) == 2);
    T_VERIFY(strcmp(string1.buffer,"-1") == 0);

    T_VERIFY(string_fromInt32(&string1,-2147483647) == 11);
    T_VERIFY(strcmp(string1.buffer,"-2147483647") == 0);

    return 0;
}

int test_NULL_string_from_int32() {
    T_VERIFY(string_fromInt32(NULL,100) == 0);

    return 0;
}

int test_string_from_int64() {
    T_VERIFY(string_fromInt64(&string1,214748364700) == 12);
    T_VERIFY(strcmp(string1.buffer,"214748364700") == 0);

    T_VERIFY(string_fromInt64(&string1,-1) == 2);
    T_VERIFY(strcmp(string1.buffer,"-1") == 0);

    T_VERIFY(string_fromInt64(&string1,-214748364700) == 13);
    T_VERIFY(strcmp(string1.buffer,"-214748364700") == 0);

    return 0;
}

int test_NULL_string_from_int64() {
    T_VERIFY(string_fromInt64(NULL,100) == 0);

    return 0;
}

int test_string_from_uint8() {
    T_VERIFY(string_fromUInt8(&string1,25) == 2);
    T_VERIFY(strcmp(string1.buffer,"25") == 0);

    T_VERIFY(string_fromUInt8(&string1,255) == 3);
    T_VERIFY(strcmp(string1.buffer,"255") == 0);

    return 0;
}

int test_NULL_string_from_uint8() {
    T_VERIFY(string_fromUInt8(NULL,100) == 0);

    return 0;
}

int test_string_from_uint16() {
    T_VERIFY(string_fromUInt16(&string1,3276) == 4);
    T_VERIFY(strcmp(string1.buffer,"3276") == 0);

    T_VERIFY(string_fromUInt16(&string1,65534) == 5);
    T_VERIFY(strcmp(string1.buffer,"65534") == 0);

    return 0;
}

int test_NULL_string_from_uint16() {
    T_VERIFY(string_fromUInt16(NULL,100) == 0);

    return 0;
}

int test_string_from_uint32() {
    T_VERIFY(string_fromUInt32(&string1,14748364) == 8);
    T_VERIFY(strcmp(string1.buffer,"14748364") == 0);

    T_VERIFY(string_fromUInt32(&string1,429496729) == 9);
    T_VERIFY(strcmp(string1.buffer,"429496729") == 0);

    return 0;
}

int test_NULL_string_from_uint32() {
    T_VERIFY(string_fromUInt32(NULL,100) == 0);

    return 0;
}

int test_string_from_uint64() {
    T_VERIFY(string_fromUInt64(&string1,21474836470) == 11);
    T_VERIFY(strcmp(string1.buffer,"21474836470") == 0);

    T_VERIFY(string_fromUInt64(&string1,214748364700) == 12);
    T_VERIFY(strcmp(string1.buffer,"214748364700") == 0);

    return 0;
}

int test_NULL_string_from_uint64() {
    T_VERIFY(string_fromUInt64(NULL,100) == 0);

    return 0;
}

int test_string_from_double() {
    T_VERIFY(string_fromDouble(&string1,100.5) == 12);
    T_VERIFY(strcmp(string1.buffer,"1.005000e+02") == 0);

    T_VERIFY(string_fromDouble(&string1,-200.25) == 13);
    T_VERIFY(strcmp(string1.buffer,"-2.002500e+02") == 0);

    return 0;
}

int test_NULL_string_from_double() {
    T_VERIFY(string_fromDouble(NULL,100.100) == 0);

    return 0;
}


int test_string_from_time() {
    struct tm datetime; // = (struct tm *)malloc(sizeof(struct tm));

    strptime("2001-11-12 18:31:01Z", "%Y-%m-%d %H:%M:%SZ", &datetime);

    T_VERIFY(string_fromTime(&string1,&datetime,"%Y-%m-%d")==10);
    T_VERIFY(strcmp(string1.buffer,"2001-11-12") == 0);

    T_VERIFY(string_fromTime(&string1,&datetime,NULL)==20);
    T_VERIFY(strcmp(string1.buffer,"2001-11-12T18:31:01Z") == 0);

    return 0;
}

/*
int test_NULL_string_from_time() {
    T_VERIFY(string_fromTime(NULL,time,NULL) == 0);

    return 0;
}
*/
