/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#define _XOPEN_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <pcb/utils/string.h>
#include <sahtest/sahtest.h>

static string_t original;
static string_t search;
static string_t replacement;

void test_string_replace_setup() {
    string_fromChar(&original,"7 horses running through the sky, when the sky is blue");
    string_fromChar(&search,"sky");
    string_initialize(&replacement,0);
}

void test_string_replace_teardown() {
    string_cleanup(&original);
    string_cleanup(&replacement);
    string_cleanup(&search);
}

int test_replace_smaller() {
    string_fromChar(&replacement,"a");
    string_replace(&original,&search,&replacement);

    T_VERIFY(strcmp(string_buffer(&original),"7 horses running through the a, when the a is blue") == 0);

    return 0;
}

int test_replace_bigger() {
    string_fromChar(&replacement,"daisy field");
    string_replace(&original,&search,&replacement);

    T_VERIFY(strcmp(string_buffer(&original),"7 horses running through the daisy field, when the daisy field is blue") == 0);

    return 0;
}

int test_replace_not_found() {
    string_fromChar(&replacement,"daisy field");
    string_fromChar(&search,"heaven");
    string_replace(&original,&search,&replacement);

    T_VERIFY(strcmp(string_buffer(&original),"7 horses running through the sky, when the sky is blue") == 0);

    return 0;
}

int test_replace_using_null_replacement() {
    string_replace(&original,&search,NULL);

    T_VERIFY(strcmp(string_buffer(&original),"7 horses running through the sky, when the sky is blue") == 0);

    return 0;
}

int test_replace_using_null_search() {
    string_replace(&original,NULL,&replacement);

    T_VERIFY(strcmp(string_buffer(&original),"7 horses running through the sky, when the sky is blue") == 0);

    return 0;
}

int test_replace_using_empty_replacement_size_0() {
    string_replace(&original,&search,&replacement);

    T_VERIFY(strcmp(string_buffer(&original),"7 horses running through the sky, when the sky is blue") == 0);

    return 0;
}

int test_replace_using_empty_original_size_0() {
    string_cleanup(&original);
    string_replace(&original,&search,&replacement);

    T_VERIFY(string_buffer(&original) == NULL);

    return 0;
}
