/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include <pcb/utils/string.h>
#include <sahtest/sahtest.h>

#include "mock.h"

static string_t string1;
static string_t string2;

void test_string_resolve_env_setup() {
    mock_reset(getenv);
    string_fromChar(&string1,"Hallo $(PERSON), you are at $(LOCATION)");
    string_fromChar(&string2,"Hallo $PERSON), you are at $(LOCATION");
}

void test_string_resolve_env_teardown() {
    string_cleanup(&string1);
    string_cleanup(&string2);
}

int test_resolve_env() {
    mock_add_expectation(getenv)->varname = "PERSON";
    mock_add_expectation(getenv)->value = "developer";
    mock_add_expectation(getenv)->varname = "LOCATION";
    mock_add_expectation(getenv)->value = "office";

    T_VERIFY(string_resolveEnvVar(&string1) == true);
    T_VERIFY(mock_verify_expectations(getenv) == true);
    T_VERIFY(strcmp(string_buffer(&string1),"Hallo developer, you are at office") == 0);

    return 0;
}

int test_resolve_env_not_available() {
    mock_add_expectation(getenv)->varname = "PERSON";
    mock_add_expectation(getenv)->value = NULL;
    mock_add_expectation(getenv)->varname = "LOCATION";
    mock_add_expectation(getenv)->value = NULL;

    T_VERIFY(string_resolveEnvVar(&string1) == true);
    T_VERIFY(mock_verify_expectations(getenv) == true);
    T_VERIFY(strcmp(string_buffer(&string1),"Hallo , you are at ") == 0);

    return 0;
}

int test_resolve_env_invalid_format() {
    T_VERIFY(string_resolveEnvVar(&string2) == false);
    T_VERIFY(strcmp(string_buffer(&string2),"Hallo $PERSON), you are at $(LOCATION") == 0);

    return 0;
}

int test_resolve_env_char() {
    mock_add_expectation(getenv)->varname = "PERSON";
    mock_add_expectation(getenv)->value = "developer";
    mock_add_expectation(getenv)->varname = "LOCATION";
    mock_add_expectation(getenv)->value = "office";

    char *txt = string_resolveEnvVarChar("Hallo $(PERSON), you are at $(LOCATION)");
    T_VERIFY(txt != NULL);
    T_VERIFY(mock_verify_expectations(getenv) == true);
    T_VERIFY(strcmp(txt,"Hallo developer, you are at office") == 0);
    free(txt);

    return 0;
}

int test_resolve_env_char_not_available() {
    mock_add_expectation(getenv)->varname = "PERSON";
    mock_add_expectation(getenv)->value = NULL;
    mock_add_expectation(getenv)->varname = "LOCATION";
    mock_add_expectation(getenv)->value = NULL;

    char *txt = string_resolveEnvVarChar("Hallo $(PERSON), you are at $(LOCATION)");
    T_VERIFY(txt != NULL);
    T_VERIFY(mock_verify_expectations(getenv) == true);
    T_VERIFY(strcmp(txt,"Hallo , you are at ") == 0);
    free(txt);

    return 0;
}

int test_resolve_env_char_invalid_format() {
    char *txt = string_resolveEnvVarChar("Hallo PERSON), you are at $(LOCATION");
    T_VERIFY(txt == NULL);

    return 0;
}
