/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#define _XOPEN_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <pcb/utils/string.h>
#include <sahtest/sahtest.h>

static string_t string_true;
static string_t string_false;
static string_t string_yes;
static string_t string_int;
static string_t string_neg_int;
static string_t string_double;
static string_t string_date_time;
static string_t string_date_time_milliseconds;
static string_t string_text;

void test_string_to_type_setup() {
    string_initialize(&string_true,0);
    string_initialize(&string_false,0);
    string_initialize(&string_yes,0);
    string_initialize(&string_int,0);
    string_initialize(&string_neg_int,0);
    string_initialize(&string_double,0);
    string_initialize(&string_date_time,0);
    string_initialize(&string_date_time_milliseconds,0);
    string_initialize(&string_text,0);

    string_fromChar(&string_true,"true");
    string_fromChar(&string_false,"false");
    string_fromChar(&string_yes,"yes");
    string_fromChar(&string_int,"127");
    string_fromChar(&string_neg_int,"-127");
    string_fromChar(&string_double,"-2.002500e+02");
    string_fromChar(&string_date_time,"2001-11-12T18:31:01Z");
    string_fromChar(&string_date_time_milliseconds,"2001-11-12T18:31:01.123Z");
    string_fromChar(&string_text,"Hallo world");
}

void test_string_to_type_teardown() {
    string_cleanup(&string_true);
    string_cleanup(&string_false);
    string_cleanup(&string_yes);
    string_cleanup(&string_int);
    string_cleanup(&string_neg_int);
    string_cleanup(&string_double);
    string_cleanup(&string_date_time);
    string_cleanup(&string_date_time_milliseconds);
    string_cleanup(&string_text);
}

int test_string_to_bool() {
    T_VERIFY(string_toBool(&string_true) == true);
    T_VERIFY(string_toBool(&string_false) == false);
    T_VERIFY(string_toBool(&string_yes) == true);
    T_VERIFY(string_toBool(&string_int) == false);
    T_VERIFY(string_toBool(&string_neg_int) == false);
    T_VERIFY(string_toBool(&string_double) == false);
    T_VERIFY(string_toBool(&string_date_time) == false);
    T_VERIFY(string_toBool(&string_date_time_milliseconds) == false);
    T_VERIFY(string_toBool(&string_text) == false);

    return 0;
}

int test_NULL_string_to_bool() {
    T_VERIFY(string_toBool(NULL) == false);

    return 0;
}

int test_string_to_char() {
    char *txt = NULL;

    txt = string_toChar(&string_true);
    T_CMP(txt,"true");
    free(txt);

    txt = string_toChar(&string_false);
    T_CMP(txt,"false");
    free(txt);

    txt = string_toChar(&string_yes);
    T_CMP(txt,"yes");
    free(txt);

    txt = string_toChar(&string_int);
    T_CMP(txt,"127");
    free(txt);

    txt = string_toChar(&string_neg_int);
    T_CMP(txt,"-127");
    free(txt);

    txt = string_toChar(&string_double);
    T_CMP(txt,"-2.002500e+02");
    free(txt);

    txt = string_toChar(&string_date_time);
    T_CMP(txt,"2001-11-12T18:31:01Z");
    free(txt);

    txt = string_toChar(&string_date_time_milliseconds);
    T_CMP(txt,"2001-11-12T18:31:01.123Z");
    free(txt);

    txt = string_toChar(&string_text);
    T_CMP(txt,"Hallo world");
    free(txt);

    return 0;
}

int test_NULL_string_to_char() {
    T_VERIFY(string_toChar(NULL) == NULL);

    return 0;
}

int test_string_to_int8() {
    T_VERIFY(string_toInt8(&string_true) == 0);
    T_VERIFY(string_toInt8(&string_false) == 0);
    T_VERIFY(string_toInt8(&string_yes) == 0);
    T_VERIFY(string_toInt8(&string_int) == 127);
    T_VERIFY(string_toInt8(&string_neg_int) == -127);
    T_VERIFY(string_toInt8(&string_double) == 0);
    T_VERIFY(string_toInt8(&string_date_time) == 0);
    T_VERIFY(string_toInt8(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toInt8(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_int8() {
    T_VERIFY(string_toInt8(NULL) == 0);

    return 0;
}

int test_string_to_int16() {
    T_VERIFY(string_toInt16(&string_true) == 0);
    T_VERIFY(string_toInt16(&string_false) == 0);
    T_VERIFY(string_toInt16(&string_yes) == 0);
    T_VERIFY(string_toInt16(&string_int) == 127);
    T_VERIFY(string_toInt16(&string_neg_int) == -127);
    T_VERIFY(string_toInt16(&string_double) == 0);
    T_VERIFY(string_toInt16(&string_date_time) == 0);
    T_VERIFY(string_toInt16(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toInt16(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_int16() {
    T_VERIFY(string_toInt16(NULL) == 0);

    return 0;
}

int test_string_to_int32() {
    T_VERIFY(string_toInt32(&string_true) == 0);
    T_VERIFY(string_toInt32(&string_false) == 0);
    T_VERIFY(string_toInt32(&string_yes) == 0);
    T_VERIFY(string_toInt32(&string_int) == 127);
    T_VERIFY(string_toInt32(&string_neg_int) == -127);
    T_VERIFY(string_toInt32(&string_double) == 0);
    T_VERIFY(string_toInt32(&string_date_time) == 0);
    T_VERIFY(string_toInt32(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toInt32(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_int32() {
    T_VERIFY(string_toInt32(NULL) == 0);

    return 0;
}

int test_string_to_int64() {
    T_VERIFY(string_toInt64(&string_true) == 0);
    T_VERIFY(string_toInt64(&string_false) == 0);
    T_VERIFY(string_toInt64(&string_yes) == 0);
    T_VERIFY(string_toInt64(&string_int) == 127);
    T_VERIFY(string_toInt64(&string_neg_int) == -127);
    T_VERIFY(string_toInt64(&string_double) == 0);
    T_VERIFY(string_toInt64(&string_date_time) == 0);
    T_VERIFY(string_toInt64(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toInt64(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_int64() {
    T_VERIFY(string_toInt64(NULL) == 0);

    return 0;
}

int test_string_to_uint8() {
    T_VERIFY(string_toUInt8(&string_true) == 0);
    T_VERIFY(string_toUInt8(&string_false) == 0);
    T_VERIFY(string_toUInt8(&string_yes) == 0);
    T_VERIFY(string_toUInt8(&string_int) == 127);
    T_VERIFY(string_toUInt8(&string_neg_int) == 0);
    T_VERIFY(string_toUInt8(&string_double) == 0);
    T_VERIFY(string_toUInt8(&string_date_time) == 0);
    T_VERIFY(string_toUInt8(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toUInt8(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_uint8() {
    T_VERIFY(string_toUInt8(NULL) == 0);

    return 0;
}

int test_string_to_uint16() {
    T_VERIFY(string_toUInt16(&string_true) == 0);
    T_VERIFY(string_toUInt16(&string_false) == 0);
    T_VERIFY(string_toUInt16(&string_yes) == 0);
    T_VERIFY(string_toUInt16(&string_int) == 127);
    T_VERIFY(string_toUInt16(&string_neg_int) == 0);
    T_VERIFY(string_toUInt16(&string_double) == 0);
    T_VERIFY(string_toUInt16(&string_date_time) == 0);
    T_VERIFY(string_toUInt16(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toUInt16(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_uint16() {
    T_VERIFY(string_toUInt16(NULL) == 0);

    return 0;
}

int test_string_to_uint32() {
    T_VERIFY(string_toUInt32(&string_true) == 0);
    T_VERIFY(string_toUInt32(&string_false) == 0);
    T_VERIFY(string_toUInt32(&string_yes) == 0);
    T_VERIFY(string_toUInt32(&string_int) == 127);
    T_VERIFY(string_toUInt32(&string_date_time) == 0);
    T_VERIFY(string_toUInt32(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toUInt32(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_uint32() {
    T_VERIFY(string_toUInt32(NULL) == 0);

    return 0;
}

int test_string_to_uint64() {
    T_VERIFY(string_toUInt64(&string_true) == 0);
    T_VERIFY(string_toUInt64(&string_false) == 0);
    T_VERIFY(string_toUInt64(&string_yes) == 0);
    T_VERIFY(string_toUInt64(&string_int) == 127);
    T_VERIFY(string_toUInt64(&string_date_time) == 0);
    T_VERIFY(string_toUInt64(&string_date_time_milliseconds) == 0);
    T_VERIFY(string_toUInt64(&string_text) == 0);

    return 0;
}

int test_NULL_string_to_uint64() {
    T_VERIFY(string_toUInt64(NULL) == 0);

    return 0;
}

int test_string_to_time() {
    struct tm *time = string_toTime(&string_date_time,NULL);
    T_VERIFY(time != NULL);
    free(time);

    time = string_toTime(&string_date_time_milliseconds,NULL);
    T_VERIFY(time != NULL);
    free(time);

    return 0;
}

int test_NULL_string_to_time() {
    T_VERIFY(string_toTime(NULL,NULL) == NULL);

    return 0;
}
