/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/string_list.h>

static string_list_t list;
static string_list_iterator_t *it[5];
static const char *parts[] = { "Hello", "World", "!","Foo", "Bar" };

void string_list_contains_setup() {
    int i;
    string_list_initialize(&list);
    for(i = 0; i <5; i++) {
        it[i] = string_list_iterator_createFromChar(parts[i]);
        string_list_append(&list,it[i]);
    }
}

void string_list_contains_teardown() {
    string_list_cleanup(&list);
}

int string_list_contains_search_valid_item() {
    string_t text;
    string_initialize(&text,0);
    string_fromChar(&text,"World");

    T_VERIFY(string_list_contains(&list,&text) == true);

    string_cleanup(&text);
    return 0;
}

int string_list_contains_search_invalid_item() {
    string_t text;
    string_initialize(&text,0);
    string_fromChar(&text,"Test");

    T_VERIFY(string_list_contains(&list,&text) == false);

    string_cleanup(&text);
    return 0;
}

int string_list_contains_null_string() {
    T_VERIFY(string_list_contains(&list,NULL) == false);

    return 0;
}

int string_list_contains_null_char() {
    T_VERIFY(string_list_containsChar(&list,NULL) == false);

    return 0;
}

int string_list_contains_using_null_list() {
    string_t text;
    string_initialize(&text,0);
    string_fromChar(&text,"Test");

    T_VERIFY(string_list_contains(NULL,&text) == false);
    T_VERIFY(string_list_containsChar(NULL,"Test") == false);

    string_cleanup(&text);
    return 0;
}

