/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/string_list.h>

int string_list_init_cleanup() {
    string_list_t list;
    T_VERIFY(string_list_initialize(&list));
    T_VERIFY(!string_list_first(&list));
    T_VERIFY(!string_list_last(&list));
    string_list_cleanup(&list);

    return 0;
}

int string_list_init_null() {
    T_VERIFY(!string_list_initialize(NULL));

    return 0;
}

int string_list_cleanup_null() {
    string_list_cleanup(NULL);

    return 0;
}

int string_list_create_iterator() {
    string_t txt;
    string_initialize(&txt,0);
    string_fromChar(&txt,"Hello World");

    string_list_iterator_t *it = NULL;
    it = string_list_iterator_create(&txt);
    T_VERIFY(it != NULL);
    T_VERIFY(!string_list_iterator_prev(it));
    T_VERIFY(!string_list_iterator_next(it));

    string_list_iterator_destroy(it);
    string_cleanup(&txt);
    return 0;
}

int string_list_create_iterator_null() {
    string_list_iterator_t *it = NULL;
    it = string_list_iterator_create(NULL);

    T_VERIFY(it != NULL);
    T_VERIFY(!string_list_iterator_prev(it));
    T_VERIFY(!string_list_iterator_next(it));

    string_list_iterator_destroy(it);
    return 0;
}

int string_list_create_iterator_char() {
    string_list_iterator_t *it = NULL;
    it = string_list_iterator_createFromChar("Hello World");
    T_VERIFY(it != NULL);
    T_VERIFY(!string_list_iterator_prev(it));
    T_VERIFY(!string_list_iterator_next(it));

    string_list_iterator_destroy(it);
    return 0;
}

int string_list_create_iterator_char_null() {
    string_list_iterator_t *it = NULL;
    it = string_list_iterator_createFromChar(NULL);

    T_VERIFY(it != NULL);
    T_VERIFY(!string_list_iterator_prev(it));
    T_VERIFY(!string_list_iterator_next(it));

    string_list_iterator_destroy(it);
    return 0;
}

int string_list_init_fill_cleanup() {
    string_list_t list;
    string_list_iterator_t *it;
    int i;
    T_VERIFY(string_list_initialize(&list));
    for(i = 0; i < 5; i++) {
        it = string_list_iterator_createFromChar("text");
        T_VERIFY(it != NULL);
        T_VERIFY(string_list_append(&list,it));
    }
    T_VERIFY(string_list_size(&list) == 5);
    string_list_cleanup(&list);
    T_VERIFY(string_list_size(&list) == 0);

    return 0;
}

int string_list_iterator_destroy_null() {
    string_list_iterator_destroy(NULL);

    return 0;
}
