/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/string_list.h>

static string_list_t list;
static string_list_t emptyList;
static string_list_iterator_t *it[5];
static const char *parts[] = { "Hello", "World", "!","Foo", "Bar" };
static string_list_iterator_t *orphan;

void string_list_insert_build_list_setup() {
    int i;
    string_list_initialize(&list);
    string_list_initialize(&emptyList);
    for(i = 0; i <5; i++) {
        it[i] = string_list_iterator_createFromChar(parts[i]);
        string_list_append(&list,it[i]);
    }
    orphan = string_list_iterator_createFromChar("Test");
}

void string_list_insert_build_list_teardown() {
    int i;
    for(i = 0; i <5; i++) {
        string_list_iterator_destroy(it[i]);
    }
    string_list_iterator_destroy(orphan);

    string_list_cleanup(&list);
    string_list_cleanup(&emptyList);
}

int string_list_insert_indexed_null_list() {
    T_VERIFY(!string_list_insertAt(NULL,0,orphan));
    T_VERIFY(!string_list_insertAt(NULL,10,orphan));

    return 0;
}

int string_list_insert_indexed_empty_list_index_0() {
    T_VERIFY(string_list_size(&emptyList) == 0);
    T_VERIFY(string_list_insertAt(&emptyList,0,orphan));
    T_VERIFY(string_list_iterator_next(orphan) == NULL);
    T_VERIFY(string_list_iterator_prev(orphan) == NULL);
    T_VERIFY(string_list_first(&emptyList) == orphan);
    T_VERIFY(string_list_last(&emptyList) == orphan);

    return 0;
}

int string_list_insert_indexed_empty_list_index_2() {
    T_VERIFY(string_list_size(&emptyList) == 0);
    T_VERIFY(string_list_insertAt(&emptyList,0,orphan));
    T_VERIFY(string_list_iterator_next(orphan) == NULL);
    T_VERIFY(string_list_iterator_prev(orphan) == NULL);
    T_VERIFY(string_list_first(&emptyList) == orphan);
    T_VERIFY(string_list_last(&emptyList) == orphan);

    return 0;
}

int string_list_insert_indexed_list_index_out_of_boundry() {
    unsigned int size = string_list_size(&list);
    T_VERIFY(string_list_insertAt(&list,size + 10,orphan));
    T_VERIFY(string_list_iterator_next(orphan) == NULL);
    T_VERIFY(string_list_iterator_prev(orphan) == it[4]);

    return 0;
}

int string_list_insert_indexed_list_index_0() {
    T_VERIFY(string_list_insertAt(&list,0,orphan));
    T_VERIFY(string_list_iterator_next(orphan) == it[0]);
    T_VERIFY(string_list_iterator_prev(orphan) == NULL);

    return 0;
}

int string_list_insert_indexed_list_index_2() {
    T_VERIFY(string_list_insertAt(&list,2,orphan));
    T_VERIFY(string_list_iterator_next(orphan) == it[2]);
    T_VERIFY(string_list_iterator_prev(orphan) == it[1]);

    return 0;
}

int string_list_insert_indexed_list_index_last() {
    unsigned int size = string_list_size(&list);
    T_VERIFY(string_list_insertAt(&list,size - 1,orphan));
    T_VERIFY(string_list_iterator_next(orphan) == it[4]);
    T_VERIFY(string_list_iterator_prev(orphan) == it[3]);

    return 0;
}
