/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/string_list.h>

static string_list_t list;
static string_list_iterator_t *it[5];
static const char *parts[] = { "Hello", "World", "!","Foo", "Bar" };
static string_list_iterator_t *orphan;

void string_list_remove_setup() {
    int i;
    string_list_initialize(&list);
    for(i = 0; i <5; i++) {
        it[i] = string_list_iterator_createFromChar(parts[i]);
        string_list_append(&list,it[i]);
    }
    orphan = string_list_iterator_createFromChar("Test");
}

void string_list_remove_teardown() {
    int i;
    for(i = 0; i <5; i++) {
        string_list_iterator_destroy(it[i]);
    }
    string_list_cleanup(&list);
    string_list_iterator_destroy(orphan);
}

int string_list_remove_take_first_null() {
    T_VERIFY(string_list_takeFirst(NULL) == NULL);
    return 0;
}

int string_list_remove_take_last_null() {
    T_VERIFY(string_list_takeLast(NULL) == NULL);
    return 0;
}

int string_list_remove_take_first_empty_list() {
    string_list_t empty;
    string_list_initialize(&empty);
    T_VERIFY(string_list_size(&empty) == 0);
    T_VERIFY(string_list_takeFirst(&empty) == NULL);
    string_list_cleanup(&empty);

    return 0;
}

int string_list_remove_take_last_empty_list() {
    string_list_t empty;
    string_list_initialize(&empty);
    T_VERIFY(string_list_size(&empty) == 0);
    T_VERIFY(string_list_takeLast(&empty) == NULL);
    string_list_cleanup(&empty);

    return 0;
}

int string_list_remove_take_first() {
    string_list_iterator_t *taken = string_list_takeFirst(&list);
    T_VERIFY(taken == it[0]);
    T_VERIFY(string_list_iterator_next(taken) == NULL);
    T_VERIFY(string_list_iterator_prev(taken) == NULL);
    T_VERIFY(string_list_first(&list) == it[1]);

    return 0;
}

int string_list_remove_take_last() {
    string_list_iterator_t *taken = string_list_takeLast(&list);
    T_VERIFY(taken == it[4]);
    T_VERIFY(string_list_iterator_next(taken) == NULL);
    T_VERIFY(string_list_iterator_prev(taken) == NULL);
    T_VERIFY(string_list_last(&list) == it[3]);

    return 0;
}

int string_list_remove_take_null() {
    T_VERIFY(string_list_iterator_take(NULL) == NULL);

    return 0;
}

int string_list_remove_take() {
    T_VERIFY(string_list_iterator_take(it[2]) == it[2]);

    return 0;
}
