/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/string_list.h>

int string_list_split_empty_string() {
    string_t text;
    string_initialize(&text,64);

    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_split(&list,&text,",",strlist_keep_empty_parts,string_case_sensitive) == 0);

    string_list_cleanup(&list);
    string_cleanup(&text);

    return 0;
}

int string_list_split_null_string() {
    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_split(&list,NULL,",",strlist_keep_empty_parts,string_case_sensitive) == 0);

    return 0;
}

int string_list_split_null_list() {
    string_t text;
    string_initialize(&text,64);
    string_fromChar(&text,"HelloSepWorldSep!");

    T_VERIFY(string_list_split(NULL,&text,",",strlist_keep_empty_parts,string_case_sensitive) == 0);

    string_cleanup(&text);
    return 0;
}

int string_list_split_null_separator() {
    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    string_t text;
    string_initialize(&text,64);
    string_fromChar(&text,"HelloSepWorldSep!");

    T_VERIFY(string_list_split(&list,&text,NULL,strlist_keep_empty_parts,string_case_sensitive) == 0);

    string_cleanup(&text);
    return 0;
}

int string_list_split_case_sensitive() {
    string_t text;
    string_initialize(&text,64);
    string_fromChar(&text,"HelloSepWorldSep!");

    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_split(&list,&text,"Sep",strlist_keep_empty_parts,string_case_sensitive) == 3);
    string_list_iterator_t *it = string_list_first(&list);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"Hello") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"World") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"!") == 0);

    string_list_cleanup(&list);
    string_cleanup(&text);

    return 0;
}

int string_list_split_case_insensitive() {
    string_t text;
    string_initialize(&text,64);
    string_fromChar(&text,"HelloSepWorldSep!");

    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_split(&list,&text,"SEP",strlist_keep_empty_parts,string_case_insensitive) == 3);
    string_list_iterator_t *it = string_list_first(&list);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"Hello") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"World") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"!") == 0);

    string_list_cleanup(&list);
    string_cleanup(&text);

    return 0;
}

int string_list_split_keep_empty() {
    string_t text;
    string_initialize(&text,64);
    string_fromChar(&text,"HelloSepSepWorldSep!Sep");

    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_split(&list,&text,"SEP",strlist_keep_empty_parts,string_case_insensitive) == 4);
    string_list_iterator_t *it = string_list_first(&list);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"Hello") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"World") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"!") == 0);

    string_list_cleanup(&list);
    string_cleanup(&text);

    return 0;
}

int string_list_split_skip_empty() {
    string_t text;
    string_initialize(&text,64);
    string_fromChar(&text,"HelloSepSepWorldSep!Sep");

    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_split(&list,&text,"SEP",strlist_skip_empty_parts,string_case_insensitive) == 3);
    string_list_iterator_t *it = string_list_first(&list);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"Hello") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"World") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"!") == 0);

    string_list_cleanup(&list);
    string_cleanup(&text);

    return 0;
}

int string_list_split_empty_char() {
    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_splitChar(&list,"",",",strlist_keep_empty_parts,string_case_sensitive) == 0);

    string_list_cleanup(&list);

    return 0;
}

int string_list_split_null_char() {
    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_splitChar(&list,NULL,",",strlist_keep_empty_parts,string_case_sensitive) == 0);

    string_list_cleanup(&list);

    return 0;
}

int string_list_split_char_null_list() {
    T_VERIFY(string_list_splitChar(NULL,"HelloSepWorldSep!","Sep",strlist_keep_empty_parts,string_case_sensitive) == 0);
    return 0;
}

int string_list_split_char_null_separator() {
    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_splitChar(&list,"HelloSepWorldSep!",NULL,strlist_keep_empty_parts,string_case_sensitive) == 0);

    string_list_cleanup(&list);
    return 0;
}

int string_list_split_char() {
    string_list_t list;
    T_VERIFY(string_list_initialize(&list));

    T_VERIFY(string_list_splitChar(&list,"HelloSepWorldSep!","Sep",strlist_keep_empty_parts,string_case_sensitive) == 3);
    string_list_iterator_t *it = string_list_first(&list);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"Hello") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"World") == 0);
    it = string_list_iterator_next(it);
    T_VERIFY(strcmp(string_buffer(string_list_iterator_data(it)),"!") == 0);

    string_list_cleanup(&list);

    return 0;
}
