/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/timer.h>

int timer_create_destroy() {
    assert_bool(pcb_timer_initializeTimers() == true);

    // create
    pcb_timer_t *timer = pcb_timer_create();
    assert_not_null(timer);
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    assert_bool(pcb_timer_start(timer,5000));
    assert_bool(pcb_timer_getState(timer) == pcb_timer_started);

    // destroy
    pcb_timer_destroy(timer);
    assert_bool(!pcb_timer_start(timer,5000));
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    pcb_timer_cleanupTimers();

    sahtest_success();
}

int timer_destroy_null_timer() {
    // should not segfault - segfaults will be captured
    pcb_timer_destroy(NULL);

    sahtest_success();
}

int timer_functions_fail_on_destroyed_timer() {
    assert_bool(pcb_timer_initializeTimers() == true);

    // create
    pcb_timer_t *timer = pcb_timer_create();
    assert_not_null(timer);
    assert_bool(pcb_timer_setUserData(timer,(void *)5));

    // destroy
    pcb_timer_destroy(timer);
    assert_bool(!pcb_timer_start(timer,5000));
    assert_bool(!pcb_timer_stop(timer));
    assert_bool(pcb_timer_remainingTime(timer) == 0);
    assert_bool(!pcb_timer_setUserData(timer,(void *)1));
    assert_bool(pcb_timer_getUserData(timer) == (void *)5);
    assert_bool(!pcb_timer_setHandler(timer,NULL));
    assert_bool(!pcb_timer_setInterval(timer,5000));

    pcb_timer_cleanupTimers();

    sahtest_success();
}

int timer_check_timers_on_destroyed_timer() {
    assert_bool(pcb_timer_initializeTimers() == true);

    // create
    pcb_timer_t *timer = pcb_timer_create();
    assert_not_null(timer);
    assert_bool(pcb_timer_setUserData(timer,(void *)5));

    // destroy
    pcb_timer_destroy(timer);
    pcb_timer_checkTimers();

    pcb_timer_cleanupTimers();

    sahtest_success();
}
