/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <unistd.h>
#include <signal.h>

#include <sahtest/sahtest.h>
#include <pcb/utils/timer.h>

static void signalHandler(int signal) {
    (void)signal;
}

void timer_state_flow_setup() {
    pcb_timer_initializeTimers();
    signal(SIGALRM,signalHandler);
}

void timer_state_flow_teardown() {
    pcb_timer_cleanupTimers();
    signal(SIGALRM,SIG_DFL);
}

int timer_check_all_states_single_shot_timer() {
    pcb_timer_t *timer = pcb_timer_create();
    assert_not_null(timer);
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);
    assert_bool(pcb_timer_remainingTime(timer) <= 0);

    assert_bool(pcb_timer_start(timer,2000));
    assert_bool(pcb_timer_getState(timer) == pcb_timer_started);
    assert_bool(pcb_timer_remainingTime(timer) == 2000);

    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_running);
    assert_bool(pcb_timer_remainingTime(timer) > 0);

    sleep(1);
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_running);
    assert_bool(pcb_timer_remainingTime(timer) > 0);

    sleep(1);
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_expired);
    assert_bool(pcb_timer_remainingTime(timer) <= 0);

    pcb_timer_checkTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    pcb_timer_destroy(timer);
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    sahtest_success();
}

int timer_check_all_states_interval_timer() {
    pcb_timer_t *timer = pcb_timer_create();
    assert_not_null(timer);
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);
    assert_bool(pcb_timer_remainingTime(timer) <= 0);

    assert_bool(pcb_timer_setInterval(timer,1000));
    assert_bool(pcb_timer_start(timer,2000));
    assert_bool(pcb_timer_getState(timer) == pcb_timer_started);
    assert_bool(pcb_timer_remainingTime(timer) == 2000);

    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_running);
    assert_bool(pcb_timer_remainingTime(timer) > 0);

    sleep(1);
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_running);
    assert_bool(pcb_timer_remainingTime(timer) > 0);

    sleep(1);
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_expired);
    assert_bool(pcb_timer_remainingTime(timer) <= 0);

    pcb_timer_checkTimers();
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_running);
    assert_bool(pcb_timer_remainingTime(timer) > 0);

    sleep(1);
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_expired);
    assert_bool(pcb_timer_remainingTime(timer) <= 0);

    pcb_timer_checkTimers();
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_running);
    assert_bool(pcb_timer_remainingTime(timer) > 0);

    pcb_timer_stop(timer);
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    sleep(1);
    pcb_timer_calculateTimers();
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    pcb_timer_destroy(timer);
    assert_bool(pcb_timer_getState(timer) == pcb_timer_off);

    sahtest_success();
}
