/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/tree.h>

static tree_t tree;
static tree_item_t item[5];
static tree_item_t orphan;

void tree_empty_size_setup() {
    int i;
    tree_initialize(&tree);
    tree_item_t *root = tree_root(&tree);
    for(i = 0; i <5; i++) {
        tree_item_initialize(&item[i]);
        tree_item_appendChild(root,&item[i]);
    }
    tree_item_initialize(&orphan);
}

void tree_empty_size_teardown() {
    tree_cleanup(&tree);
}

int tree_hasChilderen_null_tree() {
    T_VERIFY(tree_item_hasChildren(NULL) == false);

    return 0;
}

int tree_hasChilderen_empty_tree() {
    tree_cleanup(&tree);
    tree_item_t *root = tree_root(&tree);

    T_VERIFY(tree_item_hasChildren(root) == false);

    return 0;
}

int tree_hasChilderen_tree() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_hasChildren(root) == true);

    return 0;
}

int tree_child_count_null_tree() {
    T_VERIFY(tree_item_childCount(NULL) == 0);

    return 0;
}

int tree_child_count_empty_tree() {
    tree_item_t *root = tree_root(&tree);
    tree_cleanup(&tree);

    T_VERIFY(tree_item_childCount(root) == 0);

    return 0;
}

int tree_child_count_tree() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_childCount(root) == 5);

    return 0;
}
