/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/tree.h>

static tree_t tree;
static tree_item_t item[5];
static tree_item_t orphan;

void tree_inserting_setup() {
    int i;
    tree_initialize(&tree);
    for(i = 0; i <5; i++) {
        tree_item_initialize(&item[i]);
    }
    tree_item_initialize(&orphan);
}

void tree_inserting_teardown() {
    tree_cleanup(&tree);
}

void tree_insert_build_tree_setup() {
    int i;
    tree_initialize(&tree);
    tree_item_t *root = tree_root(&tree);
    for(i = 0; i <5; i++) {
        tree_item_initialize(&item[i]);
        tree_item_appendChild(root,&item[i]);
    }
    tree_item_initialize(&orphan);
}

void tree_insert_build_tree_teardown() {
    tree_cleanup(&tree);
}

int tree_append_child_null_tree() {
    T_VERIFY(!tree_item_appendChild(NULL,&orphan));

    return 0;
}

int tree_append_child_null_item() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(!tree_item_appendChild(root,NULL));

    return 0;
}

int tree_append_child_orphan_item() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_appendChild(root,&orphan));

    T_VERIFY(tree_item_childCount(root) == 1);
    T_VERIFY(tree_item_firstChild(root) == &orphan);
    T_VERIFY(tree_item_lastChild(root) == &orphan);

    return 0;
}

int tree_append_child_item_twice() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_appendChild(root,&orphan));
    T_VERIFY(tree_item_appendChild(root,&orphan));

    T_VERIFY(tree_item_childCount(root) == 1);
    T_VERIFY(tree_item_firstChild(root) == &orphan);
    T_VERIFY(tree_item_lastChild(root) == &orphan);

    return 0;
}

int tree_append_multiple_childeren() {
    tree_item_t *root = tree_root(&tree);
    int i = 0;
    for(i = 0; i < 5; i++) {
        T_VERIFY(tree_item_appendChild(root,&item[i]));
    }
    T_VERIFY(tree_item_childCount(root) == 5);

    T_VERIFY(tree_item_firstChild(root) == &item[0]);
    T_VERIFY(tree_item_lastChild(root) == &item[4]);

    T_VERIFY(tree_item_nextSibling(&item[0]) == &item[1]);
    T_VERIFY(tree_item_nextSibling(&item[1]) == &item[2]);
    T_VERIFY(tree_item_nextSibling(&item[2]) == &item[3]);
    T_VERIFY(tree_item_nextSibling(&item[3]) == &item[4]);
    T_VERIFY(tree_item_nextSibling(&item[4]) == NULL);

    T_VERIFY(tree_item_prevSibling(&item[0]) == NULL);
    T_VERIFY(tree_item_prevSibling(&item[1]) == &item[0]);
    T_VERIFY(tree_item_prevSibling(&item[2]) == &item[1]);
    T_VERIFY(tree_item_prevSibling(&item[3]) == &item[2]);
    T_VERIFY(tree_item_prevSibling(&item[4]) == &item[3]);

    T_VERIFY(tree_item_parent(&item[0]) == root);
    T_VERIFY(tree_item_parent(&item[1]) == root);
    T_VERIFY(tree_item_parent(&item[2]) == root);
    T_VERIFY(tree_item_parent(&item[3]) == root);
    T_VERIFY(tree_item_parent(&item[4]) == root);

    return 0;
}

int tree_prepend_child_null_tree() {
    T_VERIFY(!tree_item_prependChild(NULL,&orphan));

    return 0;
}

int tree_prepend_child_null_item() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(!tree_item_prependChild(root,NULL));

    return 0;
}

int tree_prepend_child_orphan_item() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_prependChild(root,&orphan));

    T_VERIFY(tree_item_childCount(root) == 1);
    T_VERIFY(tree_item_firstChild(root) == &orphan);
    T_VERIFY(tree_item_lastChild(root) == &orphan);

    return 0;
}

int tree_prepend_child_item_twice() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_prependChild(root,&orphan));
    T_VERIFY(tree_item_prependChild(root,&orphan));

    T_VERIFY(tree_item_childCount(root) == 1);
    T_VERIFY(tree_item_firstChild(root) == &orphan);
    T_VERIFY(tree_item_lastChild(root) == &orphan);

    return 0;
}

int tree_prepend_multiple_childeren() {
    tree_item_t *root = tree_root(&tree);
    int i = 0;
    for(i = 0; i < 5; i++) {
        T_VERIFY(tree_item_prependChild(root,&item[i]));
    }
    T_VERIFY(tree_item_childCount(root) == 5);

    T_VERIFY(tree_item_firstChild(root) == &item[4]);
    T_VERIFY(tree_item_lastChild(root) == &item[0]);

    T_VERIFY(tree_item_nextSibling(&item[4]) == &item[3]);
    T_VERIFY(tree_item_nextSibling(&item[3]) == &item[2]);
    T_VERIFY(tree_item_nextSibling(&item[2]) == &item[1]);
    T_VERIFY(tree_item_nextSibling(&item[1]) == &item[0]);
    T_VERIFY(tree_item_nextSibling(&item[0]) == NULL);

    T_VERIFY(tree_item_prevSibling(&item[4]) == NULL);
    T_VERIFY(tree_item_prevSibling(&item[3]) == &item[4]);
    T_VERIFY(tree_item_prevSibling(&item[2]) == &item[3]);
    T_VERIFY(tree_item_prevSibling(&item[1]) == &item[2]);
    T_VERIFY(tree_item_prevSibling(&item[0]) == &item[1]);

    T_VERIFY(tree_item_parent(&item[0]) == root);
    T_VERIFY(tree_item_parent(&item[1]) == root);
    T_VERIFY(tree_item_parent(&item[2]) == root);
    T_VERIFY(tree_item_parent(&item[3]) == root);
    T_VERIFY(tree_item_parent(&item[4]) == root);

    return 0;
}

int tree_insert_at_null_tree() {
    T_VERIFY(!tree_item_insertChildAt(NULL,0,&orphan));
    T_VERIFY(!tree_item_insertChildAt(NULL,10,&orphan));

    return 0;
}

int tree_insert_at_empty_tree_index_0() {
    tree_item_t *root = tree_root(&tree);
    tree_cleanup(&tree);
    T_VERIFY(tree_item_childCount(root) == 0);
    T_VERIFY(tree_item_insertChildAt(root,0,&orphan));
    T_VERIFY(tree_item_nextSibling(&orphan) == NULL);
    T_VERIFY(tree_item_prevSibling(&orphan) == NULL);
    T_VERIFY(tree_item_firstChild(root) == &orphan);
    T_VERIFY(tree_item_lastChild(root) == &orphan);

    return 0;
}

int tree_insert_at_empty_tree_index_2() {
    tree_item_t *root = tree_root(&tree);
    tree_cleanup(&tree);
    T_VERIFY(tree_item_childCount(root) == 0);
    T_VERIFY(tree_item_insertChildAt(root,2,&orphan));
    T_VERIFY(tree_item_nextSibling(&orphan) == NULL);
    T_VERIFY(tree_item_prevSibling(&orphan) == NULL);
    T_VERIFY(tree_item_firstChild(root) == &orphan);
    T_VERIFY(tree_item_lastChild(root) == &orphan);

    return 0;
}

int tree_insert_at_index_out_of_boundry() {
    tree_item_t *root = tree_root(&tree);
    unsigned int size = tree_item_childCount(root);
    T_VERIFY(tree_item_insertChildAt(root,size + 10,&orphan));
    T_VERIFY(tree_item_nextSibling(&orphan) == NULL);
    T_VERIFY(tree_item_prevSibling(&orphan) == &item[4]);

    return 0;
}

int tree_insert_at_index_0() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_insertChildAt(root,0,&orphan));
    T_VERIFY(tree_item_nextSibling(&orphan) == &item[0]);
    T_VERIFY(tree_item_prevSibling(&orphan) == NULL);

    return 0;
}

int tree_insert_at_index_2() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_insertChildAt(root,2,&orphan));
    T_VERIFY(tree_item_nextSibling(&orphan) == &item[2]);
    T_VERIFY(tree_item_prevSibling(&orphan) == &item[1]);

    return 0;
}

int tree_insert_at_index_last() {
    tree_item_t *root = tree_root(&tree);
    unsigned int size = tree_item_childCount(root);
    T_VERIFY(tree_item_insertChildAt(root,size - 1,&orphan));
    T_VERIFY(tree_item_nextSibling(&orphan) == &item[4]);
    T_VERIFY(tree_item_prevSibling(&orphan) == &item[3]);

    return 0;
}

int tree_insert_at_null_child() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(!tree_item_insertChildAt(root,3,NULL));

    return 0;
}
