/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/tree.h>

static tree_t tree;
static tree_item_t item[5];
static tree_item_t orphan;

void tree_iterating_setup() {
    int i;
    tree_initialize(&tree);
    tree_item_t *root = tree_root(&tree);
    for(i = 0; i <5; i++) {
        tree_item_initialize(&item[i]);
        tree_item_appendChild(root,&item[i]);
    }
    tree_item_initialize(&orphan);
}

void tree_iterating_teardown() {
    tree_cleanup(&tree);
}

int tree_iterating_first_null() {
    T_VERIFY(tree_item_firstChild(NULL) == NULL);

    return 0;
}

int tree_iterating_last_null() {
    T_VERIFY(tree_item_lastChild(NULL) == NULL);

    return 0;
}

int tree_iterating_next_null() {
    T_VERIFY(tree_item_nextSibling(NULL) == NULL);

    return 0;
}

int tree_iterating_prev_null() {
    T_VERIFY(tree_item_prevSibling(NULL) == NULL);

    return 0;
}

int tree_iterating_first_empty() {
    tree_item_t *root = tree_root(&tree);
    tree_item_cleanup(root);
    T_VERIFY(tree_item_firstChild(root) == NULL);

    return 0;
}

int tree_iterating_first() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_firstChild(root) == &item[0]);

    return 0;
}

int tree_iterating_last_empty() {
    tree_item_t *root = tree_root(&tree);
    tree_item_cleanup(root);
    T_VERIFY(tree_item_lastChild(root) == NULL);

    return 0;
}

int tree_iterating_last() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_lastChild(root) == &item[4]);

    return 0;
}

int tree_iterating_first_prev() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *iter = tree_item_firstChild(root);
    T_VERIFY(tree_item_prevSibling(iter) == NULL);

    return 0;
}

int tree_iterating_last_next() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *iter = tree_item_lastChild(root);
    T_VERIFY(tree_item_nextSibling(iter) == NULL);

    return 0;
}

int tree_iterating_first_next() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *iter = tree_item_firstChild(root);
    T_VERIFY(tree_item_nextSibling(iter) == &item[1]);

    return 0;
}

int tree_iterating_last_prev() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *iter = tree_item_lastChild(root);
    T_VERIFY(tree_item_prevSibling(iter) == &item[3]);

    return 0;
}

int tree_iterating_orphan_next() {
    T_VERIFY(tree_item_nextSibling(&orphan) == NULL);

    return 0;
}

int tree_iterating_orphan_prev() {
    T_VERIFY(tree_item_prevSibling(&orphan) == NULL);

    return 0;
}

int tree_iterating_loop_forward() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *iter = NULL;
    int i = 0;
    tree_for_each_child(iter, root) {
        T_VERIFY(i < 5);
        T_VERIFY(iter == &item[i]);
        T_VERIFY(tree_item_parent(iter) == root);
        i++;
    }

    return 0;
}

int tree_iterating_loop_reverse() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *iter = NULL;
    int i = 4;
    tree_for_each_child_reverse(iter, root) {
        T_VERIFY(i >= 0);
        T_VERIFY(iter == &item[i]);
        T_VERIFY(tree_item_parent(iter) == root);
        i--;
    }

    return 0;
}

int tree_parent_null() {
    T_VERIFY(tree_item_parent(NULL) == NULL);

    return 0;
}

int tree_parent_root() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_parent(root) == NULL);

    return 0;
}

int tree_iterating_indexed() {
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_childAt(root,3) == &item[3]);
    T_VERIFY(tree_item_childAt(root,5) == NULL);
    T_VERIFY(tree_item_childAt(root,1) == &item[1]);

    return 0;
}

int tree_iterating_indexed_null() {
    T_VERIFY(tree_item_childAt(NULL,3) == NULL);

    return 0;
}
