/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/tree.h>

static tree_t tree;
static tree_item_t item[5];
static tree_item_t orphan;

void tree_remove_setup() {
    int i;
    tree_initialize(&tree);
    tree_item_t *root = tree_root(&tree);
    for(i = 0; i <5; i++) {
        tree_item_initialize(&item[i]);
        tree_item_appendChild(root,&item[i]);
    }
    tree_item_initialize(&orphan);
}

void tree_remove_teardown() {
    tree_cleanup(&tree);
}

int tree_remove_take_first_null() {
    T_VERIFY(tree_item_takeFirstChild(NULL) == NULL);
    return 0;
}

int tree_remove_take_last_null() {
    T_VERIFY(tree_item_takeLastChild(NULL) == NULL);
    return 0;
}

int tree_remove_take_first_empty_tree() {
    tree_cleanup(&tree);
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_childCount(root) == 0);
    T_VERIFY(tree_item_takeFirstChild(root) == NULL);

    return 0;
}

int tree_remove_take_last_empty_tree() {
    tree_cleanup(&tree);
    tree_item_t *root = tree_root(&tree);
    T_VERIFY(tree_item_childCount(root) == 0);
    T_VERIFY(tree_item_takeLastChild(root) == NULL);

    return 0;
}

int tree_remove_take_first() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *taken = tree_item_takeFirstChild(root);
    T_VERIFY(taken == &item[0]);
    T_VERIFY(tree_item_nextSibling(taken) == NULL);
    T_VERIFY(tree_item_prevSibling(taken) == NULL);
    T_VERIFY(tree_item_firstChild(root) == &item[1]);

    return 0;
}

int tree_remove_take_last() {
    tree_item_t *root = tree_root(&tree);
    tree_item_t *taken = tree_item_takeLastChild(root);
    T_VERIFY(taken == &item[4]);
    T_VERIFY(tree_item_nextSibling(taken) == NULL);
    T_VERIFY(tree_item_prevSibling(taken) == NULL);
    T_VERIFY(tree_item_lastChild(root) == &item[3]);

    return 0;
}

int tree_remove_item_null() {
    T_VERIFY(tree_item_takeChild(NULL) == NULL);
    return 0;
}

int tree_remove_item() {
    T_VERIFY(tree_item_takeChild(&item[2]) == &item[2]);
    T_VERIFY(tree_item_nextSibling(&item[2]) == NULL);
    T_VERIFY(tree_item_prevSibling(&item[2]) == NULL);

    T_VERIFY(tree_item_nextSibling(&item[1]) == &item[3]);
    T_VERIFY(tree_item_prevSibling(&item[3]) == &item[1]);

    return 0;
}
