/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/uri.h>

int test_parse_null() {
    uri_t *uri = uri_parse(NULL);

    T_VERIFY(uri == NULL);

    return 0;
}

int test_parse_empty_string() {
    uri_t *uri = uri_parse("");

    T_VERIFY(uri == NULL);
    T_VERIFY(uri_getScheme(uri) == NULL);
    T_VERIFY(uri_getHost(uri) == NULL);
    T_VERIFY(uri_getPort(uri) == NULL);
    T_VERIFY(uri_getUser(uri) == NULL);
    T_VERIFY(uri_getPassword(uri) == NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    uri_destroy(NULL);

    return 0;
}

int test_parse_scheme_only() {
    uri_t *uri = uri_parse("http://");

    T_VERIFY(uri == NULL);

    return 0;
}

int test_parse_authority_no_host() {
    uri_t *uri = uri_parse("http://userame:password@:8181");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getHost(uri),"");

    uri_destroy(uri);

    return 0;
}


int test_parse_simple_authority() {
    uri_t *uri = uri_parse("http://www.google.be");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) == NULL);
    T_VERIFY(uri_getUser(uri) == NULL);
    T_VERIFY(uri_getPassword(uri) == NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getHost(uri),"www.google.be");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_with_port() {
    uri_t *uri = uri_parse("http://www.google.be:8181");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) == NULL);
    T_VERIFY(uri_getPassword(uri) == NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getHost(uri),"www.google.be");
    T_CMP(uri_getPort(uri),"8181");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_with_user() {
    uri_t *uri = uri_parse("http://username@www.google.be");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) == NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) == NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getHost(uri),"www.google.be");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_with_user_password() {
    uri_t *uri = uri_parse("http://username:password@www.google.be");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) == NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"www.google.be");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_full() {
    uri_t *uri = uri_parse("http://username:password@www.google.be:8181");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"www.google.be");
    T_CMP(uri_getPort(uri),"8181");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_full_with_curly_braces() {
    uri_t *uri = uri_parse("http://username:password@{www.google.be}:{8181}");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"www.google.be");
    T_CMP(uri_getPort(uri),"8181");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_full_with_curly_braces_special_chars() {
    uri_t *uri = uri_parse("pcb://username:password@ipc:{/var/run/[@test]:}");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"pcb");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"ipc");
    T_CMP(uri_getPort(uri),"/var/run/[@test]:");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_full_with_braces() {
    uri_t *uri = uri_parse("http://username:password@[www.google.be]:[8181]");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"www.google.be");
    T_CMP(uri_getPort(uri),"8181");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_full_with_braces_special_chars() {
    uri_t *uri = uri_parse("pcb://username:password@ipc:[/var/run/{@test}:]");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"pcb");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"ipc");
    T_CMP(uri_getPort(uri),"/var/run/{@test}:");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_full_with_host_ipv6() {
    uri_t *uri = uri_parse("http://username:password@[::1]:[8181]");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"::1");
    T_CMP(uri_getPort(uri),"8181");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_full_with_port_path() {
    uri_t *uri = uri_parse("pcb://username:password@ipc:[/var/run/mysocket]");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) != NULL);
    T_VERIFY(uri_getUser(uri) != NULL);
    T_VERIFY(uri_getPassword(uri) != NULL);
    T_VERIFY(uri_getPath(uri) == NULL);

    T_CMP(uri_getScheme(uri),"pcb");
    T_CMP(uri_getUser(uri),"username");
    T_CMP(uri_getPassword(uri),"password");
    T_CMP(uri_getHost(uri),"ipc");
    T_CMP(uri_getPort(uri),"/var/run/mysocket");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_path() {
    uri_t *uri = uri_parse("http://www.google.be/this/is/my/path");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) == NULL);
    T_VERIFY(uri_getUser(uri) == NULL);
    T_VERIFY(uri_getPassword(uri) == NULL);
    T_VERIFY(uri_getPath(uri) != NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getHost(uri),"www.google.be");
    T_CMP(uri_getPath(uri),"this/is/my/path");

    uri_destroy(uri);

    return 0;
}

int test_parse_authority_path_with_parameters() {
    uri_t *uri = uri_parse("http://www.google.be/this/is/my/path?param1=abcd&param2=1234&param3=echo");

    T_VERIFY(uri != NULL);
    T_VERIFY(uri_getScheme(uri) != NULL);
    T_VERIFY(uri_getHost(uri) != NULL);
    T_VERIFY(uri_getPort(uri) == NULL);
    T_VERIFY(uri_getUser(uri) == NULL);
    T_VERIFY(uri_getPassword(uri) == NULL);
    T_VERIFY(uri_getPath(uri) != NULL);

    T_CMP(uri_getScheme(uri),"http");
    T_CMP(uri_getHost(uri),"www.google.be");
    T_CMP(uri_getPath(uri),"this/is/my/path");

    T_VERIFY(uri_getParameter(uri,"param1") != NULL);
    T_VERIFY(uri_getParameter(uri,"param2") != NULL);
    T_VERIFY(uri_getParameter(uri,"param3") != NULL);
    T_VERIFY(uri_getParameter(uri,"param4") == NULL);

    T_CMP(uri_getParameter(uri,"param1"),"abcd");
    T_CMP(uri_getParameter(uri,"param2"),"1234");
    T_CMP(uri_getParameter(uri,"param3"),"echo");

    uri_destroy(uri);

    return 0;
}

int test_parse_and_build_string() {
    uri_t *uri = NULL;
    char *uri_str = NULL;

    uri = uri_parse("http://www.google.be");
    uri_str = uri_string(uri);
    T_CMP(uri_str,"http://[www.google.be]/");
    uri_destroy(uri);
    free(uri_str);

    uri = uri_parse("http://www.google.be:8181");
    uri_str = uri_string(uri);
    T_CMP(uri_str,"http://[www.google.be]:[8181]/");
    uri_destroy(uri);
    free(uri_str);

    uri = uri_parse("http://username@www.google.be");
    uri_str = uri_string(uri);
    T_CMP(uri_str,"http://username@[www.google.be]/");
    uri_destroy(uri);
    free(uri_str);

    uri = uri_parse("http://username:password@www.google.be");
    uri_str = uri_string(uri);
    T_CMP(uri_str,"http://username:password@[www.google.be]/");
    uri_destroy(uri);
    free(uri_str);

    uri = uri_parse("http://username:password@www.google.be:8181");
    uri_str = uri_string(uri);
    T_CMP(uri_str,"http://username:password@[www.google.be]:[8181]/");
    uri_destroy(uri);
    free(uri_str);

    uri = uri_parse("http://www.google.be/this/is/my/path?param1=abcd&param2=1234&param3=echo");
    uri_str = uri_string(uri);
    T_CMP(uri_str,"http://[www.google.be]/this/is/my/path?param1=abcd&param2=1234&param3=echo");
    uri_destroy(uri);
    free(uri_str);

    return 0;
}

int test_parse_empty_authority_path() {
    uri_t *uri = uri_parse("http:///is/my/path?param1=abcd&param2=1234&param3=echo");
    T_VERIFY(uri != NULL);
    uri_destroy(uri);

    return 0;
}
