/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/variant_map.h>

static variant_t variant_possitive;
static variant_t variant_negative;

void variant_int8_conversion_setup() {
    variant_initialize(&variant_possitive,variant_type_int8);
    variant_initialize(&variant_negative,variant_type_int8);

    variant_setInt8(&variant_possitive,127);
    variant_setInt8(&variant_negative,-1);
}

void variant_int8_conversion_teardown() {
    variant_cleanup(&variant_possitive);
    variant_cleanup(&variant_negative);
}


int variant_int8_tostring() {
    string_t result;
    string_t *string = NULL;
    string_initialize(&result,0);

    assert_bool(variant_toString(&result,&variant_possitive));
    assert_bool(strcmp(string_buffer(&result),"127") == 0);

    assert_bool(variant_toString(&result,&variant_negative));
    assert_bool(strcmp(string_buffer(&result),"-1") == 0);

    string = variant_string(&variant_possitive);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"127") == 0);
    string_cleanup(string);
    free(string);

    string = variant_string(&variant_negative);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"-1") == 0);
    string_cleanup(string);
    free(string);

    string_cleanup(&result);
    sahtest_success();
}

int variant_int8_tojson() {
    string_t result;
    string_initialize(&result,0);

    assert_bool(variant_toJSON(&result,&variant_possitive));
    assert_bool(strcmp(string_buffer(&result),"127") == 0);

    assert_bool(variant_toJSON(&result,&variant_negative));
    assert_bool(strcmp(string_buffer(&result),"-1") == 0);

    string_cleanup(&result);
    sahtest_success();
}

int variant_int8_toChar() {
    char *result = NULL;

    assert_bool(variant_toChar(&result,&variant_possitive));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"127") == 0);
    free(result);
    result = variant_char(&variant_possitive);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"127") == 0);
    free(result);

    assert_bool(variant_toChar(&result,&variant_negative));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"-1") == 0);
    free(result);
    result = variant_char(&variant_negative);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"-1") == 0);
    free(result);

    sahtest_success();
}

int variant_int8_toInt8() {
    int8_t result = 0;

    assert_bool(variant_toInt8(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_int8(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toInt8(&result,&variant_negative));
    assert_bool(result == -1);
    result = variant_int8(&variant_negative);
    assert_bool(result == -1);

    sahtest_success();
}

int variant_int8_toInt16() {
    int16_t result = 0;

    assert_bool(variant_toInt16(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_int16(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toInt16(&result,&variant_negative));
    assert_bool(result == -1);
    result = variant_int16(&variant_negative);
    assert_bool(result == -1);

    sahtest_success();
}

int variant_int8_toInt32() {
    int32_t result = 0;

    assert_bool(variant_toInt32(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_int32(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toInt32(&result,&variant_negative));
    assert_bool(result == -1);
    result = variant_int32(&variant_negative);
    assert_bool(result == -1);

    sahtest_success();
}

int variant_int8_toInt64() {
    int64_t result = 0;

    assert_bool(variant_toInt64(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_int64(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toInt64(&result,&variant_negative));
    assert_bool(result == -1);
    result = variant_int64(&variant_negative);
    assert_bool(result == -1);

    sahtest_success();
}

int variant_int8_toUInt8() {
    uint8_t result = 0;

    assert_bool(variant_toUInt8(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_uint8(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toUInt8(&result,&variant_negative));
    assert_bool(result == 1);
    result = variant_uint8(&variant_negative);
    assert_bool(result == 1);

    sahtest_success();
}

int variant_int8_toUInt16() {
    uint16_t result = 0;

    assert_bool(variant_toUInt16(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_uint16(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toUInt16(&result,&variant_negative));
    assert_bool(result == 1);
    result = variant_uint16(&variant_negative);
    assert_bool(result == 1);

    sahtest_success();
}

int variant_int8_toUInt32() {
    uint32_t result = 0;

    assert_bool(variant_toUInt32(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_uint32(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toUInt32(&result,&variant_negative));
    assert_bool(result == 1);
    result = variant_uint32(&variant_negative);
    assert_bool(result == 1);

    sahtest_success();
}

int variant_int8_toUInt64() {
    uint64_t result = 0;

    assert_bool(variant_toUInt64(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_uint64(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toUInt64(&result,&variant_negative));
    assert_bool(result == 1);
    result = variant_uint64(&variant_negative);
    assert_bool(result == 1);

    sahtest_success();
}

int variant_int8_toBool() {
    bool result = false;

    assert_bool(variant_toBool(&result,&variant_possitive));
    assert_bool(result == true);
    result = variant_bool(&variant_possitive);
    assert_bool(result == true);

    assert_bool(variant_toBool(&result,&variant_negative));
    assert_bool(result == true);
    result = variant_bool(&variant_negative);
    assert_bool(result == true);

    sahtest_success();
}

int variant_int8_toFd() {
    int result = 0;

    assert_bool(variant_toFd(&result,&variant_possitive) == false);
    assert_bool(variant_toFd(&result,&variant_negative) == false);

    assert_bool(variant_fd(&variant_possitive) == -1);
    assert_bool(variant_fd(&variant_negative) == -1);

    sahtest_success();
}

int variant_int8_toDouble() {
    double result = 0;

    assert_bool(variant_toDouble(&result,&variant_possitive));
    assert_bool(result == 127);
    result = variant_double(&variant_possitive);
    assert_bool(result == 127);

    assert_bool(variant_toDouble(&result,&variant_negative));
    assert_bool(result == -1);
    result = variant_double(&variant_negative);
    assert_bool(result == -1);

    sahtest_success();
}

int variant_int8_toList() {
    variant_list_t result;
    variant_list_t *result_list = NULL;
    variant_list_initialize(&result);

    assert_bool(variant_toList(&result,&variant_possitive) == true);
    assert_bool(variant_list_size(&result) == 1);
    result_list = variant_list(&variant_possitive);
    assert_bool(result_list != NULL);
    assert_bool(variant_list_size(result_list) == 1);

    variant_list_clear(&result);
    variant_list_clear(result_list);
    free(result_list);

    sahtest_success();
}

int variant_int8_toMap() {
    variant_map_t result;
    variant_map_t *result_map = NULL;
    variant_map_initialize(&result);

    assert_bool(variant_toMap(&result,&variant_possitive) == false);
    assert_bool(variant_map_size(&result) == 0);
    result_map = variant_map(&variant_possitive);
    assert_bool(result_map == NULL);

    sahtest_success();
}
