/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/variant_map.h>

static variant_t variant_l;

void variant_list_conversion_setup() {
    variant_initialize(&variant_l,variant_type_array);
    variant_list_t *list = variant_da_list(&variant_l);
    variant_list_addBool(list,true);
    variant_list_addUInt32(list,4096);
    variant_list_addChar(list,"Hallo World");
}

void variant_list_conversion_teardown() {
    variant_cleanup(&variant_l);
}

int variant_list_tostring() {
    string_t result;
    string_t *string = NULL;
    string_initialize(&result,0);

    assert_bool(variant_toString(&result,&variant_l));
    assert_bool(strcmp(string_buffer(&result),"[1,4096,Hallo World]") == 0);

    string = variant_string(&variant_l);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"[1,4096,Hallo World]") == 0);
    string_cleanup(string);
    free(string);

    string_cleanup(&result);
    sahtest_success();
}

int variant_list_tojson() {
    string_t result;
    string_initialize(&result,0);

    assert_bool(variant_toJSON(&result,&variant_l));
    assert_bool(strcmp(string_buffer(&result),"[true,4096,\"Hallo World\"]") == 0);

    string_cleanup(&result);
    sahtest_success();
}

int variant_list_toChar() {
    char *result = NULL;

    assert_bool(variant_toChar(&result,&variant_l));
    assert_bool(strcmp(result,"[1,4096,Hallo World]") == 0);
    free(result);

    result = variant_char(&variant_l);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"[1,4096,Hallo World]") == 0);
    free(result);

    sahtest_success();
}

int variant_list_toInt8() {
    int8_t result = 0;

    assert_bool(variant_toInt8(&result,&variant_l) == false);

    result = variant_int8(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toInt16() {
    int16_t result = 0;

    assert_bool(variant_toInt16(&result,&variant_l) == false);

    result = variant_int16(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toInt32() {
    int32_t result = 0;

    assert_bool(variant_toInt32(&result,&variant_l) == false);

    result = variant_int32(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toInt64() {
    int64_t result = 0;

    assert_bool(variant_toInt64(&result,&variant_l) == false);

    result = variant_int64(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toUInt8() {
    uint8_t result = 0;

    assert_bool(variant_toUInt8(&result,&variant_l) == false);

    result = variant_uint8(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toUInt16() {
    uint16_t result = 0;

    assert_bool(variant_toUInt16(&result,&variant_l) == false);

    result = variant_uint16(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toUInt32() {
    uint32_t result = 0;

    assert_bool(variant_toUInt32(&result,&variant_l) == false);

    result = variant_uint32(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toUInt64() {
    uint64_t result = 0;

    assert_bool(variant_toUInt64(&result,&variant_l) == false);

    result = variant_uint64(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toBool() {
    bool result = true;

    assert_bool(variant_toBool(&result,&variant_l) == false);

    result = variant_bool(&variant_l);
    assert_bool(result == false);

    sahtest_success();
}

int variant_list_toFd() {
    int result = true;

    assert_bool(variant_toFd(&result,&variant_l) == false);

    result = variant_fd(&variant_l);
    assert_bool(result == -1);

    sahtest_success();
}

int variant_list_toDouble() {
    double result = 0;

    assert_bool(variant_toDouble(&result,&variant_l) == false);

    result = variant_double(&variant_l);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_list_toList() {
    variant_list_t result;
    variant_list_t *list = NULL;
    variant_list_initialize(&result);

    assert_bool(variant_toList(&result,&variant_l) == true);
    assert_bool(variant_list_size(&result) == 3);
    variant_list_cleanup(&result);

    list = variant_list(&variant_l);
    assert_bool(list != NULL);
    assert_bool(variant_list_size(list) == 3);
    variant_list_cleanup(list);
    free(list);

    sahtest_success();
}

int variant_list_toMap() {
    variant_map_t result;
    variant_map_t *list = NULL;
    variant_map_initialize(&result);

    assert_bool(variant_toMap(&result,&variant_l) == false);
    variant_map_cleanup(&result);

    list = variant_map(&variant_l);
    assert_bool(list == NULL);

    sahtest_success();
}

