/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/variant_map.h>

static variant_t variant_unknown;

void variant_null_unknown_conversion_setup() {
    variant_initialize(&variant_unknown,variant_type_unknown);
}

void variant_null_unknown_conversion_teardown() {

}

int variant_null_unknown_tostring() {
    string_t result;
    string_t *string = NULL;
    string_initialize(&result,0);

    assert_bool(variant_toString(&result,&variant_unknown) == false);
    assert_bool(result.buffer == NULL);

    assert_bool(variant_toString(&result,NULL) == false);

    string = variant_string(&variant_unknown);
    assert_bool(string == NULL);
    string_cleanup(string);
    free(string);

    string = variant_string(NULL);
    assert_bool(string == NULL);
    string_cleanup(string);
    free(string);

    string_cleanup(&result);
    sahtest_success();
}

int variant_null_unknown_tojson() {
    string_t result;
    string_initialize(&result,0);

    assert_bool(variant_toJSON(&result,NULL) == false);
    assert_bool(result.buffer == NULL);

    assert_bool(variant_toJSON(&result,&variant_unknown) == true);
    assert_bool(result.buffer != NULL);
    assert_bool(strcmp(string_buffer(&result),"null") == 0);

    string_cleanup(&result);
    sahtest_success();
}

int variant_null_unknown_toChar() {
    char *result = NULL;

    assert_bool(variant_toChar(&result,&variant_unknown) == false);
    assert_bool(result == NULL);
    free(result);

    assert_bool(variant_toChar(&result,NULL) == false);

    assert_bool(variant_char(NULL) == NULL);
    assert_bool(variant_char(&variant_unknown) == NULL);

    sahtest_success();
}

int variant_null_unknown_toInt8() {
    int8_t result = 0;

    assert_bool(variant_toInt8(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toInt8(&result,NULL) == false);

    assert_bool(variant_int8(NULL) == 0);
    assert_bool(variant_int8(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toInt16() {
    int16_t result = 0;

    assert_bool(variant_toInt16(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toInt16(&result,NULL) == false);

    assert_bool(variant_int16(NULL) == 0);
    assert_bool(variant_int16(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toInt32() {
    int32_t result = 0;

    assert_bool(variant_toInt32(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toInt32(&result,NULL) == false);

    assert_bool(variant_int32(NULL) == 0);
    assert_bool(variant_int32(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toInt64() {
    int64_t result = 0;

    assert_bool(variant_toInt64(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toInt64(&result,NULL) == false);

    assert_bool(variant_int64(NULL) == 0);
    assert_bool(variant_int64(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toUInt8() {
    uint8_t result = 0;

    assert_bool(variant_toUInt8(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toUInt8(&result,NULL) == false);

    assert_bool(variant_uint8(NULL) == 0);
    assert_bool(variant_uint8(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toUInt16() {
    uint16_t result = 0;

    assert_bool(variant_toUInt16(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toUInt16(&result,NULL) == false);

    assert_bool(variant_uint16(NULL) == 0);
    assert_bool(variant_uint16(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toUInt32() {
    uint32_t result = 0;

    assert_bool(variant_toUInt32(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toUInt32(&result,NULL) == false);

    assert_bool(variant_uint32(NULL) == 0);
    assert_bool(variant_uint32(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toUInt64() {
    uint64_t result = 0;

    assert_bool(variant_toUInt64(&result,&variant_unknown) == false);
    assert_bool(result == 0);

    assert_bool(variant_toUInt64(&result,NULL) == false);

    assert_bool(variant_uint64(NULL) == 0);
    assert_bool(variant_uint64(&variant_unknown) == 0);

    sahtest_success();
}

int variant_null_unknown_toBool() {
    bool result = false;

    assert_bool(variant_toBool(&result,&variant_unknown) == false);
    assert_bool(result == false);

    assert_bool(variant_toBool(&result,NULL) == false);

    assert_bool(variant_bool(NULL) == false);
    assert_bool(variant_bool(&variant_unknown) == false);

    sahtest_success();
}

int variant_null_unknown_toFd() {
    int result = 0;

    assert_bool(variant_toFd(&result,&variant_unknown) == false);

    assert_bool(variant_toFd(&result,NULL) == false);

    assert_bool(variant_fd(NULL) == -1);
    assert_bool(variant_fd(&variant_unknown) == -1);

    sahtest_success();
}

int variant_null_unknown_toDouble() {
    double result = 0;

    assert_bool(variant_toDouble(&result,&variant_unknown) == false);
    assert_bool(result == 0);
    assert_bool(variant_toDouble(&result,NULL) == false);
    assert_bool(result == 0);

    result = variant_double(&variant_unknown);
    assert_bool(result == 0);
    result = variant_double(NULL);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_null_unknown_toList() {
    variant_list_t result;
    variant_list_t *result_list = NULL;
    variant_list_initialize(&result);

    assert_bool(variant_toList(&result,&variant_unknown) == true);
    assert_bool(variant_list_size(&result) == 1);
    assert_bool(variant_toList(&result,NULL) == false);
    assert_bool(variant_list_size(&result) == 1);

    result_list = variant_list(&variant_unknown);
    assert_bool(result_list != NULL);
    assert_bool(variant_list_size(result_list) == 1);
    variant_list_clear(result_list);
    free(result_list);

    result_list = variant_list(NULL);
    assert_bool(result_list == NULL);

    variant_list_clear(&result);
    variant_list_clear(result_list);
    free(result_list);

    sahtest_success();
}

int variant_null_unknown_toMap() {
    variant_map_t result;
    variant_map_t *result_map = NULL;
    variant_map_initialize(&result);

    assert_bool(variant_toMap(&result,&variant_unknown) == false);
    assert_bool(variant_map_size(&result) == 0);
    assert_bool(variant_toMap(&result,NULL) == false);
    assert_bool(variant_map_size(&result) == 0);

    result_map = variant_map(&variant_unknown);
    assert_bool(result_map == NULL);
    result_map = variant_map(NULL);
    assert_bool(result_map == NULL);

    sahtest_success();
}
