/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/variant_map.h>

static variant_t variant_text;
static variant_t variant_string_number;
static variant_t variant_string_yes;
static variant_t variant_string_true;
static variant_t variant_string_false;

void variant_string_conversion_setup() {
    variant_initialize(&variant_text,variant_type_string);
    variant_initialize(&variant_string_number,variant_type_string);
    variant_initialize(&variant_string_yes,variant_type_string);
    variant_initialize(&variant_string_true,variant_type_string);
    variant_initialize(&variant_string_false,variant_type_string);

    variant_setChar(&variant_text,"Hallo World");
    variant_setChar(&variant_string_number,"4096");
    variant_setChar(&variant_string_yes,"yes");
    variant_setChar(&variant_string_true,"true");
    variant_setChar(&variant_string_false,"false");
}

void variant_string_conversion_teardown() {
    variant_cleanup(&variant_text);
    variant_cleanup(&variant_string_number);
    variant_cleanup(&variant_string_yes);
    variant_cleanup(&variant_string_true);
    variant_cleanup(&variant_string_false);
}

int variant_string_tostring() {
    string_t result;
    string_t *string = NULL;
    string_initialize(&result,0);

    assert_bool(variant_toString(&result,&variant_text));
    assert_bool(strcmp(string_buffer(&result),"Hallo World") == 0);

    assert_bool(variant_toString(&result,&variant_string_number));
    assert_bool(strcmp(string_buffer(&result),"4096") == 0);

    assert_bool(variant_toString(&result,&variant_string_yes));
    assert_bool(strcmp(string_buffer(&result),"yes") == 0);

    assert_bool(variant_toString(&result,&variant_string_true));
    assert_bool(strcmp(string_buffer(&result),"true") == 0);

    assert_bool(variant_toString(&result,&variant_string_false));
    assert_bool(strcmp(string_buffer(&result),"false") == 0);

    string = variant_string(&variant_text);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"Hallo World") == 0);
    string_cleanup(string);
    free(string);

    string = variant_string(&variant_string_number);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"4096") == 0);
    string_cleanup(string);
    free(string);

    string = variant_string(&variant_string_yes);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"yes") == 0);
    string_cleanup(string);
    free(string);

    string = variant_string(&variant_string_true);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"true") == 0);
    string_cleanup(string);
    free(string);

    string = variant_string(&variant_string_false);
    assert_bool(string != NULL);
    assert_bool(strcmp(string_buffer(string),"false") == 0);
    string_cleanup(string);
    free(string);

    string_cleanup(&result);
    sahtest_success();
}

int variant_string_tojson() {
    string_t result;
    string_initialize(&result,0);

    assert_bool(variant_toJSON(&result,&variant_text));
    assert_bool(strcmp(string_buffer(&result),"\"Hallo World\"") == 0);

    assert_bool(variant_toJSON(&result,&variant_string_number));
    assert_bool(strcmp(string_buffer(&result),"\"4096\"") == 0);

    assert_bool(variant_toJSON(&result,&variant_string_yes));
    assert_bool(strcmp(string_buffer(&result),"\"yes\"") == 0);

    assert_bool(variant_toJSON(&result,&variant_string_true));
    assert_bool(strcmp(string_buffer(&result),"\"true\"") == 0);

    assert_bool(variant_toJSON(&result,&variant_string_false));
    assert_bool(strcmp(string_buffer(&result),"\"false\"") == 0);

    string_cleanup(&result);
    sahtest_success();
}

int variant_string_toChar() {
    char *result = NULL;

    assert_bool(variant_toChar(&result,&variant_text));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"Hallo World") == 0);
    free(result);
    result = variant_char(&variant_text);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"Hallo World") == 0);
    free(result);

    assert_bool(variant_toChar(&result,&variant_string_number));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"4096") == 0);
    free(result);
    result = variant_char(&variant_string_number);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"4096") == 0);
    free(result);

    assert_bool(variant_toChar(&result,&variant_string_yes));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"yes") == 0);
    free(result);
    result = variant_char(&variant_string_yes);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"yes") == 0);
    free(result);

    assert_bool(variant_toChar(&result,&variant_string_true));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"true") == 0);
    free(result);
    result = variant_char(&variant_string_true);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"true") == 0);
    free(result);

    assert_bool(variant_toChar(&result,&variant_string_false));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"false") == 0);
    free(result);
    result = variant_char(&variant_string_false);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"false") == 0);
    free(result);

    sahtest_success();
}

int variant_string_toInt8() {
    int8_t result = 0;

    assert_bool(!variant_toInt8(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_int8(&variant_text);
    assert_bool(result == 0);

    assert_bool(!variant_toInt8(&result,&variant_string_number));
    assert_bool(result == 0);
    result = variant_int8(&variant_string_number);
    assert_bool(result == 0);

    assert_bool(!variant_toInt8(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_int8(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toInt8(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_int8(&variant_text);
    assert_bool(result == 0);

    assert_bool(!variant_toInt8(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_int8(&variant_string_true);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toInt16() {
    int16_t result = 0;

    assert_bool(!variant_toInt16(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_int16(&variant_text);
    assert_bool(result == 0);

    assert_bool(variant_toInt16(&result,&variant_string_number));
    assert_bool(result == (int16_t)4096);
    result = variant_int16(&variant_string_number);
    assert_bool(result == (int16_t)4096);

    assert_bool(!variant_toInt16(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_int16(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toInt16(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_int16(&variant_string_true);
    assert_bool(result == 0);

    assert_bool(!variant_toInt16(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_int16(&variant_string_false);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toInt32() {
    int32_t result = 0;

    assert_bool(!variant_toInt32(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_int32(&variant_text);
    assert_bool(result == 0);

    assert_bool(variant_toInt32(&result,&variant_string_number));
    assert_bool(result == (int32_t)4096);
    result = variant_int32(&variant_string_number);
    assert_bool(result == (int32_t)4096);

    assert_bool(!variant_toInt32(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_int32(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toInt32(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_int32(&variant_string_true);
    assert_bool(result == 0);

    assert_bool(!variant_toInt32(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_int32(&variant_string_false);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toInt64() {
    int64_t result = 0;

    assert_bool(!variant_toInt64(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_int64(&variant_text);
    assert_bool(result == 0);

    assert_bool(variant_toInt64(&result,&variant_string_number));
    assert_bool(result == (int64_t)4096);
    result = variant_int64(&variant_string_number);
    assert_bool(result == (int64_t)4096);

    assert_bool(!variant_toInt64(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_int64(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toInt64(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_int64(&variant_string_true);
    assert_bool(result == 0);

    assert_bool(!variant_toInt64(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_int64(&variant_string_false);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toUInt8() {
    uint8_t result = 0;

    assert_bool(!variant_toUInt8(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_uint8(&variant_text);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt8(&result,&variant_string_number));
    assert_bool(result == 0);
    result = variant_uint8(&variant_string_number);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt8(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_uint8(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt8(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_uint8(&variant_string_true);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt8(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_uint8(&variant_string_false);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toUInt16() {
    uint16_t result = 0;

    assert_bool(!variant_toUInt16(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_uint16(&variant_text);
    assert_bool(result == 0);

    assert_bool(variant_toUInt16(&result,&variant_string_number));
    assert_bool(result == (uint16_t)4096);
    result = variant_uint16(&variant_string_number);
    assert_bool(result == (uint16_t)4096);

    assert_bool(!variant_toUInt16(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_uint16(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt16(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_uint16(&variant_string_true);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt16(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_uint16(&variant_string_false);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toUInt32() {
    uint32_t result = 0;

    assert_bool(!variant_toUInt32(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_uint32(&variant_text);
    assert_bool(result == 0);

    assert_bool(variant_toUInt32(&result,&variant_string_number));
    assert_bool(result == (uint32_t)4096);
    result = variant_uint32(&variant_string_number);
    assert_bool(result == (uint32_t)4096);

    assert_bool(!variant_toUInt32(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_uint32(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt32(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_uint32(&variant_string_true);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt32(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_uint32(&variant_string_false);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toUInt64() {
    uint64_t result = 0;

    assert_bool(!variant_toUInt64(&result,&variant_text));
    assert_bool(result == 0);
    result = variant_uint64(&variant_text);
    assert_bool(result == 0);

    assert_bool(variant_toUInt64(&result,&variant_string_number));
    assert_bool(result == (uint64_t)4096);
    result = variant_uint64(&variant_string_number);
    assert_bool(result == (uint64_t)4096);

    assert_bool(!variant_toUInt64(&result,&variant_string_yes));
    assert_bool(result == 0);
    result = variant_uint64(&variant_string_yes);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt64(&result,&variant_string_true));
    assert_bool(result == 0);
    result = variant_uint64(&variant_string_true);
    assert_bool(result == 0);

    assert_bool(!variant_toUInt64(&result,&variant_string_false));
    assert_bool(result == 0);
    result = variant_uint64(&variant_string_false);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toBool() {
    bool result = false;

    assert_bool(!variant_toBool(&result,&variant_text));
    assert_bool(result == false);
    result = variant_bool(&variant_text);
    assert_bool(result == false);

    assert_bool(!variant_toBool(&result,&variant_string_number));
    assert_bool(result == false);
    result = variant_bool(&variant_string_number);
    assert_bool(result == false);

    assert_bool(variant_toBool(&result,&variant_string_yes));
    assert_bool(result == true);
    result = variant_bool(&variant_string_yes);
    assert_bool(result == true);

    assert_bool(variant_toBool(&result,&variant_string_true));
    assert_bool(result == true);
    result = variant_bool(&variant_string_true);
    assert_bool(result == true);

    assert_bool(variant_toBool(&result,&variant_string_false));
    assert_bool(result == false);
    result = variant_bool(&variant_string_false);
    assert_bool(result == false);

    sahtest_success();
}

int variant_string_toFd() {
    int result = 0;

    assert_bool(variant_toFd(&result,&variant_text) == false);
    result = variant_fd(&variant_text);
    assert_bool(result == -1);

    assert_bool(variant_toFd(&result,&variant_string_number) == false);
    result = variant_fd(&variant_string_number);
    assert_bool(result == -1);

    sahtest_success();
}

int variant_string_toDouble() {
    double result = 0;

    assert_bool(variant_toDouble(&result,&variant_text) == false);
    result = variant_double(&variant_text);
    assert_bool(result == 0);

    assert_bool(variant_toDouble(&result,&variant_string_number) == false);
    result = variant_double(&variant_string_number);
    assert_bool(result == 0);

    sahtest_success();
}

int variant_string_toList() {
    variant_list_t result;
    variant_list_t *result_list = NULL;
    variant_list_initialize(&result);

    assert_bool(variant_toList(&result,&variant_text) == true);
    assert_bool(variant_list_size(&result) == 1);
    result_list = variant_list(&variant_text);
    assert_bool(result_list != NULL);
    assert_bool(variant_list_size(result_list) == 1);

    variant_list_clear(&result);
    variant_list_clear(result_list);
    free(result_list);

    sahtest_success();
}

int variant_string_toMap() {
    variant_map_t result;
    variant_map_t *result_map = NULL;
    variant_map_initialize(&result);

    assert_bool(variant_toMap(&result,&variant_text) == false);
    assert_bool(variant_map_size(&result) == 0);
    result_map = variant_map(&variant_text);
    assert_bool(result_map == NULL);

    sahtest_success();
}
