/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant.h>

static variant_t variant_low;
static variant_t variant_high;

void variant_uint32_conversion_setup() {
    variant_initialize(&variant_low,variant_type_uint32);
    variant_initialize(&variant_high,variant_type_uint32);

    variant_setUInt32(&variant_low,INT32_MAX);
    variant_setUInt32(&variant_high,UINT32_MAX);
}

void variant_uint32_conversion_teardown() {
    variant_cleanup(&variant_low);
    variant_cleanup(&variant_high);
}


int variant_uint32_tostring() {
    string_t result;
    string_initialize(&result,0);

    assert_bool(variant_toString(&result,&variant_low));
    assert_bool(strcmp(string_buffer(&result),"2147483647") == 0);

    assert_bool(variant_toString(&result,&variant_high));
    assert_bool(strcmp(string_buffer(&result),"4294967295") == 0);

    string_cleanup(&result);
    sahtest_success();
}

int variant_uint32_tojson() {
    string_t result;
    string_initialize(&result,0);

    assert_bool(variant_toJSON(&result,&variant_low));
    assert_bool(strcmp(string_buffer(&result),"2147483647") == 0);

    assert_bool(variant_toJSON(&result,&variant_high));
    assert_bool(strcmp(string_buffer(&result),"4294967295") == 0);

    string_cleanup(&result);
    sahtest_success();
}

int variant_uint32_toChar() {
    char *result = NULL;

    assert_bool(variant_toChar(&result,&variant_low));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"2147483647") == 0);
    free(result);
    result = variant_char(&variant_low);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"2147483647") == 0);
    free(result);

    assert_bool(variant_toChar(&result,&variant_high));
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"4294967295") == 0);
    free(result);
    result = variant_char(&variant_high);
    assert_bool(result != NULL);
    assert_bool(strcmp(result,"4294967295") == 0);
    free(result);

    sahtest_success();
}

int variant_uint32_toInt8() {
    int8_t result = 0;

    assert_bool(variant_toInt8(&result,&variant_low));
    assert_bool(result == INT8_MAX);
    result = variant_int8(&variant_low);
    assert_bool(result == INT8_MAX);

    assert_bool(variant_toInt8(&result,&variant_high));
    assert_bool(result == INT8_MAX);
    result = variant_int8(&variant_high);
    assert_bool(result == INT8_MAX);

    sahtest_success();
}

int variant_uint32_toInt16() {
    int16_t result = 0;

    assert_bool(variant_toInt16(&result,&variant_low));
    assert_bool(result == INT16_MAX);
    result = variant_int16(&variant_low);
    assert_bool(result == INT16_MAX);

    assert_bool(variant_toInt16(&result,&variant_high));
    assert_bool(result == INT16_MAX);
    result = variant_int16(&variant_high);
    assert_bool(result == INT16_MAX);

    sahtest_success();
}

int variant_uint32_toInt32() {
    int32_t result = 0;

    assert_bool(variant_toInt32(&result,&variant_low));
    assert_bool(result == INT32_MAX);
    result = variant_int32(&variant_low);
    assert_bool(result == INT32_MAX);

    assert_bool(variant_toInt32(&result,&variant_high));
    assert_bool(result == INT32_MAX);
    result = variant_int32(&variant_high);
    assert_bool(result == INT32_MAX);

    sahtest_success();
}

int variant_uint32_toInt64() {
    int64_t result = 0;

    assert_bool(variant_toInt64(&result,&variant_low));
    assert_bool(result == INT32_MAX);
    result = variant_int64(&variant_low);
    assert_bool(result == INT32_MAX);

    assert_bool(variant_toInt64(&result,&variant_high));
    assert_bool(result == UINT32_MAX);
    result = variant_int64(&variant_high);
    assert_bool(result == UINT32_MAX);

    sahtest_success();
}

int variant_uint32_toUInt8() {
    uint8_t result = 0;

    assert_bool(variant_toUInt8(&result,&variant_low));
    assert_bool(result == UINT8_MAX);
    result = variant_uint8(&variant_low);
    assert_bool(result == UINT8_MAX);

    assert_bool(variant_toUInt8(&result,&variant_high));
    assert_bool(result == UINT8_MAX);
    result = variant_uint8(&variant_high);
    assert_bool(result == UINT8_MAX);

    sahtest_success();
}

int variant_uint32_toUInt16() {
    uint16_t result = 0;

    assert_bool(variant_toUInt16(&result,&variant_low));
    assert_bool(result == UINT16_MAX);
    result = variant_uint16(&variant_low);
    assert_bool(result == UINT16_MAX);

    assert_bool(variant_toUInt16(&result,&variant_high));
    assert_bool(result == UINT16_MAX);
    result = variant_uint16(&variant_high);
    assert_bool(result == UINT16_MAX);

    sahtest_success();
}

int variant_uint32_toUInt32() {
    uint32_t result = 0;

    assert_bool(variant_toUInt32(&result,&variant_low));
    assert_bool(result == INT32_MAX);
    result = variant_uint32(&variant_low);
    assert_bool(result == INT32_MAX);

    assert_bool(variant_toUInt32(&result,&variant_high));
    assert_bool(result == UINT32_MAX);
    result = variant_uint32(&variant_high);
    assert_bool(result == UINT32_MAX);

    sahtest_success();
}

int variant_uint32_toUInt64() {
    uint64_t result = 0;

    assert_bool(variant_toUInt64(&result,&variant_low));
    assert_bool(result == 2147483647);
    result = variant_uint64(&variant_low);
    assert_bool(result == 2147483647);

    assert_bool(variant_toUInt64(&result,&variant_high));
    assert_bool(result == UINT32_MAX);
    result = variant_uint64(&variant_high);
    assert_bool(result == UINT32_MAX);

    sahtest_success();
}

int variant_uint32_toBool() {
    bool result = false;

    assert_bool(variant_toBool(&result,&variant_low));
    assert_bool(result == true);
    result = variant_bool(&variant_low);
    assert_bool(result == true);

    assert_bool(variant_toBool(&result,&variant_high));
    assert_bool(result == true);
    result = variant_bool(&variant_high);
    assert_bool(result == true);

    sahtest_success();
}
