/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/variant_map.h>

static variant_t dest;
static variant_t src;

void variant_copy_setup() {
    variant_initialize(&dest,variant_type_unknown);
    variant_initialize(&src,variant_type_unknown);
}

void variant_copy_teardown() {
    variant_cleanup(&dest);
    variant_cleanup(&src);
}

int variant_string_copy() {
    char *txt = NULL;

    assert_bool(variant_setChar(&src,"Hallo world") == true);

    variant_copy(&dest,&src);
    assert_bool(variant_type(&dest) == variant_type_string);
    txt = variant_char(&dest);
    assert_bool(strcmp(txt,"Hallo world") == 0);
    free(txt);

    sahtest_success();
}

int variant_int8_copy() {
    assert_bool(variant_setInt8(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_int8);
    assert_bool(variant_int8(&dest) == 127);

    sahtest_success();
}

int variant_int16_copy() {
    assert_bool(variant_setInt16(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_int16);
    assert_bool(variant_int16(&dest) == 127);

    sahtest_success();
}

int variant_int32_copy() {
    assert_bool(variant_setInt32(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_int32);
    assert_bool(variant_int32(&dest) == 127);

    sahtest_success();
}

int variant_int64_copy() {
    assert_bool(variant_setInt64(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_int64);
    assert_bool(variant_int64(&dest) == 127);

    sahtest_success();
}

int variant_uint8_copy() {
    assert_bool(variant_setUInt8(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_uint8);
    assert_bool(variant_uint8(&dest) == 127);

    sahtest_success();
}

int variant_uint16_copy() {
    assert_bool(variant_setUInt16(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_uint16);
    assert_bool(variant_uint16(&dest) == 127);

    sahtest_success();
}

int variant_uint32_copy() {
    assert_bool(variant_setUInt32(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_uint32);
    assert_bool(variant_uint32(&dest) == 127);

    sahtest_success();
}

int variant_uint64_copy() {
    assert_bool(variant_setUInt64(&src,127) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_uint64);
    assert_bool(variant_uint64(&dest) == 127);

    sahtest_success();
}

int variant_bool_copy() {
    assert_bool(variant_setBool(&src,true) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_bool);
    assert_bool(variant_bool(&dest) == true);

    sahtest_success();
}

int variant_double_copy() {
    assert_bool(variant_setDouble(&src,100.25) == true);

    variant_copy(&dest,&src);

    assert_bool(variant_type(&dest) == variant_type_double);
    assert_bool(variant_double(&dest) == 100.25);

    sahtest_success();
}

int variant_date_time_copy() {
    time_t now;
    time(&now);
    struct tm *t = gmtime(&now);

    assert_bool(variant_setDateTime(&src,t) == true);
    variant_copy(&dest,&src);
    assert_bool(variant_type(&dest) == variant_type_date_time);

    sahtest_success();
}

int variant_list_copy() {
    assert_bool(variant_initialize(&src,variant_type_array));

    variant_list_t *list = variant_da_list(&src);
    assert_bool(list != NULL);

    variant_list_addUInt32(list,1);
    variant_list_addUInt32(list,2);

    variant_copy(&dest,&src);

    assert_bool(variant_list_size(list) == 2);
    assert_bool(variant_type(&dest) == variant_type_array);
    list = variant_da_list(&dest);
    assert_bool(list != NULL);
    assert_bool(variant_list_size(list) == 2);

    sahtest_success();
}

int variant_map_copy() {
    assert_bool(variant_initialize(&src,variant_type_map));

    variant_map_t *map = variant_da_map(&src);
    assert_bool(map != NULL);

    variant_map_addUInt32(map,"1",1);
    variant_map_addUInt32(map,"2",2);

    variant_copy(&dest,&src);

    assert_bool(variant_map_size(map) == 2);
    assert_bool(variant_type(&dest) == variant_type_map);
    map = variant_da_map(&dest);
    assert_bool(map != NULL);
    assert_bool(variant_map_size(map) == 2);

    sahtest_success();
}

int variant_fd_copy() {
    assert_bool(variant_initialize(&src,variant_type_file_descriptor));

    variant_copy(&dest,&src);
    assert_bool(variant_type(&dest) == variant_type_file_descriptor);
    assert_bool(variant_fd(&dest) == -1);

    sahtest_success();
}

int variant_byte_array_copy() {
    char data[10] = "abcdefghij";
    variant_setByteArray(&src,data,10);
    uint32_t size = 0;

    variant_copy(&dest,&src);
    assert_bool(variant_type(&dest) == variant_type_byte_array);
    void *d2 = variant_byteArray(&dest,&size);
    assert_bool(size == 10);
    assert_bool(memcmp(d2,data,size) == 0);

    free(d2);
    sahtest_success();
}

int variant_null_copy() {
    variant_copy(NULL,&src);
    variant_copy(&dest,NULL);

    sahtest_success();
}
