/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant.h>

static variant_t var_bytearray;
static variant_t var_string;
static variant_t var_datetime;
static variant_t var_list;
static variant_t var_map;
static variant_t var_integer;
static void *data;

void variant_da_setup() {
    data = calloc(1,10);
    variant_initialize(&var_bytearray,variant_type_byte_array);
    variant_setByteArray(&var_bytearray,data,10);
    variant_initialize(&var_string,variant_type_string);
    variant_setChar(&var_string,"Hello world");
    variant_initialize(&var_datetime,variant_type_date_time);
    variant_initialize(&var_list,variant_type_array);
    variant_initialize(&var_map,variant_type_map);
    variant_initialize(&var_integer,variant_type_int32);
}

void variant_da_teardown() {
    variant_cleanup(&var_bytearray);
    variant_cleanup(&var_string);
    variant_cleanup(&var_datetime);
    variant_cleanup(&var_list);
    variant_cleanup(&var_map);
    variant_cleanup(&var_integer);
    free(data);
}

int variant_check_da_bytearray() {
    uint32_t size = 0;

    assert_bool(variant_da_byteArray(&var_bytearray,&size) != NULL);
    assert_bool(variant_da_byteArray(&var_string,&size) == NULL);
    assert_bool(variant_da_byteArray(&var_datetime,&size) == NULL);
    assert_bool(variant_da_byteArray(&var_list,&size) == NULL);
    assert_bool(variant_da_byteArray(&var_map,&size) == NULL);
    assert_bool(variant_da_byteArray(&var_integer,&size) == NULL);
    assert_bool(variant_da_byteArray(NULL,&size) == NULL);

    sahtest_success();
}

int variant_check_da_char() {
    assert_bool(variant_da_char(&var_bytearray) == NULL);
    assert_bool(variant_da_char(&var_string) != NULL);
    assert_bool(variant_da_char(&var_datetime) == NULL);
    assert_bool(variant_da_char(&var_list) == NULL);
    assert_bool(variant_da_char(&var_map) == NULL);
    assert_bool(variant_da_char(&var_integer) == NULL);
    assert_bool(variant_da_char(NULL) == NULL);

    sahtest_success();
}

int variant_check_da_datetime() {
    assert_bool(variant_da_dateTime(&var_bytearray) == NULL);
    assert_bool(variant_da_dateTime(&var_string) == NULL);
    assert_bool(variant_da_dateTime(&var_datetime) != NULL);
    assert_bool(variant_da_dateTime(&var_list) == NULL);
    assert_bool(variant_da_dateTime(&var_map) == NULL);
    assert_bool(variant_da_dateTime(&var_integer) == NULL);
    assert_bool(variant_da_dateTime(NULL) == NULL);

    sahtest_success();
}

int variant_check_da_list() {
    assert_bool(variant_da_list(&var_bytearray) == NULL);
    assert_bool(variant_da_list(&var_string) == NULL);
    assert_bool(variant_da_list(&var_datetime) == NULL);
    assert_bool(variant_da_list(&var_list) != NULL);
    assert_bool(variant_da_list(&var_map) == NULL);
    assert_bool(variant_da_list(&var_integer) == NULL);
    assert_bool(variant_da_list(NULL) == NULL);

    sahtest_success();
}

int variant_check_da_map() {
    assert_bool(variant_da_map(&var_bytearray) == NULL);
    assert_bool(variant_da_map(&var_string) == NULL);
    assert_bool(variant_da_map(&var_datetime) == NULL);
    assert_bool(variant_da_map(&var_list) == NULL);
    assert_bool(variant_da_map(&var_map) != NULL);
    assert_bool(variant_da_map(&var_integer) == NULL);
    assert_bool(variant_da_map(NULL) == NULL);

    sahtest_success();
}

int variant_check_da_string() {
    assert_bool(variant_da_string(&var_bytearray) == NULL);
    assert_bool(variant_da_string(&var_string) != NULL);
    assert_bool(variant_da_string(&var_datetime) == NULL);
    assert_bool(variant_da_string(&var_list) == NULL);
    assert_bool(variant_da_string(&var_map) == NULL);
    assert_bool(variant_da_string(&var_integer) == NULL);
    assert_bool(variant_da_string(NULL) == NULL);

    sahtest_success();
}
