/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/variant_map.h>

static variant_list_t list;
static variant_t var;

void variant_list_add_setup() {
    variant_list_initialize(&list);
    variant_initialize(&var,variant_type_unknown);
    variant_setInt32(&var,1024);
}

void variant_list_add_teardown() {
    variant_list_cleanup(&list);
}

int variant_list_add_variant() {
    assert_bool(variant_list_add(&list,&var) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_int32);

    assert_bool(variant_list_add(NULL,&var) == false);

    sahtest_success();
}

int variant_list_add_char() {
    assert_bool(variant_list_addChar(&list,"Hallo World") == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_string);

    assert_bool(variant_list_addChar(NULL,"Hallo World") == false);

    sahtest_success();
}

int variant_list_add_string() {
    string_t txt;
    string_initialize(&txt,0);
    string_fromChar(&txt,"Hallo World");

    assert_bool(variant_list_addString(&list,&txt) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_string);

    assert_bool(variant_list_addString(NULL,&txt) == false);

    string_cleanup(&txt);
    sahtest_success();
}

int variant_list_add_int8() {
    assert_bool(variant_list_addInt8(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_int8);

    assert_bool(variant_list_addInt8(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_int16() {
    assert_bool(variant_list_addInt16(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_int16);

    assert_bool(variant_list_addInt16(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_int32() {
    assert_bool(variant_list_addInt32(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_int32);

    assert_bool(variant_list_addInt32(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_int64() {
    assert_bool(variant_list_addInt64(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_int64);

    assert_bool(variant_list_addInt64(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_uint8() {
    assert_bool(variant_list_addUInt8(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_uint8);

    assert_bool(variant_list_addUInt8(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_uint16() {
    assert_bool(variant_list_addUInt16(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_uint16);

    assert_bool(variant_list_addUInt16(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_uint32() {
    assert_bool(variant_list_addUInt32(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_uint32);

    assert_bool(variant_list_addUInt32(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_uint64() {
    assert_bool(variant_list_addUInt64(&list,127) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_uint64);

    assert_bool(variant_list_addUInt64(NULL,127) == false);

    sahtest_success();
}

int variant_list_add_bool() {
    assert_bool(variant_list_addBool(&list,true) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_bool);

    assert_bool(variant_list_addBool(NULL,true) == false);

    sahtest_success();
}

int variant_list_add_double() {
    assert_bool(variant_list_addDouble(&list,125.125) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_double);

    assert_bool(variant_list_addDouble(NULL,125.125) == false);

    sahtest_success();
}

int variant_list_add_date_time() {
    time_t now;
    time(&now);
    struct tm *t = gmtime(&now);

    assert_bool(variant_list_addDateTime(&list,t) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_date_time);

    assert_bool(variant_list_addDateTime(NULL,t) == false);

    sahtest_success();
}

int variant_list_add_list_move() {
    variant_list_t l;
    assert_bool(variant_list_initialize(&l) == true);
    variant_list_addUInt32(&l,1);
    variant_list_addUInt32(&l,2);

    assert_bool(variant_list_addListMove(&list,&l) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_array);

    assert_bool(variant_list_addListMove(NULL,&l) == false);

    variant_list_cleanup(&l);

    sahtest_success();
}

int variant_list_add_list_copy() {
    variant_list_t l;
    assert_bool(variant_list_initialize(&l) == true);
    variant_list_addUInt32(&l,1);
    variant_list_addUInt32(&l,2);

    assert_bool(variant_list_addListCopy(&list,&l) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_array);

    assert_bool(variant_list_addListCopy(NULL,&l) == false);

    variant_list_cleanup(&l);

    sahtest_success();
}

int variant_list_add_map_move() {
    variant_map_t l;
    assert_bool(variant_list_initialize(&l) == true);
    variant_map_addUInt32(&l,"1",1);
    variant_map_addUInt32(&l,"2",2);

    assert_bool(variant_list_addMapMove(&list,&l) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_map);

    assert_bool(variant_list_addMapMove(NULL,&l) == false);

    variant_map_cleanup(&l);

    sahtest_success();
}

int variant_list_add_map_copy() {
    variant_map_t l;
    assert_bool(variant_map_initialize(&l) == true);
    variant_map_addUInt32(&l,"1",1);
    variant_map_addUInt32(&l,"2",2);

    assert_bool(variant_list_addMapCopy(&list,&l) == true);
    assert_bool(variant_list_isEmpty(&list) == false);
    assert_bool(variant_list_size(&list) == 1);

    variant_list_iterator_t *it = variant_list_first(&list);
    assert_bool(it != NULL);
    assert_bool(variant_type(variant_list_iterator_data(it)) == variant_type_map);

    assert_bool(variant_list_addMapCopy(NULL,&l) == false);

    variant_map_cleanup(&l);

    sahtest_success();
}
