/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_list.h>

static variant_list_t list;
static variant_list_iterator_t *it[5];
static variant_list_iterator_t *orphan;

void variant_list_append_setup() {
    int i;
    variant_list_initialize(&list);
    for(i = 0; i <5; i++) {
        it[i] = variant_list_iterator_create(NULL);
    }
    orphan = variant_list_iterator_create(NULL);
}

void variant_list_append_teardown() {
    int i;
    for(i = 0; i <5; i++) {
        variant_list_iterator_destroy(it[i]);
    }
    variant_list_iterator_destroy(orphan);

    variant_list_cleanup(&list);
}

int variant_list_append_null_list() {
    T_VERIFY(!variant_list_append(NULL,orphan));

    return 0;
}

int variant_list_append_null_it() {
    T_VERIFY(!variant_list_append(&list,NULL));

    return 0;
}

int variant_list_append_orphan() {
    T_VERIFY(variant_list_append(&list,orphan));

    T_VERIFY(variant_list_size(&list) == 1);
    T_VERIFY(variant_list_first(&list) == orphan);
    T_VERIFY(variant_list_last(&list) == orphan);

    return 0;
}

int variant_list_append_iterator_twice() {
    T_VERIFY(variant_list_append(&list,orphan));
    T_VERIFY(variant_list_append(&list,orphan));

    T_VERIFY(variant_list_size(&list) == 1);
    T_VERIFY(variant_list_first(&list) == orphan);
    T_VERIFY(variant_list_last(&list) == orphan);

    return 0;
}

int variant_list_append_multiple() {
    int i = 0;
    for(i = 0; i < 5; i++) {
        T_VERIFY(variant_list_append(&list,it[i]));
    }
    T_VERIFY(variant_list_size(&list) == 5);

    T_VERIFY(variant_list_first(&list) == it[0]);
    T_VERIFY(variant_list_last(&list) == it[4]);

    T_VERIFY(variant_list_iterator_next(it[0]) == it[1]);
    T_VERIFY(variant_list_iterator_next(it[1]) == it[2]);
    T_VERIFY(variant_list_iterator_next(it[2]) == it[3]);
    T_VERIFY(variant_list_iterator_next(it[3]) == it[4]);
    T_VERIFY(variant_list_iterator_next(it[4]) == NULL);

    T_VERIFY(variant_list_iterator_prev(it[0]) == NULL);
    T_VERIFY(variant_list_iterator_prev(it[1]) == it[0]);
    T_VERIFY(variant_list_iterator_prev(it[2]) == it[1]);
    T_VERIFY(variant_list_iterator_prev(it[3]) == it[2]);
    T_VERIFY(variant_list_iterator_prev(it[4]) == it[3]);

    return 0;
}
