/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_list.h>

static variant_list_t list;
static variant_t integer_in_list;
static variant_t integer_not_in_list;
static variant_t text;

void variant_list_contains_setup() {
    variant_setUInt32(&integer_in_list,100);
    variant_setUInt32(&integer_not_in_list,150);
    variant_setChar(&text,"-100");

    variant_list_initialize(&list);
    variant_list_addInt16(&list,-300);
    variant_list_addInt16(&list,-200);
    variant_list_addInt16(&list,-100);
    variant_list_addInt16(&list,0);
    variant_list_addInt16(&list,100);
    variant_list_addInt16(&list,200);
    variant_list_addInt16(&list,300);
}

void variant_list_contains_teardown() {
    variant_cleanup(&integer_in_list);
    variant_cleanup(&integer_not_in_list);
    variant_cleanup(&text);

    variant_list_cleanup(&list);
}

int variant_list_contains_search_valid_item() {
    assert_bool(variant_list_contains(&list,&integer_in_list) == true);
    assert_bool(variant_list_contains(&list,&text) == true);

    sahtest_success();
}

int variant_list_contains_search_invalid_item() {
    assert_bool(variant_list_contains(&list,&integer_not_in_list) == false);

    sahtest_success();
}

int variant_list_contains_null_variant() {
    assert_bool(variant_list_contains(&list,NULL) == false);

    sahtest_success();
}

int variant_list_contains_using_null_list() {
    assert_bool(variant_list_contains(NULL,&integer_in_list) == false);

    sahtest_success();
}

