/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_list.h>

static variant_list_t list;
static variant_list_t emptyList;
static variant_list_iterator_t *it[5];
static variant_list_iterator_t *orphan;

void variant_list_iterating_setup() {
    int i;
    variant_list_initialize(&list);
    variant_list_initialize(&emptyList);
    for(i = 0; i <5; i++) {
        it[i] = variant_list_iterator_create(NULL);
        variant_list_append(&list,it[i]);
    }
    orphan = variant_list_iterator_create(NULL);
}

void variant_list_iterating_teardown() {
    variant_list_iterator_take(orphan);
    variant_list_iterator_destroy(orphan);

    variant_list_cleanup(&list);
    variant_list_cleanup(&emptyList);
}

int variant_list_iterating_first_null() {
    T_VERIFY(variant_list_first(NULL) == NULL);

    return 0;
}

int variant_list_iterating_first_empty_list() {
    T_VERIFY(variant_list_first(&emptyList) == NULL);

    return 0;
}

int variant_list_iterating_last_null() {
    T_VERIFY(variant_list_last(NULL) == NULL);

    return 0;
}

int variant_list_iterating_last_empty_list() {
    T_VERIFY(variant_list_last(&emptyList) == NULL);

    return 0;
}

int variant_list_iterating_next_null() {
    T_VERIFY(variant_list_iterator_next(NULL) == NULL);

    return 0;
}

int variant_list_iterating_prev_null() {
    T_VERIFY(variant_list_iterator_prev(NULL) == NULL);

    return 0;
}

int variant_list_iterating_first() {
    T_VERIFY(variant_list_first(&list) == it[0]);

    return 0;
}

int variant_list_iterating_last() {
    T_VERIFY(variant_list_last(&list) == it[4]);

    return 0;
}

int variant_list_iterating_first_prev() {
    variant_list_iterator_t *iter = variant_list_first(&list);
    T_VERIFY(variant_list_iterator_prev(iter) == NULL);

    return 0;
}

int variant_list_iterating_last_next() {
    variant_list_iterator_t *iter = variant_list_last(&list);
    T_VERIFY(variant_list_iterator_next(iter) == NULL);

    return 0;
}

int variant_list_iterating_first_next() {
    variant_list_iterator_t *iter = variant_list_first(&list);
    T_VERIFY(variant_list_iterator_next(iter) == it[1]);

    return 0;
}

int variant_list_iterating_last_prev() {
    variant_list_iterator_t *iter = variant_list_last(&list);
    T_VERIFY(variant_list_iterator_prev(iter) == it[3]);

    return 0;
}

int variant_list_iterating_orphan_next() {
    T_VERIFY(variant_list_iterator_next(orphan) == NULL);

    return 0;
}

int variant_list_iterating_orphan_prev() {
    T_VERIFY(variant_list_iterator_prev(orphan) == NULL);

    return 0;
}

int variant_list_iterating_loop_forward() {
    variant_list_iterator_t *iter = NULL;
    int i = 0;
    variant_list_for_each(iter, &list) {
        T_VERIFY(i < 5);
        T_VERIFY(iter == it[i]);
        i++;
    }

    return 0;
}

int variant_list_iterating_loop_reverse() {
    variant_list_iterator_t *iter = NULL;
    int i = 4;
    variant_list_for_each_reverse(iter, &list) {
        T_VERIFY(i >= 0);
        T_VERIFY(iter == it[i]);
        i--;
    }

    return 0;
}

int variant_list_indexed_access() {
    int i = 0;
    for(i = 0; i < 5; i++) {
        T_VERIFY(variant_list_at(&list,i) == it[i]);
    }

    return 0;
}

int variant_list_indexed_access_null_list() {
    T_VERIFY(variant_list_at(NULL,5) == NULL);

    return 0;
}

int variant_list_indexed_access_out_of_boundry() {
    T_VERIFY(variant_list_at(&list,10) == NULL);

    return 0;
}
