/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_list.h>

static variant_list_t list;
static variant_list_t emptyList;
static variant_list_iterator_t *it[5];
static variant_list_iterator_t *orphan;

void variant_list_remove_setup() {
    int i;
    variant_list_initialize(&list);
    variant_list_initialize(&emptyList);
    for(i = 0; i <5; i++) {
        it[i] = variant_list_iterator_create(NULL);
        variant_list_append(&list,it[i]);
    }
    orphan = variant_list_iterator_create(NULL);
}

void variant_list_remove_teardown() {
    int i;
    for(i = 0; i <5; i++) {
        variant_list_iterator_destroy(it[i]);
    }
    variant_list_iterator_destroy(orphan);

    variant_list_cleanup(&list);
    variant_list_cleanup(&emptyList);
}

int variant_list_remove_take_first_null() {
    T_VERIFY(variant_list_takeFirst(NULL) == NULL);
    return 0;
}

int variant_list_remove_take_last_null() {
    T_VERIFY(variant_list_takeLast(NULL) == NULL);
    return 0;
}

int variant_list_remove_take_first_empty_list() {
    T_VERIFY(variant_list_takeFirst(&emptyList) == NULL);

    return 0;
}

int variant_list_remove_take_last_empty_list() {
    T_VERIFY(variant_list_takeLast(&emptyList) == NULL);
    return 0;
}

int variant_list_remove_take_first() {
    variant_list_iterator_t *taken = variant_list_takeFirst(&list);
    T_VERIFY(taken == it[0]);
    T_VERIFY(variant_list_iterator_next(taken) == NULL);
    T_VERIFY(variant_list_iterator_prev(taken) == NULL);
    T_VERIFY(variant_list_first(&list) == it[1]);

    return 0;
}

int variant_list_remove_take_last() {
    variant_list_iterator_t *taken = variant_list_takeLast(&list);
    T_VERIFY(taken == it[4]);
    T_VERIFY(variant_list_iterator_next(taken) == NULL);
    T_VERIFY(variant_list_iterator_prev(taken) == NULL);
    T_VERIFY(variant_list_last(&list) == it[3]);

    return 0;
}

int variant_list_remove_take_null() {
    T_VERIFY(variant_list_iterator_take(NULL) == NULL);

    return 0;
}
