/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_map.h>

static variant_map_t map;
static variant_map_t emptymap;
static variant_map_iterator_t *it[5];
static variant_map_iterator_t *orphan;

void variant_map_insert_build_map_setup() {
    int i;
    variant_map_initialize(&map);
    variant_map_initialize(&emptymap);
    for(i = 0; i <5; i++) {
        it[i] = variant_map_iterator_create("test",NULL);
        variant_map_append(&map,it[i]);
    }
    orphan = variant_map_iterator_create("test",NULL);
}

void variant_map_insert_build_map_teardown() {
    int i;
    for(i = 0; i <5; i++) {
        variant_map_iterator_destroy(it[i]);
    }
    variant_map_iterator_destroy(orphan);

    variant_map_cleanup(&map);
    variant_map_cleanup(&emptymap);
}

int variant_map_insert_indexed_null_map() {
    T_VERIFY(!variant_map_insertAt(NULL,0,orphan));
    T_VERIFY(!variant_map_insertAt(NULL,10,orphan));

    return 0;
}

int variant_map_insert_indexed_empty_map_index_0() {
    T_VERIFY(variant_map_size(&emptymap) == 0);
    T_VERIFY(variant_map_insertAt(&emptymap,0,orphan));
    T_VERIFY(variant_map_iterator_next(orphan) == NULL);
    T_VERIFY(variant_map_iterator_prev(orphan) == NULL);
    T_VERIFY(variant_map_first(&emptymap) == orphan);
    T_VERIFY(variant_map_last(&emptymap) == orphan);

    return 0;
}

int variant_map_insert_indexed_empty_map_index_2() {
    T_VERIFY(variant_map_size(&emptymap) == 0);
    T_VERIFY(variant_map_insertAt(&emptymap,0,orphan));
    T_VERIFY(variant_map_iterator_next(orphan) == NULL);
    T_VERIFY(variant_map_iterator_prev(orphan) == NULL);
    T_VERIFY(variant_map_first(&emptymap) == orphan);
    T_VERIFY(variant_map_last(&emptymap) == orphan);

    return 0;
}

int variant_map_insert_indexed_map_index_out_of_boundry() {
    unsigned int size = variant_map_size(&map);
    T_VERIFY(variant_map_insertAt(&map,size + 10,orphan));
    T_VERIFY(variant_map_iterator_next(orphan) == NULL);
    T_VERIFY(variant_map_iterator_prev(orphan) == it[4]);

    return 0;
}

int variant_map_insert_indexed_map_index_0() {
    T_VERIFY(variant_map_insertAt(&map,0,orphan));
    T_VERIFY(variant_map_iterator_next(orphan) == it[0]);
    T_VERIFY(variant_map_iterator_prev(orphan) == NULL);

    return 0;
}

int variant_map_insert_indexed_map_index_2() {
    T_VERIFY(variant_map_insertAt(&map,2,orphan));
    T_VERIFY(variant_map_iterator_next(orphan) == it[2]);
    T_VERIFY(variant_map_iterator_prev(orphan) == it[1]);

    return 0;
}

int variant_map_insert_indexed_map_index_last() {
    unsigned int size = variant_map_size(&map);
    T_VERIFY(variant_map_insertAt(&map,size - 1,orphan));
    T_VERIFY(variant_map_iterator_next(orphan) == it[4]);
    T_VERIFY(variant_map_iterator_prev(orphan) == it[3]);

    return 0;
}
