/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_map.h>

static variant_map_t map;
static variant_map_t emptymap;
static variant_map_iterator_t *it[5];
static variant_map_iterator_t *orphan;

void variant_map_iterating_setup() {
    int i;
    variant_map_initialize(&map);
    variant_map_initialize(&emptymap);
    for(i = 0; i <5; i++) {
        it[i] = variant_map_iterator_create("test",NULL);
        variant_map_append(&map,it[i]);
    }
    orphan = variant_map_iterator_create("test",NULL);
}

void variant_map_iterating_teardown() {
    variant_map_iterator_take(orphan);
    variant_map_iterator_destroy(orphan);

    variant_map_cleanup(&map);
    variant_map_cleanup(&emptymap);
}

int variant_map_iterating_first_null() {
    T_VERIFY(variant_map_first(NULL) == NULL);

    return 0;
}

int variant_map_iterating_first_empty_map() {
    T_VERIFY(variant_map_first(&emptymap) == NULL);

    return 0;
}

int variant_map_iterating_last_null() {
    T_VERIFY(variant_map_last(NULL) == NULL);

    return 0;
}

int variant_map_iterating_last_empty_map() {
    T_VERIFY(variant_map_last(&emptymap) == NULL);

    return 0;
}

int variant_map_iterating_next_null() {
    T_VERIFY(variant_map_iterator_next(NULL) == NULL);

    return 0;
}

int variant_map_iterating_prev_null() {
    T_VERIFY(variant_map_iterator_prev(NULL) == NULL);

    return 0;
}

int variant_map_iterating_first() {
    T_VERIFY(variant_map_first(&map) == it[0]);

    return 0;
}

int variant_map_iterating_last() {
    T_VERIFY(variant_map_last(&map) == it[4]);

    return 0;
}

int variant_map_iterating_first_prev() {
    variant_map_iterator_t *iter = variant_map_first(&map);
    T_VERIFY(variant_map_iterator_prev(iter) == NULL);

    return 0;
}

int variant_map_iterating_last_next() {
    variant_map_iterator_t *iter = variant_map_last(&map);
    T_VERIFY(variant_map_iterator_next(iter) == NULL);

    return 0;
}

int variant_map_iterating_first_next() {
    variant_map_iterator_t *iter = variant_map_first(&map);
    T_VERIFY(variant_map_iterator_next(iter) == it[1]);

    return 0;
}

int variant_map_iterating_last_prev() {
    variant_map_iterator_t *iter = variant_map_last(&map);
    T_VERIFY(variant_map_iterator_prev(iter) == it[3]);

    return 0;
}

int variant_map_iterating_orphan_next() {
    T_VERIFY(variant_map_iterator_next(orphan) == NULL);

    return 0;
}

int variant_map_iterating_orphan_prev() {
    T_VERIFY(variant_map_iterator_prev(orphan) == NULL);

    return 0;
}

int variant_map_iterating_loop_forward() {
    variant_map_iterator_t *iter = NULL;
    int i = 0;
    variant_map_for_each(iter, &map) {
        T_VERIFY(i < 5);
        T_VERIFY(iter == it[i]);
        i++;
    }

    return 0;
}

int variant_map_iterating_loop_reverse() {
    variant_map_iterator_t *iter = NULL;
    int i = 4;
    variant_map_for_each_reverse(iter, &map) {
        T_VERIFY(i >= 0);
        T_VERIFY(iter == it[i]);
        i--;
    }

    return 0;
}

int variant_map_indexed_access() {
    int i = 0;
    for(i = 0; i < 5; i++) {
        T_VERIFY(variant_map_at(&map,i) == it[i]);
    }

    return 0;
}

int variant_map_indexed_access_null_map() {
    T_VERIFY(variant_map_at(NULL,5) == NULL);

    return 0;
}

int variant_map_indexed_access_out_of_boundry() {
    T_VERIFY(variant_map_at(&map,10) == NULL);

    return 0;
}
