/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_map.h>

static variant_map_t map;
static variant_map_iterator_t *it[5];
static variant_map_iterator_t *orphan;

void variant_map_prepend_setup() {
    int i;
    variant_map_initialize(&map);
    for(i = 0; i <5; i++) {
        it[i] = variant_map_iterator_create("test",NULL);
    }
    orphan = variant_map_iterator_create("test",NULL);
}

void variant_map_prepend_teardown() {
    int i;
    for(i = 0; i <5; i++) {
        variant_map_iterator_destroy(it[i]);
    }
    variant_map_iterator_destroy(orphan);

    variant_map_cleanup(&map);
}

int variant_map_prepend_null_map() {
    T_VERIFY(!variant_map_prepend(NULL,orphan));

    return 0;
}

int variant_map_prepend_null_it() {
    T_VERIFY(!variant_map_prepend(&map,NULL));

    return 0;
}

int variant_map_prepend_orphan() {
    T_VERIFY(variant_map_prepend(&map,orphan));

    T_VERIFY(variant_map_size(&map) == 1);
    T_VERIFY(variant_map_first(&map) == orphan);
    T_VERIFY(variant_map_last(&map) == orphan);

    return 0;
}

int variant_map_prepend_iterator_twice() {
    T_VERIFY(variant_map_prepend(&map,orphan));
    T_VERIFY(variant_map_prepend(&map,orphan));

    T_VERIFY(variant_map_size(&map) == 1);
    T_VERIFY(variant_map_first(&map) == orphan);
    T_VERIFY(variant_map_last(&map) == orphan);

    return 0;
}

int variant_map_prepend_multiple() {
    int i = 0;
    for(i = 0; i < 5; i++) {
        T_VERIFY(variant_map_prepend(&map,it[i]));
    }
    T_VERIFY(variant_map_size(&map) == 5);

    T_VERIFY(variant_map_first(&map) == it[4]);
    T_VERIFY(variant_map_last(&map) == it[0]);

    T_VERIFY(variant_map_iterator_next(it[4]) == it[3]);
    T_VERIFY(variant_map_iterator_next(it[3]) == it[2]);
    T_VERIFY(variant_map_iterator_next(it[2]) == it[1]);
    T_VERIFY(variant_map_iterator_next(it[1]) == it[0]);
    T_VERIFY(variant_map_iterator_next(it[0]) == NULL);

    T_VERIFY(variant_map_iterator_prev(it[4]) == NULL);
    T_VERIFY(variant_map_iterator_prev(it[3]) == it[4]);
    T_VERIFY(variant_map_iterator_prev(it[2]) == it[3]);
    T_VERIFY(variant_map_iterator_prev(it[1]) == it[2]);
    T_VERIFY(variant_map_iterator_prev(it[0]) == it[1]);

    return 0;
}
