/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#include <sahtest/sahtest.h>
#include <pcb/utils/variant_map.h>

static variant_map_t map;
static variant_map_t emptymap;
static variant_map_iterator_t *it[5];
static variant_map_iterator_t *orphan;

void variant_map_remove_setup() {
    int i;
    variant_map_initialize(&map);
    variant_map_initialize(&emptymap);
    for(i = 0; i <5; i++) {
        it[i] = variant_map_iterator_create("test",NULL);
        variant_map_append(&map,it[i]);
    }
    orphan = variant_map_iterator_create("test",NULL);
}

void variant_map_remove_teardown() {
    int i;
    variant_map_iterator_destroy(orphan);

    for(i = 0; i <5; i++) {
        variant_map_iterator_destroy(it[i]);
    }

    variant_map_cleanup(&map);
    variant_map_cleanup(&emptymap);
}

int variant_map_remove_take_first_null() {
    T_VERIFY(variant_map_takeFirst(NULL) == NULL);
    return 0;
}

int variant_map_remove_take_last_null() {
    T_VERIFY(variant_map_takeLast(NULL) == NULL);
    return 0;
}

int variant_map_remove_take_first_empty_map() {
    T_VERIFY(variant_map_takeFirst(&emptymap) == NULL);

    return 0;
}

int variant_map_remove_take_last_empty_map() {
    T_VERIFY(variant_map_takeLast(&emptymap) == NULL);
    return 0;
}

int variant_map_remove_take_first() {
    variant_map_iterator_t *taken = variant_map_takeFirst(&map);
    T_VERIFY(taken == it[0]);
    T_VERIFY(variant_map_iterator_next(taken) == NULL);
    T_VERIFY(variant_map_iterator_prev(taken) == NULL);
    T_VERIFY(variant_map_first(&map) == it[1]);

    return 0;
}

int variant_map_remove_take_last() {
    variant_map_iterator_t *taken = variant_map_takeLast(&map);
    T_VERIFY(taken == it[4]);
    T_VERIFY(variant_map_iterator_next(taken) == NULL);
    T_VERIFY(variant_map_iterator_prev(taken) == NULL);
    T_VERIFY(variant_map_last(&map) == it[3]);

    return 0;
}

int variant_map_remove_take_null() {
    T_VERIFY(variant_map_iterator_take(NULL) == NULL);

    return 0;
}
