#include "cfgfile.h"

#include <debug/sahtrace.h>
#include <errno.h>
#include <libgen.h>
#include <string.h>
#include <stdlib.h>


int cfg_initialize(struct cfg_file_writer *cw)
{
  if (cw->config_path == NULL) {
    SAH_TRACE_ERROR("config path cannot be null");
    goto error;
  }

  if (cw->hostname == NULL) {
    SAH_TRACE_ERROR("hostname cannot be null");
    goto error;
  }
  
  if (cw->log_level == NULL) {
    SAH_TRACE_ERROR("log_level cannot be null");
    goto error;
  }

  if (cw->request_root == NULL) {
    SAH_TRACE_ERROR("request_root cannot be null");
    goto error;
  }

  if (cw->printer_name == NULL) {
    SAH_TRACE_ERROR("printer_name cannot be null");
    goto error;
  }

  if (cw->printer_uri == NULL) {
    SAH_TRACE_ERROR("printer_uri cannot be null");
    goto error;
  }

  char *path_cupsd_conf = (char *)calloc(strlen(cw->config_path) + strlen("cupsd.conf") + 1, sizeof(char));
  if(!path_cupsd_conf) {
    SAH_TRACE_ERROR("Memory full");
    goto error;
  }
  strcpy(path_cupsd_conf, cw->config_path);
  strcat(path_cupsd_conf, "cupsd.conf");

  char *path_printers_conf = (char *)calloc(strlen(cw->config_path) + strlen("printers.conf") + 1, sizeof(char));
  if(!path_printers_conf) {
    SAH_TRACE_ERROR("Memory full");
    free(path_cupsd_conf);
    goto error;
  }
  strcpy(path_printers_conf, cw->config_path);
  strcat(path_printers_conf, "printers.conf");

  if ((cw->fd_cupsd_conf = fopen(path_cupsd_conf, "w")) == NULL) {
    SAH_TRACE_ERROR("unable to open config file %s: %s", path_cupsd_conf, strerror(errno));
    goto error;
  }

  if ((cw->fd_printers_conf = fopen(path_printers_conf, "w")) == NULL) {
    SAH_TRACE_ERROR("unable to open config file %s: %s", path_printers_conf, strerror(errno));
    fclose(cw->fd_cupsd_conf);
    goto error;
  }

  if(cw->interface_name==NULL) {
    cw->interface_name = strdup("bridge");
  }

  if(cw->ip_address==NULL) {
    cw->ip_address = strdup("192.168.1.1");
  }

  if(cw->hostname==NULL) {
    cw->hostname = strdup("LIVEBOX");
  }
  free(path_cupsd_conf);
  free(path_printers_conf);
  return 0;

 error:
  return -1;
}

void cfg_release(struct cfg_file_writer *cw)
{
  fclose(cw->fd_cupsd_conf);
  fclose(cw->fd_printers_conf);
  free(cw->interface_name);
  cw->interface_name = NULL;
  free(cw->ip_address);
  cw->ip_address = NULL;
}

int cfg_write_cupsd_conf(struct cfg_file_writer *cw)
{
  int rc = 0;

  rc = fprintf(cw->fd_cupsd_conf,
               "LogLevel %s\n"
               "SystemGroup root\n"
               "# Allow remote access\n"
               "RequestRoot %s\n"
               "# Enable printer sharing and shared printers.\n"
               "Browsing On\n"
               "BrowseOrder allow,deny\n"
               "BrowseAllow @IF(%s)\n"
               "BrowseAddress @IF(%s)\n"
               "DefaultAuthType Basic\n"
               "AutoPurgeJobs Yes \n"
               "PreserveJobHistory Off \n"
               "PreserveJobFiles Off \n"
               "MaxJobs 1\n"
               "MaxLogSize %d\n"
               "Listen %s:631\n"
               "<Location />\n"
               "# Allow shared printing...\n"
               "  Order allow,deny\n"
               "  Allow from @IF(%s)\n"
               "</Location>\n"
               "<Location /admin>\n"
               "# Restrict access to the admin pages...\n"
               "  Order allow,deny\n"
               "</Location>\n"
               "<Location /admin/conf>\n"
               "  AuthType Default\n"
               "  Require user @SYSTEM\n"
               "# Restrict access to the configuration files...\n"
               "  Order allow,deny\n"
               "</Location>\n"
               "<Policy default>\n"
               "  <Limit Send-Document Send-URI Hold-Job Release-Job Restart-Job Purge-Jobs Set-Job-Attributes Create-Job-Subscription Renew-Subscription Cancel-Subscription Get-Notifications Reprocess-Job Cancel-Current-Job Suspend-Current-Job Resume-Job CUPS-Move-Job>\n"
               "    Require user @OWNER @SYSTEM\n"
               "    Order deny,allow\n"
               "  </Limit>\n"
               "  <Limit CUPS-Add-Modify-Printer CUPS-Delete-Printer CUPS-Add-Modify-Class CUPS-Delete-Class CUPS-Set-Default>\n"
               "    AuthType Default\n"
               "    Require user @SYSTEM\n"
               "    Order deny,allow\n"
               "  </Limit>\n"
               "  <Limit Pause-Printer Resume-Printer Enable-Printer Disable-Printer Pause-Printer-After-Current-Job Hold-New-Jobs Release-Held-New-Jobs Deactivate-Printer Activate-Printer Restart-Printer Shutdown-Printer Startup-Printer Promote-Job Schedule-Job-After CUPS-Accept-Jobs CUPS-Reject-Jobs>\n"
               "    AuthType Default\n"
               "    Require user @SYSTEM\n"
               "    Order deny,allow\n"
               "  </Limit>\n"
               "  <Limit Cancel-Job CUPS-Authenticate-Job>\n"
               "    Require user @OWNER @SYSTEM\n"
               "    Order deny,allow\n"
               "  </Limit>\n"
               "  <Limit All>\n"
               "    Order deny,allow\n"
               "  </Limit>\n"
               "</Policy>\n"
               , cw->log_level, cw->request_root, cw->interface_name, cw->interface_name, cw->log_size, cw->ip_address, cw->interface_name);
  
  if (rc < 0) {
    SAH_TRACE_ERROR("unable to write to cupsd.conf file: %s", strerror(errno));
    goto error;
  }

  return 0;

 error:
  return -1;
}

int cfg_write_printers_conf(struct cfg_file_writer *cw)
{
  int rc = 0;

  rc = fprintf(cw->fd_printers_conf,
               "# Printer configuration file for CUPS\n"
               "<DefaultPrinter %s>\n"
               "Info My Gateway Printer\n"
               "DeviceURI file:%s\n"
               "State Idle\n"
               "StateTime 1238429418\n"
               "Accepting Yes\n"
               "Shared Yes\n"
               "JobSheets none none\n"
               "QuotaPeriod 0\n"
               "PageLimit 0\n"
               "KLimit 0\n"
               "OpPolicy default\n"
               "ErrorPolicy retry-job\n"
               "</Printer>\n"
               , cw->printer_name, cw->printer_uri);

  if (rc < 0) {
    SAH_TRACE_ERROR("unable to write to printers.conf file: %s", strerror(errno));
    goto error;
  }

  return 0;

 error:
  return -1;
}

