
include $(STAGINGDIR)/components.config

PREFIX = /usr
EXEC_PREFIX = $(PREFIX)/sbin

EXTENSIONS=xt_conntrack ipt_LOG ipt_MASQUERADE ipt_REDIRECT ipt_REJECT ipt_SNAT ipt_DNAT\
			ipt_icmp xt_CLASSIFY xt_connmark ipt_ah\
			xt_CONNMARK xt_dscp xt_DSCP xt_mark xt_MARK ipt_addrtype \
			xt_NFQUEUE xt_SKIPLOG xt_standard xt_tcp xt_udp xt_state xt_pkttype \
			xt_physdev xt_TCPMSS xt_mac xt_length xt_limit xt_esp xt_sctp xt_dccp \
			xt_NOTRACK

ifdef CONFIG_IPTABLES_IPV6
EXTENSIONS += ip6t_icmp6 ip6t_LOG ip6t_REJECT ip6t_rt ip6t_ah ip6t_hl
endif

CFLAGS += -I$(STAGINGDIR)/include
LDFLAGS += -L$(STAGINGDIR)/lib

ifdef CONFIG_IPTABLES_XTRIGGER
	EXTENSIONS += ipt_TRIGGER
endif

ifdef CONFIG_IPTABLES_TC_EXTENSIONS
	CFLAGS += -I$(STAGINGDIR)/include/linux/fusiv_src/kernel/drivers/ipqos
	ENABLE_TC_FUSIV = --enable-tc_fusiv
	EXTENSIONS += ipt_fusivTC
ifdef CONFIG_IPTABLES_IPV6
	EXTENSIONS += ip6t_fusivTC
endif
endif

ifdef CONFIG_IPTABLES_TC_IMARK
	ENABLE_TC_IMARK = --enable-tc_imark
	EXTENSIONS += ipt_IMARK
ifdef CONFIG_IPTABLES_IPV6
	EXTENSIONS += ip6t_IMARK
endif
endif

ifdef CONFIG_IPTABLES_DISABLE_IPV4
	DISABLE_IPV4 = --disable-ipv4
endif

ifndef CONFIG_IPTABLES_IPV6
	DISABLE_IPV6 = --disable-ipv6
endif

ifdef ARCH
	TYPE = --host=$(ARCH)-linux
endif

ifdef AUTOCONF_HOST
	TYPE = --host=$(AUTOCONF_HOST)
endif

	TMP=$(addsuffix .so, $(EXTENSIONS))
	IPTEXTENSIONS=$(addprefix lib, $(TMP))

all: Makefile
	$(MAKE) DESTDIR=$(STAGINGDIR) $(@)

Makefile: $(STAGINGDIR)/components.config
	LIBS="-ldl" INSTALL="$(INSTALL) -p" ./configure $(TYPE) --disable-static --enable-shared --disable-largefile --enable-devel \
		--prefix=$(PREFIX) --includedir=/include/ --libdir=/lib/ CFLAGS="$(CFLAGS)" $(DISABLE_IPV4) $(DISABLE_IPV6) $(ENABLE_TC_FUSIV) $(ENABLE_TC_IMARK)


clean:
	if [ -f Makefile ]; then $(MAKE) distclean; fi

install:
	$(MAKE) DESTDIR=$(STAGINGDIR) $(@)
	cp -Rfvp include/net/ include/linux/ $(STAGINGDIR)/include/

release:
	if [ ! -d $(ROOT_FS_DIR)$(EXEC_PREFIX) ]; then \
		$(INSTALL) -m 755 -d $(ROOT_FS_DIR)$(EXEC_PREFIX); \
	fi
	$(INSTALL) -m 755 -d $(ROOT_FS_DIR)/lib
	$(INSTALL) -m 755 $(STAGINGDIR)/lib/libip4tc.so.0.0.0 $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libip4tc.so.0.0.0
	(cd $(ROOT_FS_DIR)/lib; \
		ln -sf libip4tc.so.0.0.0 libip4tc.so.0; \
		ln -sf libip4tc.so.0.0.0 libip4tc.so;)
	$(INSTALL) -m 755 $(STAGINGDIR)/lib/libip6tc.so.0.0.0 $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libip6tc.so.0.0.0
	(cd $(ROOT_FS_DIR)/lib; \
		ln -sf libip6tc.so.0.0.0 libip6tc.so.0; \
		ln -sf libip6tc.so.0.0.0 libip6tc.so;)
	$(INSTALL) -m 755 $(STAGINGDIR)/lib/libiptc.so.0.0.0 $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libiptc.so.0.0.0
	(cd $(ROOT_FS_DIR)/lib; \
		ln -sf libiptc.so.0.0.0 libiptc.so.0; \
		ln -sf libiptc.so.0.0.0 libiptc.so;)
	$(INSTALL) -m 755 $(STAGINGDIR)/lib/libxtables.so.5.0.0 $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libxtables.so.5.0.0
	(cd $(ROOT_FS_DIR)/lib; \
		ln -sf libxtables.so.5.0.0 libxtables.so.5; \
		ln -sf libxtables.so.5.0.0 libxtables.so;)
	$(INSTALL) -m 755 -d $(ROOT_FS_DIR)/usr/libexec/xtables
	(cd $(STAGINGDIR)/usr/libexec/xtables/; \
		cp -a $(IPTEXTENSIONS) $(ROOT_FS_DIR)/usr/libexec/xtables/;)
	$(STRIP) $(ROOT_FS_DIR)/usr/libexec/xtables/*.so

	if [ -z "$(DISABLE_IPV4)" ]; then \
		prog_base=iptables; \
		$(INSTALL) -m 755 \
			$(STAGINGDIR)$(EXEC_PREFIX)/$${prog_base}-multi \
			$(ROOT_FS_DIR)$(EXEC_PREFIX)/$${prog_base}-multi; \
		(cd $(ROOT_FS_DIR)$(EXEC_PREFIX); \
			ln -sf $${prog_base}-multi $${prog_base}; \
			ln -sf $${prog_base}-multi $${prog_base}-save; \
			ln -sf $${prog_base}-multi $${prog_base}-restore); \
		$(STRIP) $(ROOT_FS_DIR)$(EXEC_PREFIX)/iptables-multi; \
	fi
	if [ -z "$(DISABLE_IPV6)" ]; then \
		prog_base=ip6tables; \
		$(INSTALL) -m 755 \
			$(STAGINGDIR)$(EXEC_PREFIX)/$${prog_base}-multi \
			$(ROOT_FS_DIR)$(EXEC_PREFIX)/$${prog_base}-multi; \
		(cd $(ROOT_FS_DIR)$(EXEC_PREFIX); \
			ln -sf $${prog_base}-multi $${prog_base}; \
			ln -sf $${prog_base}-multi $${prog_base}-save; \
			ln -sf $${prog_base}-multi $${prog_base}-restore); \
		$(STRIP) $(ROOT_FS_DIR)$(EXEC_PREFIX)/ip6tables-multi; \
	fi

.PHONY: all clean install release


