# Include this makefile to compile autoconfigured opensource packages
# you need to define the following from the calling Makefile :
#
# AUTO_ARCHIVE       : the source archive to be expanded
# AUTO_SRCPATH       : the sources path once extraction is done
# AUTO_PATCHES       : list of patches to apply to the source tree
# AUTO_RELEASE       : list of files to be copied in the release directory
# AUTO_CONF          : the configuration flags to be passed to the ./configure
# AUTO_ISAUTOCONF    : flag to say if standard autoconf is used or not, default (y)
# AUTO_RECONF        : force enable/disable reconfigure
# AUTO_RUN_CONFIGURE : enable/disable running the ./configure, default (y)
# AUTO_BUILD_INPLACE : flag to say if inplace compilation is to be used default (n)
# AUTO_BUILDUTILS_DIR: location of the buildutils dir (useful when overriding STAGINGDIR)
# AUTO_CONFINSTALL   : list of host-dependent *-config files

AUTO_MK_SAVED_DEFAULT_GOAL := $(.DEFAULT_GOAL)

# newer build system exports [CPP|C|CXX|LD]FLAGS from Makefile.global
BS_HAS_FLAGS_EXPORT=$(CONFIG_BUILD_MODE_RELWITHDBGINFO)$(CONFIG_BUILD_MODE_RELEASE)$(CONFIG_BUILD_MODE_DEBUG)

# Will unpack again if archive name, archive path or list of patches changes
ECHO=/bin/echo
AUTO_COOKIE_UNPACK_FILE=auto_cookie_unpack
AUTO_COOKIE_UNPACK_MD5=$(shell $(ECHO) $(AUTO_ARCHIVE) $(AUTO_SRCPATH) \
                                    $(AUTO_PATCHES) \
                         | md5sum - | awk '{print $$1}')

# Will reconf again if unpacked
AUTO_COOKIE_RECONF_FILE=auto_cookie_reconf
AUTO_COOKIE_RECONF_MD5=$(shell $(ECHO) $(AUTO_COOKIE_UNPACK_MD5) $(AUTO_RECONF) \
                         | md5sum - | awk '{print $$1}')

# Will configure again if unpacked or if conf changed
AUTO_COOKIE_CONF_FILE=auto_cookie_conf
# CONFIG_CROSS_COMPILE_CFLAGS and CONFIG_CROSS_COMPILE_LDFLAGS are for legacy build system in DEV/STB
AUTO_COOKIE_CONF_MD5=$(shell $(ECHO) $(AUTO_COOKIE_RECONF_MD5) $(AUTO_CONF) \
                                  $(CC) $(CXX) \
                                  $(CFLAGS) \
                                  $(LDFLAGS) \
                                  $(CONFIG_CROSS_COMPILE_CFLAGS) \
                                  $(CONFIG_CROSS_COMPILE_LDFLAGS) \
                       | md5sum - | awk '{print $$1}')

ifeq ($(AUTO_BUILD_INPLACE),y)
AUTO_BUILDDIR=$(AUTO_SRCPATH)
else
AUTO_BUILDDIR=build
endif

# By default patches are applied from the source path
AUTO_PATCHFROM?=$(AUTO_SRCPATH)
# By default archives are unpacked from the component path
AUTO_UNPACKFROM?=

CONFIG_SYSTEM_TOOLS_BUILDUTILS_FORCE_AUTO_RECONF?=
AUTO_RECONF?=$(CONFIG_SYSTEM_TOOLS_BUILDUTILS_FORCE_AUTO_RECONF)

AUTO_RUN_CONFIGURE?=y

AUTO_CONFINSTALL?=

AUTO_BUILDUTILS_DIR:=$(STAGINGDIR)/buildutils
export ACLOCAL:=$(AUTO_BUILDUTILS_DIR)/fixed-aclocal

# do not export empty FLAGS variables
# otherwise configure script will not use their internal defaults
ifeq (,$(strip $(CPPFLAGS)))
unexport CPPFLAGS
endif
ifeq (,$(strip $(CFLAGS)))
unexport CFLAGS
endif
ifeq (,$(strip $(CXXFLAGS)))
unexport CXXFLAGS
endif
ifeq (,$(strip $(LDFLAGS)))
unexport LDFLAGS
endif

ifeq (,$(BS_HAS_FLAGS_EXPORT))
# for legacy build system in DEV/STB
ifdef CONFIG_CROSS_COMPILE_CPPFLAGS
export CONFIG_CROSS_COMPILE_CPPFLAGS
endif
ifdef CONFIG_CROSS_COMPILE_CFLAGS
export CONFIG_CROSS_COMPILE_CFLAGS
endif
ifdef CONFIG_CROSS_COMPILE_CXXFLAGS
export CONFIG_CROSS_COMPILE_CXXFLAGS
endif
ifdef CONFIG_CROSS_COMPILE_LDFLAGS
export CONFIG_CROSS_COMPILE_LDFLAGS
endif
endif

include $(AUTO_BUILDUTILS_DIR)/tmpdir.mk
include $(AUTO_BUILDUTILS_DIR)/mk/common.mk

auto_all: auto_make

auto_check:
	@# Check mandatory vars
	@[ "$(AUTO_ARCHIVE)" ]
	@[ "$(AUTO_SRCPATH)" ]

auto_unpack: auto_check
	@# Cookie does not exist, or is different
	@# Unpack
	@# Apply patches
	@# Reconfigure if patches applied
	@set -e ; \
	if    [ ! -f "$(AUTO_COOKIE_UNPACK_FILE)" ] \
	   || [    "$$(cat $(AUTO_COOKIE_UNPACK_FILE))" \
	        != "$(AUTO_COOKIE_UNPACK_MD5)" ] ; \
	then \
		[ -d "$(AUTO_SRCPATH)" ] && rm -rf "$(AUTO_SRCPATH)" ; \
		printf "\nUnpacking %s in %s...\n" $(AUTO_ARCHIVE) $(AUTO_SRCPATH) ; \
		auto_comproot="$$(pwd)" ; \
		if [ "$(AUTO_UNPACKFROM)" ] ; \
		then \
			mkdir -p "$(AUTO_UNPACKFROM)" ; \
			cd "$(AUTO_UNPACKFROM)" ; \
		fi ; \
		case "$(AUTO_ARCHIVE)" in \
			*.zip) unzip   "$$auto_comproot/$(AUTO_ARCHIVE)" ;; \
			*)     tar -xf "$$auto_comproot/$(AUTO_ARCHIVE)" ;; \
		esac ; \
		if [ "$(AUTO_UNPACKFROM)" ] ; \
		then \
			cd - ; \
		fi ; \
		if [ "$(AUTO_PATCHES)" ] ; \
		then \
			printf "\nPatching...\n" ; \
			cd "$(AUTO_PATCHFROM)" ; \
			for file in $(AUTO_PATCHES) ; \
			do \
				printf " - %s\n" $$file ; \
				patch -p1 -g0 -i "$$auto_comproot/$$file" ; \
			done ; \
			cd - ; \
		fi ; \
		printf "Done" ; \
		printf "$(AUTO_COOKIE_UNPACK_MD5)" \
		        > "$(AUTO_COOKIE_UNPACK_FILE)" ; \
	fi

auto_reconf: auto_unpack
	@set -e ; \
	if    [ ! -f "$(AUTO_COOKIE_RECONF_FILE)" ] \
	   || [    "$$(cat $(AUTO_COOKIE_RECONF_FILE))" \
	        != "$(AUTO_COOKIE_RECONF_MD5)" ] ; \
	then \
		if [ "$(AUTO_RECONF)" = "y" ] ; \
		then \
			cd "$(AUTO_SRCPATH)" ; \
			printf "\nRunning autoreconf...\n" ; \
			$(AUTO_BUILDUTILS_DIR)/sah-autoreconf ; \
			printf "Done\n" ; \
			cd - ; \
		fi ; \
		printf "$(AUTO_COOKIE_RECONF_MD5)" \
		        > "$(AUTO_COOKIE_RECONF_FILE)" ; \
	fi

AUTO_ISCROSSCOMPILE?=y
CONFIG_SYSTEM_TOOLS_BUILDUTILS_DISABLE_RPATH_LINK?=

ifeq ($(CONFIG_X86_32),y)
AUTO_HOST = i586-linux-gnu
else
AUTO_HOST = $(shell $(CC) -dumpmachine)
endif

auto_builddir: auto_reconf
	if    [ ! -d "./$(AUTO_BUILDDIR)" ] ; \
	then \
		mkdir ./$(AUTO_BUILDDIR) ; \
	fi

expand_if_defined = $(if $(filter undefined,$(origin $(1))),,$(2) ; )

auto_conf: auto_builddir
	@set -e ; \
	if    [ ! -f "$(AUTO_COOKIE_CONF_FILE)" ] \
	   || [    "$$(cat $(AUTO_COOKIE_CONF_FILE))" \
	        != "$(AUTO_COOKIE_CONF_MD5)" ] ; \
	then \
		cd $(AUTO_BUILDDIR) ; \
		CONF="" ; \
		if [ "$(AUTO_ISAUTOCONF)" != "n" ] ; \
		then \
			if    [ "$(AUTO_ISCROSSCOMPILE)" = "y" ] \
			   && [ -e "../$(AUTO_SRCPATH)/config.guess" ] ; \
			then \
				BUILD="$$($(SHELL) ../$(AUTO_SRCPATH)/config.guess)" ; \
				CONF="$$CONF --build=$$BUILD" ; \
			fi ; \
			CONF="$$CONF --host=$(AUTO_HOST)" ; \
			CONF="$$CONF --prefix=/usr" ; \
			CONF="$$CONF --sysconfdir=/etc --localstatedir=/var" ; \
			CONF="$$CONF --disable-docs --disable-nls" ; \
		fi ; \
		CONF="$$CONF $(AUTO_CONF)" ; \
		printf "\nConfigure options: $$CONF\n\n" ; \
		export CC="$(CC)" ; \
		export CXX="$(CXX)"	; \
		$(call expand_if_defined,CPPFLAGS, CPPFLAGS="$(CPPFLAGS)" ) \
		$(call expand_if_defined,CFLAGS,   CFLAGS="$(CFLAGS)" ) \
		$(call expand_if_defined,CXXFLAGS, CXXFLAGS="$(CXXFLAGS)" ) \
		$(call expand_if_defined,LDFLAGS,  LDFLAGS="$(LDFLAGS)" ) \
		if [ "$(AUTO_ISCROSSCOMPILE)" = "y" ] ; \
		then \
			CPPFLAGS="$$CPPFLAGS -isystem $(STAGINGDIR)/include" ; \
			CPPFLAGS="$$CPPFLAGS -isystem $(STAGINGDIR)/usr/include" ; \
			LDFLAGS="$$LDFLAGS -L$(STAGINGDIR)/lib" ; \
			LDFLAGS="$$LDFLAGS -L$(STAGINGDIR)/usr/lib" ; \
			if [ "$(CONFIG_SYSTEM_TOOLS_BUILDUTILS_DISABLE_RPATH_LINK)" != "y" ] ; \
			then \
				LDFLAGS="$$LDFLAGS -Wl,-rpath-link,$(STAGINGDIR)/lib" ; \
				LDFLAGS="$$LDFLAGS -Wl,-rpath-link,$(STAGINGDIR)/usr/lib" ; \
			fi; \
		fi ; \
		if [ -z "$(BS_HAS_FLAGS_EXPORT)" ] ; \
		then \
			$(call expand_if_defined,CONFIG_CROSS_COMPILE_CPPFLAGS, CPPFLAGS="$$CPPFLAGS "$(CONFIG_CROSS_COMPILE_CPPFLAGS) ) \
			$(call expand_if_defined,CONFIG_CROSS_COMPILE_CFLAGS,   CFLAGS="$$CFLAGS "$(CONFIG_CROSS_COMPILE_CFLAGS) ) \
			$(call expand_if_defined,CONFIG_CROSS_COMPILE_CXXFLAGS, CXXFLAGS="$$CXXFLAGS "$(CONFIG_CROSS_COMPILE_CXXFLAGS) ) \
			$(call expand_if_defined,CONFIG_CROSS_COMPILE_LDFLAGS,  LDFLAGS="$$LDFLAGS "$(CONFIG_CROSS_COMPILE_LDFLAGS) ) \
			: ; \
		fi ; \
		export CPPFLAGS ; \
		export CFLAGS ; \
		export CXXFLAGS ; \
		export LDFLAGS ; \
		export CONFIG_SHELL="$(SHELL)" ; \
		export CONFIG_SITE="$${CONFIG_SITE:-$(AUTO_BUILDUTILS_DIR)/config.site.dummy}" ; \
		if [ "$(filter 1 2,$(V))" ] ; then \
		printf "Environment variables: " ; \
		printf "CC           %s\n" $$CC ; \
		printf "CXX          %s\n" $$CXX ; \
		printf "CPPFLAGS     %s\n" $${CPPFLAGS-[unset]} ; \
		printf "CFLAGS       %s\n" $${CFLAGS-[unset]} ; \
		printf "CXXFLAGS     %s\n" $${CXXFLAGS-[unset]} ; \
		printf "LDFLAGS      %s\n" $${LDFLAGS-[unset]} ; \
		fi ; \
		"../$(AUTO_SRCPATH)/configure" $$CONF ; \
		cd - ; \
		printf "$(AUTO_COOKIE_CONF_MD5)" \
		        > "$(AUTO_COOKIE_CONF_FILE)" ; \
	fi

ifeq ($(AUTO_RUN_CONFIGURE),y)
auto_make: auto_conf
endif

auto_make: auto_builddir
	cd ./$(AUTO_BUILDDIR) && $(MAKE) $(AUTO_MAKEOPTS)

#IDESTPREFIX:=$(shell $(ECHO) $$PWD)/$(AUTO_BUILDDIR)/dest
IDESTPREFIX:=$(realpath .)/$(AUTO_BUILDDIR)/dest
TMPDIR_SRC:=$(IDESTPREFIX)/
TMPDIR_TEMPFILE:=$(IDESTPREFIX).list
TMPDIR_DST:=$(STAGINGDIR)

auto_subinstall:
	rm -rf "$(TMPDIR_SRC)"
	mkdir -p "$(TMPDIR_SRC)"
	# Install in dest
	$(MAKE) -C ./$(AUTO_BUILDDIR) install DESTDIR="$(TMPDIR_SRC)"
auto_cleaninstall: auto_subinstall
	@# Clean dest
	@# ..."*.la"
	@find "$(TMPDIR_SRC)" -name "*.la" -exec rm {} \;
	@# ...man and doc
	@rm -rf "$(TMPDIR_SRC)/share/man" \
	        "$(TMPDIR_SRC)/usr/share/man" \
	        "$(TMPDIR_SRC)/share/doc" \
	        "$(TMPDIR_SRC)/usr/share/doc" \
	        "$(TMPDIR_SRC)/share/gtk-doc" \
	        "$(TMPDIR_SRC)/usr/share/gtk-doc"
tmpdir_install: auto_cleaninstall
auto_wrappers: tmpdir_install
	# Create *-conf wrappers
	set -e ; \
	if [ "$(AUTO_CONFINSTALL)" ] ; \
	then \
		install $(INSTALL_V) -d $(AUTO_BUILDUTILS_DIR)/pkgconfig ; \
		for each in $(AUTO_CONFINSTALL) ; \
		do \
			OUTCONF="$(AUTO_BUILDUTILS_DIR)/pkgconfig/$$(basename $$each)" ; \
			sed -e 's|^prefix=.*$$|prefix=$(STAGINGDIR)/usr|' "$(STAGINGDIR)/$$each" > "$$OUTCONF" ; \
			chmod +x "$$OUTCONF" ; \
		done ; \
	fi
auto_install: tmpdir_install auto_wrappers

auto_release:
	@set -e ; \
	for src in $(addprefix $(STAGINGDIR)/,$(AUTO_RELEASE)) ; \
	do \
		file="$${src#$(STAGINGDIR)}" ; \
		mkdir -p "$$(dirname $(ROOT_FS_DIR)/$$file)" ; \
		if [ ! -L "$$src" ] && $(OBJDUMP) -f "$$src"  > /dev/null 2>&1 ; \
		then \
			echo "OBJCOPY $$file" ; \
			$(OBJCOPY) $(OBJCOPY_V) $(OBJCOPY_STRIP) "$$src" "$(ROOT_FS_DIR)/$$file" ; \
		else \
			echo "COPY $$file" ; \
			cp -af "$$src" "$(ROOT_FS_DIR)/$$file" ; \
		fi ; \
	done

auto_clean:
	rm -f auto_*
	rm -rf $(AUTO_SRCPATH)
	rm -rf $(AUTO_BUILDDIR)
	rm -rf $(TMPDIR_SRC)
	rm -f $(TMPDIR_TEMPFILE)

#Make sure we don't set the default goal (if we are included before it's defined)
.DEFAULT_GOAL := $(AUTO_MK_SAVED_DEFAULT_GOAL)
