
# This Makefile is based on Paul D. Smith (GNU make maintainer) system
# More information on http://make.paulandlesley.org/multi-arch.html
# (You may have to use archive.org)

CC ?= $(CROSS_COMPILE)gcc # MKCONF cross compiler
CXX ?= $(CROSS_COMPILE)g++

CCMACHINE := $(shell $(CC) -dumpmachine)
CCVERSION := $(shell $(CC) -dumpversion)

DEBUG?=0
PROFILING?=0
ifeq ($(DEBUG),1)
CCKIND=debug
else
ifeq ($(PROFILING),1)
CCKIND=profiling
else
CCKIND=release
endif
endif

BUILDDIR ?= build/$(CCMACHINE)/$(CCVERSION)/$(CCKIND) # MKCONF build directory
DESTDIR ?= dest/$(CCMACHINE)/$(CCVERSION)/$(CCKIND) # MKCONF install directory

REL_BUILDDIR:=build/$(CCMACHINE)/$(CCVERSION)/$(CCKIND)
REL_DESTDIR:=dest/$(CCMACHINE)/$(CCVERSION)/$(CCKIND)

CP ?= cp
INSTALL ?= install

ifneq ($(filter %$(BUILDDIR),$(CURDIR)),)

# We are in the build directory, include target
include $(MKDIR)/scan.mk

else

# We are not in the build directory

# Disable implicit rules
.SUFFIXES:

# Compute absolute path before entering the sub-makefile
override BUILDDIR:=$(abspath $(BUILDDIR))
override DESTDIR:=$(abspath $(DESTDIR))
override CONF:=$(if $(CONF),$(abspath $(CONF)),)
override MULTIARCH_TARGET_MK:=$(abspath $(MULTIARCH_TARGET_MK))

THIS_MAKEFILE:=$(abspath $(patsubst $(CURDIR)/%,%,$(lastword $(MAKEFILE_LIST))))

MAKECMDGOALS?=
SUBMAKECMDGOALS:=$(filter-out clean,$(MAKECMDGOALS))

# $(BUILDDIR) is a *target* to run the sub-make
.PHONY: $(BUILDDIR)
$(BUILDDIR):
	+@if [ -z "$(MAKECMDGOALS)" -o -n "$(SUBMAKECMDGOALS)" ] ; \
	  then \
	  [ -d "$@" ] || mkdir -p "$@" ; \
	  $(MAKE) \
	      --warn-undefined-variables \
	      --no-print-directory -C $@ \
	      -f $(THIS_MAKEFILE) \
	      MKDIR=$(MKDIR) \
	      BASEDIR=$(CURDIR) \
	      DESTDIR=$(DESTDIR) \
	      CONF=$(CONF) \
	      MULTIARCH_TARGET_MK=$(MULTIARCH_TARGET_MK) \
	      $(SUBMAKECMDGOALS) ; \
	  fi

all: $(BUILDDIR)
install: $(BUILDDIR)
release: $(BUILDDIR)
Makefile : ;
%.mk :: ;
#% :: $(BUILDDIR) ; @:

.PHONY: clean
clean:
	rm -rf $(BUILDDIR)
	@if [ "$(wildcard add-on/*/*)" ] ; \
	then \
		for d in $(wildcard add-on/*/*) ; \
		do \
			$(MAKE) --no-print-directory -f $(MKDIR)/main.mk -C "$$d" clean ; \
		done ; \
	fi

endif

