
# If BASEDIR is not set by multiarch.mk (stand-alone?)
BASEDIR?=.

# Define CPPFLAGS and TARGET_ARCH for generic gcc rules
CPPFLAGS?=
CFLAGS?=
CXXFLAGS?=
LDFLAGS?=
TARGET_ARCH?=

SCAN_MK_SAVED_CPPFLAGS := $(CPPFLAGS)
SCAN_MK_SAVED_CFLAGS   := $(CFLAGS)
SCAN_MK_SAVED_CXXFLAGS := $(CXXFLAGS)
SCAN_MK_SAVED_LDFLAGS  := $(LDFLAGS)

#----- FUNCTIONS -----
,=,# To allow commas as $(,)
echolist=$(shell $(ECHO) 1>&2 \
           '$(1) $(if $(2),$(addprefix \n    [,$(addsuffix ],$(2))),None)')
echoone=$(shell $(ECHO) 1>&2 '$(1)')
defined_and_not_empty = $(and $(filter-out $(origin $(1)),undefined),$($(1)))
define newline


endef
#newlines are replaced by spaces by make, so we must cheat to still have separated lines for eval
eval_shell = $(eval $(subst %,$(newline),$(shell $(1) | tr '\n' '%')))
#----- ----- ----- -----

name?=$(shell $(ECHO) $(BASEDIR)/ | sed -r -e 's|^.*/([^/]+)/*$$|\1|')
type?=lib

-include $(BASEDIR)/mk.spec

ifndef out
ifeq ($(type),bin)
out:=$(name).bin
else ifeq ($(type),lib)
out:=lib$(name).so
else ifeq ($(type),dllib)
out:=lib$(name).so
else ifeq ($(type),staticlib)
out:=lib$(name).a
else ifeq ($(type),resource)
out:=
endif
endif

DEBUG?=0

ifneq ($(DEBUG),1)
RESOURCEKIND=release
else
RESOURCEKIND=debug
endif


CORE_SRC_DIRS:=$(wildcard $(BASEDIR)/src/     $(BASEDIR)/core/src/)
CORE_SRC_DIRS:=$(shell find $(CORE_SRC_DIRS) -type d)
CORE_INC_DIRS:=$(wildcard $(BASEDIR)/include/ $(BASEDIR)/core/include/)

RESOURCEDIR:=$(BASEDIR)/resource/$(CCMACHINE)/$(CCVERSION)
RES_LIB_DIRS:=$(wildcard $(RESOURCEDIR)/$(RESOURCEKIND)/lib/)
RES_INC_DIRS:=$(wildcard $(RESOURCEDIR)/$(RESOURCEKIND)/include/)

AVAILABLE_IMPLS:=$(shell ls -d $(BASEDIR)/add-on/*/src $(BASEDIR)/add-on/*/*/src 2>/dev/null | sed -e 's|^$(BASEDIR)/add-on/||' | while read dir ; do addon="$$(dirname $$dir)" ; parent="$$(dirname $$addon)" ; [ "$$parent" = "." ] && $(ECHO) "./$$addon" || $(ECHO) "$$addon" ; done)
AVAILABLE_FEATURES:=$(shell for dir in $(AVAILABLE_IMPLS) ; do $(ECHO) $$(dirname $$dir) ; done | sort | uniq)

# Add staging dir includes if it is defined
STAGINGDIR?=
ifdef STAGINGDIR
-include $(STAGINGDIR)/components.config
STAGE_LIB_DIRS:=$(wildcard $(STAGINGDIR)/lib $(STAGINGDIR)/usr/lib $(STAGINGDIR)/usr/local/lib)
STAGE_INC_DIRS:=$(wildcard $(STAGINGDIR)/include $(STAGINGDIR)/usr/include $(STAGINGDIR)/usr/local/include)
else
STAGE_LIB_DIRS?=
STAGE_INC_DIRS?=
endif

V?=0
ifneq ($(filter $(strip $(V)),2),)
$(call echoone,Summary:)
$(call echoone,* Name: $(name)$(,) type: $(type)$(,) out: $(out))
$(call echolist,* Core src dirs:,$(CORE_SRC_DIRS))
$(call echolist,* Core inc dirs:,$(CORE_INC_DIRS))
$(call echolist,* Resource lib dirs:,$(RES_LIB_DIRS))
$(call echolist,* Resource inc dirs:,$(RES_INC_DIRS))
ifdef STAGINGDIR
$(call echolist,* Staging inc dirs:,$(STAGE_INC_DIRS))
$(call echolist,* Staging lib dirs:,$(STAGE_LIB_DIRS))
endif
$(call echoone,* Build dir: $(CURDIR))
$(call echoone,* Add-ons:,)
$(foreach feature,$(AVAILABLE_FEATURES),$(call echoone,  - "$(feature)": $(foreach i,$(filter $(feature)/%,$(AVAILABLE_IMPLS)),$(if $(filter $(addprefix $(feature)/,$(value ADDON_$(feature))),$(i)),[$(notdir $(i))],$(notdir $(i))))))
$(call echoone,)
endif

# Check Add-on selection
$(foreach feature,$(AVAILABLE_FEATURES), \
    $(if $(filter-out $(AVAILABLE_IMPLS),$(addprefix $(feature)/,$(value ADDON_$(feature)))), \
        $(if $(filter "file", ($(origin ADDON_$(feature)))), \
            $(error Invalid implementation choice for feature "$(feature):$(value ADDON_$(feature))" in "$(notdir $(value $BASEDIR))"), \
            $(call echoone, Skipping unavailable implementation choice for feature "$(feature):$(value ADDON_$(feature))" in "$(notdir $(value BASEDIR))"))))

# Get Add-on list. skip any requested implementation that is not available
ADDONS:=$(foreach feature,$(AVAILABLE_FEATURES), \
    $(foreach active_imp,$(value ADDON_$(feature)), \
        $(if $(filter $(AVAILABLE_IMPLS),$(addprefix $(feature)/,$(value active_imp))), \
            $(addprefix $(feature)/,$(value active_imp)))))

# Read mk.spec vars from addon X in ADDON_X_type/ADDON_X_name
$(foreach addon,$(ADDONS),\
    $(call eval_shell,[ -f "$(BASEDIR)/add-on/$(addon)/mk.spec" ] && sed -n 's!^\(type\|name\)=!ADDON_$(addon)_\0!p' "$(BASEDIR)/add-on/$(addon)/mk.spec"))

# Check Add-on vars
$(foreach addon, $(ADDONS),\
    $(if $(call defined_and_not_empty,ADDON_$(addon)_type),,$(error "type" is not defined in $(BASEDIR)/add-on/$(addon)/mk.spec)))
$(foreach addon, $(ADDONS),\
    $(if $(call defined_and_not_empty,ADDON_$(addon)_name),,$(error "name" is not defined in $(BASEDIR)/add-on/$(addon)/mk.spec)))

# The build directory for add-ons is the same than our
ADDONS_BUILDDIR:=$(REL_BUILDDIR)

# Add-on flags
ADDONS_CPPFLAGS+=$(addprefix -I$(BASEDIR)/add-on/,$(addsuffix /include,$(ADDONS)))
ADDONS_LDFLAGS+=$(addprefix -L$(BASEDIR)/add-on/,$(addsuffix /$(ADDONS_BUILDDIR),$(ADDONS)))
# Dynamic libs
ADDONS_LIBS+=$(foreach addon,$(ADDONS),$(if $(filter $(ADDON_$(addon)_type),lib),-l$(ADDON_$(addon)_name)))
# Static libs
ADDONS_LIBS+=$(foreach addon,$(ADDONS),$(if $(filter $(ADDON_$(addon)_type),staticlib),-Wl$(,)--whole-archive$(,)-l$(ADDON_$(addon)_name)$(,)--no-whole-archive))

# Add core source dirs
# > Use this to put all .o in a single folder.
# > This fails if two files have the same name
# > vpath %.c $(CORE_SRC_DIRS)
VPATH+=$(BASEDIR)/

# Add core include dir and resource include dir in CFLAGS
CPPFLAGS+=$(addprefix -I,$(CORE_INC_DIRS) $(RES_INC_DIRS))
CPPFLAGS+=$(addprefix -isystem ,$(STAGE_INC_DIRS))

# Add resource lib dir
CONFIG_SYSTEM_TOOLS_BUILDUTILS_DISABLE_RPATH_LINK?=
LDFLAGS+=$(addprefix -L,. $(RES_LIB_DIRS) $(if $(STAGINGDIR),$(STAGE_LIB_DIRS)))
ifneq ($(CONFIG_SYSTEM_TOOLS_BUILDUTILS_DISABLE_RPATH_LINK),y)
LDFLAGS+=$(addprefix -Wl$(,)-rpath-link$(,),$(RES_LIB_DIRS) $(if $(STAGINGDIR),$(STAGE_LIB_DIRS)))
endif

# Add add-on flags
SUBADDON_CPPFLAGS?=
SUBADDON_CFLAGS?=
SUBADDON_CXXFLAGS?=
SUBADDON_LDFLAGS?=
ADDONS_CPPFLAGS?=
ADDONS_CFLAGS?=
ADDONS_CXXFLAGS?=
ADDONS_LDFLAGS?=
CPPFLAGS+=$(ADDONS_CPPFLAGS) $(SUBADDON_CPPFLAGS)
CFLAGS+=$(ADDONS_CFLAGS) $(SUBADDON_CFLAGS)
CXXFLAGS+=$(ADDONS_CXXFLAGS) $(SUBADDON_CXXFLAGS)
LDFLAGS+=$(ADDONS_LDFLAGS) $(ADDONS_LIBS) $(SUBADDON_LDFLAGS)

SIMPLE_CCVERSION:=`echo $(CCVERSION) | sed -e "s/\([0-9]\+\)\.\([0-9]\+\).*/\1.\2/"`
ifeq ($(shell echo "$(SIMPLE_CCVERSION) >= 4.2" | bc),1)
CPPFLAGS+=-Wall -Wextra -Wformat=2
CPPFLAGS+=-Wno-error=unused-parameter
LDFLAGS+=-Wl,--no-undefined
endif
ifeq ($(shell echo "$(SIMPLE_CCVERSION) >= 4.5" | bc),1)
CFLAGS+=-Wjump-misses-init -Wno-error=jump-misses-init
endif

ifneq ($(filter $(strip $(V)),2),)
$(info Debug:)
$(info - CPPFLAGS=$(CPPFLAGS))
$(info - LDFLAGS=$(LDFLAGS))
$(info )
endif
# Define objects
# > Use this to put all .o in a single folder
# > This fails if two files have the same name
# > OBJS:=$(patsubst %.c,%.o,$(notdir $(wildcard $(addsuffix /*.c,$(CORE_SRC_DIRS)))))
SRCS_C:=$(subst $(BASEDIR)/,,$(wildcard $(addsuffix /*.c,$(CORE_SRC_DIRS))))
SRCS_CXX:=$(subst $(BASEDIR)/,,$(wildcard $(addsuffix /*.cpp,$(CORE_SRC_DIRS))))
SRCS:=$(SRCS_C) $(SRCS_CXX)
OBJS:=$(shell $(ECHO) $(patsubst %.c,%.o,$(SRCS_C)) $(patsubst %.cpp,%.o,$(SRCS_CXX)) \
              | sed -e 's|/\+|/|g')

ADDONS_RULES:=$(addsuffix .addon,$(ADDONS))
ADDONS_NOTDLLIB_RULES := $(foreach addon,$(ADDONS),$(if $(filter-out $(ADDON_$(addon)_type),dllib),$(addon).addon))
ADDONS_INSTALL:=$(addsuffix .addon-install,$(ADDONS))
ADDONS_RELEASE:=$(addsuffix .addon-release,$(ADDONS))

ALL=$(ADDONS_RULES) $(out)
$(out): $(ADDONS_NOTDLLIB_RULES)

-include $(BASEDIR)/mk.inc
include $(MKDIR)/common.mk
include $(MKDIR)/build.mk

# Define the "all" default rule
.DEFAULT_GOAL := all
.PHONY: all install_headers install_out
all: $(ALL)

PREFIX?=usr/local

INSTALL_TYPE_bin=bin
INSTALL_TYPE_lib=lib
INSTALL_TYPE_staticlib=lib
INSTALL_TYPE_dllib=lib
INSTALL_TYPE=$(value INSTALL_TYPE_$(type))

ifndef install_dir
ifeq ($(type),dllib)
ifdef SUBADDON_PARENT
install_dir:=$(PREFIX)/$(INSTALL_TYPE)/$(SUBADDON_PARENT)
else
install_dir:=$(PREFIX)/$(INSTALL_TYPE)/$(name)
endif
else
install_dir:=$(PREFIX)/$(INSTALL_TYPE)
endif
endif

install: install_headers $(ADDONS_INSTALL)
release:                 $(ADDONS_RELEASE)

ifneq ($(out),)
# Don't install static libs for addons
ifneq ($(type),staticlib)
install: install_out
else ifndef SUBADDON_PARENT
install: install_out
endif
# Don't release static libs
ifneq ($(type),staticlib)
release: $(ROOT_FS_DIR)/$(install_dir)/$(out)
endif
endif

%.addon:
	$(MAKEV)env \
		CPPFLAGS="$(SCAN_MK_SAVED_CPPFLAGS)" \
		CFLAGS="$(SCAN_MK_SAVED_CFLAGS)" \
		CXXFLAGS="$(SCAN_MK_SAVED_CXXFLAGS)" \
		LDFLAGS="$(SCAN_MK_SAVED_LDFLAGS)" \
	$(MAKE) -f $(MKDIR)/main.mk -C $(BASEDIR)/add-on/$(@:.addon=) \
	    SUBADDON_PARENT="$(name)" \
	    SUBADDON_CPPFLAGS="$(addprefix -I,$(CORE_INC_DIRS) $(RES_INC_DIRS))"\
	    SUBADDON_LDFLAGS="$(addprefix -L,$(RES_LIB_DIRS)) $(addprefix -Wl$(,)-rpath-link$(,),$(RES_LIB_DIRS))"

%.addon-install:
	$(MAKEV)$(MAKE) -f $(MKDIR)/main.mk -C $(BASEDIR)/add-on/$(@:.addon-install=) \
	    SUBADDON_PARENT="$(name)" \
	    install

%.addon-release:
	$(MAKEV)$(MAKE) -f $(MKDIR)/main.mk -C $(BASEDIR)/add-on/$(@:.addon-release=) \
	    SUBADDON_PARENT="$(name)" \
	    release

install_out:
	$(INSTALLV)$(INSTALL) $(INSTALL_V) -D $(out) $(DESTDIR)/$(install_dir)/$(out)

$(ROOT_FS_DIR)/$(install_dir)/$(out): $(STAGINGDIR)/$(install_dir)/$(out)
	$(INSTALLV)$(INSTALL) $(INSTALL_V) -d $(ROOT_FS_DIR)/$(install_dir)
	$(OBJCOPYV)$(OBJCOPY) $(OBJCOPY_V) $(OBJCOPY_STRIP) $(STAGINGDIR)/$(install_dir)/$(out) $(ROOT_FS_DIR)/$(install_dir)/$(out)


install_headers:
	$(INSTALLV)$(INSTALL) $(INSTALL_V) -d $(DESTDIR)/$(PREFIX)/include/
	$(INSTALLV)if [ -d "$(BASEDIR)/include" ] ; \
	then \
	find $(BASEDIR)/include/ -type f -name "*.h" \
	| while read f ; \
	do \
		REL="$$($(ECHO) "$$f" | sed -e 's|$(BASEDIR)/*||')" ; \
		$(ECHO) "$$REL" ; \
		$(INSTALL) $(INSTALL_V) -m 644 -D "$$f" "$(DESTDIR)/$(PREFIX)/$$REL" ; \
	done ; \
	fi

