# Include this makefile to compile simple components
#
# Simple components are defined this way:
#  - include/ contains public headers
#  - src/ contains source files (including subfolders)
#  - mk.spec contains basic configuration:
#    * name=<basename>
#    * type=<lib|bin>
#    (if type is lib, the final binary will be lib<basename>.so)
#    (if type is bin, the final binary will be <basename>.bin)
#  - mk.inc contains Makefile code to modify vars, for example:
#    EXTRA_CFLAGS
#    EXTRA_LDFLAGS
#
# Possible configuration:
# SIMPLE_PREFIX: the prefix to use (default /usr)

SIMPLE_MK_SAVED_DEFAULT_GOAL := $(.DEFAULT_GOAL)

export ECHO=/bin/echo -e
SIMPLE_PREFIX?=/usr

export PREFIX=$(SIMPLE_PREFIX)
export WERROR=1

ifneq (,$(CONFIG_BUILD_MODE_RELWITHDBGINFO)$(CONFIG_BUILD_MODE_RELEASE)$(CONFIG_BUILD_MODE_DEBUG))
# MAIN buildsystem
DEBUG=$(if $(filter y,$(CONFIG_BUILD_MODE_DEBUG)),1,0)
else
# old DEV/STB build system
DEBUG=$(if $(filter y,$(CONFIG_DEBUG)),1,0)
EXTRA_CPPFLAGS=$(shell $(ECHO) $(CONFIG_CROSS_COMPILE_CPPFLAGS))
EXTRA_CFLAGS=$(shell $(ECHO) $(CONFIG_CROSS_COMPILE_CFLAGS))
EXTRA_CXXFLAGS=$(shell $(ECHO) $(CONFIG_CROSS_COMPILE_CXXFLAGS))
EXTRA_LDFLAGS=$(shell $(ECHO) $(CONFIG_CROSS_COMPILE_LDFLAGS))
endif

export DEBUG
export EXTRA_CPPFLAGS
export EXTRA_CFLAGS
export EXTRA_CXXFLAGS
export EXTRA_LDFLAGS

include $(dir $(lastword $(MAKEFILE_LIST)))/tmpdir.mk

simple_all:
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk

#IDESTPREFIX:=$(shell $(ECHO) $$PWD)/dest
IDESTPREFIX:=$(realpath .)/dest
TMPDIR_SRC:=$(IDESTPREFIX)/
TMPDIR_TEMPFILE:=$(IDESTPREFIX).list
TMPDIR_DST:=$(STAGINGDIR)

simple_subinstall:
	rm -rf "$(TMPDIR_SRC)"
	mkdir -p "$(TMPDIR_SRC)"
	# Install in dest
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk install \
	        DESTDIR=$(TMPDIR_SRC)
tmpdir_install: simple_subinstall
simple_install: tmpdir_install

simple_release:
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk release

simple_clean:
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk clean
	rm -rf $(TMPDIR_SRC)
	rm -f $(TMPDIR_TEMPFILE)

#Make sure we don't set the default goal (if we are included before it's defined)
.DEFAULT_GOAL := $(SIMPLE_MK_SAVED_DEFAULT_GOAL)
