# This implements the mechanism to copy files from a tmpdir to the final
# staging dir without overwriting header files if not necessary, and cleaning
# .la files in the process

TMPDIR_MK_SAVED_DEFAULT_GOAL := $(.DEFAULT_GOAL)

ECHO=/bin/echo -e 
TMPDIR_SRC?=
TMPDIR_DST?=
TMPDIR_TEMPFILE?=$(shell mktemp -u)
TMPDIR_TEMPFILE:=$(TMPDIR_TEMPFILE) # Apply the shell command now and once

#make sure $(TMPDIR_TEMPFILE) is removed on exit if in /tmp/
ifneq ($(filter /tmp/%,$(TMPDIR_TEMPFILE)),)
.INTERMEDIATE: $(TMPDIR_TEMPFILE)
endif
$(TMPDIR_TEMPFILE):
tmpdir_install: $(TMPDIR_TEMPFILE)

tmpdir_install:
	@# ...dirs
	@cd $(TMPDIR_SRC) && \
	find -type d | sed -e 's|^|$(TMPDIR_DST)/|' | sort | tr '\n' '\0' | xargs -0 mkdir -pv
	@# ...files: use dest.list
	@$(ECHO) -n > $(TMPDIR_TEMPFILE)
	@# ...files: headers
	@cd $(TMPDIR_SRC) && \
	find -type f -name "*.h" | while read file ; \
	do \
		if [ ! -e "$(TMPDIR_DST)/$$file" ] ; \
		then \
			$(ECHO) "$$file" ; \
		elif ! diff -q "$$file" "$(TMPDIR_DST)/$$file" 1> /dev/null; \
		then \
			$(ECHO) "$$file"; \
		fi ; \
	done \
	>> $(TMPDIR_TEMPFILE)
	@# ...files: other files
	@cd $(TMPDIR_SRC) && \
	find -not \( \
		-type d \
		-or \( -type f -name "*.h" \) \
		\) \
	>> $(TMPDIR_TEMPFILE)
	@# ...files: copy now
	@set -e ; \
	if [ "$$(head -n 1 $(TMPDIR_TEMPFILE))" ] ; \
	then \
		cd $(TMPDIR_SRC) && \
		xargs cp -af -v --parents -t "$(TMPDIR_DST)" \
		<$(TMPDIR_TEMPFILE) ; \
	fi

#Make sure we don't set the default goal (if we are included before it's defined)
.DEFAULT_GOAL := $(TMPDIR_MK_SAVED_DEFAULT_GOAL)
